=== modified file 'Makefile'
--- Makefile	2017-01-27 16:31:13 +0000
+++ Makefile	2017-02-02 14:23:26 +0000
@@ -227,7 +227,10 @@
 test+lxd: lxd $(strip $(test-scripts))
 	utilities/isolated-make-test
 
-test: $(strip $(test-scripts))
+test: bin/test.parallel
+	@bin/test.parallel
+
+test-serial: $(strip $(test-scripts))
 	@bin/maas-region makemigrations --dry-run --exit && exit 1 ||:
 	@$(RM) coverage.data .failed
 	$(foreach test,$^,$(test-template);)
@@ -263,8 +266,8 @@
 	@bin/maas-region makemigrations --dry-run --exit && exit 1 ||:
 	@bin/test.rack --stop
 
-test+coverage: export NOSE_WITH_COVERAGE = 1
-test+coverage: test
+test-serial+coverage: export NOSE_WITH_COVERAGE = 1
+test-serial+coverage: test-serial
 
 coverage-report: coverage/index.html
 	sensible-browser $< > /dev/null 2>&1 &
@@ -280,10 +283,12 @@
 	    --title "MAAS r$(revno)" --directory $(@D)
 
 coverage.data:
-	@$(error Use `$(MAKE) test+coverage` to generate coverage data, \
-	    or invoke a test script using the `--with-coverage` flag)
+	@$(error Use `$(MAKE) test-serial+coverage` to generate coverage \
+            data, or invoke a test script using the `--with-coverage` flag)
 
-lint: lint-py lint-py-complexity lint-py-imports lint-js lint-doc lint-rst
+lint: \
+    lint-py lint-py-complexity lint-py-imports \
+    lint-js lint-junk lint-doc lint-rst
 
 pocketlint = $(call available,pocketlint,python-pocket-lint)
 
@@ -333,6 +338,11 @@
 	    '(' -name '*.html' -o -name '*.js' ')' -print0 \
 		| xargs -r0 -n20 -P4 $(pocketlint)
 
+# Sometimes junk gets added to MAAS, like .moved files.
+lint-junk:
+	@if bzr ls --recursive --versioned | egrep '[.]moved$$'; then \
+	    echo "^ Junk found. Please remove it from the tree." >&2 ; fi
+
 # Apply automated formatting to all Python files.
 format: sources = $(wildcard *.py contrib/*.py) src templates twisted utilities etc
 format:
@@ -456,6 +466,7 @@
   lint-css
   lint-doc
   lint-js
+  lint-junk
   lint-py
   lint-py-complexity
   lint-py-imports
@@ -469,10 +480,11 @@
   sudoers
   syncdb
   test
-  test+coverage
   test+lxd
   test-failed
   test-initial-data
+  test-serial
+  test-serial+coverage
 endef
 
 #

=== removed file 'docs/about.rst'
--- docs/about.rst	2016-04-11 16:23:26 +0000
+++ docs/about.rst	1970-01-01 00:00:00 +0000
@@ -1,50 +0,0 @@
-About this documentation
-========================
-
-This is the documentation for Canonical's MAAS software. If you aren't
-sure what that is, you should probably skip everything else and head
-straight to the :ref:`orientation` section where it is explained.
-Like any software though, it can be frustrating if you don't know how
-bits of it work, how to achieve certain goals or what to do when
-things go wrong. Amongst its various sections, this manual aims to
-answer all those questions and plenty more you haven't even thought of
-yet.
-
-
-Getting it
-----------
-
-In a cunning move, the current documentation always lives, and is
-built from, the main MAAS source code (in the top-level ``docs/``
-directory). That means that whatever MAAS package you have installed,
-or even if you are really living life on the edge and have checked out
-a development version from Launchpad, this documentation should be the
-latest and most appropriate version for the software you are running.
-However, it is also possible that there have been additional sections,
-or more helpful and clearer bits added since the package you are using
-was made. For this reason you can always find the latest documentation
-online here: `http://maas.io/`_.
-
-.. _http://maas.io/:
-  http://maas.io
-
-
-Contributing
-------------
-
-If you have some extra information to add, or think you have spotted
-an error or something out of date, we really want to hear about
-it.  Please `File a bug report`_ or `contact us directly`_.  In addition
-you can talk to us on the Freenode IRC channel #maas.
-
-.. _File a bug report:
-  https://bugs.launchpad.net/maas/+filebug
-
-.. _contact us directly:
-  https://launchpad.net/~maas-maintainers/+contactuser
-
-If you see something wrong with this documentation, you can help us fix it.
-Download the source to MAAS by following the instructions in
-:doc:`the hacking guide <hacking>`, make your changes, and propose a merge
-against lp:maas on Launchpad.  The documentation source lives in the top-level
-``docs/`` directory.

=== removed file 'docs/bootsources.rst'
--- docs/bootsources.rst	2016-09-27 22:26:08 +0000
+++ docs/bootsources.rst	1970-01-01 00:00:00 +0000
@@ -1,113 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _bootsources:
-
-Boot images import configuration
-================================
-
-The configuration for where a region downloads its images is defined by
-a set of "sources".  Each "source" defines a Simplestreams repository
-location (``url``) from which images can be downloaded and a
-``keyring_filename`` (or ``keyring_data``) for validating index and image
-signatures from that location.  For each source, you can define a series of
-filters (``selections``) specifying which images should be downloaded from
-that source.
-
-The following example use the MAAS CLI to list the boot sources and the boot
-source selections.  Assuming the CLI ``PROFILE`` is the name of the profile
-under which you're logged in to the server::
-
-    $ maas $PROFILE boot-sources read
-    [
-        {
-            "url": "http://images.maas.io/ephemeral-v3/daily/",
-            "keyring_data": "",
-            "resource_uri": "<url omitted for readability>",
-            "keyring_filename": "/usr/share/keyrings/ubuntu-cloudimage-keyring.gpg",
-            "id": 1
-        }
-    ]
-
-    $ maas $PROFILE boot-source-selections read 1
-    [
-        {
-            "labels": [
-                "release"
-            ],
-            "arches": [
-                "amd64"
-            ],
-            "subarches": [
-                "*"
-            ],
-            "release": "trusty",
-            "id": 1,
-            "resource_uri": "<url omitted for readability>"
-        }
-    ]
-
-
-Restricting the images being downloaded
----------------------------------------
-
-Let's say you want to add a previous LTS release to images being downloaded.
-Starting from the configuration described above, you would need to:
-
-- Add the "Precise" selection (the selection '1' of the source '1')::
-
-    $ maas $PROFILE boot-source-selections create 1 os="ubuntu" release="precise" arches="amd64" subarches="*" labels="*"
-
-After you've selected the additional boot sources you need to tell MAAS to
-start the import process by running the command::
-
-    $ maas $PROFILE boot-resources import
-
-Downloading the images from a different source
-----------------------------------------------
-
-Let's say you want to import the images from a different location.  You would
-need to to change the source's url and keyring::
-
-    $ maas $PROFILE boot-source update 1 url="http://custom.url" keyring_filename="" keyring_data@=./custom_keyring_file
-    {
-        "url": "http://custom.url/",
-        "keyring_data": "<base64 encoded content of `custom_keyring_file`>",
-        "resource_uri": "<url omitted for readability>",
-        "keyring_filename": "",
-        "id": 1
-    }
-
-Adding a source
----------------
-
-You can also add a new source::
-
-    $ maas $PROFILE boot-sources create url=http://my.url keyring_filename="" keyring_data@=./ custom_keyring_file
-    {
-        "url": "http://my.url/",
-        "keyring_data": "ZW1wdHkK",
-        "keyring_filename": "",
-        "id": 2,
-        "resource_uri": "<url omitted for readability>"
-    }
-
-Inside that newly created source ('2') you can add selections::
-
-    $ maas $PROFILE boot-source-selections create 2 os="ubuntu" release="trusty" arches="amd64" subarches="*" labels='*'
-    {
-        "labels": ["*"],
-        "arches": ["amd64"],
-        "subarches": ["*"],
-        "release": "trusty",
-        "id": 3,
-        "resource_uri": "<url omitted for readability>"
-    }
-
-Deleting a source
------------------
-
-Let's say you need to delete the newly added source.
-
-To delete the source::
-
-    $ maas $PROFILE boot-source delete 2

=== modified file 'docs/changelog.rst'
--- docs/changelog.rst	2016-11-04 10:14:36 +0000
+++ docs/changelog.rst	2017-01-31 14:41:46 +0000
@@ -2,954 +2,103 @@
 Changelog
 =========
 
-2.1.1
-=====
-
-Bugs fixed in this release
---------------------------
-
-LP: #1554636    maas serving old image to nodes
-
-LP: #1593991    [2.0b7, API/CLI] allows you to add machines with power_type and no power_parameters
-
-LP: #1602482    [2.0rc2] Incorrect DNS records
-
-LP: #1620478    [2.1, UI] Broken validation on VLAN MTU
-
-LP: #1630636    [2.1 ipv6] YAML error when maas_url has an IPv6 IP
-
-LP: #1630679    [2.1b1] Can't use custom image repository
-
-LP: #1633717    [2.1] DHCP probe needs to be smarter about interface selection to avoid log spam
-
-LP: #1636858    [2.1.1, trunk, bzr5510] Can't enlist machines
-
-LP: #1636861    [2.1.1, trunk, bzr5510] UI error when adding a chassis
-
-LP: #1636992    [2.1] Deleting all boot sources and creating a new boot source, does not update the cache
-
-LP: #1598175    [2.0] If the machine is deployed, I cannot update NIC's nor storage
-
-LP: #1603563    [2.0 RC2] Multiple failures to release nodes
-
-LP: #1606508    [2.1] Failover peers must be IPv4 for use with ISC dhcpd
-
-LP: #1632853    [2.1] Observed neighbours should be avoided when assigning IP addresses
-
-LP: #1633401    [2.1] In device discovery in Settings, remove the header of the first dropdown field (Host discovery and network observation)
-
-LP: #1633452    [2.1] In settings, rename the option Disabled (suppress active scanning) in the Active discovery interval field to Never (disabled).
-
-LP: #1633462    [2.1] In settings in the device discovery section reduce the text of the explanation of the fields.
-
-LP: #1633822    [2.1] Device discovery ignores reverse DNS
-
-LP: #1636250    [2.1] machines allocate API returns a random machine if incorrect - parameters are used
-
-LP: #1636873    [2.1.0] Creating a bond doesn't work and no feedback is provided if fabric in interfaces is 'disconnected'
-
-LP: #1636874    [2.1, Yakkety] Plus '+' button is not visible when you hover over an interface in machine details
-
-LP: #1637009    [2.0,2.1] Node acquisition constraints API documentation needs to be updated to match reality
-
-LP: #1637182    Help and documentation 'list of unicodes' inconsistent
-
-LP: #1637192    [2.0,2.1] Allocate using subnets or not_subnets with space fails
-
-LP: #1637246    MaaS should use configured names for VLAN interfaces
-
-LP: #1638284    [2.1.1pre] Debug logging shown by default in regiond.conf
-
-LP: #1638288    [2.1.1 pre] A lot of repeated logging
-
-LP: #1638589    [2.1] commissioning doesn't find the second address family on the boot interface
-
-LP: #1600328    [2.0rc1, API/CLI] When adding a new machine and no rack controller can connect to the BMC, no error message is displayed.
-
-LP: #1633378    [Device Discovery] Rename the section Header in the Settings page
-
-LP: #1633600    [2.1.1] Docs do not mention the need to mirror bootloaders
-
-LP: #1636251    resolv.conf search path doesn't match the domain for the host
-
-
-2.1.0
-=====
+2.2.0 (beta1)
+=============
 
 Important announcements
 -----------------------
 
-**New MAAS dashboard, now including discovered devices!**
- In MAAS 2.1, administrators will be redirected to the new MAAS dashboard
- after they log in to the Web UI. On the dashboard, administrators are guided
- through where to go to quickly get MAAS up and running. In addition,
- administrators can view hosts that have been discovered on the network, and
- quickly convert them to a device in MAAS.
-
-**Image streams have been upgraded to v3. (Important: update your mirrors!)**
- In order to support the new kernels, MAAS has moved to a new format for image
- streams. Previous releases used stream in “v2” format. Starting from MAAS 2.1,
- the “v3” format image stream will be used.
-
- Users upgrading from earlier versions of MAAS who are using the default images
- URL will be automatically migrated to the new “v3” URL.
-
- For users with custom mirrors, MAAS will not migrate the image URL
- automatically. Users must ensure their image mirrors contain the “v3” stream,
- which is available at http://images.maas.io/ephemeral-v3/. Old images
- downloaded from the “v2” stream will continue to work until the mirror is
- updated, but the MAAS team only supports MAAS 2.1 users using the “v3” stream.
- Please note that bootloaders are now included in the mirror; be sure to mirror
- them in addition to the images.
-
-**New hardware enablement kernel naming convention**
- Starting from MAAS 2.1 and Ubuntu 16.04 "Xenial", MAAS is adhering to a new
- naming convention for hardware enablement kernels. On Xenial and above, MAAS
- will not support HWE kernels with the old naming convention, but it will
- support HWE kernel channel. For example, given Ubuntu 16.04 "Xenial" and
- Yakkety, currently available kernels in MAAS are:
-
-  * ga-<version>
-    The GA, or general availability kernel is the kernel which the Ubuntu
-    release shipped with. For example ‘ga-16.04’ is the default 4.4 kernel
-    which shipped on Ubuntu 16.04 "Xenial". The ga kernel contains all bug and
-    security fixes provided by the Ubuntu archives. Deployments which use the
-    ga kernel will stay at the same kernel version through upgrades until the
-    entire release is upgraded with ‘do-release-upgrade.’
-
-  * hwe-<version>
-    The latest Hardware Enablement Kernel currently available in a given
-    Ubuntu release. As new Hardware Enable Kernels are released with new Ubuntu
-    releases the hwe-<version> kernel will be upgraded up until the next LTS.
-    For example hwe-16.04 is currently the 16.04 GA kernel. Once 16.10 is
-    released the hwe-16.04 kernel will be upgraded to the 16.10 GA kernel. The
-    kernel will continue to be upgraded up until and including the 18.04 GA
-    kernel.
-
-**Commissioning-user-data and pxe/uefi templates no longer available**
- In the past, MAAS stored commissioning-user-data and pxe/uefi templates in
- /etc/maas/templates. As of MAAS 2.1, these templates are no longer available
- under /etc/maas.
-
-Major new features
-------------------
-
-**First user configuration journey (UI)**
- MAAS now provides the ability for administrators to perform some initial
- configuration immediately after they log-in into the MAAS UI for the first
- time. The configuration includes:
-
-  * Ability to change the name of your MAAS
-  * Ability to configure options that affect connectivity:
-    * Option to select an Upstream DNS Server (Optional)
-    * Option to input different Ubuntu Mirrors (Required)
-    * Option to input an external proxy (Optional)
-  * Ability to select additional images to download
-  * Ability to import SSH keys from Launchpad or Github
-
-**Device discovery**
- MAAS will now automatically listen to the network and report any discovered
- devices. Devices are identified when the MAAS rack observes them
- communicating on an attached IPv4 subnet. Discovered devices that do not
- correspond to machines and devices already known to MAAS are shown on the
- dashboard. If a device advertises a hostname using mDNS (such as with avahi
- or Bonjour), MAAS will also present the discovered hostname in the dashboard.
- Using the dashboard, a discovery can quickly be added to MAAS as a device or
- as a network interface to a machine or device.
-
-**Active subnet mapping**
- The device discovery feature was designed to operate passively by default.
- While MAAS will not send any traffic on attached networks for discovery
- purposes unless instructed to, there are two ways to instruct MAAS to map
- your networks:
-
-  * On-demand: administrators can choose to map their subnet using an action
-    on the subnet details page. This action will scan the subnet just once, so
-    that observed devices on that subnet may quickly be seen in the dashboard.
-    This feature is useful after initially installing MAAS, to quickly populate
-    the list of discoveries with active devices on the network.
-
-  * Periodically (recommended): by enabling active discovery on a per-subnet
-    basis, subnets will be scanned at a user-specified interval. (default is
-    every three hours) This allows MAAS to maintain current information about
-    which IP addresses are in use on each subnet.
-
- Before actively mapping any networks, it is recommended that the ‘nmap’
- package be installed on each MAAS rack controller. Doing so results in faster
- scans that require less network traffic. (If ‘nmap’ is not installed, MAAS
- will resort to scanning using the ‘ping’ utility.)
-
-**Offline deployment and customizable APT repositories**
- MAAS 2.1 improves its offline deployment capabilities by adding support for
- Ubuntu derived repositories, PPAs, and custom APT repositories. This enables
- MAAS to configure deployed machines with the correct APT repositories and
- keyrings, without being dependent on Internet connectivity.
-
-  * Ubuntu Derived Repositories (from an Ubuntu Mirror)
-    MAAS 2.0 and earlier versions only allowed users to change the Ubuntu
-    archive to use. This was limited to defining the location of an official
-    Ubuntu mirror.
-
-    Derived repositories are based on an Ubuntu mirror, but have had packages
-    added or removed, which requires signing the repository with an unofficial
-    GPG key. MAAS now allow users to provide GPG key fingerprints to support
-    this type of repository. These fingerprints are required in order for
-    the derived repository to be trusted, and will be added to the APT keyring
-    on each machine.
-
-  * PPAs
-    PPAs can now be specified, which will be added to the APT sources on
-    deployed machines. Users may define a GPG key fingerprint in order for
-    the machine to trust the PPA, for cases where the deployed machine cannot
-    access the Ubuntu key server.
-
-  * Custom repositories
-    Custom repositories can be specified to add additional packages to deployed
-    machines. For custom repositories, the distribution and component can be
-    customized as appropriate. For example, users would be able to add the
-    Google Chrome repository, which is as follows:
-
-    deb http://dl.google.com/linux/chrome/deb/ stable main
-
-    In this case, the distribution is “stable”, and the component is “main”.
-    (Multiple components may also be specified.)
-
-**MAAS time sync, NTP services and configuration**
- MAAS now provides managed NTP services (with ntpd) for all region and rack
- controllers. This allows MAAS to both keep its own controllers synchronized,
- and keep deployed machines synchronized well.
-
-  * Region controllers synchronize time externally
-    The MAAS region controller configures the NTP service (ntpd) to keep its
-    time synchronized from one or more external sources. By default, the MAAS
-    region controller uses ntp.ubuntu.com. This can be customized on the
-    settings page.
-
-  * Rack controllers synchronize time from the region controller
-    The rack controllers also configure the NTP service (ntpd). Unlike the
-    region controllers, rack controllers synchronize their time from region
-    controllers, rather than accessing an external time source.
-
-    Rack controllers also configure DHCP with the correct NTP information, so
-    that any machine on the network that obtains a DHCP lease from MAAS will
-    benefit from the enhanced NTP support.
-
-  * Controllers and Machines can synchronize time for external time sources only.
-    MAAS 2.1 also provides the ability to directly use external time sources
-    without using the Rack Controller as a source of time for machines. This
-    means that administrators who already have their own NTP infrastructure,
-    and they don’t want MAAS to provide NTP services, they can tell all
-    machines and controllers to sync their time from the external time source.
-    This can be done by selecting the "External Only" option on the Settings
-    page.
-
-**Advanced networking: static routes**
- MAAS 2.1 introduces the ability to define static routes. This allows
- administrators to configure reachability to a subnet from a source subnet.
- Administrators can define routes on a per-subnet basis to use a particular
- gateway, using a configured destination and metric.
-
-**Machine networking: bridge configuration**
- MAAS now supports the creation of bridge interfaces. This support is limited
- to the ability to create a bridge against a single interface, such as for the
- purpose of eventually deploying virtual machines or containers on the machine.
-
- Automatic bridge creation on all configured interfaces can also be performed
- at allocation time using the API.
-
-**Rescue mode**
- MAAS 2.1 supports a new state in the machine lifecycle: rescue mode. Rescue
- mode allows users to boot a Deployed or a Broken using an ephemeral image.
- (That is, Ubuntu is running in memory on the machine.) This allows
- administrators to SSH to the machine for maintenance purposes, similar to
- running Ubuntu from a USB stick.
-
-**Enhanced images user interface**
- The MAAS images page has been completely redesigned. Improvements include:
-
-  * Supports selecting the image source (maas.io or custom repository).
-  * Now shows the image releases and architectures available in a repository before the import starts.
-  * Now displays detailed status throughout the image import process.
-  * The Boot Images section in the settings page has been removed.
-
-Minor new features
-------------------
-
-**Disk erasing improvements and secure erase**
- In 1.7 (and later) MAAS introduced the ability to erase disks on machine
- release. This support was limited to erasing the whole disk and could only
- be enabled (or disabled) globally.
-
- Starting from MAAS 2.1, it now supports the ability to request disk erasure
- on a per-machine basis, at the time the machine is released. In addition, new
- options for the disk erase mode have been added:
-
-  * Secure erase - If this option is enabled, MAAS will attempt to erase via
-    secure erase (if the storage device support it), otherwise, it will perform
-    a full erase or a quick erase (depending on the options provided).
-
-  * Quick erase - If this option is enabled, MAAS will only erase the beginning
-    and the end of each storage device.
-
-**Machine networking: - SR-IOV auto-tagging, tags UI**
- MAAS now allows the definition of tags per network interface via the WebUI.
- Additionally, MAAS also attempts to auto-detect and tag SR-IOV NIC cards.
-
-**Support for low latency kernels**
- Starting from Ubuntu 16.04 “Xenial” and later, “low latency” kernels available
- on i386 and amd64 for both GA and HWE kernels. The currently available
- lowlatency kernels are:
-
-  * hwe-x-lowlatency - For using the Xenial Lowlatency kernel on Trusty
-  * ga-16.04-lowlatency - For using the GA lowlatency kernel on Xenial
-  * hwe-16.04-lowlatency - For using the HWE lowlatency kernel on Xenial.
-
-**Bootloaders are now provided in the image stream**
- Previously, bootloaders where downloaded on the rack controller from the
- Ubuntu archives for each architecture MAAS had images for. Starting from MAAS
- 2.1, bootloaders are downloaded with the images. All rack controllers retrieve
- all supported bootloaders from the region controller. MAAS no longer directly
- interacts with the Ubuntu archives.
-
- In the case that bootloaders are missing from the stream, MAAS will attempt
- to locate previous downloads of the bootloader as well as package installs of
- the bootloader. Users with image mirrors must ensure image their mirrors
- include the bootloaders in order to be running the latest supported versions.
-
-**SSH keys can be imported from Launchpad or GitHub**
- All users will now have the ability to import their SSH public keys from the
- UI. Users who log-in to MAAS for the first time will be prompted to import
- their SSH keys, if desired. Alternatively, users can import keys later on
- their user profile page, or continue to upload keys manually.
-
-Other notable changes
----------------------
-
-**Better error surfacing for DHCP snippets and package repositories**
- Both the DHCP Snippets section and the Package Repositories section have been
- improved in order to show errors in a more user-friendly way.
-
-**Vanilla framework: HTML and CSS updates, smoother look and feel**
- The HTML templates and CSS frameworks in MAAS have been completely rebuilt
- with the Vanilla CSS framework. Icons and interactions in MAAS have greatly
- improved; users will notice smoother, more intuitive interactions with the UI.
-
- The MAAS team would like to thank the Canonical design and web teams for their
- contributions in this area.
-
-Issues fixed in this release
-----------------------------
-
-A full list of issues fixed in this release is available in the Launchpad 2.1.0
-Milestone page
-
-  https://launchpad.net/maas/+milestone/2.1.0
-
-
-2.1.0 (RC1)
-===========
-
-Issues fixed in this release
-----------------------------
-
-LP: #1569365    TestPartition.test_get_partition_number_returns_starting_at_2_for_ppc64el fails spuriously
-
-LP: #1598470    "Deployed" and "Deploying" are too similar
-
-LP: #1536354    Users' maas api keys do not have a name
-
-LP: #1631358    [2.1] Incorrect logging message - showing SERVICE_STATE.ON
-
-LP: #1631420    [2.1 UI] Images page "Queued for download" is confusing when selections are not saved
-
-LP: #1631024    [2.1b1] Dashboard column widths for discovered items are wonky
-
-LP: #1631022    [2.1b1] 'Registering existing rack controller'
-
-LP: #1629604    [2.1] Command 'interface link-subnet' does not work
-
-LP: #1628114    [FUJ] SSH input field not indicated for invalid username & the error is incomprehensible
-
-LP: #1629475    [2.1 ipv6] DHCP generation should not fail when address-family conflicts are present
-
-LP: #1603466    [2.0rc2] Commissioning node with gateway_link_v4 set fails.
-
-LP: #1608555    [2.1, 2.0 UI] Error when using dhcp range with pre-existing dynamic reservation
-
-LP: #1632815    [2.1b2] Node failed to be released, because of the following error: 'NoneType' object has no attribute 'addErrback'
-
-LP: #1632395    [2.1, Yakkety, UI] UI error when adding a chassis
-
-LP: #1631079    [2.0, 2.1 UI] Other reserved IP ranges disappear when one of them is deleted on Subnet details page.
-
-LP: #1630667    [2.1b1] MAAS fails to deploy systems with 3+ TB disks
-
-LP: #1630633    [2.1 Yakkety UI] Unable to select nodes
-
-LP: #1629061    [2.0, 2.1] Release and list IPs reserved by another user
-
-LP: #1605476    [2.0rc2] Changing DNSSEC validation does not trigger configuration file update
-
-
-2.1.0 (beta2)
-=============
-
-Issues fixed in this release
-----------------------------
-
-LP: #1630394    [2.1] Bootloaders not downloaded on initial import
-
-LP: #1611949    cryptic error when PXE-boot requires an image not yet imported
-
-LP: #1625676    [2.0, 2.1, UI] MAAS webui allows boot disk to be changed on an already deployed node
-
-LP: #1630591    Rename "Networks" tab to "Subnets"
-
-LP: #1628761    [2.1] netaddr assumes MAC OUI is ascii
-
-LP: #1619262    [2.1, 2.0] Can't input dynamic range on "Enable DHCP" after I deleted the dynamic range
-
-LP: #1630636    [2.1 ipv6] YAML error when maas_url has an IPv6 IP
-
-LP: #1612203    Machine unable to pxe with no-such-image while non-related images are being imported
-
-LP: #1628645    External DHCP detection is broken for a variety of reasons
-
-LP: #1627362    [2.1] expected string or bytes-like object
-
-LP: #1614659    [2.1] When trying to release a node, it gets stuck in releasing if there is no rack controller to power control
-
-LP: #1445941    WebUI needs a filter for deployed OS
-
-
-2.1.0 (beta1)
-=============
-
-Important Announcements
------------------------
-
-**New Hardware Enablement Kernels naming convention**
- Starting from MAAS 2.1 and Ubuntu Xenial, MAAS is adhering to a new naming
- convention for hardware enablement kernels. On Xenial and above, MAAS will
- not support HWE kernels with the old naming convention, but it will support
- HWE kernel channel. For Ubuntu Xenial and Yakkety, currently available
- kernels in MAAS now are:
-
- * ga-<version>
-   The GA, or general availability kernel is the kernel which the Ubuntu
-   release shipped with. For example ‘ga-16.04’ is the default 4.4 kernel
-   which shipped on Ubuntu Xenial. The ga kernel contains all bug and security
-   fixes provided by the Ubuntu archives. Deployments which use the ga
-   kernel will stay at the same kernel version through upgrades until the
-   entire release is upgraded with ‘do-release-upgrade.’
-
- * hwe-<version>
-   The latest Hardware Enablement Kernel currently available in a given
-   Ubuntu release. As new Hardware Enable Kernels are released with new
-   Ubuntu releases the hwe-<version> kernel will be upgraded up until the
-   next LTS. For example hwe-16.04 is currently the 16.04 GA kernel. Once
-   16.10 is released the hwe-16.04 kernel will be upgraded to the 16.10 GA
-   kernel. The kernel will continue to be upgraded up until and including
-   the 18.04 GA kernel.
-
-**New Simplestreams Version - Update your images & your Image repositories**
- In order to support the new kernels, MAAS has updated the version of the
- MAAS Image streams. Previously MAAS has been using the Streams Version 2,
- and as of MAAS 2.1 it will use the MAAS Streams Version 3.
-
- All users who upgrade from an earlier version of MAAS who have been using
- the default image mirror, will be automatically migrated to the new version
- of streams.
-
- For all those users running a custom mirror, MAAS won’t make the migration
- automatically. Users are requested to update their image mirror to match the
- latest images. This image mirror is now available at
- http://images.maas.io/ephemeral-v3/.
-
-Major new features
-------------------
-
-**Support for Low Latency kernels.**
- Starting from MAAS 2.1 Beta 1 and Ubuntu Xenial, MAAS will be making available
- the ability to install low latency kernels. Low latency kernels are available
- on i386 and amd64 for both GA and HWE kernels. The currently available
- lowlatency kernels are
-
-  * hwe-x-lowlatency - For using the Xenial Lowlatency kernel on Trusty
-  * ga-16.04-lowlatency - For using the GA lowlatency kernel on Xenial
-  * hwe-16.04-lowlatency - For using the HWE lowlatency kernel on Xenial.
-
-**Bootloaders are now provided in the SimpleStream.**
- Previously bootloaders where downloaded on the Rack Controller from the Ubuntu
- archives for each architecture MAAS had images for. Starting with MAAS 2.1
- Beta 1 bootloaders are downloaded with the images from the SimpleStream. All
- Rack Controllers retrieve all supported bootloaders from the Region Controller.
- MAAS no longer directly interacts with the Ubuntu archives.
-
- In the case that bootloaders are missing from the SimpleStream MAAS will
- attempt to locate previous downloads of the bootloader as well as package
- installs of the bootloader.
-
-Minor new features
-------------------
-
-**Active Device Discovery - Map your subnet**
- To complete MAAS’ Active Discovery, starting from beta 1 MAAS 2.1 now allows
- the user to ‘Map a subnet’, both automatically at regular intervals, or
- manually:
-
-  * Manually
-    Administrators can now Map a subnet manually from the Subnet Details page
-    under the ‘Take Action’ option. This will allow administrators to map
-    their subnet. By default, this will use ping but if nmap is installed,
-    it will automatically use nmap.
-
-  * Automatically
-    Administrators can now chose to Map their subnets Automatically at regular
-    intervals. This allows administrators to have MAAS always probe on the
-    network to find new devices. By default, the automatic mechanism is enabled,
-    but no subnet is enabled by default.
-
- To automatically map each subnet, please refer to the ‘Active Discovery’
- section on the subnet details page.
-
-Bugs fixed in this release
---------------------------
-
-LP: #1392763    When changing sync-url via the UI, it's not obvious if syncing starts on its own or not
-LP: #1508975    maas deletes products/images locally that do not exist remotely
-LP: #1481285    1.8 Boot images tick boxes should be orange
-LP: #1629402    [2.1] please cleanup log format for new interface monitoring state
-LP: #1629011    Missing punctuation in disk erasing options
-LP: #1629008    Missing preposition sentence disk erasing options
-LP: #1629004    Typo: "futher"
-LP: #1628052    [2.1, FUJ] Help text in input fields is missing 'e.g'
-LP: #1459888    Too much spacing between checkboxes/releases in the 'Images'
-LP: #1627039    [2.1] Discovery object and view doesn't set a flag when the device is the DHCP server
-LP: #1627038    [2.1] SSH key import should use the specified HTTP proxy if one exists
-LP: #1625714    DHCP services on rack controllers only publishes external NTP servers
-LP: #1625711    Peer selection for NTP servers on region controllers is broken
-LP: #1593388    Changing a boot source URL while images are being download doesn't interrupt current downloads to use the new URL
-LP: #1623878    [2.1] mDNS label contains disallowed characters
-LP: #1394792    MAAS could do a better job of reporting image download status
-LP: #1623110    [2.1] Networks page doesn't load fully on yakkety
-LP: #1629896    [2.1] Deployment defaulting to hwe-16.04 instead of ga-16.04
-LP: #1629491    [2.1] After upgrade to latest trunk (pre-beta1) and after updating images, machines fail to pxe boot due to missing hwe-x kernel
-LP: #1629142    2.1 DHCP reported as enabled but not running
-LP: #1629045    [2.1] When failing to download images, MAAS leaves old files in the fs
-LP: #1629022    [2.1, UI] Broken 'images page' link
-LP: #1629019    [2.1 ipv6] log_host needs to be ipv6 when booting ipv6-only
-LP: #1628298    [2.1 UI] SSH keys not listed on user page and no button to add keys
-LP: #1628213    [2.1 yakkety UI] First user journey doesn't display and can't be skipped
-LP: #1627363    [2.1] 'NoneType' object has no attribute 'external_dhcp'
-LP: #1627019    [2.1, rev5385] NTP services on region/rack keep showing as ON/OFF constantly
-LP: #1623634    [2.1, UX] Trying to cancel an image import from the new Images page results on it not being cancelled on the backend.
-LP: #1589640    [2.0b6] MAAS should validate a boot source path actually provides images
-
-Known issues and workarounds
-Trusty images not available on fresh installs
-The MAAS Images V3 streams do not yet have Ubuntu Trusty available. These are currently being built to be made available.
-
-User’s upgrading from a previous version of MAAS that had originally imported Trusty images will continue to be able to deploy Trusty. Once the images are made available, MAAS will automatically update (if so configured).
-
-LP: #1627362 - expected string or bytes-like object
-In some situations after an upgrade, accessing the dashboard might yield error above. This is a difficult to easily reproduce issue. If you come across it please contact a MAAS developer immediately.
-
-https://bugs.launchpad.net/maas/+bug/1627362
-
-
-2.1.0 (alpha4)
-==============
-
-Important Announcements
------------------------
-
-**MAAS Landing page - Let’s see what’s on your network!**
- As of MAAS 2.1 alpha 4, administrative users have a new landing page. Once
- administrators log in they will be redirected to the MAAS dashboard.
-
- This dashboard is where administrators will have some basic information
- and the ability to see the observed and discovered devices.
-
-Major new features
-------------------
-
-**Device discovery UI**
- MAAS 2.1 alpha 4 introduces the MAAS Device Discovery UI. As part of the
- dashboard, administrative users will be able to see all the observed and
- discovered devices.
-
- MAAS will also allow administrator to properly register those discoveries
- as MAAS known devices, and be able to select the IP address allocation for
- them, if MAAS is to manage them.
-
-**Active Device Discovery - map your network (API only)**
- As of MAAS 2.1 alpha 2, networks attached to rack controllers are observed
- for device discovery purposes. MAAS listens to ARP requests and replies to
- determine which IPv4 addresses are in-use on attached networks, and will
- resolve their hostnames if possible (when advertised using the mDNS
- protocol).
-
- As of MAAS 2.1 alpha 4, MAAS now has the ability to actively probe subnets.
- This allows MAAS to keep its knowledge of which devices are on the network
- up-to-date, and discover “quiet” devices that MAAS would not be able to
- observe passively. If ‘nmap’ is installed, MAAS will prefer to use it for
- scanning (since the scan is faster and will transmit fewer packets). If
- ‘nmap’ is not installed, MAAS will fall back to using parallel ‘ping’ requests.
-
- Scanning is available using the API at the following URL:
-
-    POST /MAAS/api/2.0/discovery/?op=scan
-
- To scan using the command-line interface, you can use the following syntax:
-
-    maas <profile> discoveries scan [cidr=<cidr> [cidr=<cidr>....] [force=true] [always_use_ping=true] [slow=true] [threads=<num-concurrent-scanning-threads>]
-
- If you want to scan particular subnets, specify one or more using the cidr
- option. For example, ‘cidr=192.168.0.0/24’ would scan for neighbours on
- 192.168.0.0/24 on any rack controller configured with an address in that
- network. The cidr option can be specified multiple times, such as
- ‘cidr=192.168.0.0/24 cidr=192.168.1.0/24’.
-
- If you want to scan all networks attached to all rack controllers, you must
- specify the “force=true” option. (This is not allowed by default, since some
- network operators do not allow active neighbour scanning.)
-
- If your organization has a policy against using ‘nmap’, you will want to use
- the ‘always_use_ping’ option, in case ‘nmap’ has been installed on a rack
- controller by mistake.
-
- If quickly scanning your network using ‘nmap’ may raise alerts with an
- intrusion detection system, you can use the ‘slow=true’ argument to slow
- down scanning. This option has no effect when using ‘ping’, since scanning
- using ‘ping’ is already slower. If using ‘ping’, scans can be slowed down or
- sped up, if desired, by using the threads option, such as by specifying
- “threads=2”. Using the threads option has less impact on nmap threads, which
- use a single thread to scan an entire network.
-
-Minor new features
-------------------
-
-**First User Journey - Import your SSH keys from Launchpad or Github**
- The ability to import SSH keys from Launchpad or Github was introduced in
- MAAS alpha 3. As of alpha 4, you can do so via the Front-end.
-
- All users will now have the ability to import their SSH keys from the UI.
- All users who log-in to MAAS for the first time will be prompted to import
- their SSH keys, if they so desire. Alternatively, users can do so via their
- user profile page.
-
-Other notable changes
----------------------
-
-**NTP Improvements - MAAS NTP vs External**
- MAAS now provides the ability to decide between using solely an external NTP
- server or a MAAS run NTP server. MAAS run NTP services is the preferred
- configuration, but, in order to maintain backwards compatibility,
- administrators can chose to use external NTP organizations. This will only
- be suitable for scenarios where administrators have restricted communication
- between their machines and the MAAS rack controllers.
-
-Bugs fixed in this release
---------------------------
-
-#1625668    [2.1] When trying to add SSH keys for a GH user that doesn't exist, there's no feedback
-#1626748    [2.1] maas admin discoveries scan API output shows rack controller ids instead of names
-#1626722    [2.1] DHPv6 addresses do not have netmasks: do not create /128 subnets for them
-#1625812    [2.1] Error message is not user friendly
-#1625689    [2.1] default gateway cannot be set to fe80::/64 via web ui
-#1626727    [2.1] You can define distribution or component for 'ubuntu archive' or 'ubuntu extra architectures'
-#1625671    [2.1] Need better error message when trying to add SSH keys for LP/GH user that doesn't exist
-#1623994    [2.1] DHCP configuration breaks when NTP servers are unresolvable.
-#1626669    [2.1] Can't logout, create users and do other actions
-#1625674    [2.1] No feedback when there are no keys to import from LP/GH
-
-Known issues and workarounds
-----------------------------
-
-**LP: #1623634: Unable to cancel the image import.**
- When downloading images, MAAS will fail to cancel the import of all or
- any of the images being imported. MAAS will first download all the images
- before the user is able to remove them.
-
- See bug `1617596`_ for more information.
-
-.. _1617596:
-  http://launchpad.net/bugs/1617596
-
-**LP: 1624693: Rack failed to run/register on fresh install**
- The MAAS Rack Controller is unable to register after a fresh install due to
- being unable to parse network interfaces. After manual restart of maas-rackd,
- the rack was successfully registered.
-
- See bug `1624693`_ for more information.
-
-.. _1624693:
-  http://launchpad.net/bugs/1624693
-
-
-2.1.0 (alpha 3)
-===============
-
-Major new features
-------------------
-
-**First User Configuration Journey (UI)**
- Starting from alpha 3, MAAS now provides the ability for administrators to
- perform some initial configuration when they log-in into the UI for the
- first time. The configuration includes:
-
-  * Ability to change the name of your MAAS.
-  * Ability to configure options that affect connectivity:
-  * Option to select an Upstream DNS Server (Optional)
-  * Option to input different Ubuntu Mirrors (Required).
-  * Option to input an external proxy (Optional)
-  * Ability to select additional images to download.
-
-**MAAS time sync, NTP services and configuration**
- Starting from alpha 3, MAAS now provides managed NTP services (with ntpd) in
- both the Region and Rack controller. This allows MAAS to not only keep its
- own controllers time synced, but the deployed machines as well.
-
- * Region Controller time syncs from external source
-   The Region Controller configures the NTP service (ntpd) to keep its time
-   sync from one or various external sources. By default, the MAAS region
-   controller syncs its time from ntp.ubuntu.com. The default can be changed
-   by one or multiple external NTP servers from the Settings page, under the
-   Network Configuration section.
-
- * Rack Controller time syncs from the Region Controller
-   The Rack Controllers also configure the NTP service (ntpd). Unlike the
-   Region Controllers, the Rack Controllers sync their time from the Region
-   Controller(s) instead of accessing directly to the external time source.
-
-   Additionally, the Rack Controllers also configure DHCP with the correct
-   NTP information, so that any machine on the network that DHCP’s from MAAS
-   can benefit of the NTP configuration.
-
- * Machines configured to sync time from external NTP (transitional).
-   MAAS also configures deployed machines with NTP configuration. This is done
-   by cloud-init via MAAS vendor data.
-
-   During the transition period, MAAS will configure machines to use the
-   external time source (configured under the Settings page). Note that this
-   is transitional, as in future releases the machines will default to the
-   Rack Controller for NTP.
-
-**MAAS Images page re-written in AngularJS**
- Continuing the transition from YUI to AngularJS, the MAAS Images page has now
- been completely redesigned and reimplemented in AngularJS. Improvements to
- the Image page include:
-
- * Ability to select the image source (maas.io or custom repository).
-   Show the releases and architectures available in the custom repository
-   before the import starts.
-
- * Ability to view the status of the image in the import process.
-   Show percentage based progress on the image import.
-
- Additionally, the ‘Boot Images’ section in the Settings page has been removed.
-
-**Minor new features**
-
- * (Backend) Ability to import SSH keys from Launchpad and Github
-   MAAS now provides the ability to import SSH keys for a user from Launchpad
-   and Github. This is currently supported via the API or via the user
-   creation process. Users can import their SSH keys when creating their user
-   for Launchpad or Github:
-
-    maas createadmin --ssh-import lp:<user-id>
-    maas createadmin --ssh-import gh:<user-id>
-
-   Or via the API based CLI with:
-
-    maas <maas username> sshkeys import protocol=lp auth_id=<user-id>
-    maas <maas username> sshkeys import protocol=gh auth_id=<user-id>
-
- * MAAS now provides cloud-init vendor data for NTP
-   As of MAAS 2.1 alpha 3, MAAS now provide cloud-init vendor data. Vendor
-   data allows cloud-init to do some initial configurations on the system
-   before user data is being run. As of 2.1, MAAS will provide NTP
-   configuration which is delivered via vendor data. Note that this is
-   dependent on the latest version of cloud-init (0.7.8-1-g3705bb5-0ubuntu1).
-   This is currently available in Yakkety and is in progress to be available
-   in Xenial.
-
- * Add ability to enable or disable network discovery
-   MAAS now provides the ability to disable the discovery of networks and
-   devices. By default, discovery is enabled. This setting can be changed
-   under the Settings page, or via the MAAS CLI and API using the
-   “network_discovery” configuration key. (Accepted values are “enabled” and
-   “disabled”.) When discovery is disabled, mDNS records and ARP requests will
-   no longer be stored in MAAS, and the listening processes on each rack
-   controller will be shut down.
-
-Other notable changes
----------------------
-
-**HTML template updates**
- In MAAS 2.1 alpha 3, the HTML templates and SCSS framework has been
- completely rebuilt and using the current Vanilla CSS framework as its base.
- This includes all design patterns and utility classes which would be expected
- of a powerful frontend GUI framework.
-
- HTML and CSS templates have been completely redesigned to use the new
- “Vanilla” styles. This brings a refreshed look of icons and interactions that
- benefit the UI. While users may not see much difference, it has been
- completely re-implemented in the backend.
-
- Thank you the design and web teams for their contribution to MAAS.
-
-Known issues and workarounds
-----------------------------
-
-**Unable to cancel the image import.**
- When downloading images, MAAS will fail to cancel the import of all or any of
- the images being imported. MAAS will first download all the images before the
- user is able to remove them.
-
- See bug `1623634`_ for more information.
-
-.. _1623634:
-  http://launchpad.net/bugs/1623634
-
-**Unable to enable DHCP if NTP server is unresolvable.**
- If the NTP server(s) are unresolvable, DHCP will fail to enable. This is
- because DHCP doesn’t accept DNS names for DHCP’s NTP configuration, and as
- such, MAAS tries to resolve the domain before it is able to set it in the
- configuration.
-
- See bug `1623994`_ for more information.
-
-.. _1623994:
-  http://launchpad.net/bugs/1623994
-
-**Rack failed to run/register on fresh install**
- The MAAS Rack Controller is unable to register after a fresh install due to
- being unable to parse network interfaces. After manual restart of maas-rackd,
- the rack was successfully registered.
-
- See bug `1624693`_ for more information.
-
-.. _1624693:
-  http://launchpad.net/bugs/1624693
-
-
-2.1.0 (alpha2)
-==============
-
-Important Announcements
------------------------
-
-**commissioning-user-data and pxe/uefi templates no longer available**
- In the past, MAAS stored commissioning-user-data and pxe/uefi templates
- in `/etc/maas/templates`. As of MAAS 2.1.0 Alpha 2, these templates are
- no longer available under /etc/maas.
-
-Major New Features
-------------------
-
-**(Backend) Device Discovery**
- As of MAAS 2.1.0 Alpha 2, MAAS will automatically listen to the network
- and report any observed devices.
-
-  * New discovery API can be used to get information about what MAAS has
-    discovered. This API can be used from the command line interface as
-    follows:
-
-    * maas <profile> discoveries read - Lists all MAC, IP bindings
-      (discoveries) that MAAS has seen, and attempts to correlate those
-      discoveries with hostnames advertised by mDNS.
-    * maas <profile> discoveries by-unknown-mac - Lists all discoveries,
-      but filters out discoveries where the MAC belongs to an interface
-      known to MAAS.
-    * maas <profile> discoveries by-unknown-ip - Lists all discoveries,
-      but filters out discoveries where the IP address is known to MAAS
-      (such as reserved by a user, or assigned to a node).
-    * maas <profile> discoveries by-unknown-ip-and-mac - Lists all discoveries,
-      but applies the filters for both unknown MACs and unknown IP addresses.
-
-  * Note that the discovery API is currently read-only. It brings together
-    data from several different sources, including observed network neighbours,
-    resolved mDNS hostnames, and controller interface information.
-  * New maas-rack commands have been added, which are called internally by
-    MAAS in order to gather information about observed devices on the network.
-    MAAS administrators should not normally need to use these commands
-    (although they could be helpful for supportability).
-
-    * maas-rack observe-mdns [--verbose]
-    * sudo maas-rack observe-arp <interface> [--verbose]
-
-  * Note: this feature intentionally does not place any network devices into
-    “promiscuous mode”, or actively probe. MAAS controllers listen to ARP
-    traffic they would have received anyway. Therefore, if a MAAS admin wants
-    to keep MAAS’s knowledge of the network up-to-date, a command such as one
-    of the following could be run periodically (such as from a script invoked
-    by a crontab); MAAS will listen to any ARP replies and update its knowledge
-    of the network:
-
-     * To actively probe one or more subnet CIDRs on an interface:
-       sudo nmap -e <interface> -sn -n -oX - -PR <cidr> [cidr2] [...]
-
-     * To actively probe for a single IP address from a particular interface
-       (regardless of whether or not the IP address is routable on-link on that
-       interface):
-       ping -r -I <interface> <ip-address> -c 3 -w 1 -i 0.2 -D -O
-
-  * MAAS now depends on the avahi-utils and tcpdump packages in order to provide
-    this functionality. (Before MAAS 2.1.0 is released, the MAAS team will consider
-    making these optional dependencies, in case MAAS administrators do not want
-    to run the avahi daemon, or require that tcpdump not be installed.)
-
-Important Bugs Fixed in this Release
-------------------------------------
-
-**Bug #1617596: [2.1] Rack(relay) Controller is rejected after upgrade to 2.1**
- Fixes a regression regarding registering rack controllers which have bonds
- interfaces which are not currently bonding any interfaces.
-
- See bug `1617596`_ for more information.
-
-.. _1617596:
-  http://launchpad.net/bugs/1617596
-
-**Bug #1615618: [2.1] 'SERVICE_STATE' object has no attribute 'getStatusInfo'**
- Fixes a regression in the service tracking mechanism, where it would fail to
- successfully track the status of some services.
-
- See bug `1615618`_ for more information.
-
-.. _1615618:
-  http://launchpad.net/bugs/1615618
-
-
-Other Notable Changes
----------------------
-
-**WebUI - Better error surfacing for DHCP snippets and Package Repositories**
- Both the DHCP Snippets Section and the Package Repositories section have now
- been improvement and will surface better errors.
-
-Ongoing Work
-------------
-
- * First User Journery - WebUI
- * Device Discovery - WebUI
- * Improved IPv6 Support
- * MAAS Services - NTP
- * MAAS Image Consolidation
- * Support for HWE Rolling Kernels
-
-Known Issues and Workarounds
-----------------------------
-
-**Configuring APT key’s in ephemeral environment (overlayfs) fails.**
- A regression preventing cloud-init from configuring APT's key in a
- ephemeral environment, prevents MAAS from enlisting, commissioning and
- deploying `only` when using Derived Repositories or Custom Mirrors that
- require a new key.
-
- See bug `1618572`_ for more information.
-
-.. _1618572:
-  http://launchpad.net/bugs/1618572
+**Migrating MAAS L3 to L2 spaces**
+ MAAS 2.2 has changed the definition of spaces from a Layer 3 concept to a
+ Layer 2 concept.
+
+ The spaces definition in MAAS (first introduced in MAAS 1.9) is “a set of
+ subnets that can mutually communicate”. The assumption is that these spaces
+ can route to each other, and have appropriate firewall rules for their
+ purposes. (For example, a dmz space might contain subnets with internet
+ access, and a storage space might contain subnets that can access the same
+ storage networks.) Juju uses the current definition in order to ensure that
+ deployed applications have access to networks appropriate for the services
+ they provide.
+
+ The current definition of spaces as a L3 concept is problematic, in that
+ sometimes Juju wants to deploy applications that themselves create a Layer 3
+ subnet. Therefore, it was decided that the concept of spaces will be pushed
+ down a layer (to apply to VLANs in MAAS).
+
+ With spaces as a Layer 2 concept, it is is now “a set of VLANs whose subnets
+ can mutually communicate”.
+
+ As such, starting from MAAS 2.2b1:
+
+  * VLANs will gain a ‘space’ reference, and subnets will have their spaces
+    migrated to the VLANs they are on. On upgrades, if two subnets on the same
+    VLAN are in different spaces, the most recently created space will be used
+    for both.
+
+  * Spaces will become optional. Fresh installs will not have a default space
+    (e.g. space-0). On upgrades, if only the default space (space-0) exists,
+    it will be removed.
+
+ The following API changes will occur in MAAS 2.2:
+
+  * Editing a subnet's space will no longer be possible (breaks backwards
+    compatibility). Spaces must now be edited each VLAN. For backward
+    compatibility, the subnets endpoint will present the underlying VLAN’s space.
+
+ Recommended actions for MAAS administrators prior to upgrading to MAAS 2.2:
+
+  * Ensure that no two subnets in the same VLAN are in different spaces, so that
+    the upgrade path migrates the expected space to the VLAN. Ensure that each
+    VLAN with an assigned space will contain subnets which can mutually
+    communicate with other subnets whose VLAN is in the same space. This will
+    allow backward compatibility with Juju charms which use the Layer 3 definition
+    of spaces.
+
+ NOTE: Please note that not breakage is expected, provided that most people are not
+ using spaces. For those who we know are, they are using them in a compatible way.
+ If you experience some type of issue, please contact us.
+
+Major new features
+------------------
+
+**DHCP Relay support**
+ The ability to model the usage of DHCP relays in your networking configuration has
+ been added to MAAS. The allows an administrator to identify which VLANs will be
+ relayed through another VLAN running a MAAS DHCP server. This will configure the
+ MAAS DHCP server running on the primary and/or secondary rack controller to include
+ the shared network statement for that VLAN. Note: MAAS does not run a DHCP relay
+ service, it is up to the administrator to configure the DHCP relay service on the
+ VLAN and point it at the primary and/or secondary rack controller running the MAAS DHCP.
+
+**Unmanaged subnets**
+ In MAAS 2.0, the concept of a “static range” (a specific range of addresses in which
+ MAAS was allowed to freely allocate addresses from) was removed from MAAS, in favor
+ of the idea that MAAS managing entire subnets. As such, the only way to tell MAAS to
+ not allocate certain sections of a subnet is to add a reserved IP range.
+
+ Starting from MAAS 2.2b1, however, MAAS enhances this functionality by introducing a
+ new concept, called unamanged subnets. Setting a Subnet in MAAS as unmanaged, allows
+ administrators to prevent MAAS from using that subnet for automatic IP assignment.
+ In other words, it is a way to tell MAAS that it knows about a subnet but that it
+ shouldn’t use it.
+
+Other notable changes
+---------------------
+
+**MAAS is now responsive**
+ For all of those users that use (or would like to use) MAAS WebUI from their Phone
+ or Tablet, will now have a better user experience, provided that starting from
+ 2.2b1, MAAS is now responsive.
+
+ Phone or Table users will see a new slick design for those devices. Thanks for
+ the Ubuntu Web team for putting the effort into making MAAS look great in smaller
+ devices.
+
+Known issues and workarounds
+----------------------------
+
+**Cannot add a device from the dashboard**
+ Please see LP #1659959 for more information.
+
+ https://bugs.launchpad.net/maas/+bug/1659959

=== removed file 'docs/configure.rst'
--- docs/configure.rst	2016-03-28 13:54:47 +0000
+++ docs/configure.rst	1970-01-01 00:00:00 +0000
@@ -1,135 +0,0 @@
-Additional Configuration
-========================
-
-
-Choosing a series to install
-----------------------------
-
-You may have some specific reason to choose a particular version of Ubuntu
-to install on your nodes, perhaps based around package availability,
-hardware support or some other reason.
-
-It is possible to choose a specific series from those available in a
-number of ways.
-
-From the user interface
-^^^^^^^^^^^^^^^^^^^^^^^
-
-The web-based user interface makes it easy to select which Ubuntu series you
-wish to install on an individual node. When either adding a node
-manually, or on the node page when the node has been automatically
-discovered but before it is accepted, there is a drop down menu to select
-the version of Ubuntu you wish to install.
-
-.. image:: media/series.*
-
-The menu will always list all the currently available series according
-to which boot images are available.
-
-Using the maas command
-^^^^^^^^^^^^^^^^^^^^^^
-
-It is also possible to select a series using the maas command. This
-can be done on a per node basis with::
-
- $ maas <profile> machine update <system_id> distro_series="<value>"
-
-Where the string contains one of the valid, available distro series (e.g.
-"trusty") or is empty for the default value.
-
-
-.. _preseed:
-
-Altering the Preseed file
--------------------------
-
-.. warning::
-  Do not try to alter the preseed files if you don't have a good
-  understanding of what you are doing. Altering the installed version
-  of Ubuntu can prevent MAAS from working as intended, and may have
-  security and stability consequences.
-
-When MAAS commissions a node it installs a version of Ubuntu. The
-installation is performed using a 'preseed' file, which is
-effectively a list of answers to the questions you would get were
-you to run the installer manually.
-The preseed file used by MAAS is carefully made so that the
-target node can be brought up and do all the jobs expected of it.
-However, in exceptional circumstances, you may wish to alter the
-pressed file to work around some issue.
-There are actually two preseed files, stored here::
-
-  /etc/maas/preseeds/curtin_userdata
-
-The preseed file is used to customize the installation of a machine
-based on different options. Users can set early_commands or late_commands
-according to what they need, or customize it based on nodes, architecture
-and other variables. The preseeds offers a concept that will allow users
-to configure it as required. This is based on a node's ::
-
-    node.system_id
-    node.hostname
-    node.domain
-    node.owner
-    node.bios_boot_method
-    node.osystem
-    node.distro_series
-    node.architecture
-    node.min_hwe_kernel
-    node.hwe_kernel
-    node.zone
-    node.cpu_count
-    node.memory
-
-You can configure the preseed to add late_commands. For example,
-you can configure the preseed to install a package based on the hostname,
-and after the installation has been completed::
-
-    late_commands:
-    {{if node.hostname == 'node01'}}
-        package_install: ["curtin", "in-target", "--", "apt-get", "-y", "install", "mysql"]
-    {{endif}}
-
-
-Installing Additional Rack Controllers
---------------------------------------
-
-In an environment comprising large numbers of nodes, it is likely that you will
-want to organise the nodes on a more distributed basis. The standard install of
-the MAAS region controller includes a rack controller, but it is
-possible to add additional rack controllers to the configuration, as
-shown in the diagram below:
-
-.. image:: media/orientation_architecture-diagram.*
-
-Each rack controller will need to run on a separate Ubuntu server.
-Installing and configuring the software is straightforward though::
-
-  $ sudo apt-get install maas-rack-controller
-
-This meta-package will install all the basic requirements of the system.
-
-
-Configuring the Rack Controller
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-Follow the instructions at :doc:`rack-configuration` to configure
-additional Rack Controllers.
-
-
-Client-side DNS configuration
------------------------------
-
-When using a third party tool such as ``juju`` it will need to be able to
-resolve the hostnames that the MAAS API returns to it.  In order for this to
-happen, *client-side DNS* must be configured to point to MAAS's DNS
-server.  Generally speaking, this is a simple case of adding the following
-line to the ``/etc/resolv.conf`` file on your client host::
-
-  nameserver <IP OF MAAS DNS HOST>
-
-replacing the <IP OF MAAS DNS HOST> with the actual IP address of the host
-running the MAAS DNS server.
-
-However, for hosts using the ``resolvconf`` package, please read its
-documentation for more information.

=== removed file 'docs/devices.rst'
--- docs/devices.rst	2015-05-20 14:56:13 +0000
+++ docs/devices.rst	1970-01-01 00:00:00 +0000
@@ -1,28 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _devices:
-
-Devices
-========
-
-.. note::
-
-  This feature is available in MAAS versions 1.8 and above.
-  If you're writing a client application, you can check if MAAS
-  supports this feature via the web API; see the documentation for the
-  ``devices-management`` capability :ref:`here<cap-devices-management>`.
-
-In addition to nodes, a MAAS cluster controller can manage *devices*. Devices
-represent non-installable machines. This feature can be used to track
-routers, virtual machines, etc. within MAAS.
-
-Same as nodes, devices can be assigned IP addresses and DNS names. IP
-addresses can be fixed, in which case the device should be configured to use
-the defined IP address, or dynamic, in which case the device can obtain an
-IP address from the MAAS DHCP server and will receive the configured IP
-address.
-
-Devices can also be assigned a parent node and will be automatically deleted
-(along with all the IP address reservations associated with it) when the
-parent node is deleted or released. This is designed to model and manage the
-virtual machines or containers running on a MAAS-deployed node.
\ No newline at end of file

=== removed file 'docs/dhcpsnippets.rst'
--- docs/dhcpsnippets.rst	2016-03-28 16:52:32 +0000
+++ docs/dhcpsnippets.rst	1970-01-01 00:00:00 +0000
@@ -1,111 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _dhcpsnippets:
-
-=========================
-DHCP Snippets
-=========================
-
-.. note::
-
-  This feature is available in MAAS versions 2.0 and above. Modifications made
-  directly to dhcpd.conf.template or dhcpd6.conf.template are no longer
-  supported.
-
-MAAS allows customizing dhcpd.conf and dhcpd6.conf through the use of DHCP
-snippets. DHCP snippets are user defined dhcpd.conf configuration options which
-are inserted into /var/lib/maas/dhcpd.conf and /var/lib/maas/dhcpd6.conf by
-MAAS. Custom dhcpd.conf configuration options can be inserted either globally,
-on a subnet, or on a node. For information about dhcpd.conf options refer to
-the dhcpd.conf man or info page.
-
-Creating a DHCP Snippet
------------------------
-
-Administrators can create DHCP Snippets over the API using the following command:::
-
-  $ maas <profile> dhcpsnippets create name=<DHCP Snippet Name> value=<valid DHCP configuration options>
-
-The name of the DHCP snippet will be added to /var/lib/maas/dhcpd.conf and
-/var/lib/maas/dhcpd6.conf as a comment above the value. Optionally a
-description can also be specified as such::
-
-  $ maas <profile> dhcpsnippets create name=<DHCP Snippet Name> value=<valid DHCP configuration options> description=<DHCP Snippet description>
-
-Global DHCP Snippets
-^^^^^^^^^^^^^^^^^^^^
-
-If no subnet or node is specified, the DHCP Snippet will be considered global.
-A global DHCP Snippet is a configuration option which is applied to all VLANS,
-subnets, and nodes. The global_snippet flag can also be used to force a DHCP
-Snippet to be global::
-
-  $ maas <profile> dhcpsnippets create name=<DHCP Snippet Name> value=<valid DHCP configuration options> global_snippet=true
-
-Subnet DHCP Snippets
-^^^^^^^^^^^^^^^^^^^^
-
-DHCP Snippets can be applied only to a specific subnet as follows::
-
-  $ maas <profile> dhcpsnippets create name=<DHCP Snippet Name> value=<valid DHCP configuration options> subnet=<subnet id or cidr>
-
-Node DHCP Snippets
-^^^^^^^^^^^^^^^^^^
-
-DHCP Snippets can be applied only to a specific node. When a node is specified,
-each snippet will be added to the host entry for each interface. A node can be
-specified as follows::
-
-  $ maas <profile> dhcpsnippets create name=<DHCP Snippet Name> value=<valid DHCP configuration options> node=<system_id or hostname>
-
-DHCP Snippet Enablement
-^^^^^^^^^^^^^^^^^^^^^^^
-
-DHCP Snippets can be turned off by using the enabled flag option as follows::
-
-  $ maas <profile> dhcpsnippets create name=<DHCP Snippet Name> value=<valid DHCP configuration options> enabled=false
-
-Listing DHCP Snippets
----------------------
-
-To list all DHCP Snippets use the following command::
-
-  $ maas <profile> dhcpsnippets read
-
-To list a particular DHCP Snippet use the following command.::
-
-  $ maas <profile> dhcpsnippet read <DHCP Snippet id or name>
-
-Updating a DHCP Snippet
------------------------
-
-Administrators can update the DHCP Snippet attributes using the following
-command::
-
-  $ maas <profile> dhcpsnippet update <DHCP Snippet id or name> <options>
-
-DHCP Snippet Value History
---------------------------
-
-MAAS stores the complete history of changes made to the DHCP Snippet's
-value. MAAS only uses the latest revision of the value when writing
-dhcpd.conf.
-
-Reverting a Value
-^^^^^^^^^^^^^^^^^
-
-.. warning::
-  Reverting a value will result in all later versions being deleted!
-
-The revert operation allows the user to revert to a previous value. When
-specifying what to revert to the user can either provide the value id or a
-negative number representing how many revivisions to go back::
-
-  $ maas <profile> dhcpsnippet revert <DHCP Snippet id or name> to=<value id or negative number>
-
-Deleting a DHCP Snippet
------------------------
-
-Administrators can delete a DHCP Snippet using the following command::
-
-  $ maas <profile> dhcpsnippet delete <DHCP Snippet id or name>

=== removed file 'docs/getting-help.rst'
--- docs/getting-help.rst	2014-08-20 12:07:13 +0000
+++ docs/getting-help.rst	1970-01-01 00:00:00 +0000
@@ -1,49 +0,0 @@
-************
-Getting help
-************
-
-Where to get help
------------------
-
-The two channels you can use to get help debugging a MAAS issue are:
-
-- The `Ask Ubuntu`_ website.
-- The `Freenode #maas`_ IRC channel.
-
-.. _Ask Ubuntu:
-  http://askubuntu.com/questions/ask?tags=maas
-
-.. _Freenode #maas:
-  http://webchat.freenode.net/?channels=maas
-
-
-Gathering debugging information
--------------------------------
-
-If your question is related to a problem related to a specific MAAS installation, we encourage you to gather debugging information and make it available before you head over to Ask Ubuntu or the IRC channel.  This way, you'll have all the required information handy for people who can help you.
-
-Gathering debugging information is a fully automated process that is performed with the help of `sosreport`_.
-
-If your MAAS server is running Ubuntu version 14.04 (Trusty Tahr) or later, you can get sosreport from the official archives::
-
-  # Install sosreport.
-  sudo apt-get install -y sosreport
-  # Create the report.
-  sudo sosreport -o maas
-
-Alternatively, if your MAAS server is running a previous Ubuntu release, you'll need to install sosreport manually::
-
-  # Install git.
-  sudo apt-get install -y git
-  # Get the latest version of sosreport.
-  git clone https://github.com/sosreport/sosreport.git /tmp/sosreport
-  # Create the report.
-  sudo /tmp/sosreport/sosreport -o maas
-
-.. _sosreport:
-  https://github.com/sosreport/sosreport
-
-This will create a tarball containing MAAS' log files, MAAS' configuration files and a dump of MAAS' database.  By default, the tarball will end up in /tmp but you can change the location, see sosreport's manpage for details.  If there are things you do not wish to share publicly, feel free to edit the tarball.
-
-Now, the last step is to make this file available by any means at your disposal (openly accessible FTP server, Dropbox, etc.) in order for the people who will help you to be able to get their hands on it.
-

=== removed file 'docs/ha.rst'
--- docs/ha.rst	2016-06-03 23:10:17 +0000
+++ docs/ha.rst	1970-01-01 00:00:00 +0000
@@ -1,290 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _ha:
-
-High Availability (HA) Configuration
-====================================
-
-MAAS 2.0+ supports high availability (HA) across its entire architecture. HA
-can be enabled at the rack controller level allowing the ability for
-controlling BMCs, providing DHCP, and deploying machines to span across
-multiple rack controllers. At the region controller level, HA can be provided
-for the API, WebUI, DNS, and APT proxy.
-
-
-Rack Controller HA
-------------------
-
-Rack controller HA is provided for BMCs out of the box. MAAS will
-automatically identify which rack controllers can communicate to which BMCs
-and pass BMC actions to the correct rack controllers.
-
-To provide HA for deploying machines, DHCP HA must be turned on. DHCP HA in
-MAAS allows a primary and a secondary DHCP server to run on the same VLAN. This
-allows a deploying machine to request an IP address from either DHCP server
-(primary first) for PXE booting. The responding rack controller will then
-handle PXE booting the machine and loading the required files to complete the
-deployment. All lease information will be replicated between the primary and
-secondary rack controller, allowing one rack controller to fail without
-interrupting MAAS operation.
-
-Enabling DHCP HA
-^^^^^^^^^^^^^^^^
-
-DHCP works at the VLAN level, providing interfaces connected to that VLAN the
-ability to get an IP address on a subnet. This requires a subnet be defined
-on the VLAN where DHCP will be enabled, along with a range of IPs that will be
-assigned to unknown interfaces (called the 'dynamic range'). Enabling DHCP HA
-is the same as enabling DHCP without HA; setting a secondary rack controller
-will turn DHCP HA on. Follow the instructions at :doc:`rack-configuration` to
-enable DHCP HA.
-
-
-Region Controller HA
---------------------
-
-Region controller HA is more complicated, in that MAAS allows you to configure
-this in many different configurations. Any number of region controllers can
-be added to MAAS, as long as they connect to the same PostgreSQL database. The
-only state that is held in MAAS is in the database, allowing region controllers
-to be scaled up and scaled down at will.
-
-Initial Region Controller
-^^^^^^^^^^^^^^^^^^^^^^^^^
-
-The first step to enabling HA for the MAAS region controller is installing the
-``maas-region-controller`` package. This package configures everything for you
-to start using the MAAS region controller on one machine. It initializes a new
-database, runs the migrations, and sets up the default configuration files for
-the new MAAS region controller to run. ::
-
-    $ sudo apt update
-    $ sudo apt install maas-region-controller
-
-.. note::
-
-  This installs and configures the PostgreSQL database on the same machine as
-  the region controller. When possible, this is the recommended configuration.
-  MAAS requires database access to be fast, since all MAAS state (including
-  operating system images, which will be synchronized with each rack
-  controller).
-
-Enabling PostgreSQL HA
-^^^^^^^^^^^^^^^^^^^^^^
-
-MAAS stores all state information in the PostgreSQL database, so it is key to
-place this into an HA configuration. PostgreSQL supports many different HA
-configurations, but hot standby is recommended as it allows the secondary
-database to become the primary quickly and the MAAS region controller can
-quickly switch to using the secondary database server as the primary. For
-configuring hot standby, the following wiki provides a quick start guide on
-setting it up: `https://wiki.postgresql.org/wiki/Hot_Standby`_.
-
-.. _https://wiki.postgresql.org/wiki/Hot_Standby:
-  https://wiki.postgresql.org/wiki/Hot_Standby
-
-Extra Region Controller
-^^^^^^^^^^^^^^^^^^^^^^^
-
-Since this new region will not be setting up its own database, use the
-following packages to set up the new region controller::
-
-    maas-region-api maas-dns
-
-In order to add another region controller to MAAS, you will also need the
-*regiond.conf* from the initial region controller. This will allow the new
-region controller to connect to the same PostgreSQL database. You will want to
-adjust the database_host in the *regiond.conf* to point to the primary database
-for PostgreSQL.::
-
-    $ sudo apt update
-    $ sudo apt install maas-region-api maas-dns
-    $ sudo systemctl stop maas-regiond
-    $ sudo rm /var/lib/maas/{maas_id,secret}
-    $ sudo scp <ubuntu@initial-region>:/etc/maas/regiond.conf /etc/maas/regiond.conf
-    $ sudo chown root:maas /etc/maas/regiond.conf
-    $ sudo chmod 640 /etc/maas/regiond.conf
-    $ sudo maas-region local_config_set --database-host <postgresql-primary-ip>
-    $ sudo systemctl restart maas-regiond
-
-Now if you check the MAAS UI or the API, you will see that MAAS has another
-region controller. But one issue still exists: the ``bind9`` service will
-refuse to start. This is because both BIND and MAAS define some of the same
-options in their respective configuration files. Since BIND will not allow you
-to use the same option more than once, MAAS needs to migrate the options from
-the BIND configuration. ::
-
-    $ sudo maas-region edit_named_options --migrate-conflicting-options
-    $ sudo systemctl restart bind9
-
-After that, you should have another fully functioning MAAS region controller!
-The DNS configuration and proxy configuration across all region controllers
-will stay synced, and you have another server to access the MAAS WebUI and API
-endpoints.
-
-Load Balancing Between Regions (optional)
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-With multiple region controllers, there are several options for load balancing.
-One approach is to only use one region controller all the time, and fail over
-to another using a virtual IP (VIP). Another approach is to use a load-balancer
-and a VIP, to distribute the workload across all active region controllers.
-In this example, we will use ``haproxy`` to configure load balancing across the
-MAAS region controllers.
-
-First (on each region controller) ``apache2`` needs to be disabled. While the
-``maas-region-api`` package depends on ``apache2`` for its default
-configuration, it will conflict with ``haproxy`` (or any other load balancer
-running on port 80) if it is enabled. ::
-
-    $ sudo systemctl stop apache2
-    $ sudo systemctl disable apache2
-
-Now that ``apache2`` is disabled, ``haproxy`` can be installed and configured.
-You will want to change the <server-name> and <server-ip> in the proxy
-configuration below to match your infrastructure. ::
-
-    $ sudo apt install haproxy
-    $ sudo sh -c 'cat >>/etc/haproxy/haproxy.cfg <<EOL
-
-    frontend maas
-        bind    *:80
-        retries 3
-        option  redispatch
-        option  http-server-close
-        default_backend maas
-
-    backend maas
-        timeout server 30s
-        balance roundrobin
-        server localhost localhost:5240 check
-        server <server-name-1> <server-ip-1>:5240 check
-        server <server-name-2> <server-ip-2>:5240 check
-    EOL'
-    $ sudo systemctl restart haproxy
-
-.. note::
-
-  It is recommended to run a load-balancer on every region controller server
-  and place a VIP between the servers. This will ensure that the load is
-  balanced between the servers, and ensure that (if a failure occurs) the
-  VIP moves over to the another server (which could then distribute
-  requests to the remaining servers).
-
-VIP between the Regions
-^^^^^^^^^^^^^^^^^^^^^^^
-
-Whether you configured a load-balancer or not, a VIP (virtual IP) is needed.
-The VIP will be used by the rack controllers (and the deploying machines) to
-access the region controller API endpoint. In this example, we will show how to
-use ``keepalived`` to configure a VIP.
-
-You will need to adjust checks below, depending on if you are using
-``apache2``, or ``haproxy``. You will also need to configure the
-interface_name, priority, random_password, and the VIP in the configuration
-below. The priority needs to be between 1-255. Larger priority numbers
-indicate a greater preference for the server to claim the VIP. ::
-
-    $ sudo apt install keepalived
-    $ sudo modprobe ip_vs
-    $ sudo sh -c 'echo modprobe ip_vs >> /etc/modules'
-    $ sudo sh -c 'echo net.ipv4.ip_nonlocal_bind=1' > /etc/sysctl.d/60-keepalived-nonlocal.conf
-    $ sudo systemctl restart procps
-    $ sudo sh -c 'cat >>/etc/keepalived/keepalived.conf <<EOL
-
-    # Un-comment when using haproxy.
-    #vrrp_script chk_haproxy {
-    #    script "killall -0 haproxy"
-    #    interval 2
-    #}
-
-    # Un-comment when using apache2.
-    #vrrp_script chk_apache2 {
-    #    script "killall -0 apache2"
-    #    interval 2
-    #}
-
-    vrrp_script chk_named {
-        script "killall -0 named"
-        interval 2
-    }
-
-    vrrp_instance maas_region {
-        state MASTER
-        interface <interface_name>
-        priority <priority>
-        virtual_router_id 51
-
-        authentication {
-            auth_type PASS
-            auth_pass <random_password>
-        }
-
-        track_script {
-            # Un-comment when using haproxy
-            #chk_haproxy
-            # Un-comment when using apache2
-            #chk_apache2
-            chk_named
-        }
-
-        virtual_ipaddress {
-            <vip>
-        }
-    }'
-    $ sudo systemctl restart keepalived
-
-.. note::
-
-  If you are enabling this inside of a container, the host of the container
-  needs the ip_vs module loaded and the sysctl change. A restart of the
-  container is required once the change has been made in the host.
-
-Once ``keepalived`` has been configured you will want to adjust the
-MAAS_URL on all region controllers and rack controllers to point to that VIP.
-That will ensure that all clients and machines use that IP address for
-communication.
-
-On the rack controller's::
-
-    $ sudo maas-rack config --region-url http://<vip>/MAAS
-    $ sudo systemctl restart maas-rackd
-
-On the region controller's::
-
-    $ sudo maas-region local_config_set --maas-url http://<vip>/MAAS
-    $ sudo systemctl restart maas-regiond
-
-
-Deploying HA with Juju
-----------------------
-
-Now that you have an understanding of how to configure HA manually, it
-is possible to use Juju to deploy MAAS in an HA configuration. Using Juju
-allows you to quickly scale up or scale down the MAAS infrastructure in a
-configuration supported by the MAAS team.
-
-.. note::
-
-  Using Juju to deploy MAAS is not normally how Juju would be used.
-  Normally, you would first install MAAS, then use Juju to deploy services on
-  MAAS-managed machines. However, we can use Juju's manual provisioning
-  support to deploy MAAS to existing Ubuntu systems.
-
-In the following example, Juju is bootstrapped with manual provisioning,
-the machines intended to be used for MAAS services are added, and the services
-are deployed and linked together. Be sure to adjust the given numbers based
-on what you see in the "juju status" command. (See the Juju documentation
-for more details.) The following commands could be used to deploy MAAS::
-
-    $ juju bootstrap maas manual/<ip-of-server>
-    $ juju add-machine ssh:<ip-of-server>
-    ... add required machines ...
-    $ juju deploy postgresql --to 0
-    $ juju add-unit postgresql --to 1
-    $ juju deploy maas-region --to 0
-    $ juju add-unit maas-region --to 1
-    $ juju add-relation maas-region:db postgresql:db
-    $ juju deploy maas-rack --to 3
-    $ juju add-unit maas-rack --to 4
-    $ juju add-relation maas-region:rpc maas-rack:rpc

=== removed file 'docs/hardware-enablement-kernels.rst'
--- docs/hardware-enablement-kernels.rst	2015-08-26 02:32:49 +0000
+++ docs/hardware-enablement-kernels.rst	1970-01-01 00:00:00 +0000
@@ -1,80 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _hardware-enablement-kernels:
-
-=================================
-Using hardware-enablement kernels
-=================================
-
-.. note::
-
-  As of MAAS 1.9 this feature is configured by setting the hwe_kernel variable
-  instead of the architecture variable.
-
-MAAS allows you to use hardware enablement kernels when booting nodes
-with Ubuntu that require them.
-
-What are hardware-enablement kernels?
--------------------------------------
-
-Brand new hardware gets released all the time. We want that hardware to
-work well with Ubuntu and MAAS, even if it was released after the latest
-release of MAAS or Ubuntu. Hardware Enablement (HWE) is all about keeping
-pace with the new hardware.
-
-Ubuntu's solution to this is to offer newer kernels for older releases.
-There are at least two kernels on offer for Ubuntu releases: the
-"generic" kernel -- i.e. the kernel released with the current series --
-and the Hardware Enablement kernel, which is the most recent kernel
-release.
-
-There are separate HWE kernels for each release of Ubuntu, referred to
-as ``hwe-<release letter>``. So, the 14.04 / Trusty Tahr HWE kernel is
-called ``hwe-t``, the 12.10 / Quantal Quetzal HWE kernel is called
-``hwe-q`` and so on. This allows you to use newer kernels with older
-releases, for example running Precise with a Saucy (hwe-s) kernel.
-
-For more information see the `LTS Enablement Stack`_ page on the Ubuntu
-wiki.
-
-.. _LTS Enablement Stack:
-   https://wiki.ubuntu.com/Kernel/LTSEnablementStack
-
-
-Booting hardware-enablement kernels
------------------------------------
-
-MAAS imports hardware-enablement kernels along with its generic boot
-images. These hardware-enablement kernels are specified by using min_hwe_kernel
-or hwe_kernel variables.
-
-The min_hwe_kernel variable is used to instruct MAAS to ensure the release to
-be deployed uses a kernel version at or above the value of min_hwe_kernel. For
-example if min_hwe_kernel is set to hwe-t when deploying any release before
-Trusty the hwe-t kernel will be used. For any release after Trusty the default
-kernel for that release will be used. If hwe-t or newer is not availible for
-the specified release MAAS will not allow that release to be deployed and throw
-an error.
-
-min_hwe_kernel can be set by running the command::
-
-  $ maas <profile-name> node update <system-id>
-    min_hwe_kernel=hwe-<release letter>
-
-It's also possible to set the min_hwe_kernel from the MAAS web UI, by visiting
-the Node's page and clicking ``Edit node``. Under the Minimum Kernel field, you
-will be able to select any HWE kernels that have been imported onto that node's
-cluster controller.
-
-.. image:: media/min_hwe_kernel.png
-
-You can also set the hwe_kernel during deployment. MAAS checks that the
-specified kernel is avalible for the release specified before deploying the
-node. You can set the hwe_kernel when deploying by using the command::
-
-  $ maas <profile-name> node start <system-id> distro_series=<distro>
-  hwe_kernel=hwe-<release letter>
-
-Or through the web interface as seen below.
-
-.. image:: media/hwe_kernel.png

=== modified file 'docs/index.rst'
--- docs/index.rst	2016-08-15 09:54:43 +0000
+++ docs/index.rst	2017-02-02 20:19:50 +0000
@@ -39,54 +39,10 @@
 .. toctree::
    :maxdepth: 2
 
-   about
-   orientation
    releases
    changelog
 
 
-************************
-Setting up a MAAS server
-************************
-
-.. toctree::
-   :maxdepth: 2
-
-   install
-   configure
-   rack-configuration
-   ha
-   static-ips
-   ipv6
-   bootsources
-   nodes
-   hardware-enablement-kernels
-   sstreams-mirror
-   networking
-   kernel-options
-   installing-ubuntu
-   os-support
-   storage
-   networking
-   proxy
-   devices
-   dhcpsnippets
-   packagerepositories
-   power-driver-capabilities
-
-
-******************
-Deploying services
-******************
-
-.. toctree::
-   :maxdepth: 2
-
-   juju-quick-start
-   tags
-   physical-zones
-
-
 ***********************
 API / CLI Documentation
 ***********************
@@ -100,17 +56,6 @@
    version
 
 
-***************
-Troubleshooting
-***************
-
-.. toctree::
-   :maxdepth: 2
-
-   getting-help
-   troubleshooting
-
-
 ******************
 Command-line Tools
 ******************

=== removed symlink 'docs/install.rst'
=== target was '../INSTALL.txt'
=== removed file 'docs/installing-ubuntu.rst'
--- docs/installing-ubuntu.rst	2016-05-11 19:01:48 +0000
+++ docs/installing-ubuntu.rst	1970-01-01 00:00:00 +0000
@@ -1,36 +0,0 @@
-=====================================
-Installing Ubuntu and deploying nodes
-=====================================
-
-Once a node has been accepted into MAAS and is ready for use, users can
-deploy services to that node.
-
-Prior to deployment, MAAS is responsible for:
-
-1. Powering up the node.
-2. Installing Ubuntu on the node (with the help of :ref:`curtin
-   <curtin-installer>`).
-3. Installing the user's SSH keys on the node.
-
-Once these steps have been completed, the node is ready to have services
-deployed to it, either manually or by using a tool like Juju_.
-
-.. _Juju: http://juju.ubuntu.com
-
-
-.. _curtin-installer:
-
-Curtin: The Curt Installer
---------------------------
-
-As the name suggests this installs Ubuntu on a node more quickly than
-would be possible using the traditional Debian installer.
-
-Curtin is enabled by default and is the only mechanism for installing
-Ubuntu supported by MAAS. It copies a pre-built Ubuntu image to the
-node, with all the packages installed that would be normally found in an
-Ubuntu installation. It's very fast!
-
-For more information about curtin, see its `project page`_ on Launchpad.
-
-.. _project page: https://launchpad.net/curtin

=== removed file 'docs/ipv6.rst'
--- docs/ipv6.rst	2014-10-10 13:36:03 +0000
+++ docs/ipv6.rst	1970-01-01 00:00:00 +0000
@@ -1,150 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _ipv6:
-
-Managing IPv6 Networks
-======================
-
-.. note::
-
-  This feature is available in MAAS versions 1.7 and above, starting with
-  lp:maas revision 2992.  If you're writing a client application that makes use
-  of this feature, you can query the region-server API for the
-  ``ipv6-deployment-ubuntu`` :ref:`capability<cap-ipv6-deployment-ubuntu>`.
-
-MAAS has limited IPv6 support for networking nodes.  It works much like IPv4
-support, but with a number of limitations:
-
-* Nodes still boot, register, and install using the IPv4 network.
-* IPv6 addresses are only configured when using the default Ubuntu installer.
-* Most BMCs can only be controlled (e.g. to power nodes on/off) using IPv4.
-* MAAS still uses IPv4 for its internal operation, installing nodes, etc.
-* For now, MAAS only supports IPv6 on networks where it also manages IPv4 DHCP.
-* A network interface on a node can only be on one IPv6 subnet.
-* A network interface on a cluster controller can manage only one IPv6 subnet.
-
-The web user interface and REST API can be accessed in the same way on both
-IPv4 and IPv6.  To use an IPv6 address as the hostname in a URL, in your
-browser or elsewhere, surround it with square brackets.  For example, on the
-local machine (``::1``, the IPv6 equivalent of ``localhost``) you might
-request::
-
-    http://[::1]/MAAS/
-
-If your MAAS server has a DNS hostname that resolves to both IPv4 and IPv6
-addresses, your browser may already be accessing the UI through IPv6 without
-you noticing.
-
-
-Enabling IPv6
--------------
-
-You enable IPv6 networking in the same way that you enable IPv4 networking:
-configure a separate cluster interface for your IPv6 subnet, in addition to the
-one you need for your IPv4 subnet.  The IPv6 cluster interface must define a
-static address range.  Provided that you already have a functioning IPv6
-network, that's all there is to it.  The following sections will go into more
-detail about what is supported, what is needed, and what to do if you don't yet
-have a functioning IPv6 network.
-
-An IPv6 cluster interface can use the same network interface on the cluster
-controller as an existing IPv4 network interface.  It just defines a different
-subnet, with IPv6 addressing.  A node that's connected to the IPv4 subnet will
-also be connected to the IPv6 subnet on the same network segment.
-
-
-Configuring your IPv6 subnet
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-When you configure your IPv6 cluster interface, be sure to define a static IP
-address range.  Deployed nodes on the subnet will get static addresses in this
-range.
-
-IPv6 networks are normally 64 bits wide, so you can be generous with the
-ranges' sizes.  It also means that you can leave the netmask field blank.
-
-(There are no broadcast addresses in IPv6, so leave the broadcast address field
-blank as well.)
-
-You may want MAAS to manage DHCP and DNS, but it's not required.  In fact nodes
-do not need a DHCP server at all for IPv6; MAAS configures static IPv6
-addresses on the node's network interfaces while deploying it.  A DHCPv6 server
-can provide addresses for containers or virtual machines running on the nodes,
-as well as devices on the network that are not managed by MAAS, but it is not
-needed for the nodes themselves.  MAAS will not be aware of any addresses
-issued by DHCP, and does not guarantee that they will stay unchanged.
-
-
-.. _ipv6-routing:
-
-Routing
-^^^^^^^
-
-In IPv6, clients do not discover routes through DHCP.  Routers make themselves
-known on their networks by sending out *route advertisements*.  These *RAs*
-contain other configuration as well: whether clients should statelessly
-configure their own unique IP addresses based on their MAC addresses; whether
-they should request stateless configuration from a DHCP server; and finally,
-whether they should request a stateful IP address from a DHCP server.  Since a
-network interface can have any number of IPv6 addresses even on a single
-subnet, several of these address assignment mechanisms can be combined.
-
-However, when MAAS configures IPv6 networking on a node, it does not rely on
-RAs.  it statically configures your nodes' default IPv6 route to use the router
-that is configured on the cluster interface, so that the nodes will know their
-default gateway.  They do not need DHCP and will not autoconfigure global
-addresses.
-
-However, if you are planning to operate DHCPv6 clients as well, e.g. on
-machines not managed by MAAS or on virtual machines hosted by MAAS nodes, you
-may still want to have RAs configured to make those clients obtain
-configuration over DHCP.
-
-If you need RAs but your gateway does not send them, install and configure
-``radvd`` somewhere on the network to advertise its route.
-
-
-Other installers and operating systems
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-Static IPv6 addresses are currently only configured on Ubuntu, when installed
-using the "fast" installer.  Other operating systems, or Ubuntu with the
-classic Debian installer, will not have their IPv6 addresses configured.
-The same applies when a user manually installs an operating system on a node,
-or overwrites its networking configuration: the node will no longer have its
-static IPv6 address configured, even if MAAS has allocated it to the node.
-
-However, as long as the address remains allocated to the node, you may still
-configure its operating system to use that address.  The node can then use that
-address as if it had been configured by MAAS.
-
-
-Disabling IPv4
---------------
-
-For advanced users, there is an experimental capability to deploy nodes with
-pure IPv6, with IPv4 networking disabled.  To enable this on a node, check the
-"Disable IPv4 when deployed" box on the node's Edit page.  The process of
-managing and deploying the node will still largely work through IPv4, but once
-deployed, the node will have IPv6 networking only.
-
-In practice nodes may not be functional without IPv4 networking.  A few things
-are known to be needed in any case:
-
-
-Configuring the MAAS URL
-^^^^^^^^^^^^^^^^^^^^^^^^
-
-The *maas-cluster-controller* package has a configuration item for the URL
-where nodes and cluster controllers can reach the MAAS region API.
-
-By default, this URL is set based on the region controller's IPv4 address.  To
-make it work for nodes that won't have IP4, you must set the MAAS URL to use
-a hostname instead of an IP address.  The hostname must resolve to both IPv4
-and IPv6 addresses, and both on the cluster controller and on the nodes.
-
-To change this setting, run::
-
-    dpkg-reconfigure maas-cluster-controller
-
-It will prompt you for the URL, with its current setting as the initial value.

=== removed file 'docs/juju-quick-start.rst'
--- docs/juju-quick-start.rst	2016-01-25 14:10:29 +0000
+++ docs/juju-quick-start.rst	1970-01-01 00:00:00 +0000
@@ -1,145 +0,0 @@
-Juju Quick Start
-================
-
-These instructions will help you deploy your first charm with Juju to
-a MAAS cluster.
-
-In the following, we assume that you have a MAAS cluster set-up with at least
-2 nodes enlisted with it.
-
-
-Your API key, SSH key, and environments.yaml
---------------------------------------------
-
-You'll need an API key from MAAS so that the Juju client can access
-it. Each user account in MAAS can have as many API keys as desired.
-One hard and fast rule is that you'll need to use a different API key
-for each Juju *environment* you set up within a single MAAS cluster.
-
-There is no need to explicitly add an SSH key to MAAS when using Juju;
-it will automatically put your public key on any hosts that it starts up.
-
-**Note**: You do not need to use the MAAS web UI or API to allocate
-a node to yourself, Juju will do this for you.
-
-
-Getting a key
-^^^^^^^^^^^^^
-
-To get the API key:
-
-#. Go to your MAAS preferences page (go to your MAAS home page
-   ``http://${my-maas-server}:80/MAAS/`` and choose *Preferences* from the
-   drop-down menu that appears when clicking your username at the top-right
-   of the page).
-
-#. Optionally add a new MAAS key. Do this if you're setting up another
-   environment within the same MAAS cluster.
-
-The ``${my-maas-server}`` slot should be replaced with the hostname of your
-MAAS server.
-
-
-Adding an SSH key
-^^^^^^^^^^^^^^^^^
-
-While you're still on the MAAS preferences page, add your SSH key
-by clicking *Add SSH key*. Use the public half of your SSH key, the
-content of ``~/.ssh/id_rsa.pub`` for example; don't paste the private
-half.
-
-
-Creating environments.yaml
-^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-Create or modify ``~/.juju/environments.yaml`` with the following content::
-
-  environments:
-    maas:
-      type: maas
-      maas-server: 'http://${my-maas-server}:80/MAAS'
-      maas-oauth: '${maas-api-key}'
-      admin-secret: ${your-admin-secret}
-      default-series: precise
-
-Substitute the API key from earlier into the ``${maas-api-key}``
-slot, and the hostname of your MAAS server into the ``${my-maas-server}``
-slot.
-
-The ``${your-admin-secret}`` slot should be replaced with a random pass-phrase,
-there is no default.
-
-
-Now Juju
---------
-
-If juju-core is not yet installed on the client machine, run::
-
-  $ sudo apt-get install juju-core
-
-Now, use juju to display the status of the default environment::
-
-  $ juju status
-
-As you've not bootstrapped you ought to see::
-
-  error: Unable to connect to environment "".
-  Please check your credentials or use 'juju bootstrap' to create a new environment.
-
-**Note**: if Juju complains that there are multiple environments and
-no explicit default, add ``-e ${environment-name}`` after each
-command, e.g.::
-
-  $ juju status -e maas
-
-Bootstrap::
-
-  $ juju sync-tools
-  $ juju bootstrap
-
-If bootstrapping on a version of juju older than 1.14.0 then use::
-
-  $ juju bootstrap --upload-tools
-
-This will return quickly, but the master node may take a *long* time
-to come up. It has to completely install Ubuntu and Juju on it and
-reboot before it'll be available for use. It's probably worth either
-trying a ``juju status`` once in a while to check on progress, or
-following the install on the node directly.
-
-  **Beware** of `bug 413415`_ - *console-setup hangs under chroot
-  debootstrap with a console login on ttyX* - when monitoring an
-  installation on the node.
-
-.. _bug 413415:
-  https://bugs.launchpad.net/ubuntu/+source/console-setup/+bug/413415
-
-Once the boostrap node has been installed a status command should
-come up with something a bit more interesting::
-
-  environment: maas
-  machines:
-    "0":
-      agent-state: started
-      agent-version: 1.13.3.1
-      dns-name: kmhwd.master
-      instance-id: /MAAS/api/2.0/nodes/node-5c5b713a-1afc-11e3-9904-525400123456/
-      series: precise
-  services: {}
-
-Now it's possible to deploy a charm::
-
-  $ juju deploy mysql
-  $ juju status
-
-If you have another node free you can finish off the canonical and by
-now familiar example::
-
-  $ juju deploy wordpress
-  $ juju add-relation wordpress mysql
-  $ juju expose wordpress
-  $ juju status
-
-Note that each charm runs on its own host, so each deployment will
-actually take as long as it took to bootstrap. Have a beer, drown your
-sorrows in liquor, or, my preference, have another cup of tea.

=== removed file 'docs/kernel-options.rst'
--- docs/kernel-options.rst	2014-02-11 05:21:58 +0000
+++ docs/kernel-options.rst	1970-01-01 00:00:00 +0000
@@ -1,38 +0,0 @@
-===========================
-Setting kernel boot options
-===========================
-
-MAAS is able to send specific kernel options to booting nodes on both
-a global basis and a per-node basis.
-
-Global kernel options
----------------------
-
-As an admin, click on the gear icon at the top right and scroll down to
-the Global Kernel Parameters section, as shown here:
-
-.. image:: media/global_kernel_opts.png
-
-Whatever you set here is sent as-is to all booting nodes.
-
-Per-node kernel options
------------------------
-
-Per-node kernel options are set using tags.  The easiest way of doing this
-is to use the ``maas`` command.  You will need to :ref:`be
-logged in to the API first <api-key>` and then you can add a tag which has its
-``kernel_opts`` value set, like this::
-
-  $ maas maas tags new name='nomodeset' \
-    comment='nomodeset kernel option' kernel_opts='nomodeset vga'
-
-Once the tag is defined, you can add it to a node or nodes::
-
-  $ maas maas tag update-nodes nomodeset add=<system_id_1> \
-    add=<system_id_2>
-
-.. note::
-
-  Any per-node kernel options set will completely override the global options.
-  If multiple tags attached to a node have the kernel_opts defined, the first
-  one ordered by name is used.

=== removed file 'docs/networking.rst'
--- docs/networking.rst	2016-07-08 14:38:44 +0000
+++ docs/networking.rst	1970-01-01 00:00:00 +0000
@@ -1,270 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _networking:
-
-=========================
-Networking
-=========================
-
-.. note::
-
-  This feature is available in MAAS versions 1.9 and above on Ubuntu
-  deployments. If you're writing a client application, you can check
-  if MAAS supports this feature via the web API; see the documentation
-  for the ``network-deployment-ubuntu`` capability
-  :ref:`here<cap-network-deployment-ubuntu>`.
-
-MAAS 1.9 adds support for the modeling of a wide variety of networking concepts
-and configurations.
-
-Networking Concepts
--------------------
-
-At a high level, MAAS supports the following networking concepts:
-
-Fabrics
-^^^^^^^
-
-A fabric is a set of interconnected VLANs that are capable of mutual
-communication. A fabric can be thought of as a logical grouping in which VLANs
-can be considered unique.
-
-For example, a distributed network may have a fabric in London containing
-VLAN 100, while a separate fabric in San Francisco may contain a VLAN 100,
-whose attached subnets are completely different and unrelated.
-
-A "Default Fabric" is created when MAAS is installed (or upgraded).
-
-Subnets
-^^^^^^^
-
-A subnet is a "layer 3" network defined by a particular network prefix, plus
-a network mask length (in bits). This notation is also referred to as a *CIDR*.
-
-MAAS supports IPv4 and IPv6 subnets.
-
-Examples::
-
-    10.0.0.0/8
-    172.16.0.0/12
-    192.168.0.0/16
-    2001:db8:4d41:4153::/64
-
-Subnets can be annotated with a descriptive name, their default gateway,
-and/or their DNS server(s).
-
-A subnet can be in a single space.
-
-Subnets are considered managed if a cluster interface specifies the cluster
-network range.
-
-**IP Ranges**
-
-MAAS assumes it can allocate all unreserved IPs in a subnet. For example,
-MAAS will assign IPs out of the unreserved pool to node interfaces configured
-"IP Address" type "Auto assign". You can reserve IPs for other purposes by
-adding one or more "reserved ranges" in your subnet configuration.
-
-There are two kinds of reserved ranges: "Reserved Range" and "Reserved Dynamic
-Range". Use a "Reserved Range" to block out a set of static IPs that you plan
-to use for other devices on your network (e.g. managed switches or routers).
-
-Use a "Reserved Dynamic Range" to block out a set of IP addresses that you
-plan to hand out using a DHCP server. If you enable DHCP on your subnet's
-VLAN, MAAS will provide a DHCP for IPs in this range, otherwise you are
-responsible for proving an external DHCP server for this range.
-
-Note that a "Reserved Dynamic Range" with active DHCP is *required* if you
-have nodes that require DHCP to network boot (e.g. PXE clients), or if you
-have configured any node interfaces as "IP Address" type "DHCP".
-
-
-VLANs
-^^^^^
-
-VLANs (Virtual LANs) are a common way to create logically separate networks
-using the same physical infrastructure.
-
-Managed switches can assign VLANs to each port in either a "tagged" or an
-"untagged" manner. A VLAN is said to be "untagged" on a particular port when
-it is the default VLAN for that port, and requires no special configuration
-in order to access.
-
-"Tagged" VLANs (traditionally used by network administrators in order to
-aggregate multiple networks over inter-switch "trunk" lines) can also be used
-with nodes in MAAS. That is, if a switch port is configured such that "tagged"
-VLAN frames can be sent and received by a MAAS node, that MAAS node can be
-configured to automatically bring up VLAN interfaces, so that the deployed node
-can make use of them.
-
-A "Default VLAN" is created for every Fabric, to which every new VLAN-aware
-object in the fabric will be associated to by default (unless otherwise
-specified).
-
-Spaces
-^^^^^^
-
-A Space is a logical grouping of subnets that should be able to communicate
-with each other. Subnets within each space need not belong to the same fabric.
-For example, you may have a "DMZ" space in both your London and San Francisco
-fabrics, and a "Storage" space to indicate subnets attached to your storage
-network.
-
-A "space-0" is created when MAAS is installed (or upgraded), which
-every subnet will belong to by default (unless otherwise specified).
-
-Interfaces
-^^^^^^^^^^
-
-**Physical**
-
-After a node is commissioned, MAAS discovers its physical interfaces. In
-addition, devices are created with physical interfaces.
-
-Prior to deployment, a MAAS administrator can specify additional interfaces
-to be configured on the node, including one or more of the below types.
-
-**Bond**
-
-A bond interface is capable of aggregating two or more physical interfaces
-into a single logical interface. Bonds can be used in conjunction with a
-managed switch (using Link Aggregation and Control Protocol, or LACP), or
-independently (software bonds).
-
-**VLAN**
-
-A VLAN interface can be used to connect to a tagged VLAN, if the switch port
-the node is connected to is authorized to access it.
-
-**Unknown**
-
-Unknown interfaces cannot be created by users.
-
-Sometimes, *unknown* interfaces are discovered by MAAS. (For example, when
-MAAS learns of a new DHCP lease that is not associated with any known node
-or device.)
-
-How to Configure Nodes for Networking
--------------------------------------
-
-Creating a Bond Interface
-^^^^^^^^^^^^^^^^^^^^^^^^^
-
-Use the ``node-interfaces create-bond`` API to create a bond. For example::
-
-    $ maas admin node-interfaces create-bond node-d83ce230-4b50-11e5-a267-00163eb185eb name=bond0 vlan=0 parents=30 parents=31 mac_address=00:50:56:2b:60:53
-    Success.
-    Machine-readable output follows:
-    {
-        "name": "bond0",
-        "links": [],
-        "tags": [],
-        "vlan": {
-            "id": 0,
-            "resource_uri": "/MAAS/api/2.0/fabrics/0/vlans/0/",
-            "name": "Default VLAN",
-            "vid": 0,
-            "fabric": "fabric-0"
-        },
-        "enabled": true,
-        "id": 41,
-        "parents": [
-            "eth0",
-            "eth1"
-        ],
-        "mac_address": "00:50:56:2b:60:53",
-        "type": "bond",
-        "children": [],
-        "resource_uri": "/MAAS/api/2.0/nodes/node-d83ce230-4b50-11e5-a267-00163eb185eb/interfaces/41/"
-    }
-
-Note that in the example above, the bond contains two interfaces because
-the ``parents`` parameter was specified twice, such as
-``parents=30 parents=31``.
-
-Creating a VLAN Interface
-^^^^^^^^^^^^^^^^^^^^^^^^^
-
-To create a VLAN interface, use the ``node-interfaces create-vlan`` API.
-
-Be aware that the ``vlan`` paremeter does not indicate a VLAN ID that
-corresponds to the VLAN tag. You must first create the VLAN, and then
-associate it with the interface. For example::
-
-    $ maas admin vlans create 0 name="Storage network" vid=100
-    Success.
-    Machine-readable output follows:
-    {
-        "id": 1,
-        "resource_uri": "/MAAS/api/2.0/fabrics/0/vlans/1/",
-        "name": "Storage network",
-        "vid": 100,
-        "fabric": "fabric-0"
-    }
-
-Note that the ``0`` in the command above indicates the ``fabric_id``. If you
-have not defined any additional fabrics, the ``fabric_id`` of the default
-fabric will be ``0``.
-
-Now that a VLAN is created, it may be associated with a new interface::
-
-    $ maas admin node-interfaces create-vlan node-d83ce230-4b50-11e5-a267-00163eb185eb vlan=1 parent=30
-    Success.
-    Machine-readable output follows:
-    {
-        "name": "eth0.100",
-        "links": [],
-        "tags": [],
-        "vlan": {
-            "id": 1,
-            "resource_uri": "/MAAS/api/2.0/fabrics/0/vlans/1/",
-            "name": "Storage network",
-            "vid": 100,
-            "fabric": "fabric-0"
-        },
-        "enabled": true,
-        "id": 44,
-        "parents": [
-            "eth0"
-        ],
-        "mac_address": "00:50:56:2b:60:53",
-        "type": "vlan",
-        "children": [],
-        "resource_uri": "/MAAS/api/2.0/nodes/node-d83ce230-4b50-11e5-a267-00163eb185eb/interfaces/44/"
-    }
-
-
-Deleting an Interface
-^^^^^^^^^^^^^^^^^^^^^
-
-To delete an interface, use the ``node-interface delete`` API. For example::
-
-    $ maas admin node-interface delete node-d83ce230-4b50-11e5-a267-00163eb185eb 41
-    Success.
-
-Machine Interface Configuration
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-**Auto Assign**
-
-Interfaces configured as "Auto assign" will be deployed with a static
-(non-DHCP) network configuration. MAAS will choose an IP from the subnet
-that does not fall into a defined reserved range.
-
-**Static**
-
-When you configure an interfaces as "Static", you will provide an IP address
-for that interface to use when deployed.
-
-**DHCP**
-
-Interfaces configured as "DHCP" will be configured to use DHCP to request
-configuration information at boot. In order for the interface to successfully
-configure, you will need to insure that a "Dynamic Range" has been reserved
-on the associated subnet, and that either you have configured MAAS to provide
-DHCP services on the associated VLAN, or that you have provided an external
-DHCP server to do so.
-
-**(Unconfigured)**
-
-These interfaces will be left unconfigured.

=== removed file 'docs/nodes.rst'
--- docs/nodes.rst	2016-03-28 13:54:47 +0000
+++ docs/nodes.rst	1970-01-01 00:00:00 +0000
@@ -1,98 +0,0 @@
-Adding nodes to the system
-==========================
-
-Now that the MAAS controller is running, we need to make the nodes
-aware of MAAS and vice-versa. If you have set up DHCP correctly, and
-your nodes can boot via PXE_ then things really couldn't be much easier
-and you can use :ref:`the automatic discovery procedure <auto-enlist>`.
-You do not need to install Ubuntu on nodes that you wish to add to MAAS
-prior to enlistment.
-
-To learn more about setting up DHCP, read the :ref:`cluster configuration
-documentation <rack-configuration>`.
-
-.. _PXE: http://en.wikipedia.org/wiki/Preboot_Execution_Environment
-
-.. _auto-enlist:
-
-Automatic Discovery
--------------------
-
-With nodes set to boot from a PXE image, they will start, look for a
-DHCP server, receive the PXE boot details, boot the image, contact the
-MAAS server and shut down.
-
-During this process, the MAAS server will be passed information about
-the node, including the architecture, MAC address and other details
-which will be stored in the database of nodes. You can accept and
-commission the nodes via the web interface.  When the nodes have been
-accepted the selected series of Ubuntu will be installed.
-
-To save time, you can also accept and commission all nodes from the
-commandline. This requires that you first log in with the API key,
-which :ref:`you can retrieve from the web interface <api-key>`::
-
-   $ maas maas nodes accept-all
-
-
-.. _enlist-via-boot-media:
-
-
-Manually add nodes
-------------------
-
-If you know the MAC address of a node, you can manually enter details
-about the node through the web interface. Click the ``Add Node`` button
-to be taken to the "Add Node" form:
-
-.. image:: media/add-node.png
-
-
-Virtual machine nodes
----------------------
-
-If you're setting up virtual machines to use as nodes with MAAS, you need
-to configure the power type as ``virsh``.  For MAAS to be able to use
-virsh, make sure you have the ``libvirt-bin`` package installed.
-
-.. note::
-
-  If you are assembling a set of VMs for testing or development, make
-  sure they have at least 512 MB (768 MB If you are deploying 15.10)
-  to avoid failures during deployment.
-
-The virsh power type takes two parameters:
-
-Power ID
-    The Power ID is the name of the virtual machine shown by
-    ``sudo virsh list --all``
-
-Address
-    This is a libvirt connection string, such as
-    ``qemu+ssh://ubuntu@10.0.0.2/system`` or ``qemu:///system``
-
-.. image:: media/virsh-config.png
-
-If you want to use ssh you'll need to generate a ssh key pair for the maas
-user.  By default there is no home directory created for the maas user::
-
-    $ sudo mkdir /home/maas
-    $ sudo chown maas:maas /home/maas
-
-Add a login shell for the maas user::
-
-    $ sudo chsh -s /bin/bash maas
-
-Become the maas user and generate a SSH keypair::
-
-    $ sudo su - maas
-    $ ssh-keygen -f ~/.ssh/id_rsa -N ''
-
-Then add the public key to ``/ubuntu/.ssh/authorized_keys`` on the vm server
-so virsh can use ssh without a password::
-
-    $ ssh-copy-id -i ~/.ssh/id_rsa ubuntu@10.0.0.2
-
-As the maas user, test virsh commands against libvirt at 10.0.0.2::
-
-    $ virsh -c qemu+ssh://ubuntu@10.0.0.2/system list --all

=== removed file 'docs/orientation.rst'
--- docs/orientation.rst	2014-08-22 00:13:24 +0000
+++ docs/orientation.rst	1970-01-01 00:00:00 +0000
@@ -1,107 +0,0 @@
-.. _orientation:
-
-Orientation
-===========
-
-
-MAAS in Brief
--------------
-
-Canonical’s MAAS brings the dynamism of cloud computing to the world
-of physical provisioning and Ubuntu. Connect, commission and deploy
-physical servers in record time, re-allocate nodes between services
-dynamically, and keep them up to date and in due course, retire them
-from use.
-
-MAAS is a new way of thinking about physical infrastructure. Compute,
-storage and network are commodities in the virtual world, and for
-large-scale deployments the same is true of the metal. “Metal as a
-service” lets you treat farms of servers as a malleable resource for
-allocation to specific problems, and re-allocation on a dynamic basis.
-
-In conjunction with the Juju service orchestration software (see
-https://juju.ubuntu.com/docs/), MAAS will enable you to get the most
-out of your physical hardware and dynamically deploy complex services
-with ease and confidence.
-
-
-Do I Need MAAS?
----------------
-
-MAAS certainly isn't for everyone, but why not ask yourself these
-questions?
-
-You probably *SHOULD* use MAAS if any or all of the following
-statements are true:
-
-    * You are trying to manage many physical servers.
-    * You want to deploy services with the minimum fuss.
-    * You need to get the most from your resources.
-    * You want things to work, repeatably and reliably.
-
-You probably don't need MAAS if any or all of these statements are
-true:
-
-    * You don't need to manage physical hardware
-    * You relish time spent in the server room
-    * You like trying to set up complicated, critical services without any help
-
-
-.. _setup:
-
-A Typical MAAS setup
---------------------
-
-MAAS is designed to work with your physical hardware, whether your
-setup includes thousands of server boxes or only a few. The key
-components of the MAAS software are:
-
-  * Region controller
-  * Cluster controller(s)
-  * Nodes
-
-The nodes are the computers you manage using MAAS.  These can range from just
-a handful to many thousands of systems.
-
-For small (in terms of number of nodes) setups, you will probably just
-install the Region controller and a cluster controller on the same
-server - it is only worth having multiple region controllers if you
-need to organise your nodes into different subnets (e.g. if you have a
-lot of nodes).  If you install the ``maas`` package, it will include both a
-region controller and a cluster controller, and they will be automatically
-set up to work together.
-
-.. image:: media/orientation_architecture-diagram.*
-
-
-How MAAS is used
-----------------
-
-MAAS manages a pool of nodes.  After registering a new system with the MAAS
-and preparing it for service ("commissioning"), the new system joins this pool.
-
-From the moment a node is accepted into the MAAS, any operating system,
-software, or data that it may have had installed before is meant to be
-overwritten.  A node in the pool is under MAAS's sole control, and off-limits
-to users.
-
-Once you have nodes in the pool, users of the MAAS can allocate them for their
-own use.  At that point, the nodes are installed with the selected operating
-system and set up with the user's login credentials for remote access.  This
-is referred to as "starting" a node in the browser interface, and as
-"acquiring" (and, as a separate step, "starting") a node in the API.
-
-When allocating from the API, you can specify constraints such as how much
-memory you need, how many CPUs, what networks the node should be connected to,
-what physical zone they should be in, and so on.  API commands can also be
-issued through the ``maas`` command-line utility.
-
-An allocated node is not like a virtual instance in a cloud: you get complete
-control, including hardware drivers and root access.  To upgrade a BIOS, for
-example, an administrator could allocate a node to themselves, and run a
-vendor-supplied upgrade utility.  Needless to say, you also get full hardware
-performance and tweaking!
-
-Once you are done with a node you have allocated, you can release it back to
-the pool.  Once again any data, software, or operating system will no longer
-be available.

=== removed file 'docs/os-support.rst'
--- docs/os-support.rst	2015-09-08 18:41:57 +0000
+++ docs/os-support.rst	1970-01-01 00:00:00 +0000
@@ -1,121 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _os-support:
-
-=========================
-Operating Systems Support
-=========================
-
-.. note::
-
-  This feature is available in MAAS versions 1.7 and above.
-
-Besides Ubuntu, MAAS allows users to deploy different operating
-systems, such as CentOS. Ubuntu Advantage customers can
-additionally deploy Redhat Enterprise Linux (RHEL), OpenSUSE, SLES,
-Window Server and Windows HyperV.
-
-
-Generated Images vs. Custom Images
-----------------------------------
-
-MAAS supports two different classes of OS images, `Generated images`
-and `Custom images`.
-
-`Generated images` are images of the OS' that MAAS team fully
-supports. Currently supported OS' are CentOS and Windows.
-
-`Custom images` are images that MAAS can deploy, but may have been
-customized and differ from those that the MAAS team supports. Custom
-images can include any Ubuntu, CentOS or Windows image. Additionally,
-Redhat Enterprise Linux (RHEL), OpenSUSE and SLES images also fall
-under this category.
-
-
-Installing MAAS Images
-----------------------
-
-Installing Generated Images
-^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-Installing MAAS Generated Images (CentOS or Windows) can be done with
-the command::
-
-    $ maas admin boot-resources create name=<os/series>
-      architecture=<architecture> [filetype=ddtgz]
-      content@=<image-name>
-
-The list of supported Operating Systems under `Generated Images` is:
-
- * CentOS 6.5 (centos/centos65)
- * CentOS 7 (centos/centos7)
- * Windows Server 2012 (windows/win2012)
- * Windows Server 2012 R2 (windows/win2012r2)
- * Windows Hyper-V (windows/win2012hv)
- * Windows Hyper-V R2 (windows/win2012hvr2)
-
-Examples::
-
-    $ maas admin boot-resources create name=centos/centos7
-      architecture=amd64/generic content@=centos7-amd64-root-tgz
-
-    $ maas admin boot-resources create name=windows/win2012
-      architecture=amd64/generic filetype=ddtgz
-      content@=win2012-amd64-ddtgz
-
-Installing Custom Images
-^^^^^^^^^^^^^^^^^^^^^^^^
-
-Installing custom images require the user to specify a unique
-identifier for the OS and Release, as well as the title of the
-Image::
-
-    $ maas admin boot-resources create name=custom/<os-release-id>
-      title=<title> architecture=amd64/generic content@=<image-name>
-
-Examples::
-
-    $ maas admin boot-resources create name=custom/rhel7
-      title="RedHat Enterprise Linux 7" architecture=amd64/generic
-      content@=rhel7-amd64-root-tgz
-
-The list of tested Operating Systems under `Custom Images` is:
-
- * OpenSUSE
- * SLES 11 and SLES 12
- * Redhat Enterprise Linux 7 (RHEL7)
-
-
-MAAS Image Builder
-------------------
-
-The MAAS Image Builder is a script that allows users to generate their
-own images. However, ``maas-image-builder`` only supports the generation
-of CentOS and RHEL images. To install ``maas-image-builder``, please
-follow the instructions::
-
-  $ sudo apt-add-repository ppa:maas-maintainers/stable
-  $ sudo apt-get install maas-image-builder
-
-
-Creating and Installing CentOS MAAS Images
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-.. note::
-
-  Supported CentOS versions are CentOS 6.5 (centos65) and CentOS 7 (centos7).
-
-
-To create a MAAS CentOS image, no ISO is needed as all of the required
-software to create the image is downloaded (from CentOS repositories) as part
-of the image generation process. To create the MAAS CentOS image,
-``maas-image-builder`` can be used as::
-
-    $ maas-image-builder -a amd64 -o centos7-amd64-root-tgz centos --edition 7
-
-The created image, ``centos7-amd64-root-tgz``, needs to be uploaded into the
-MAAS Region Controller. This image will be listed under the `Generated` Image
-Section in the `MAAS Image` page. This image can be uploaded with::
-
-    $ maas admin boot-resources create name=centos/centos7
-      architecture=amd64/generic content@=./build-output/centos7-amd64-root-tgz

=== removed file 'docs/packagerepositories.rst'
--- docs/packagerepositories.rst	2016-08-15 09:54:43 +0000
+++ docs/packagerepositories.rst	1970-01-01 00:00:00 +0000
@@ -1,71 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _packagerepositories:
-
-====================
-Package Repositories
-====================
-
-.. note::
-
-  This feature is available in MAAS versions 2.0 and above.
-
-MAAS allows the configuring of multiple additional package repositories, which
-will find their way into the /etc/apt/sources.list on MAAS deployed machines.
-
-Creating a Package Repository
------------------------------
-
-Administrators can create Package Repositories over the API using the following command:::
-
-  $ maas <profile> package-repositories create name=<Package Repository Name> url=<URL locating the package repository>
-
-These are the minimally required parameters - see the online API help,
-described below, for the complete list of parameters.
-
-Package Repository Enablement
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-
-Package Repositories can be turned off by using the enabled flag option as follows:::
-
-  $ maas <profile> package-repositories create name=<Package Repository Name> url=<URL locating the package repository> enabled=false
-
-The enabled flag can be modified on an existing repository by using the update
-API, described below.
-
-Listing Package Repositories
-----------------------------
-
-To list all Package Repositories use the following command:::
-
-  $ maas <profile> package-repositories read
-
-To list a particular Package Repository use the following command:::
-
-  $ maas <profile> package-repository read <Package Repository id or name>
-
-Updating a Package Repository
------------------------------
-
-Administrators can update the Package Repository attributes using the following
-command:::
-
-  $ maas <profile> package-repository update <Package Repository id or name> <options>
-
-Deleting a Package Repository
------------------------------
-
-Administrators can delete a Package Repository using the following command:::
-
-  $ maas <profile> package-repository delete <Package Repository id or name>
-
-Getting Help
-------------
-
-Online help is available for all API's in MAAS. For example:::
-
-  $ maas <profile> package-repository --help
-  $ maas <profile> package-repository <command> --help
-
-  $ maas <profile> package-repositories --help
-  $ maas <profile> package-repositories <command> --help

=== removed file 'docs/physical-zones.rst'
--- docs/physical-zones.rst	2014-08-22 00:13:24 +0000
+++ docs/physical-zones.rst	1970-01-01 00:00:00 +0000
@@ -1,132 +0,0 @@
-Physical Zones
-==============
-
-.. note::
-  This feature is only available in MAAS versions 1.5 and above.
-
-To help you maximise fault-tolerance and performance of the services you
-deploy, MAAS administrators can define *physical zones* (or just *zones*
-for short), and assign nodes to them.  When a user requests a node, they can
-ask for one that is in a specific zone, or one that is not in a specific zone.
-
-It's up to you as an administrator to decide what a physical zone should
-represent: it could be a server rack, a room, a data centre, machines attached
-to the same UPS, or a portion of your network.  Zones are most useful when they
-represent portions of your infrastructure.  But you could also use them simply
-to keep track of where your systems are located.
-
-Each node is in one and only one physical zone.  Each MAAS instance ships with
-a default zone to which nodes are attached by default.  If you do not
-need this feature, you can simply pretend it does not exist.
-
-
-Applications
-------------
-
-Since you run your own MAAS, its physical zones give you more flexibility
-than those of a third-party hosted cloud service.  That means that you get to
-design your zones and define what they mean.  Below are some examples of how
-physical zones can help you get the most out of your MAAS.
-
-
-Using Zones for Fault Tolerance
-...............................
-
-If you are concerned about availability of services you deploy through MAAS, a
-physical zone could be on one power supply, or it could be an entire data
-centre location, or an area of your network that is unlikely to suffer problems
-when another zone experiences an outage.
-
-For example, you might roll out separate instances of the same web application
-into different physical zones of the same MAAS, and either load-balance
-between them or keep one as a warm stand-by.  If one zone suffers a power loss,
-is cut off from the internet, or is affected by a natural disaster, the other
-instance of your application may still be available in the other zone, and
-ready to take over.
-
-For fault tolerance, machines that work together in order to provide one
-instance of a service should generally be allocated in the same zone.  The
-entire service should have a backup instance in another zone.
-
-
-Using Zones for Performance
-...........................
-
-Even if fault tolerance is not an issue, you may still want to divide your
-nodes into groups that communicate efficiently through a high-performance local
-network, or share efficient access to external resources.
-
-When it comes to performance, machines should generally be allocated in the
-zone closest to performance-critical resources they need.
-
-For example, for applications that are highly sensitive to network latency, it
-may make sense to divide your MAAS into smaller physical networks, and
-represent each of those networks as a physical zone.  Now, you can use the
-physical zones to allocate nodes just where they get the best performance
-when communicating with specific other nodes.
-
-In another example, your application may rely on a third-party service
-available on the internet.  If your MAAS is spread out across multiple data
-centre locations, you may want the application to be deployed in the location
-with the best access to that service.
-
-
-Using Zones for Power Management
-................................
-
-If you are concerned about power density and cooling, you can lay out your
-physical zones to match server racks.  Instead of allocating machines that
-share a physical zone, you can spread out the load by ensuring that "hot"
-systems are in located different zones.
-
-
-Creating a Zone
----------------
-
-Only administrators can create and manage zones.  To create a physical
-zone in the web user interface, log in as an administrator and browse to the
-"Zones" section in the top bar.  This will takes you to the zones listing page.
-At the bottom of the page is a button for creating a new zone:
-
-.. image:: media/add-zone.*
-
-Or to do it in the :ref:`region-controller API <region-controller-api>`, POST
-your zone definition to the *"zones"* endpoint.
-
-
-Assigning Nodes to a Zone
--------------------------
-
-Once you have created one or more physical zones, you can set nodes' zones
-from the nodes listing page in the UI.  Select the nodes for which you wish to
-set a zone, and choose "Set physical zone" from the "Bulk action" dropdown
-list near the top.  A second dropdown list will appear, to let you select which
-zone you wish to set.  Leave it blank to clear nodes' physical zones.
-Clicking "Go" will apply the change to the selected nodes.
-
-You can also set an individual node's zone on its "Edit node" page.  Both ways
-are available in the API as well: edit an individual node through a ``PUT``
-request to the node's URI, or set the zone on multiple nodes at once by calling
-the ``set_zone`` operation on the ``nodes`` endpoint.
-
-
-Allocating a Node in a Zone
----------------------------
-
-To deploy in a particular zone, call the ``acquire`` method in the
-:ref:`region-controller API <region-controller-api>` as before, but pass the
-``zone`` parameter with the name of the zone.  The method will allocate a node
-in that zone, or fail with an HTTP 409 ("conflict") error if the zone has no
-nodes available that match your request.
-
-Alternatively, you may want to request a node that is *not* in a particular
-zone, or one that is not in any of several zones.  To do that, specify the
-``not_in_zone`` parameter to ``acquire``.  This parameter takes a list of zone
-names; the allocated node will not be in any of them.  Again, if that leaves no
-nodes available that match your request, the call will return a "conflict"
-error.
-
-It is possible, though not usually useful, to combine the ``zone`` and
-``not_in_zone`` parameters.   If your choice for ``zone`` is also present in
-``not_in_zone``, no node will ever match your request.  Or if it's not, then
-the ``not_in_zone`` values will not affect the result of the call at all.

=== removed file 'docs/power-driver-capabilities.rst'
--- docs/power-driver-capabilities.rst	2016-05-27 22:46:58 +0000
+++ docs/power-driver-capabilities.rst	1970-01-01 00:00:00 +0000
@@ -1,48 +0,0 @@
-.. -*- mode: rst -*-
-
-*************************
-Power Driver Capabilities
-*************************
-
-
-**2016-05-23, newell**
-
-MAAS has internal support for power controlling differents types of hardware.
-Not all the power drivers in MAAS have the same capabilities and the table below
-details the main differences amongst the power drivers.
-
-
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Power Driver                          | Set PXE On  | Power     |    Chassis    | Enhanced UI     |    BMC     |
-|                    X=Yes, Blank=No    |  Next Boot  | Querying  | Configuration | Error Reporting | Enlistment |
-+=========================+=============+=============+===========+===============+=================+============+
-| American Power Conversion (APC) - PDU |             |           |               |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Cisco UCS Manager                     |      X      |     X     |       X       |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Digital Loggers, Inc. - PDU           |             |           |               |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| HP Moonshot - iLO Chassis Manager     |      X      |     X     |       X       |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| HP Moonshot - iLO4 (IPMI)             |      X      |     X     |               |                 |     X      |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| IBM Hardware Management Console (HMC) |      X      |     X     |               |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| IPMI                                  |      X      |     X     |               |       X         |     X      |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Intel AMT                             |      X      |     X     |               |       X         |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Manual                                |             |           |               |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Microsoft OCS - Chassis Manager       |      X      |     X     |       X       |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| OpenStack Nova                        |             |     X     |               |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| SeaMicro 15000                        |      X      |     X     |       X       |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Sentry Switch CDU - PDU               |             |           |               |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| VMWare                                |      X      |     X     |       X       |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+
-| Virsh (virtual systems)               |      X      |     X     |       X       |                 |            |
-+---------------------------------------+-------------+-----------+---------------+-----------------+------------+

=== removed file 'docs/proxy.rst'
--- docs/proxy.rst	2016-04-07 17:04:26 +0000
+++ docs/proxy.rst	1970-01-01 00:00:00 +0000
@@ -1,81 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _proxy:
-
-Proxy Configuration
-===================
-
-In most cases, the machines deployed in a MAAS installation will use a proxy to
-access the archive.  Alternatively, MAAS can be told to use an externall
-configured proxy, or none at all.
-
-As of MAAS 2.0, the default MAAS region controller configuration will create a
-maas-proxy configuration that explicitly allows all of the subnets in the
-configuration to use it for a proxy.
-
-
-Assumptions
------------
-
-It is generally assumed that the MAAS region controller is not exposed to
-untusted networks.  (That is, that there is a firewall between the MAAS region
-controller and any such networks.)  Prior to MAAS 2.0, the only option if this
-was not the case was to manually edit /etc/maas/maas-proxy.conf, or use
-iptables to create a firewall on the host.
-
-As of MAAS 2.0, permission to use the proxy can be managed on a per-subnet
-basis. (The default is to allow proxying.)
-
-.. warning:: If your MAAS Region is connected to an untrusted network, you should disable that subnet in the proxy, as shown below.
-
-MAAS Proxy Installation
------------------------
-
-When you install a Region Controller, maas-proxy will be installed.  The
-configuration of the proxy relies on maas-proxy being on the same machine as
-the Region Controller.
-
-
-Using an external proxy
------------------------
-If you want to use an external proxy, see the Network Configuration section of
-the web user interface to set the proxy url.
-
-.. image:: media/external-proxy.png
-
-Alternatively, the following
-command will specify squid.example.com as the proxy, using port 3128::
-
- $ maas admin maas set-config name=http_proxy value=http://squid.example.com:3128/
-
-Disable proxying for a Subnet
------------------------------
-As an example, we will configure MAAS to disable the proxy for a subnet::
-
- $ maas admin subnet update 192.168.0.0/22 allow_proxy=False
-
-Enable proxying for a Subnet
-----------------------------
-To re-enable the proxy for that subnet::
-
- $ maas admin subnet update 192.168.0.0/22 allow_proxy=True
-
-Allow an arbitrary subnet to use the Proxy
-------------------------------------------
-Since the default is to allow proxying, simply create the subnet in MAAS::
-
- $ maas admin subnets create cidr=192.168.100.0/23
-
-You may wish to set other parameters on the subnet.
-
-Disabling proxying
-------------------
-If you want to not use any proxy, see the Network Configuration section of
-the web user interface and uncheck the "Enable the use of an APT and HTTP/HTTPS
-proxy" box.
-
-.. image:: media/no-proxy.png
-
-Alternatively, the following command will disable the proxy::
-
- $ maas admin maas set-config name=enable_http_proxy value=False

=== removed file 'docs/rack-configuration.rst'
--- docs/rack-configuration.rst	2016-04-26 17:31:11 +0000
+++ docs/rack-configuration.rst	1970-01-01 00:00:00 +0000
@@ -1,192 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _rack-configuration:
-
-Rack Controller Configuration
-=============================
-
-Before any of MAAS's features can be used for the first time, you must have
-a Rack Controller and configure at least one VLAN to manage DHCP for at least
-a subnet. Each machine can be attached to any number of subnets, which may or
-may not be managed by MAAS.
-
-Managing a VLAN normally means that MAAS will server DHCP from the Rack
-Controller, for the purpose of providing IP address to machines that
-are being enlisted or commissioned. Also, any other DHCP client that is
-on the VLAN will obtain a dynamic IP address from the MAAS DHCP server.
-
-**Do this only on a network that was set up with this in mind**. Enabling
-your own DHCP server that competes with an existing one that's
-being managed by MAAS can cause serious disruption, and it can be hard
-for administrator to track the source of the problem.  Worse, the problems
-may not become immediately noticeable.  Make sure you understand the
-implications of running a DHCP server before doing this.  If MAAS detects any
-DHCP servers already running on these networks, it will show them on the
-rack's configuration page.
-
-
-Network requirements
---------------------
-
-The Rack Controller manages DHCP for subnet(s) in the VLAN(s) that it is
-connected to via one of its interfaces defined in MAAS. Rack Controller
-interfaces are discovered automatically, when MAAS is installed, and any
-future changes are automatically communicated to the region.
-
-When a Rack Controller manages machines on the network through one of the
-VLANs it is connected to, the machines must be in the same subnet as the
-Rack Controller interface connected to that VLAN. This is because:
-
-#. If the VLAN that the Rack Controller is connected to is configured to
-   provide DHCP, the nodes must be able to configure their own network
-   interfaces using MAAS's DHCP server. This means that either they must
-   be on the same subnet, or that DHCP packets are being specially routed
-   between the machine's subnet and MAAS' DHCP server.
-
-
-Registration
-------------
-
-If you install your first Rack Controller on the same system as the Region
-Controller, as is the case when you install the full "maas" ubuntu package,
-it will be automatically accepted by default (but not yet configured, see
-below).  Any other Rack Controllers you set up will show up automatically
-after they have been manually added to MAAS.
-
-To install a new MAAS Rack Controller, at the command line, type::
-
- $ sudo apt-get install maas-rack-controller
-
-Once installed, you will need to input the shared secret, in order
-to allow the Rack Controller to authenticate with the Region Controller.
-You can obtain the shared secret from the Region Controller. The
-shared secret can be copied from the Region Controller in::
-
- /var/lib/maas/secret
-
-Once you have copied the secret, you can install it in the Rack
-Controller. At the command line, type::
-
- $ sudo maas-rack install-shared-secret
-
-Once installed, you will need to reconfigure the Rack Controller
-to correctly point to the API hostname or IP address of the Region
-Controller that's on the same network as the Rack Controller::
-
- $ sudo dpkg-reconfigure maas-rack-controller
-
-The configuration for the MAAS Rack Controller will automatically
-run and pop up this config screen:
-
-.. image:: media/install_cluster-config.*
-
-This is where the MAAS Rack Controller is prompting for the URL
-of the MAAS Region Controller. Once entered, the Rack Controller
-configuration will be complete.
-
-
-Interface Management
---------------------
-
-MAAS automatically recognises the network interfaces on each Rack
-Controller.  Some (though not necessarily all) of these will be connected
-to subnets on a VLAN inside a Fabric. In other words, the Rack Controllers
-will be connected to VLANs, and the subnets being served on these.
-
-Once a new Rack Controller is connected, it will try to autodetect in
-what Subnet, VLAN and even Fabric the interface is connected to. If these
-have not being created, new Subnets, VLAN's and Fabrics and Spaces will
-be created.
-
-If Fabrics, VLANs and Subnets are already created, once MAAS automatically
-recognises the Rack Controller network interfaces, it will try to determine
-to which these are connected to before being able to provide services.
-
-As such, each Rack Controller interface will determine whether a Rack
-Controller can provide DHCP on an specific VLAN, or for advanced configuration,
-a Rack Controller interface will determine whether a Rack Controller
-can be a primary or backup Rack on an HA configuration.
-
-If for any reason, the Rack Controller interfaces are mis-identified
-and are in the correct fabric, the user can manually change that by
-editing the Rack Controller Fabric information:
-
-.. image:: media/rack-interface-edit.png
-
-
-In order for MAAS to be able to manage a machine throught its lifecycle,
-it needs to provide DHCP for at least one subnet, by configuring the
-corresponding VLAN to which the Rack Controller is connected to.
-
-
-Providing DHCP + HA
--------------------
-In order for MAAS to be able to manage machines on the network, and more
-specifically, in order to be able to enlist, commission and deploy machines
-it needs to provide and manage DHCP. As such, Rack Controller(s) can
-provide DHCP on the different VLANs it is connected to.
-
-Dynamic range & addresses
-^^^^^^^^^^^^^^^^^^^^^^^^^
-Starting from MAAS 2.0, a Dynamic range needs to be defined in order for
-MAAS to be able to provide DHCP for machines. The purpose of the dynamic
-range is to:
-
- * provide DHCP for machines that are performing auto-registration (also
-   called enlistment), or
- * provide DHCP for machines that are being commissioned.
-
- This means that when a machine is being enlisted or commissioned, they
- DHCP from the dynamic range in order to perform those two steps.
-
- In turn, however, machines that are being deployed will, by default,
- obtain static IP addresses (DHCP or otherwise) from the section of the
- subnet that has not been reserved for the Dynamic Range on such VLAN.
- This allows the machine to obtain IP addresses that will remain
- allocated to a machine throughout the rest of its deployment.
-
-Enabling a DHCP on a VLAN (optional HA)
-^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-As an example, we will configure MAAS to provide DHCP on a Rack
-Controller.
-
-The first thing that we need to do is select in what VLAN and subnet
-we would like to configure DHCP. In this case, in order to be able
-to PXE boot machines, we need to configure DHCP for an 'untagged' VLAN
-under any fabric. In this example, we will chose the 'untagged' VLAN on
-'fabric-0'. The subnet '192.168.10.0/24' is on this 'untagged' VLAN. We
-can see that the Rack Controller has access to this VLAN via its 'eth0'
-interface.
-
-The second step is to create a dynamic range::
-
- $ maas admin ipranges create type=dynamic start_ip=192.168.10.28 end_ip=192.168.10.100
-
-The third, and last step is to enable DHCP on a VLAN. For this to be
-effective we need to at least select the Primary Rack controller that will
-provide DHCP::
-
- $ maas admin vlan update fabric-0 untagged dhcp_on=True primary_rack=node01
-
-If enabling DHCP HA is something you need for your MAAS deployment, then
-following operation would enable HA::
-
- $ maas admin vlan update fabric-0 untagged dhcp_on=True primary_rack=node01 secondary_rack=node02
-
-Note that if you are enabling DHCP over the CLI, but the subnet doesn't have a
-Gateway IP defined, you can define the gateway using::
-
-  $ maas admin subnet update 192.168.10.0/24 gateway_ip=192.168.10.1
-
-You can also do the same configuration via the WebUI on the VLAN details page:
-
-.. image:: media/vlan_provide_dhcp.png
-
-
-Multiple networks
------------------
-
-A single Rack Controller can be connected to more than one VLAN, each from a
-different Rack Controller interface.  This allows one Rack Controller to
-manage different subnets to help scale your Rack Controller or to satisfy
-your network architecture.

=== modified file 'docs/releases.rst'
--- docs/releases.rst	2016-04-11 16:23:26 +0000
+++ docs/releases.rst	2017-02-02 20:19:50 +0000
@@ -1,62 +1,29 @@
+*************
 MAAS Releases
-=============
-
-
-MAAS currently has the following releases:
-
-
-1.5
----
-
-* Status: Deprecated.
-* Last release: 1.5.4
-* Location: ubuntu/trusty
-* Support:
-
-  * Release has been deprecated.
-  * No longer providing updates.
-  * Users need to upgrade to 1.7 or 1.9.
-
-
-1.7
----
-
-* Status: Old Stable Release
-* Current Release: 1.7.6
-* Location: ubuntu/trusty-updates
-* Support:
-
-  * It is currently only supported on Ubuntu trusty-updates.
-  * It will be supported until 1.9 series is SRU'd into Trusty.
-  * It will be deprecated once supreseeded by 1.9 around February 2016.
-
-
-1.8
----
-
-* Status: Deprecated
-* Initial release data: June 2015
-* Current release: 1.8.3
-* Location: No longer available
-* Support:
-
-  * Release has been deprecated.
-  * No longer providing updates.
-  * Users need to upgrade to 1.9.
-
+*************
+
+Stable Release
+==============
 
 1.9
 ---
 
+ * Status: Old Stable Release
+ * Latest version: 1.9.4
+ * Support: Ubuntu Trusty LTS. Only critical issues.
+
+2.1
+---
+
  * Status: Current Stable Release
- * Initial Release date: November 2015
- * Current release: 1.9.0
- * Location: ppa:maas/stable
- * Support: This will be an LTS release.
-
-
-2.0
+ * Latest version: 2.1.3
+ * Support: Ubuntu Xenial LTS.
+
+Development release
+===================
+
+2.2
 ---
 
- * Status: New Development Release
- * Estimated Release Date: March or April 2016
+ * Status: Current development release
+ * Localtion: ppa:maas/next

=== removed file 'docs/sstreams-mirror.rst'
--- docs/sstreams-mirror.rst	2016-10-17 20:43:34 +0000
+++ docs/sstreams-mirror.rst	1970-01-01 00:00:00 +0000
@@ -1,64 +0,0 @@
-Local Mirroring of Boot Images
-==============================
-
-Boot images are delivered to MAAS via the simplestreams protocol.  It is
-useful in some situations, such as testing, to mirror the images locally
-so that you don't need to repeatedly pull them down over a slower Internet
-link.
-
-First, install the required packages on the host where you wish to store
-the mirrored images::
-
-  $ sudo apt-get install simplestreams ubuntu-cloudimage-keyring apache2
-
-Now you can pull the images over using the mirroring tools for simplestreams.
-This example gets the daily trusty (14.04) and xenial (16.04) images for the
-amd64 architecture::
-
-  $ sudo sstream-mirror --keyring=/usr/share/keyrings/ubuntu-cloudimage-keyring.gpg https://images.maas.io/ephemeral-v3/daily/ /var/www/html/maas/images/ephemeral-v3/daily 'arch=amd64' 'release~(trusty|xenial)' --max=1
-
-This may take a while as hundreds of megabytes will be downloaded.
-
-As of MAAS 2.1 bootloaders are now included in the simplestream. In order for
-MAAS to work bootloaders must be mirrored as well. It is recommended that all
-bootloaders are mirrored::
-
-  $ sudo sstream-mirror --keyring=/usr/share/keyrings/ubuntu-cloudimage-keyring.gpg https://images.maas.io/ephemeral-v3/daily/ /var/www/html/maas/images/ephemeral-v3/daily 'os~(grub*|pxelinux)' --max=1
-
-The images will be written to the local disk and you can verify their
-presence by browsing to
-``http://<server>/maas/images/ephemeral-v3/daily/streams/v1/index.sjson``
-(replace ``<server>`` with your own server's name).
-
-It is a good idea to configure a ``cron`` job to repeat this import on a
-regular basis to keep your mirror up-to-date.
-
-
-Configuring MAAS to use the local mirror
-----------------------------------------
-
-You can do this using the API or the web UI.  To do this via the API you
-can use the ``maas`` (see :doc:`maascli`) command, logged in as the admin
-user::
-
-  $ maas <profile> boot-sources create url=http://<server>/images/ephemeral-v3/daily/ keyring_filename=/usr/share/keyrings/ubuntu-cloudimage-keyring.gpg
-
-Note that if you wish to use older images (which change far less frequently,
-but will be lacking security updates), you can use the ``releases`` stream,
-such as::
-
-  $ maas <profile> boot-sources create url=http://<server>/images/ephemeral-v3/releases/ keyring_filename=/usr/share/keyrings/ubuntu-cloudimage-keyring.gpg
-
-And then initiate the download with::
-
-  $ maas <profile> boot-resources import
-
-See :doc:`bootsources` for more detail.
-
-In the web UI, browse to the Settings tab as the admin user and scroll down
-to the "Boot Images" section.  There you will find input boxes for the
-Sync URL and the keyring path, which should be set as the values in the API
-example above.
-
-The import is initiated by browsing to the Images tab and following the
-instructions on that page.

=== removed file 'docs/static-ips.rst'
--- docs/static-ips.rst	2016-03-28 13:54:47 +0000
+++ docs/static-ips.rst	1970-01-01 00:00:00 +0000
@@ -1,53 +0,0 @@
-.. _static-ips:
-
-Static IPs
-==========
-
-.. note::
-
-  This feature is available in MAAS versions 1.6 and above.
-  If you're writing a client application, you can check if MAAS
-  supports this feature via the web API; see the documentation for the
-  ``static-ipaddresses`` capability :ref:`here<cap-static-ipaddresses>`.
-
-Previously, MAAS relied on the DHCP server to allocate its own IP
-addresses to nodes, using the IP range defined on the relevant cluster
-interface. This was found to be unreliable since the IPs were only known
-once the node had booted and requested an address, and had race conditions
-when the lease expired and another machine was looking for its own IP.
-
-MAAS now defines an additional range on the cluster for static IPs that
-are allocated by MAAS itself (see :doc:`rack-configuration` for more
-information about this).
-
-In normal operation, MAAS will automatically choose and allocate a static IP to
-any node network interfaces where it knows on which cluster interface that node
-interface is connected.
-
-The :doc:`api` contains functions to request additional static IPs, which are
-summarised here:
-
-Sticky IPs
-----------
-Synopsis:
-  ``POST /api/2.0/nodes/{system_id}/`` ``op=claim_sticky_ip_address``
-
-Normally, IPs are released back into the pool of available IPs once a node
-is released by a user.  A "Sticky" IP is one which is allocated to an interface
-on a node that does not change unless the sticky IP is removed.  This enables
-more predictable IPs at the cost of wasting IPs when the node is not in use.
-
-Unmanaged User-allocated IPs
-----------------------------
-Synopsis:
-  ``GET /api/2.0/ipaddresses/``
-
-  ``POST /api/2.0/ipaddresses/`` ``op=release``
-
-  ``POST /api/2.0/ipaddresses/`` ``op=reserve``
-
-This API allows users to request an ad-hoc IP address for use in any way they
-see fit.  The IP is not tied to any node in MAAS and is guaranteed not to be
-in use by MAAS itself.
-
-See the full :doc:`api` documentation for precise technical details.

=== removed file 'docs/storage.rst'
--- docs/storage.rst	2016-01-25 14:10:29 +0000
+++ docs/storage.rst	1970-01-01 00:00:00 +0000
@@ -1,679 +0,0 @@
-.. -*- mode: rst -*-
-
-.. _storage:
-
-=========================
-Storage
-=========================
-
-.. note::
-
-  This feature is available in MAAS versions 1.9 and above on Ubuntu
-  deployments.  If you're writing a client application, you can check
-  if MAAS supports this feature via the web API; see the documentation
-  for the ``storage-deployment-ubuntu`` capability
-  :ref:`here<cap-storage-deployment-ubuntu>`.
-
-MAAS has the ability to configure any storage layout during node deployment.
-MAAS doesn't just do simple partitioning it supports complex storage layouts,
-including setting up and configuring Bcache, RAID, and LVM. This gives users
-unlimited possibilities on the storage configurations they want to deploy.
-
-Layouts
--------
-
-When a node is acquired by a user it gets a default storage layout. This layout
-provides the basic storage configuration to allow a node to deploy
-successfully. The default storage layout can also be adjusted allowing an
-administrator to make the decision on which layout will be the default.
-
-The users deploying nodes are not limited by the default. They can set an
-explicit storage layout when they acquire a node or after they have acquried a
-node with the set-storage-layout API. The user acquiring a node or performing
-the set-storage-layout API calls can also customize the layout generation. Each
-layout has a set of options that can be set to adjust the generated layout.
-
-Below list all the available storage layouts and the available options for
-each.
-
-LVM Layout
-^^^^^^^^^^
-
-Creates a volume group `vgroot` on a partition that spans the entire boot disk.
-A logical volume `lvroot` is created for the full size of the volume group. The
-`lvroot` is formatted with `ext4` and set as the `/` mount point.
-::
-
-    NAME        SIZE        TYPE    FSTYPE         MOUNTPOINT
-    sda         100G        disk
-      sda1      512M        part    fat32          /boot/efi
-      sda2      99.5G       part    lvm-pv(vgroot)
-    vgroot      99.5G       lvm
-      lvroot    99.5G       lvm     ext4           /
-
-The following options are supported for this layout.
-::
-
-    boot_size: Size of the boot partition on the boot disk. Default is 0,
-        meaning not to create the boot partition. The '/boot' will be placed
-        on the root filesystem.
-    root_device: The block device to place the root partition on. Default is
-        the boot disk.
-    root_size: Size of the root partition. Default is 100%, meaning the
-        entire size of the root device.
-    vg_name: Name of the created volume group. Default is `vgroot`.
-    lv_name: Name of the created logical volume. Default is `lvroot`.
-    lv_size: Size of the created logical volume. Default is 100%, meaning
-        the entire size of the volume group.
-
-Flat Layout
-^^^^^^^^^^^
-
-Creates a partition that spans the entire boot disk. The partition is formatted
-with `ext4` and set as the `/` mount point.
-::
-
-  NAME        SIZE        TYPE    FSTYPE         MOUNTPOINT
-  sda         100G        disk
-    sda1      512M        part    fat32          /boot/efi
-    sda2      99.5G       part    ext4           /
-
-The following options are supported for this layout.
-::
-
-  boot_size: Size of the boot partition on the boot disk. Default is 0,
-      meaning not to create the boot partition. The '/boot' will be placed
-      on the root filesystem.
-  root_device: The block device to place the root partition on. Default is
-      the boot disk.
-  root_size: Size of the root partition. Default is 100%, meaning the
-      entire size of the root device.
-
-Bcache Layout
-^^^^^^^^^^^^^
-
-Creates a Bcache using a partition that spans the entire boot disk as the
-backing device. Uses the smallest block device tagged with `ssd` as the cache
-device. The Bcache device is formatted with `ext4` and set as the `/` mount
-point. If no block devices exists on the node that are tagged with `ssd` then
-the Bcache device will not be created and the `flat` layout will be used.
-::
-
-  NAME        SIZE        TYPE    FSTYPE         MOUNTPOINT
-  sda         100G        disk
-    sda1      512M        part    fat32          /boot/efi
-    sda2      99.5G       part    bc-backing
-  sdb         50G         disk
-    sdb1      50G         part    bc-cache
-  bcache0     99.5G       disk    ext4           /
-
-The following options are supported for this layout.
-::
-
-  boot_size: Size of the boot partition on the boot disk. Default is 0,
-      meaning not to create the boot partition. The '/boot' will be placed
-      on the root filesystem.
-  root_device: The block device to place the root partition on. Default is
-      the boot disk.
-  root_size: Size of the root partition. Default is 100%, meaning the
-      entire size of the root device.
-  cache_device: The block device to use as the cache device. Default
-      is the smallest block device tagged ssd.
-  cache_mode: The cache mode to set the created Bcache device to. Default
-      is `writethrough`.
-  cache_size: The size of the partition on the cache device. Default is
-      100%, meaning the entire size of the cache device.
-  cache_no_part: Whether or not to create a partition on the cache device.
-      Default is false, meaning to create a partition using the given
-      `cache_size`. If set to true no partition will be created and the raw
-      cache device will be used as the cache.
-
-.. note::
-
-  The `/boot/efi` partition on all layouts will only be created on nodes that
-  deploy with UEFI.
-
-
-Setting the Layout
-------------------
-
-The following are a couple of was the storage layout can be changed either
-globally, on acquire, or after acquire.
-
-Globally
-^^^^^^^^
-
-The global default storage layout can be set using the API and the UI. This
-will change the default storage layout for when a node is acquired. `It will
-not adjust the layout of any node that is already passed the acquire stage.`::
-
-  $ maas my-maas-session maas set_config name=default_storage_layout value=flat
-
-Set Storage Layout
-^^^^^^^^^^^^^^^^^^
-
-If a node is already acquired and you want to adjust the storage layout the
-set_storage_layout API call can be used. The options for this API call do not
-require the `storage_layout_` prefix.::
-
-  $ maas my-maas-session node set-storage-layout <system-id> storage_layout=lvm lv_size=50%
-
-.. note::
-
-  This will completely remove any previous storage configuration on all block
-  devices.
-
-Block Devices
--------------
-
-Once the initial storage layout has been configure on a node you can perform
-many operations to view and adjust the entire storage layout for the node. In
-MAAS there are two different types of block devices.
-
-**Physical**
-
-A physical block device is a physically attached block device. This being true
-storage on a machine. E.g. A 100G hard drive in a server.
-
-**Virtual**
-
-A virtual block device is a block device that is exposed by the Linux kernel
-when an operation is performed. Almost all the operations on a physical block
-device can be performed on a virtual block device. E.g. A RAID device exposed
-as `md0`.
-
-List Block Devices
-^^^^^^^^^^^^^^^^^^
-To view all block devices on a node use the `read` operation. This list both
-physical and virtual block devices.::
-
-  $ maas my-maas-session block-devices read node-f4e2281c-d19a-11e4-a5ac-00163edde41f
-  [
-      {
-          "size": 21474836480,
-          "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/",
-          "uuid": null,
-          "tags": [
-              "ssd",
-              "rotary"
-          ],
-          "name": "sda",
-          "partition_table_type": "MBR",
-          "id_path": "/dev/disk/by-id/ata-QEMU_HARDDISK_QM00001",
-          "path": "/dev/disk/by-dname/sda",
-          "model": "QEMU HARDDISK",
-          "block_size": 4096,
-          "type": "physical",
-          "id": 6,
-          "serial": "QM00001",
-          "partitions": [
-              {
-                  "uuid": "e94ca09a-d83e-4521-8bac-833da2ed0b3e",
-                  "bootable": false,
-                  "filesystem": {
-                      "label": null,
-                      "mount_point": null,
-                      "uuid": "61d447c2-387d-4fb1-885a-65eeef91e92a",
-                      "fstype": "lvm-pv"
-                  },
-                  "path": "/dev/disk/by-dname/sda-part1",
-                  "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/1",
-                  "type": "partition",
-                  "id": 1,
-                  "size": 21471690752
-              }
-          ]
-      },
-      {
-          "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/11/",
-          "uuid": "f58b8fb2-dcf2-4ba9-a01c-60409829a64e",
-          "tags": [],
-          "partitions": [],
-          "name": "vgroot-lvroot",
-          "partition_table_type": null,
-          "filesystem": {
-              "label": "root",
-              "mount_point": "/",
-              "uuid": "9f09e3fd-2484-4da5-bb56-a72a0c478d06",
-              "fstype": "ext4"
-          },
-          "id_path": null,
-          "path": "/dev/disk/by-dname/lvroot",
-          "model": null,
-          "block_size": 4096,
-          "type": "virtual",
-          "id": 11,
-          "serial": null,
-          "size": 21470642176
-      }
-  ]
-
-Read Block Device
-^^^^^^^^^^^^^^^^^
-
-If you want to read just one block device instead of listing all block devices
-the `read` operation on the `block-device` endpoint provides that information.
-::
-
-  $ maas my-maas-session block-device read node-f4e2281c-d19a-11e4-a5ac-00163edde41f 12
-  {
-      "size": 21474836480,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/12/",
-      "uuid": null,
-      "tags": [],
-      "name": "sdb",
-      "partition_table_type": null,
-      "id_path": "",
-      "path": "/dev/disk/by-dname/sdb",
-      "model": "QEMU HARDDISK",
-      "block_size": 4096,
-      "type": "physical",
-      "id": 12,
-      "serial": "QM00001",
-      "partitions": []
-  }
-
-It is also possible to use the name of the block device instead of its ID.::
-
-  $ maas my-maas-session block-device read node-f4e2281c-d19a-11e4-a5ac-00163edde41f sdb
-  {
-      "size": 21474836480,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/12/",
-      "uuid": null,
-      "tags": [],
-      "name": "sdb",
-      "partition_table_type": null,
-      "id_path": "",
-      "path": "/dev/disk/by-dname/sdb",
-      "model": "QEMU HARDDISK",
-      "block_size": 4096,
-      "type": "physical",
-      "id": 12,
-      "serial": "QM00001",
-      "partitions": []
-  }
-
-.. note::
-
-  MAAS allows the name of a block device to be changed. If the block device
-  name has changed then the API call needs to use the new name. Using the ID
-  is safer as it never changes.
-
-Create Block Device
-^^^^^^^^^^^^^^^^^^^
-
-This operation only allows an administrator to add a physical block device to
-a node. It is not recommended to create a block device as you need very
-specific information for each block device. It is recommended to
-re-commissioning the machine for MAAS to gather the required information. If
-MAAS does not provide the required information this API exists only as a
-fallback.::
-
-  $ maas my-maas-session block-devices create node-f4e2281c-d19a-11e4-a5ac-00163edde41f name=sdb model="QEMU HARDDISK" serial="QM00001" size=21474836480 block_size=4096
-  {
-      "size": 21474836480,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/12/",
-      "uuid": null,
-      "tags": [],
-      "name": "sdb",
-      "partition_table_type": null,
-      "id_path": "",
-      "path": "/dev/disk/by-dname/sdb",
-      "model": "QEMU HARDDISK",
-      "block_size": 4096,
-      "type": "physical",
-      "id": 12,
-      "serial": "QM00001",
-      "partitions": []
-  }
-
-.. note::
-
-  The serial number is what MAAS will use when a node is deployed to find the
-  specific block device. Its very important that this be absolutely correct.
-  In a rare chance that your block device does not provide a model or serial
-  number you can provide an id_path. The id_path should be a path that is
-  always the same, no matter the kernel version.
-
-Update Block Device
-^^^^^^^^^^^^^^^^^^^
-
-Provides the ability for an administrator needs to update the information of a
-physical block device and a standard user to update information of a virtual
-block device. A standard user cannot update the information of a physical
-block device.::
-
-  $ maas my-maas-session block-device update node-f4e2281c-d19a-11e4-a5ac-00163edde41f 11 name=newroot
-  {
-      "size": 21470642176,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/11/",
-      "uuid": "f58b8fb2-dcf2-4ba9-a01c-60409829a64e",
-      "tags": [],
-      "name": "vgroot-newroot",
-      "partition_table_type": null,
-      "filesystem": {
-          "label": "root",
-          "mount_point": "/",
-          "uuid": "9f09e3fd-2484-4da5-bb56-a72a0c478d06",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/vgroot-newroot",
-      "model": null,
-      "block_size": 4096,
-      "type": "virtual",
-      "id": 11,
-      "serial": null,
-      "partitions": []
-  }
-
-Delete Block Device
-^^^^^^^^^^^^^^^^^^^
-
-Allows an adminstrator to delete a physical block device and a standard user
-to delete a virtual block device.::
-
-  $ maas my-maas-session block-device delete node-f4e2281c-d19a-11e4-a5ac-00163edde41f 12
-
-Format Block Device
-^^^^^^^^^^^^^^^^^^^
-
-Format the entire block device with a file system.::
-
-  $ maas my-maas-session block-device format node-f4e2281c-d19a-11e4-a5ac-00163edde41f 11 fstype=ext4
-  {
-      "size": 21470642176,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/11/",
-      "uuid": "f58b8fb2-dcf2-4ba9-a01c-60409829a64e",
-      "tags": [],
-      "name": "vgroot-newroot",
-      "partition_table_type": null,
-      "filesystem": {
-          "label": null,
-          "mount_point": null,
-          "uuid": "b713af05-3f1c-4ddc-b4dd-a7878e6af14f",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/vgroot-newroot",
-      "model": null,
-      "block_size": 4096,
-      "type": "virtual",
-      "id": 11,
-      "serial": null,
-      "partitions": []
-  }
-
-.. note::
-
-  You cannot format a block device that contains partitions or is used to make
-  another virtual block device.
-
-Unformat Block Device
-^^^^^^^^^^^^^^^^^^^^^
-
-Remove the file system from the block device.::
-
-  $ maas my-maas-session block-device unformat node-f4e2281c-d19a-11e4-a5ac-00163edde41f 11
-  {
-      "size": 21470642176,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/11/",
-      "uuid": "f58b8fb2-dcf2-4ba9-a01c-60409829a64e",
-      "tags": [],
-      "name": "vgroot-newroot",
-      "partition_table_type": null,
-      "path": "/dev/disk/by-dname/vgroot-newroot",
-      "model": null,
-      "block_size": 4096,
-      "type": "virtual",
-      "id": 11,
-      "serial": null,
-      "partitions": []
-  }
-
-Mount Block Device
-^^^^^^^^^^^^^^^^^^
-
-Mount the block device at the given mount point. Block device is required to
-have a filesystem.::
-
-  $ maas my-maas-session block-device mount node-f4e2281c-d19a-11e4-a5ac-00163edde41f 11 mount_point=/srv
-  {
-      "size": 21470642176,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/11/",
-      "uuid": "f58b8fb2-dcf2-4ba9-a01c-60409829a64e",
-      "tags": [],
-      "name": "vgroot-newroot",
-      "partition_table_type": null,
-      "filesystem": {
-          "label": null,
-          "mount_point": "/srv",
-          "uuid": "b892e5c3-8bea-4371-a456-bde11df3df40",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/vgroot-newroot",
-      "model": null,
-      "block_size": 4096,
-      "type": "virtual",
-      "id": 11,
-      "serial": null,
-      "partitions": []
-  }
-
-Unmount Block Device
-^^^^^^^^^^^^^^^^^^^^
-
-Remove the mount point from the block device.::
-
-  $ maas my-maas-session block-device unmount node-f4e2281c-d19a-11e4-a5ac-00163edde41f 11
-  {
-      "size": 21470642176,
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/11/",
-      "uuid": "f58b8fb2-dcf2-4ba9-a01c-60409829a64e",
-      "tags": [],
-      "name": "vgroot-newroot",
-      "partition_table_type": null,
-      "filesystem": {
-          "label": null,
-          "mount_point": null,
-          "uuid": "b892e5c3-8bea-4371-a456-bde11df3df40",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/vgroot-newroot",
-      "model": null,
-      "block_size": 4096,
-      "type": "virtual",
-      "id": 11,
-      "serial": null,
-      "partitions": []
-  }
-
-Set as Boot Disk
-^^^^^^^^^^^^^^^^
-
-MAAS by default picks the first added block device to the node as the boot
-disk. In most cases this works as expected as the BIOS enumerates the boot disk
-as the first block device. There are cases where this fails and the boot disk
-needs to be set to another disk. This API allow setting which block device on
-a node MAAS should use as the boot disk.::
-
-  $ maas my-maas-session block-device set-boot-disk node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6
-  OK
-
-.. note::
-
-  Only an administrator can set which block device should be used as the boot
-  disk and only a physical block device can be set as a the boot disk. This
-  operation should be done before a node is acquired or the storage layout will
-  be applied to the previous boot disk.
-
-Partitions
-----------
-
-List Partitions
-^^^^^^^^^^^^^^^
-View all the partitions on a block device.::
-
-  $ maas my-maas-session partitions read node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6
-  [
-      {
-          "uuid": "e94ca09a-d83e-4521-8bac-833da2ed0b3e",
-          "bootable": false,
-          "filesystem": {
-              "label": null,
-              "mount_point": null,
-              "uuid": "61d447c2-387d-4fb1-885a-65eeef91e92a",
-              "fstype": "lvm-pv"
-          },
-          "path": "/dev/disk/by-dname/sda-part1",
-          "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/1",
-          "type": "partition",
-          "id": 1,
-          "size": 21471690752
-      }
-  ]
-
-Read Partition
-^^^^^^^^^^^^^^
-
-If you want to read just one partition on a block device instead of listing all
-partitions `read` operation on the `partition` endpoint provides that
-information.
-::
-
-  $ maas my-maas-session partition read node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 1
-  {
-      "uuid": "e94ca09a-d83e-4521-8bac-833da2ed0b3e",
-      "bootable": false,
-      "filesystem": {
-          "label": null,
-          "mount_point": null,
-          "uuid": "61d447c2-387d-4fb1-885a-65eeef91e92a",
-          "fstype": "lvm-pv"
-      },
-      "path": "/dev/disk/by-dname/sda-part1",
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/1",
-      "type": "partition",
-      "id": 1,
-      "size": 21471690752
-  }
-
-Create Partition
-^^^^^^^^^^^^^^^^
-
-Creates a partition on a block device.::
-
-  $ maas my-maas-session partitions create node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 size=2G
-  {
-      "uuid": "fc06be78-1665-4fd2-95d3-f588aaad3575",
-      "bootable": false,
-      "path": "/dev/disk/by-dname/sda-part1",
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/2",
-      "type": "partition",
-      "id": 2,
-      "size": 2000003072
-  }
-
-Delete Partition
-^^^^^^^^^^^^^^^^
-
-Deletes a partition from a block device.::
-
-  $ maas my-maas-session partition delete node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 2
-
-Format Partition
-^^^^^^^^^^^^^^^^
-
-Format the partition with a file system.::
-
-  $ maas my-maas-session partition format node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 3 fstype=ext4
-  {
-      "uuid": "fb468be6-64bd-434a-b95b-b8c932610960",
-      "bootable": false,
-      "filesystem": {
-          "label": "",
-          "mount_point": null,
-          "uuid": "8fbb4e35-cb65-49f7-8377-f00f48ac9da9",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/sda-part1",
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/3",
-      "type": "partition",
-      "id": 3,
-      "size": 2000003072
-  }
-
-.. note::
-
-  You cannot format partitions that are used to make another virtual block
-  device.
-
-Unformat Partition
-^^^^^^^^^^^^^^^^^^
-
-Remove the file system from the partition.::
-
-  $ maas my-maas-session partition unformat node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 3
-  {
-      "uuid": "fb468be6-64bd-434a-b95b-b8c932610960",
-      "bootable": false,
-      "path": "/dev/disk/by-dname/sda-part1",
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/3",
-      "type": "partition",
-      "id": 3,
-      "size": 2000003072
-  }
-
-Mount Partition
-^^^^^^^^^^^^^^^
-
-Mount the partition at the given mount point. Partition is required to
-have a filesystem.::
-
-  $ maas my-maas-session partition mount node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 3 mount_point=/srv
-  {
-      "uuid": "fb468be6-64bd-434a-b95b-b8c932610960",
-      "bootable": false,
-      "filesystem": {
-          "label": "",
-          "mount_point": "/srv",
-          "uuid": "b59ad2c3-cffa-4cda-830f-276df4151c1c",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/sda-part1",
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/3",
-      "type": "partition",
-      "id": 3,
-      "size": 2000003072
-  }
-
-Unmount Partition
-^^^^^^^^^^^^^^^^^
-
-Remove the mount point from the partition.::
-
-  $ maas my-maas-session partition unmount node-f4e2281c-d19a-11e4-a5ac-00163edde41f 6 3
-  {
-      "uuid": "fb468be6-64bd-434a-b95b-b8c932610960",
-      "bootable": false,
-      "filesystem": {
-          "label": "",
-          "mount_point": null,
-          "uuid": "b59ad2c3-cffa-4cda-830f-276df4151c1c",
-          "fstype": "ext4"
-      },
-      "path": "/dev/disk/by-dname/sda-part1",
-      "resource_uri": "/MAAS/api/2.0/nodes/node-f4e2281c-d19a-11e4-a5ac-00163edde41f/blockdevices/6/partition/3",
-      "type": "partition",
-      "id": 3,
-      "size": 2000003072
-  }
-
-Restrictions
-------------
-
-There are only a couple of restrictions that exists in the storage
-configuration. These restrictions are only in place because they are known
-to not allow a successful deployment.
-
-  * EFI partition is required to be on the boot disk for UEFI.
-  * Cannot place partitions on logical volumes.
-  * Cannot use a logical volume as a Bcache backing device.

=== removed file 'docs/tags.rst'
--- docs/tags.rst	2014-08-22 00:13:24 +0000
+++ docs/tags.rst	1970-01-01 00:00:00 +0000
@@ -1,124 +0,0 @@
-.. _deploy-tags:
-
-Making use of Tags
-==================
-
-MAAS implements a system of tags based on the physical properties of
-the nodes.  The idea behind this is that you can use the tags to
-identify nodes with particular abilities which may be useful when it
-comes to deploying services.
-
-A real world example of this might be to identify nodes which have
-fast GPUs installed, if you were planning on deploying software which
-used CUDA or OpenCL which would make use of this hardware.
-
-
-Tag definitions
----------------
-
-Before we can create a tag we need to know how we will select which nodes it
-gets applied to. MAAS collects hardware information from the nodes using the
-`"lshw" utility <http://ezix.org/project/wiki/HardwareLiSter>`_ to return
-detailed information in XML format. The definitions used in creating a tag are
-then constructed using XPath expressions.
-If you are unfamiliar with XPath expressions, it is well worth checking out the
-`w3schools documentation <http://www.w3schools.com/xpath/xpath_syntax.asp>`_.
-For the lshw XML, we will just check all the available nodes for some
-properties.
-In our example case, we might want to find GPUs with a clock speed of over
-1GHz.
-In this case, the relevant XML node from the output will be labelled "display"
-and does have a property called clock, so it will look like this::
-
- //node[@id="display"]/clock > 1000000000
-
-Now we have a definition, we can go ahead and create a tag.
-
-
-Creating a tag
---------------
-
-Once we have sorted out what definition we will be using, creating the
-tag is easy using the ``maas`` command. You will need to :ref:`be
-logged in to the API first <api-key>`::
-
-  $ maas maas tags new name='gpu' \
-    comment='GPU with clock speed >1GHz for running CUDA type operations.' \
-    definition='//node[@id="display"]/clock > 1000000000'
-
-The comment is really for your benefit. It pays to keep the actual tag name
-short and to the point as you will be using it frequently in commands, but it
-may subsequently be hard to work out what exactly was the difference between
-tags like "gpu" and "fastgpu" unless you have a good comment. Something which
-explains the definition in plain language is always a good idea!
-
-To check which nodes this tag applies to we can use the tag command::
-
-  $ maas maas tag nodes gpu
-
-The process of updating the tags does take some time  - not a lot of time, but
-if nothing shows up straight away, try running the command again after a minute
-or so.
-
-
-Using the tag
--------------
-
-You can use the tag in the web interface to discover applicable nodes, but the
-real significance of it is when using juju to deploy services. Tags can be used
-with juju constraints to make sure that a particular service only gets deployed
-on hardware with the tag you have created.
-
-Example:
-To use the 'gpu' tag we created to run a service called 'cuda' we would use::
-
-  $ juju deploy --constraints tags=gpu cuda
-
-You could list several tags if required, and mix in other juju constraints if
-needed::
-
-  $ juju deploy --constraints "mem=1024 tags=gpu,intel" cuda
-
-
-Manually assigning tags
------------------------
-
-MAAS supports the creation of arbitrary tags which don't depend on XPath
-definitions ("nodes which make a lot of noise" perhaps). If a tag is created
-without specifying the definition parameter then it will simply be ignored by
-tag refresh mechanism, but the MAAS administrator will be able to manually add
-and remove the tag from specific nodes.
-
-In this example we are assuming you are using the 'maas' profile and you want
-to create a tag called 'my_tag'::
-
-  $ maas maas tags new name='my_tag' comment='nodes which go ping'
-  $ maas maas tag update-nodes my_tag add="<system_id>"
-
-The first line creates a new tag but omits the definition, so no nodes are
-automatically added to it. The second line applies that tag to a specific node
-referenced by its system id property.
-
-You can easily remove a tag from a particular node, or indeed add
-and remove them at the same time::
-
-  $ maas maas tag update-nodes my_tag add=<system_id_1> \
-    add=<system_id_2> add=<system_id_3> remove=<system_id_4>
-
-As the rule is that tags without a definition are ignored when rebuilds are
-done, it is also possible to create a normal tag with a definition, and then
-subsequently edit it to remove the definition. From this point the tag behaves
-as if you had manually created it, but it still retains all the existing
-associations it has with nodes. This is particularly useful if you have some
-hardware which is conceptually similar but doesn't easily fit within a single
-tag definition::
-
-  $ maas maas tag new name='my_tag' comment='nodes I like ' \
-     definition='contains(//node[@id=network]/vendor, "Intel")'
-  $ maas maas tag update my_tag definition=''
-  $ maas mass tag update-nodes my_tag add=<system_id>
-
-.. tip::
-
-   If you add and remove the same node in one operation, it ends up having
-   the tag removed (even if the tag was in place before the operation).

=== removed file 'docs/troubleshooting.rst'
--- docs/troubleshooting.rst	2016-12-07 12:46:14 +0000
+++ docs/troubleshooting.rst	1970-01-01 00:00:00 +0000
@@ -1,195 +0,0 @@
-********************
-MAAS Troubleshooting
-********************
-
-Some parts of MAAS may still be a little confusing, and sometimes you might be
-trying to do things that are just plain impossible. This section covers some of
-the most commonly encountered problems and tries its best to make them gone.
-
-.. contents:: Contents
- :depth: 1
- :local:
-
-
-Nodes hang on "Commissioning"
-=============================
-
-
-Possible Cause: Timing issues
------------------------------
-
-Various parts of MAAS rely on OAuth to negotiate a connection to nodes. If the
-current time reported by the hardware clock on your node differs significantly
-from that on the MAAS server, the connection will not be made.
-
-**SOLUTION:** Check that the hardware clocks are consistent, and if necessary,
-adjust them. This can usually be done from within the system BIOS, without
-needing to install an OS
-
-
-Possible Cause: Network drivers
--------------------------------
-
-Sometimes the hardware can boot from PXE, but fail to load correct drivers when
-booting the received image. This is sometimes the case when no open source
-drivers are available for the network hardware.
-
-**SOLUTION:** The best fix for this problem is to install a Linux-friendly
-network adapter. It *is* theoretically possible to modify the boot image to
-include proprietary drivers, but it is not a straightforward task.
-
-
-Nodes fail to PXE boot
-======================
-
-
-Possible Cause: Using an incorrectly configured VM
---------------------------------------------------
-
-Some Virtual Machine setups include emulation of network hardware that does not
-support PXE booting, and in most setups, you will need to explicitly set up the
-VM to boot via PXE.
-
-**SOLUTION**: Consult the VM docs for details of PXE booting.
-
-
-Possible Cause: DHCP conflict
------------------------------
-
-If you are using MAAS in a setup with an existing DHCP, *DO NOT SET UP THE MAAS
-DHCP SERVER* as this will cause no end of confusion to the rest of your network
-and most likely won't discover any nodes either.
-
-**SOLUTION**: You will need to configure your existing DHCP server to
-point to the MAAS server.
-
-
-Can't log in to node
-====================
-
-Sometimes you may wish to login directly to a node on your system. If
-you have set up Juju and MAAS, the attached nodes will automatically
-receive existing ssh keys and sets up ssh on the node to authenticate
-via key, so you can just login with no password from the server.
-There is also an option in the MAAS web interface to add new ssh keys
-to the nodes (via Preferences in the drop down menu which appears when
-clicking your username in the top-right of the page).
-
-
-Forgot MAAS superuser password
-==============================
-
-As long as you have sudo privileges, this is not a disaster. You can
-use the ``maas`` command to change the password for the MAAS superuser
-on the MAAS server:
-
-    ``sudo maas changepassword root``
-
-
-Need to reconfigure server IP address
-=====================================
-
-If you made a mistake during setup or you just need to reconfigure your MAAS
-server, you can simply run the setup again:
-
-    ``sudo dpkg-reconfigure maas-region-controller``
-
-
-Can't find MAAS webpage
-=======================
-
-The default webpage is located at ``http://<hostname>/MAAS/``. If you can't
-access it, there are a few things to try:
-
-  #. Check that the webserver is running - By default the web interface uses
-     Apache, which runs under the service name *apache2*. To check it, on the
-     MAAS server box you can run ``sudo /etc/init.d/apache2 status``.
-  #. Check that the hostname is correct - It may seem obvious, but check that
-     the hostname is being resolved properly. Try running a browser (even a text
-     mode one like lynx) on the same box as the MAAS server and navigating to
-     the page. If that doesn't work, try ``http://127.0.0.1/MAAS/``, which will
-     always point at the local server.
-  #. If you are still getting "404 - Page not found" errors, check that the MAAS
-     web interface has been installed in the right place. There should be a file
-     called ``urls.py`` in ``/usr/lib/python3/dist-packages/maasserver/djangosettings/``.
-
-Debugging ephemeral image
-=========================
-
-Backdoor (add a login) to ephemeral images
-------------------------------------------
-
-If you cannot login to an instance, you might have to "backdoor it" in order
-to see what is going wrong. Scott Moser wrote a simple utility that injects a
-user and password into an image. Here's how to add a 'backdoor' user with a
-password to your images::
-
- sudo apt-get install --assume-yes bzr
- bzr branch lp:~maas-maintainers/maas/backdoor-image backdoor-image
-
- imgs=$(echo /var/lib/maas/boot-resources/*/*/*/*/*/*/root-image)
- for img in $imgs; do
-     [ -f "$img.dist" ] || sudo cp -a --sparse=always $img $img.dist
- done
-
- for img in $imgs; do
-     sudo ./backdoor-image/backdoor-image -v --user=backdoor --password-auth --password=ubuntu $img
- done
-
-Inside the ephemeral image
---------------------------
-
-Important files for debugging (Someone is likely to ask you for these
-things to help debug)::
-
- /var/log/cloud-init.log
- /var/log/boot.log
- /var/log/cloud-init-output.log
-
-After enlistment or commissioning, the user-data from maas instructs the system
-to power off. To stop that from happening, you can just create a file in /tmp::
-
- touch /tmp/block-poweroff
-
-MAAS credentials
-----------------
-
-MAAS credentials can be found in 2 places:
-
-#. from the cmdline you'll see a ``url=`` or ``cloud-config-url=``
-   parameter. You can get the cloud-config from that url, which will have
-   credentials::
-
-    $ sed -n 's,.*url=\([^ ]*\).*,\1,p' /proc/cmdline
-    http://10.55.60.194/MAAS/metadata/latest/enlist-preseed/?op=get_enlist_preseed
-
-#. from ``/etc/cloud/cloud.cfg.d/91_kernel_cmdline_url``. The file was pulled
-   from ``url=`` parameter by cloud-init::
-
-    $ sudo cat /etc/cloud/cloud.cfg.d/91_kernel_cmdline
-
-MAAS datasource
----------------
-
-The cloud-init datasource for MAAS can be invoked as a 'main' for debugging
-purposes. To do so, you need to know the url for the MAAS datasource and a
-config file that contains credentials::
-
- cfg=$(echo /etc/cloud/cloud.cfg.d/*_cmdline_url.cfg)
- echo $cfg /etc/cloud/cloud.cfg.d/91_kernel_cmdline_url.cfg
-
-Now get the metadata_url from there::
-
- url=$(sudo awk '$1 == "metadata_url:" { print $2 }' $cfg)
- echo $url http://10.55.60.194/MAAS/metadata/enlist
-
-Invoke the client /usr/share/pyshared/cloudinit/sources/DataSourceMAAS.py
-The client has --help Usage also, but here is an example of how to use it::
-
- $ maasds="/usr/share/pyshared/cloudinit/sources/DataSourceMAAS.py"
- $ sudo python $maasds --config=$cfg get $url
- == http://10.55.60.194/MAAS/metadata/enlist ==
- 2012-03-01
- latest
- $ sudo python $maasds --config=$cfg get $url/latest/meta-data/local-hostname
- maas-enlisting-node

=== modified file 'src/maasserver/api/results.py'
--- src/maasserver/api/results.py	2017-01-25 17:19:32 +0000
+++ src/maasserver/api/results.py	2017-01-31 20:57:12 +0000
@@ -15,6 +15,7 @@
 )
 from maasserver.enum import NODE_PERMISSION
 from maasserver.models import Node
+from metadataserver.enum import SCRIPT_STATUS
 from metadataserver.models import ScriptResult
 
 
@@ -67,7 +68,10 @@
             if (result_type is not None and
                     script_set.result_type != result_type):
                 continue
-            for script_result in script_set:
+            for script_result in script_set.scriptresult_set.filter(
+                    status__in=(
+                        SCRIPT_STATUS.PASSED, SCRIPT_STATUS.FAILED,
+                        SCRIPT_STATUS.TIMEOUT)):
                 if names is not None and script_result.name not in names:
                     continue
                 results.append({

=== modified file 'src/maasserver/api/tests/test_commissioning.py'
--- src/maasserver/api/tests/test_commissioning.py	2017-01-26 19:23:07 +0000
+++ src/maasserver/api/tests/test_commissioning.py	2017-02-03 23:46:57 +0000
@@ -19,6 +19,7 @@
 from maastesting.utils import sample_binary_data
 from metadataserver.enum import (
     SCRIPT_STATUS,
+    SCRIPT_STATUS_CHOICES,
     SCRIPT_TYPE,
 )
 from metadataserver.fields import Bin
@@ -169,11 +170,16 @@
 
     def test_list_returns_expected_fields(self):
         node = factory.make_Node(with_empty_script_sets=True)
+        for script_set in (
+                node.current_commissioning_script_set,
+                node.current_testing_script_set,
+                node.current_installation_script_set):
+            for script_result in script_set:
+                self.store_result(
+                    script_result, exit_status=0, stdout=factory.make_string())
+
         script_set = node.current_commissioning_script_set
         script_result = script_set.scriptresult_set.first()
-        self.store_result(
-            script_result, exit_status=0,
-            stdout=factory.make_name('stdout'))
 
         url = reverse('node_results_handler')
         response = self.client.get(url, {'system_id': [node.system_id]})
@@ -309,6 +315,7 @@
                 self.store_result(script_result, exit_status=0)
                 script_results.append(script_result)
             for script_result in node.current_installation_script_set:
+                self.store_result(script_result, exit_status=0)
                 script_results.append(script_result)
 
         url = reverse('node_results_handler')
@@ -318,3 +325,26 @@
         self.assertItemsEqual(
             {script_result.id for script_result in script_results},
             {parsed_result['id'] for parsed_result in parsed_results})
+
+    def test_list_only_displayed_completed_results(self):
+        node = factory.make_Node(with_empty_script_sets=True)
+        expected_results = []
+        for script_set in (
+                node.current_commissioning_script_set,
+                node.current_testing_script_set,
+                node.current_installation_script_set):
+            for status, _ in SCRIPT_STATUS_CHOICES:
+                script_result = factory.make_ScriptResult(
+                    script_set=script_set, status=status)
+                if status in (
+                        SCRIPT_STATUS.PASSED, SCRIPT_STATUS.FAILED,
+                        SCRIPT_STATUS.TIMEOUT):
+                    expected_results.append(script_result)
+
+        url = reverse('node_results_handler')
+        response = self.client.get(url)
+        self.assertThat(response, HasStatusCode(http.client.OK))
+        parsed_results = json_load_bytes(response.content)
+        self.assertItemsEqual(
+            {script_result.id for script_result in expected_results},
+            {parsed_result['id'] for parsed_result in parsed_results})

=== added file 'src/maasserver/migrations/builtin/maasserver/0110_notification_category.py'
--- src/maasserver/migrations/builtin/maasserver/0110_notification_category.py	1970-01-01 00:00:00 +0000
+++ src/maasserver/migrations/builtin/maasserver/0110_notification_category.py	2017-02-02 17:10:32 +0000
@@ -0,0 +1,23 @@
+from django.db import (
+    migrations,
+    models,
+)
+
+
+class Migration(migrations.Migration):
+
+    dependencies = [
+        ('maasserver', '0109_bmc_names_unique'),
+    ]
+
+    operations = [
+        migrations.AddField(
+            model_name='notification',
+            name='category',
+            field=models.CharField(
+                choices=[
+                    ('error', 'Error'), ('warning', 'Warning'),
+                    ('success', 'Success'), ('info', 'Informational'),
+                ], max_length=10, default='info'),
+        ),
+    ]

=== modified file 'src/maasserver/models/notification.py'
--- src/maasserver/models/notification.py	2017-01-23 14:40:44 +0000
+++ src/maasserver/models/notification.py	2017-02-02 17:10:32 +0000
@@ -7,6 +7,8 @@
     'Notification',
 ]
 
+from functools import wraps
+
 from django.contrib.auth.models import User
 from django.db import connection
 from django.db.models import (
@@ -23,46 +25,73 @@
 from maasserver.models.timestampedmodel import TimestampedModel
 
 
+def _create(method, category):
+    """Return a wrapped `method` that passes `category` as first argument."""
+    @wraps(method)
+    def call_with_category(self, *args, **kwargs):
+        return method(self, category, *args, **kwargs)
+    return call_with_category
+
+
 class NotificationManager(Manager):
     """Manager for `Notification` class."""
 
-    def create_for_user(self, message, user, *, context=None, ident=None):
+    def _create_for_user(
+            self, category, message, user, *, context=None, ident=None):
         """Create a notification for a specific user."""
         if ident is not None:
             self.filter(ident=ident).update(ident=None)
 
         notification = self._create(
-            user, False, False, ident, message, context)
+            category, user, False, False, ident, message, context)
         notification.save()
 
         return notification
 
-    def create_for_users(self, message, *, context=None, ident=None):
+    create_error_for_user = _create(_create_for_user, "error")
+    create_warning_for_user = _create(_create_for_user, "warning")
+    create_success_for_user = _create(_create_for_user, "success")
+    create_info_for_user = _create(_create_for_user, "info")
+
+    def _create_for_users(
+            self, category, message, *, context=None, ident=None):
         """Create a notification for all users and admins."""
         if ident is not None:
             self.filter(ident=ident).update(ident=None)
 
         notification = self._create(
-            None, True, True, ident, message, context)
+            category, None, True, True, ident, message, context)
         notification.save()
 
         return notification
 
-    def create_for_admins(self, message, *, context=None, ident=None):
+    create_error_for_users = _create(_create_for_users, "error")
+    create_warning_for_users = _create(_create_for_users, "warning")
+    create_success_for_users = _create(_create_for_users, "success")
+    create_info_for_users = _create(_create_for_users, "info")
+
+    def _create_for_admins(
+            self, category, message, *, context=None, ident=None):
         """Create a notification for all admins, but not users."""
         if ident is not None:
             self.filter(ident=ident).update(ident=None)
 
         notification = self._create(
-            None, False, True, ident, message, context)
+            category, None, False, True, ident, message, context)
         notification.save()
 
         return notification
 
-    def _create(self, user, users, admins, ident, message, context):
+    create_error_for_admins = _create(_create_for_admins, "error")
+    create_warning_for_admins = _create(_create_for_admins, "warning")
+    create_success_for_admins = _create(_create_for_admins, "success")
+    create_info_for_admins = _create(_create_for_admins, "info")
+
+    def _create(self, category, user, users, admins, ident, message, context):
         return self.model(
-            ident=ident, user=user, users=users, admins=admins,
-            message=message, context={} if context is None else context)
+            category=category, ident=ident, message=message,
+            user=user, users=users, admins=admins, context=(
+                {} if context is None else context))
 
     def find_for_user(self, user):
         """Find notifications for the given user.
@@ -111,6 +140,11 @@
         format-style template; see `context`.
     :ivar context: A dict (that can be serialised to JSON) that's used with
         `message`.
+    :ivar category: The category of this notification. The "success" category
+        is used to reinforce a positive action or event, giving good news. The
+        meaning of the "warning" and "error" categories are fairly obvious.
+        The "info" category might be used to reaffirm a small action, like "10
+        partitions were created on machine foo".
     """
 
     class Meta(DefaultMeta):
@@ -129,6 +163,13 @@
 
     message = TextField(null=False, blank=True)
     context = JSONObjectField(null=False, blank=True, default=dict)
+    category = CharField(
+        null=False, blank=False, default="info", max_length=10,
+        choices=[
+            ("error", "Error"), ("warning", "Warning"),
+            ("success", "Success"), ("info", "Informational"),
+        ],
+    )
 
     def render(self):
         """Render this notification's message using its context."""
@@ -155,10 +196,10 @@
         self.render()  # Just check it works.
 
     def __repr__(self):
-        return "<Notification user=%s users=%r admins=%r %r>" % (
-            ("None" if self.user is None else repr(self.user.username)),
-            self.users, self.admins, self.render(),
-        )
+        username = "None" if self.user is None else repr(self.user.username)
+        return "<Notification %s user=%s users=%r admins=%r %r>" % (
+            self.category.upper(), username, self.users, self.admins,
+            self.render())
 
 
 class NotificationDismissal(Model):

=== modified file 'src/maasserver/models/tests/test_notification.py'
--- src/maasserver/models/tests/test_notification.py	2017-01-23 14:40:44 +0000
+++ src/maasserver/models/tests/test_notification.py	2017-02-02 17:10:32 +0000
@@ -5,13 +5,13 @@
 
 __all__ = []
 
+import itertools
 import random
 
 from django.db.models.query import QuerySet
 from maasserver.models.notification import Notification
 from maasserver.testing.factory import factory
 from maasserver.testing.testcase import MAASServerTestCase
-from maasserver.utils.orm import reload_object
 from testtools.matchers import (
     AfterPreprocessing,
     Equals,
@@ -24,131 +24,78 @@
 )
 
 
-class TestNotificationManager(MAASServerTestCase):
-    """Tests for the `NotificationManager`."""
-
-    def test_create_new_notification_for_user(self):
-        user = factory.make_User()
-        message = factory.make_name("message")
-        notification = Notification.objects.create_for_user(message, user)
-        self.assertThat(
-            reload_object(notification), MatchesStructure(
-                ident=Is(None), user=Equals(user), users=Is(False),
-                admins=Is(False), message=Equals(message), context=Equals({}),
-            ))
-
-    def test_create_new_notification_for_user_with_ident(self):
-        user = factory.make_User()
-        ident = factory.make_name("ident")
-        message = factory.make_name("message")
-        notification = Notification.objects.create_for_user(
-            message, user, ident=ident)
-        self.assertThat(
-            reload_object(notification), MatchesStructure(
-                ident=Equals(ident), user=Equals(user), users=Is(False),
-                admins=Is(False), message=Equals(message), context=Equals({}),
-            ))
-
-    def test_create_new_notification_for_user_with_reused_ident(self):
-        # A new notification is created, and the ident is moved.
-        user = factory.make_User()
-        ident = factory.make_name("ident")
-        message = factory.make_name("message")
-        n1 = Notification.objects.create_for_user(message, user, ident=ident)
-        n2 = Notification.objects.create_for_user(message, user, ident=ident)
-        self.assertThat(n2, Not(Equals(n1)))
-        self.assertThat(
-            reload_object(n1), MatchesStructure(
-                ident=Is(None), user=Equals(user), users=Is(False),
-                admins=Is(False), message=Equals(message), context=Equals({}),
-            ))
-        self.assertThat(
-            reload_object(n2), MatchesStructure(
-                ident=Equals(ident), user=Equals(user), users=Is(False),
-                admins=Is(False), message=Equals(message), context=Equals({}),
-            ))
-        self.assertThat(
-            Notification.objects.filter(ident=ident),
-            HasLength(1))
-
-    def test_create_new_notification_for_users(self):
-        message = factory.make_name("message")
-        notification = Notification.objects.create_for_users(message)
-        self.assertThat(
-            reload_object(notification), MatchesStructure(
-                ident=Is(None), user=Is(None), users=Is(True),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-
-    def test_create_new_notification_for_users_with_ident(self):
-        message = factory.make_name("message")
-        ident = factory.make_name("ident")
-        notification = Notification.objects.create_for_users(
-            message, ident=ident)
-        self.assertThat(
-            reload_object(notification), MatchesStructure(
-                ident=Equals(ident), user=Is(None), users=Is(True),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-
-    def test_create_new_notification_for_users_with_reused_ident(self):
-        # A new notification is created, and the ident is moved.
-        ident = factory.make_name("ident")
-        message = factory.make_name("message")
-        n1 = Notification.objects.create_for_users(message, ident=ident)
-        n2 = Notification.objects.create_for_users(message, ident=ident)
-        self.assertThat(n2, Not(Equals(n1)))
-        self.assertThat(
-            reload_object(n1), MatchesStructure(
-                ident=Is(None), user=Is(None), users=Is(True),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-        self.assertThat(
-            reload_object(n2), MatchesStructure(
-                ident=Equals(ident), user=Is(None), users=Is(True),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-        self.assertThat(
-            Notification.objects.filter(ident=ident),
-            HasLength(1))
-
-    def test_create_new_notification_for_admins(self):
-        message = factory.make_name("message")
-        notification = Notification.objects.create_for_admins(message)
-        self.assertThat(
-            reload_object(notification), MatchesStructure(
-                ident=Is(None), user=Is(None), users=Is(False),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-
-    def test_create_new_notification_for_admins_with_ident(self):
-        message = factory.make_name("message")
-        ident = factory.make_name("ident")
-        notification = Notification.objects.create_for_admins(
-            message, ident=ident)
-        self.assertThat(
-            reload_object(notification), MatchesStructure(
-                ident=Equals(ident), user=Is(None), users=Is(False),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-
-    def test_create_new_notification_for_admins_with_reused_ident(self):
-        # A new notification is created, and the ident is moved.
-        ident = factory.make_name("ident")
-        message = factory.make_name("message")
-        n1 = Notification.objects.create_for_admins(message, ident=ident)
-        n2 = Notification.objects.create_for_admins(message, ident=ident)
-        self.assertThat(n2, Not(Equals(n1)))
-        self.assertThat(
-            reload_object(n1), MatchesStructure(
-                ident=Is(None), user=Is(None), users=Is(False),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
-        self.assertThat(
-            reload_object(n2), MatchesStructure(
-                ident=Equals(ident), user=Is(None), users=Is(False),
-                admins=Is(True), message=Equals(message), context=Equals({}),
-            ))
+class TestNotificationManagerCreateMethods(MAASServerTestCase):
+    """Tests for the `NotificationManager`'s create methods."""
+
+    create_methods = tuple(
+        (category, target, "create_%s_for_%s" % (category.lower(), target))
+        for category, target in itertools.product(
+            ("error", "warning", "success", "info"),
+            ("user", "users", "admins"),
+        )
+    )
+
+    scenarios = tuple(
+        (method_name, {
+            "category": category,
+            "method_name": method_name,
+            "target_name": target_name,
+            "targets_user": target_name == "user",
+            "targets_users": target_name == "users",
+            "targets_admins": target_name in {"users", "admins"},
+        })
+        for category, target_name, method_name in create_methods
+    )
+
+    def makeNotification(self, *, ident=None, context=None):
+        method = getattr(Notification.objects, self.method_name)
+        message = factory.make_name("message")
+
+        if self.targets_user:
+            user = factory.make_User()
+            notification = method(message, user, context=context, ident=ident)
+        else:
+            user = None
+            notification = method(message, context=context, ident=ident)
+
+        self.assertThat(notification, MatchesStructure(
+            user=Is(None) if user is None else Equals(user),
+            message=Equals(message)))
+
+        return notification
+
+    def assertNotification(self, notification, *, ident):
+        self.assertThat(notification, MatchesStructure(
+            users=Is(self.targets_users), admins=Is(self.targets_admins),
+            user=Not(Is(None)) if self.targets_user else Is(None),
+            ident=Is(None) if ident is None else Equals(ident),
+            category=Equals(self.category)))
+
+    def test_create_new_notification_without_context(self):
+        notification = self.makeNotification()
+        self.assertNotification(notification, ident=None)
+        self.assertThat(notification.context, Equals({}))
+
+    def test_create_new_notification_with_context(self):
+        context = {factory.make_name("key"): factory.make_name("value")}
+        notification = self.makeNotification(context=context)
+        self.assertNotification(notification, ident=None)
+        self.assertThat(notification.context, Equals(context))
+
+    def test_create_new_notification_with_ident(self):
+        ident = factory.make_name("ident")
+        notification = self.makeNotification(ident=ident)
+        self.assertNotification(notification, ident=ident)
+
+    def test_create_new_notification_with_reused_ident(self):
+        # A new notification is created, and the ident is moved.
+        ident = factory.make_name("ident")
+        n1 = self.makeNotification(ident=ident)
+        n2 = self.makeNotification(ident=ident)
+        n1.refresh_from_db()  # Get current value of `ident`.
+        self.assertThat(n2, Not(Equals(n1)))
+        self.assertNotification(n1, ident=None)
+        self.assertNotification(n2, ident=ident)
         self.assertThat(
             Notification.objects.filter(ident=ident),
             HasLength(1))
@@ -160,9 +107,9 @@
     def notify(self, user):
         message = factory.make_name("message")
         return (
-            Notification.objects.create_for_user(message, user),
-            Notification.objects.create_for_users(message),
-            Notification.objects.create_for_admins(message),
+            Notification.objects.create_error_for_user(message, user),
+            Notification.objects.create_error_for_users(message),
+            Notification.objects.create_error_for_admins(message),
         )
 
     def assertNotifications(self, user, notifications):
@@ -263,30 +210,34 @@
 class TestNotificationRepresentation(MAASServerTestCase):
     """Tests for the `Notification` representation."""
 
+    scenarios = tuple(
+        (category, dict(category=category))
+        for category in ("error", "warning", "success", "info")
+    )
+
     def test_for_user(self):
         notification = Notification(
-            user=factory.make_User("foobar"),
-            message="The cat in the {place}",
-            context=dict(place="bear trap"))
+            user=factory.make_User("foobar"), message="The cat in the {place}",
+            context=dict(place="bear trap"), category=self.category)
         self.assertThat(
             notification, AfterPreprocessing(repr, Equals(
-                "<Notification user='foobar' users=False admins=False "
-                "'The cat in the bear trap'>")))
+                "<Notification %s user='foobar' users=False admins=False "
+                "'The cat in the bear trap'>" % self.category.upper())))
 
     def test_for_users(self):
         notification = Notification(
             users=True, message="The cat in the {place}",
-            context=dict(place="blender"))
+            context=dict(place="blender"), category=self.category)
         self.assertThat(
             notification, AfterPreprocessing(repr, Equals(
-                "<Notification user=None users=True admins=False "
-                "'The cat in the blender'>")))
+                "<Notification %s user=None users=True admins=False "
+                "'The cat in the blender'>" % self.category.upper())))
 
     def test_for_admins(self):
         notification = Notification(
             admins=True, message="The cat in the {place}",
-            context=dict(place="lava pit"))
+            context=dict(place="lava pit"), category=self.category)
         self.assertThat(
             notification, AfterPreprocessing(repr, Equals(
-                "<Notification user=None users=False admins=True "
-                "'The cat in the lava pit'>")))
+                "<Notification %s user=None users=False admins=True "
+                "'The cat in the lava pit'>" % self.category.upper())))

=== removed file 'src/maasserver/static/css/build.css.moved'
--- src/maasserver/static/css/build.css.moved	2017-01-11 11:02:10 +0000
+++ src/maasserver/static/css/build.css.moved	1970-01-01 00:00:00 +0000
@@ -1,1 +0,0 @@
-﻿*{font-smoothing:subpixel-antialiased;box-sizing:border-box}button,input[type='button'],input[type='reset'],input[type='submit'],.button--primary,.button--secondary,.button--neutral,.button--inline-neutral,.button--positive,.button--inline-positive,.button--destructive,.button--inline-destructive,.button--base,.button--inline-base{font-smoothing:subpixel-antialiased;font-size:1em;display:inline-block;width:100%;margin:0;padding:11px 24px;text-align:center;text-decoration:none;border:0;border-radius:2px;outline:none;font-weight:300;cursor:pointer}button:hover,input[type='button']:hover,input[type='reset']:hover,input[type='submit']:hover,.button--primary:hover,.button--secondary:hover,.button--neutral:hover,.button--inline-neutral:hover,.button--positive:hover,.button--inline-positive:hover,.button--destructive:hover,.button--inline-destructive:hover,.button--base:hover,.button--inline-base:hover{text-decoration:none}@media only screen and (min-width: 768px){button,input[type='button'],input[type='reset'],input[type='submit'],.button--primary,.button--secondary,.button--neutral,.button--inline-neutral,.button--positive,.button--inline-positive,.button--destructive,.button--inline-destructive,.button--base,.button--inline-base{width:auto}}.clearfix{display:block}.clearfix:after{clear:both;content:'\0020';display:block;height:0;overflow:hidden;visibility:hidden}textarea,input[type='text'],input[type='number'],input[type='search'],input[type='password'],input[type='email'],input[type='url'],input[type='tel'],select{-webkit-appearance:textfield;border-radius:2px;border:1px solid #cdcdcd;box-shadow:inset 0 1px 2px rgba(0,0,0,0.12);font-size:16px;margin:0;outline:none;padding:.6956522em .869565em;vertical-align:baseline;font-weight:300}textarea:active,input[type='text']:active,input[type='number']:active,input[type='search']:active,input[type='password']:active,input[type='email']:active,input[type='url']:active,input[type='tel']:active,select:active,textarea:focus,input[type='text']:focus,input[type='number']:focus,input[type='search']:focus,input[type='password']:focus,input[type='email']:focus,input[type='url']:focus,input[type='tel']:focus,select:focus{border-color:#888;outline:none}textarea::placeholder,input[type='text']::placeholder,input[type='number']::placeholder,input[type='search']::placeholder,input[type='password']::placeholder,input[type='email']::placeholder,input[type='url']::placeholder,input[type='tel']::placeholder,select::placeholder{color:contrast-friendly-search-color(#f7f7f7)}textarea[disabled="disabled"],input[disabled="disabled"][type='text'],input[disabled="disabled"][type='number'],input[disabled="disabled"][type='search'],input[disabled="disabled"][type='password'],input[disabled="disabled"][type='email'],input[disabled="disabled"][type='url'],input[disabled="disabled"][type='tel'],select[disabled="disabled"]{opacity:.5}input[type='checkbox'],input[type='radio']{border-radius:2px;border:1px solid #cdcdcd;box-shadow:inset 0 1px 2px rgba(0,0,0,0.12);height:14px;padding:0;vertical-align:middle;width:14px}.inner-wrapper{margin:0 auto;max-width:1030px}[class*='-col'].last-col,[class*='-col'] [class*='-col'].last-col{margin-right:0}.one-col,.two-col,.three-col,.four-col,.five-col,.six-col,.seven-col,.eight-col,.nine-col,.ten-col,.eleven-col,.twelve-col{clear:none;margin-bottom:20px;margin-right:0;padding:0;display:inline-block;position:relative;width:100%}@media only screen and (min-width: 768px){.one-col,.two-col,.three-col,.four-col,.five-col,.six-col,.seven-col,.eight-col,.nine-col,.ten-col,.eleven-col,.twelve-col{float:left}}.list-ticks,.list{list-style:none;margin-left:0}.list-ticks li,.list li{border-bottom:1px dotted #888;margin-bottom:0;padding:10px 0}.list-ticks li:last-of-type,.list li:last-of-type,.list-ticks .last-item,.list .last-item{border:0;margin-bottom:0;padding-bottom:0}.list-ticks li{background-repeat:no-repeat;background-position:0 1em;padding-left:25px}.box{background:#fff;border:1px solid #cdcdcd;padding:1.25em 20px}@media only screen and (min-width: 768px){.equal-height,.equal-height--vertical-divider{display:flex;flex-wrap:wrap;flex-direction:row;width:100%}.equal-height .equal-height__item,.equal-height--vertical-divider .equal-height__item{display:flex;flex:auto;flex-direction:column}}button,input[type='button'],input[type='reset'],input[type='submit'],.button--primary,.button--secondary,.button--neutral,.button--inline-neutral,.button--positive,.button--inline-positive,.button--destructive,.button--inline-destructive,.button--base,.button--inline-base{font-size:1em;box-sizing:border-box;width:100%;margin:0;padding:11px 24px;text-align:center;text-decoration:none;border-radius:2px;outline:none;font-weight:300;cursor:pointer;line-height:1em}button:disabled,input[type='button']:disabled,input[type='reset']:disabled,input[type='submit']:disabled,.button--primary:disabled,.button--secondary:disabled,.button--neutral:disabled,.button--inline-neutral:disabled,.button--positive:disabled,.button--inline-positive:disabled,.button--destructive:disabled,.button--inline-destructive:disabled,.button--base:disabled,.button--inline-base:disabled,button.button--disabled,input.button--disabled[type='button'],input.button--disabled[type='reset'],input.button--disabled[type='submit'],.button--disabled.button--primary,.button--disabled.button--secondary,.button--disabled.button--neutral,.button--disabled.button--inline-neutral,.button--disabled.button--positive,.button--disabled.button--inline-positive,.button--disabled.button--destructive,.button--disabled.button--inline-destructive,.button--disabled.button--base,.button--disabled.button--inline-base{opacity:.5;cursor:default}button,input[type='button'],input[type='reset'],input[type='submit'],.button--primary,.button--secondary,.button--neutral,.button--inline-neutral,.button--positive,.button--inline-positive,.button--destructive,.button--inline-destructive,.button--base,.button--inline-base{line-height:20px;white-space:nowrap;vertical-align:middle;touch-action:manipulation;cursor:pointer;user-select:none;padding:8px 14px;height:36px;box-sizing:border-box}button[disabled],input[disabled][type='button'],input[disabled][type='reset'],input[disabled][type='submit'],[disabled].button--primary,[disabled].button--secondary,[disabled].button--neutral,[disabled].button--inline-neutral,[disabled].button--positive,[disabled].button--inline-positive,[disabled].button--destructive,[disabled].button--inline-destructive,[disabled].button--base,[disabled].button--inline-base,button.button--disabled,input.button--disabled[type='button'],input.button--disabled[type='reset'],input.button--disabled[type='submit'],.button--disabled.button--primary,.button--disabled.button--secondary,.button--disabled.button--neutral,.button--disabled.button--inline-neutral,.button--disabled.button--positive,.button--disabled.button--inline-positive,.button--disabled.button--destructive,.button--disabled.button--inline-destructive,.button--disabled.button--base,.button--disabled.button--inline-base{pointer-events:none;opacity:.5}button.button--inline,input.button--inline[type='button'],input.button--inline[type='reset'],input.button--inline[type='submit'],.button--inline.button--primary,.button--inline.button--secondary,.button--inline.button--neutral,.button--inline.button--inline-neutral,.button--inline.button--positive,.button--inline.button--inline-positive,.button--inline.button--destructive,.button--inline.button--inline-destructive,.button--inline.button--base,.button--inline.button--inline-base{display:inline-block;width:auto}@media only screen and (max-width: 768px){button.button--inline,input.button--inline[type='button'],input.button--inline[type='reset'],input.button--inline[type='submit'],.button--inline.button--primary,.button--inline.button--secondary,.button--inline.button--neutral,.button--inline.button--inline-neutral,.button--inline.button--positive,.button--inline.button--inline-positive,.button--inline.button--destructive,.button--inline.button--inline-destructive,.button--inline.button--base,.button--inline.button--inline-base{width:100%}}@media only screen and (max-width: 768px){button,input[type='button'],input[type='reset'],input[type='submit'],.button--primary,.button--secondary,.button--neutral,.button--inline-neutral,.button--positive,.button--inline-positive,.button--destructive,.button--inline-destructive,.button--base,.button--inline-base{width:100%}}html,body,div,span,applet,object,iframe,h1,h2,h3,h4,h5,h6,p,blockquote,pre,a,acronym,address,big,cite,code,del,dfn,em,img,ins,kbd,q,s,samp,small,strike,strong,sub,sup,tt,var,b,u,i,center,dl,ol,ul,li,form,label,legend,table,caption,tbody,tfoot,thead,tr,th,td,article,aside,canvas,details,embed,figure,figcaption,footer,header,menu,nav,output,ruby,section,summary,time,mark,audio,video{border:0;margin:0;padding:0;vertical-align:baseline}article,aside,details,figcaption,figure,footer,header,nav,section{display:block}audio,canvas,video{display:inline-block}audio:not([controls]){display:none}[hidden]{display:none}html{font-size:100%;overflow-y:scroll;text-size-adjust:100%}blockquote,q{quotes:none}legend{border:0}figure{margin:0}abbr,acronym{cursor:help}.link-arrow:after{content:'\0000a0›'}nav ul li h2 a:after{content:'\0000a0›'}nav ul li a:after,.carousel ul li a:after,ul li p a:after{content:''}svg:not(:root){overflow:hidden}img{border:0;height:auto;max-width:100%}img .left{margin-right:20px}img .right{margin-left:20px}.middle img{vertical-align:middle;margin-top:4em}ins{background:#fffbeb;text-decoration:none}.left{float:left}.right{float:right}.no-border{border:0}.link-top{font-size:.875em;clear:both;margin-bottom:40px;margin-top:-40px}.link-top a{background:#fff;margin-right:10px;margin-top:-35px;padding:5px;float:right}.caps{text-transform:uppercase}.touch-border,.touch-bottom{float:left}@media only screen and (min-width: 768px){.touch-border,.touch-bottom{margin-bottom:-20px}}@media only screen and (min-width: 1440px){.touch-border,.touch-bottom{margin-bottom:-40px}}.accessibility-aid,.off-left{position:absolute;left:-999em}.external{background-size:.7em .7em;padding-right:.9em;background-image:url("https://assets.ubuntu.com/v1/f24a8aa0-external-link-orange.svg");background-position:100% top;background-repeat:no-repeat}.no-svg .external{background-image:url("https://assets.ubuntu.com/v1/f24a8aa0-external-link-orange.svg")}.text-center,.align-center{text-align:center}.no-margin-bottom{margin-bottom:0}.no-padding-bottom{padding-bottom:0}.pull-bottom-right{float:right;margin-right:-10px}@media only screen and (min-width: 768px){.pull-bottom-right{margin-right:-40px;margin-bottom:-40px}}@media only screen and (min-width: 1440px){.pull-bottom-right{margin-right:-40px;margin-bottom:-60px}}.pull-bottom-left{float:left;margin-left:-10px}@media only screen and (min-width: 768px){.pull-bottom-left{margin-left:-40px;margin-bottom:-40px}}@media only screen and (min-width: 1440px){.pull-bottom-left{margin-left:-40px;margin-bottom:-60px}}@media only screen and (max-width: 767px){.priority-0,.not-for-small{display:none}}@media only screen and (min-width: 768px){.for-mobile,.for-small{display:none}}.video-container{position:relative;padding-bottom:56.25%;padding-top:30px;height:0;overflow:hidden}.video-container iframe{position:absolute;top:0;left:0;width:100%;height:100%}.video-container+.video-title{margin-top:20px}.pull-right{float:right;margin-right:-40px}@media only screen and (max-width: 768px){.pull-right{margin-bottom:20px}}.pull-left{margin-left:-40px}.for-tablet,.for-medium{display:none}@media only screen and (min-width: 768px){.for-tablet,.for-medium{display:block}}@media only screen and (min-width: 768px) and (max-width: 1440px){.med-six-col .three-col{width:48.93617%}.med-six-col .three-col:nth-of-type(2n){margin-right:0}}blockquote{margin:0}blockquote>p{font-size:1em;margin:0 0 .4em}blockquote small{font-size:.8125em;line-height:1.4}.pull-quote{text-indent:0}.pull-quote p{font-size:1.5em;line-height:1.3;margin-left:.4em;padding-left:10px;padding-right:10px;text-indent:-.4em}.pull-quote p span{color:#f7f7f7;font-weight:bold;left:-5px;line-height:0;position:relative}.pull-quote p span:last-of-type{left:5px}.pull-quote p cite{display:block;font-size:.75em;font-weight:300;margin:10px 0 0;text-indent:0}@media only screen and (min-width: 768px){.pull-quote p{padding-left:0;padding-right:0;text-indent:-.7em}.pull-quote p span{top:5px;font-size:1.391304348em}.pull-quote p cite{margin-left:0;text-indent:0}}@media only screen and (min-width: 1440px){.pull-quote p span{top:10px}}.row-quote{border-radius:0}.row-quote blockquote{border-radius:4px;margin:0;padding:0}.row-quote blockquote p{color:#333;line-height:1.3;margin-bottom:.75em;padding-left:10px;padding-right:10px;text-indent:0}.row-quote blockquote span{color:#f7f7f7;font-weight:bold;left:-5px;line-height:0;position:relative}.row-quote blockquote span+span{left:5px}.row-quote blockquote cite{color:#333;font-size:.75em;font-style:normal;margin-bottom:0;text-indent:0}@media only screen and (min-width: 768px){.row-quote{text-indent:-.4em}.row-quote blockquote{text-indent:-7px}.row-quote blockquote p{font-size:1.5em;padding-left:0;padding-right:0;text-indent:-.4em}.row-quote blockquote span{top:5px;font-size:1.391304348em}.row-quote blockquote cite{margin-left:0;text-indent:0}}@media only screen and (min-width: 1440px){.row-quote blockquote{padding:0 80px 20px;text-indent:-10px}.row-quote blockquote p span{top:10px}}button,input[type='button'],input[type='reset'],input[type='submit'],.button--primary{color:#fff;background-color:#dd4814}button:hover,input[type='button']:hover,input[type='reset']:hover,input[type='submit']:hover,.button--primary:hover{background-color:#c03f11}.button--secondary{color:#dd4814;background-color:#fff;border:1px solid #cdcdcd}.button--secondary:hover{background-color:#efefef}@media only screen and (min-width: 620px){.nav-secondary{border:1px solid #d2d2d2;border-top:0}}.nav-secondary .breadcrumb{margin-bottom:0;margin-left:0;padding-top:2px;padding-bottom:3px}@media only screen and (min-width: 620px){.nav-secondary .breadcrumb{float:left;margin-left:4px}}.nav-secondary .breadcrumb li{color:#fff;display:inline-block;margin:0;width:100%}@media only screen and (min-width: 620px){.nav-secondary .breadcrumb li{display:inline-block;width:auto}}.nav-secondary .breadcrumb li:first-of-type{border-bottom:1px solid #d2d2d2}@media only screen and (min-width: 620px){.nav-secondary .breadcrumb li:first-of-type{border-bottom:0}}.nav-secondary .breadcrumb li a{color:#888;font-size:1em;display:inline-block;width:100%;margin-right:0;padding:8px 10px;text-decoration:none}@media only screen and (min-width: 620px){.nav-secondary .breadcrumb li a{font-size:.875em}}@media only screen and (min-width: 620px){.nav-secondary .breadcrumb li .breadcrumb-link,.nav-secondary .breadcrumb li .breadcrumb-link--second-level,.nav-secondary .breadcrumb li .active{display:inline-block;float:none;width:auto}}.nav-secondary .breadcrumb li .breadcrumb-link:after,.nav-secondary .breadcrumb li .breadcrumb-link--second-level:after{content:'\203A';display:inline-block;height:5px;margin-left:8px;position:absolute;width:5px}.nav-secondary .second-level-nav,.nav-secondary .third-level-nav{display:none;overflow:hidden;width:100%;margin:0;padding-top:10px;padding-bottom:10px;border-bottom:1px solid #d2d2d2}@media only screen and (min-width: 620px){.nav-secondary .second-level-nav,.nav-secondary .third-level-nav{display:inline;float:none;margin-bottom:0;padding:0;width:auto;border-bottom:0}}.nav-secondary .second-level-nav li,.nav-secondary .third-level-nav li{margin:0}@media only screen and (min-width: 620px){.nav-secondary .second-level-nav li,.nav-secondary .third-level-nav li{float:none;width:auto}}.nav-secondary .second-level-nav li a,.nav-secondary .third-level-nav li a{font-size:.875em;display:block;width:100%;height:100%;padding:10px;color:#333}@media only screen and (max-width: 620px){.nav-secondary .second-level-nav li a.active,.nav-secondary .third-level-nav li a.active{font-weight:500}}@media only screen and (min-width: 620px){.nav-secondary .second-level-nav li a,.nav-secondary .third-level-nav li a{padding:8px 10px 0}.nav-secondary .second-level-nav li a.active,.nav-secondary .third-level-nav li a.active{color:#f7f7f7}}@media only screen and (min-width: 620px){.nav-secondary .second-level-nav li{width:auto}.nav-secondary .second-level-nav li .active,.nav-secondary .second-level-nav li .breadcrumb-link--second-level{color:#888}.nav-secondary .second-level-nav li .third-level-nav li a{color:#333}.nav-secondary .second-level-nav li .third-level-nav li a:hover{color:#f7f7f7}.nav-secondary .second-level-nav li .third-level-nav li .active{color:#f7f7f7;float:none}.nav-secondary .second-level-nav li .third-level-nav li .active:after{content:''}}.third-level-nav li a:hover{color:#f7f7f7}.third-level-nav li .active{color:#f7f7f7;float:none}@media only screen and (max-width: 619px){#breadcrumbs:hover .second-level-nav,#breadcrumbs:hover .third-level-nav,#breadcrumbs:hover .active{border:0;display:block;overflow:visible;float:left;width:100%;position:relative;clear:both}#breadcrumbs:hover li{border:0;float:left}#breadcrumbs:hover .second-level-nav{border-top:1px solid #d2d2d2}#breadcrumbs:hover .second-level-nav li{float:left}#breadcrumbs:hover .third-level-nav{border-bottom:1px solid #d2d2d2;padding-top:0}#breadcrumbs:hover .third-level-nav li{width:50%;display:block;padding-left:10px}}label{cursor:pointer;display:block;margin-bottom:4px}label.has-error{color:#df382c}label.has-warning{color:#eca918}label.has-success{color:#38b44a}label.has-information{color:#19b6ee}input[type='reset']{display:none}input[type='search']{-webkit-appearance:none;border-radius:0}input[type='checkbox']+label,input[type='radio']+label{display:inline;margin-left:5px;vertical-align:middle;width:auto}input.has-error{border:1px solid #df382c}input.has-warning{border:1px solid #eca918}input.has-success{border:1px solid #38b44a}input.has-information{border:1px solid #19b6ee}form ul{margin-left:0;list-style:none}textarea{overflow:auto;vertical-align:top}textarea[readonly='readonly']{color:#888;cursor:default}textarea[readonly='readonly']:active,textarea[readonly='readonly']:focus{border-color:#888;outline:none}fieldset{background-color:#f7f7f7;background-position:-15px -15px;background-repeat:no-repeat;border-radius:4px;border:0;margin-bottom:8px;padding:15px 20px}@media only screen and (min-width: 1440px){fieldset{padding:15px 20px}}fieldset h3{border-bottom:1px dotted #cdcdcd;margin-bottom:9px;padding-bottom:10px}form input,form select,form textarea{width:100%}@media only screen and (min-width: 768px){.one-col{float:left;width:6.40351%;margin-right:2.10526%}.one-col .one-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.two-col{float:left;width:14.91228%;margin-right:2.10526%}.two-col .one-col{float:left;width:43.81443%;margin-right:12.37113%}.two-col .two-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.three-col{float:left;width:23.42105%;margin-right:2.10526%}.three-col .one-col{float:left;width:27.83505%;margin-right:8.24742%}.three-col .two-col{float:left;width:63.91753%;margin-right:8.24742%}.three-col .three-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.four-col{float:left;width:31.92982%;margin-right:2.10526%}.four-col .one-col{float:left;width:20.36082%;margin-right:6.18557%}.four-col .two-col{float:left;width:46.90722%;margin-right:6.18557%}.four-col .three-col{float:left;width:73.45361%;margin-right:6.18557%}.four-col .four-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.five-col{float:left;width:40.4386%;margin-right:2.10526%}.five-col .one-col{float:left;width:16.04124%;margin-right:4.94845%}.five-col .two-col{float:left;width:37.03093%;margin-right:4.94845%}.five-col .three-col{float:left;width:58.02062%;margin-right:4.94845%}.five-col .four-col{float:left;width:79.01031%;margin-right:4.94845%}.five-col .five-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.six-col{float:left;width:48.94737%;margin-right:2.10526%}.six-col .one-col{float:left;width:13.23024%;margin-right:4.12371%}.six-col .two-col{float:left;width:30.58419%;margin-right:4.12371%}.six-col .three-col{float:left;width:47.93814%;margin-right:4.12371%}.six-col .four-col{float:left;width:65.2921%;margin-right:4.12371%}.six-col .five-col{float:left;width:82.64605%;margin-right:4.12371%}.six-col .six-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.seven-col{float:left;width:57.45614%;margin-right:2.10526%}.seven-col .one-col{float:left;width:11.25605%;margin-right:3.53461%}.seven-col .two-col{float:left;width:26.04671%;margin-right:3.53461%}.seven-col .three-col{float:left;width:40.83737%;margin-right:3.53461%}.seven-col .four-col{float:left;width:55.62802%;margin-right:3.53461%}.seven-col .five-col{float:left;width:70.41868%;margin-right:3.53461%}.seven-col .six-col{float:left;width:85.20934%;margin-right:3.53461%}.seven-col .seven-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.eight-col{float:left;width:65.96491%;margin-right:2.10526%}.eight-col .one-col{float:left;width:9.79381%;margin-right:3.09278%}.eight-col .two-col{float:left;width:22.68041%;margin-right:3.09278%}.eight-col .three-col{float:left;width:35.56701%;margin-right:3.09278%}.eight-col .four-col{float:left;width:48.45361%;margin-right:3.09278%}.eight-col .five-col{float:left;width:61.34021%;margin-right:3.09278%}.eight-col .six-col{float:left;width:74.2268%;margin-right:3.09278%}.eight-col .seven-col{float:left;width:87.1134%;margin-right:3.09278%}.eight-col .eight-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.nine-col{float:left;width:74.47368%;margin-right:2.10526%}.nine-col .one-col{float:left;width:8.66743%;margin-right:2.74914%}.nine-col .two-col{float:left;width:20.084%;margin-right:2.74914%}.nine-col .three-col{float:left;width:31.50057%;margin-right:2.74914%}.nine-col .four-col{float:left;width:42.91714%;margin-right:2.74914%}.nine-col .five-col{float:left;width:54.33372%;margin-right:2.74914%}.nine-col .six-col{float:left;width:65.75029%;margin-right:2.74914%}.nine-col .seven-col{float:left;width:77.16686%;margin-right:2.74914%}.nine-col .eight-col{float:left;width:88.58343%;margin-right:2.74914%}.nine-col .nine-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.ten-col{float:left;width:82.98246%;margin-right:2.10526%}.ten-col .one-col{float:left;width:7.7732%;margin-right:2.47423%}.ten-col .two-col{float:left;width:18.02062%;margin-right:2.47423%}.ten-col .three-col{float:left;width:28.26804%;margin-right:2.47423%}.ten-col .four-col{float:left;width:38.51546%;margin-right:2.47423%}.ten-col .five-col{float:left;width:48.76289%;margin-right:2.47423%}.ten-col .six-col{float:left;width:59.01031%;margin-right:2.47423%}.ten-col .seven-col{float:left;width:69.25773%;margin-right:2.47423%}.ten-col .eight-col{float:left;width:79.50515%;margin-right:2.47423%}.ten-col .nine-col{float:left;width:89.75258%;margin-right:2.47423%}.ten-col .ten-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.eleven-col{float:left;width:91.49123%;margin-right:2.10526%}.eleven-col .one-col{float:left;width:7.04609%;margin-right:2.2493%}.eleven-col .two-col{float:left;width:16.34148%;margin-right:2.2493%}.eleven-col .three-col{float:left;width:25.63687%;margin-right:2.2493%}.eleven-col .four-col{float:left;width:34.93227%;margin-right:2.2493%}.eleven-col .five-col{float:left;width:44.22766%;margin-right:2.2493%}.eleven-col .six-col{float:left;width:53.52305%;margin-right:2.2493%}.eleven-col .seven-col{float:left;width:62.81844%;margin-right:2.2493%}.eleven-col .eight-col{float:left;width:72.11383%;margin-right:2.2493%}.eleven-col .nine-col{float:left;width:81.40922%;margin-right:2.2493%}.eleven-col .ten-col{float:left;width:90.70461%;margin-right:2.2493%}.eleven-col .eleven-col{width:100%;margin-right:0}}@media only screen and (min-width: 768px){.prepend-one{margin-left:8.50877%}.prepend-two{margin-left:17.01754%}.prepend-three{margin-left:25.52632%}.prepend-four{margin-left:34.03509%}.prepend-five{margin-left:42.54386%}.prepend-six{margin-left:51.05263%}.prepend-seven{margin-left:59.5614%}.prepend-eight{margin-left:68.07018%}.prepend-nine{margin-left:76.57895%}.prepend-ten{margin-left:85.08772%}.prepend-eleven{margin-left:93.59649%}}@media only screen and (min-width: 768px){.append-one{margin-right:8.50877%}.append-two{margin-right:17.01754%}.append-three{margin-right:25.52632%}.append-four{margin-right:34.03509%}.append-five{margin-right:42.54386%}.append-six{margin-right:51.05263%}.append-seven{margin-right:59.5614%}.append-eight{margin-right:68.07018%}.append-nine{margin-right:76.57895%}.append-ten{margin-right:85.08772%}.append-eleven{margin-right:93.59649%}}@media only screen and (min-width: 768px){.push-one{float:left;position:relative;margin:0 -8.50877% 0 8.50877%}.push-two{float:left;position:relative;margin:0 -19.12281% 0 19.12281%}.push-three{float:left;position:relative;margin:0 -27.63158% 0 27.63158%}.push-four{float:left;position:relative;margin:0 -36.14035% 0 36.14035%}.push-five{float:left;position:relative;margin:0 -44.64912% 0 44.64912%}.push-six{float:left;position:relative;margin:0 -53.15789% 0 53.15789%}.push-seven{float:left;position:relative;margin:0 -61.66667% 0 61.66667%}.push-eight{float:left;position:relative;margin:0 -70.17544% 0 70.17544%}.push-nine{float:left;position:relative;margin:0 -78.68421% 0 78.68421%}.push-ten{float:left;position:relative;margin:0 -87.19298% 0 87.19298%}.push-eleven{float:left;position:relative;margin:0 -95.70175% 0 95.70175%}}@media only screen and (min-width: 768px){.pull-one{float:left;position:relative;margin-left:-6.40351%}.pull-two{float:left;position:relative;margin-left:17.01754%}.pull-three{float:left;position:relative;margin-left:25.52632%}.pull-four{float:left;position:relative;margin-left:34.03509%}.pull-five{float:left;position:relative;margin-left:42.54386%}.pull-six{float:left;position:relative;margin-left:51.05263%}.pull-seven{float:left;position:relative;margin-left:59.5614%}.pull-eight{float:left;position:relative;margin-left:68.07018%}.pull-nine{float:left;position:relative;margin-left:76.57895%}.pull-ten{float:left;position:relative;margin-left:85.08772%}.pull-eleven{float:left;position:relative;margin-left:93.59649%}}.banner{background:#f7f7f7;border-bottom:0;border-top:0;box-shadow:inset 0 -1px 0 #ccc;display:block;min-width:100%;position:relative;width:auto;z-index:2}@media only screen and (min-width: 620px){.banner{border-bottom:1px solid #ccc;box-shadow:none}}.banner .logo{position:relative;display:table;float:left;overflow:hidden;height:48px;margin:0 10px;border-left:0}@media only screen and (min-width: 620px){.banner .logo{margin:0 15px}}.banner .logo a{display:table-cell;height:100%;vertical-align:middle;color:#fff}.banner .logo a span{display:inline-block}.banner h2{font-size:1.563em;position:relative;top:14px;left:4px;display:block;margin-bottom:0;text-transform:lowercase}.banner h2 a,.banner h2 a:link,.banner h2 a:visited{float:left;text-decoration:none;color:#fff}.banner .nav-primary{overflow:hidden;margin:0 auto;border:0}@media only screen and (min-width: 620px){.banner .nav-primary{max-width:1440px}}.banner .nav-primary span{display:none}.banner .nav-primary ul{display:none;float:left;width:100%;margin:0;padding:0;border-top:1px solid #ccc;box-shadow:inset 0 -1px 0 #ccc;background-color:#ebebeb}@media only screen and (min-width: 620px){.banner .nav-primary ul{position:static;display:block;width:auto;border-top:0;background-color:transparent;box-shadow:none}}.banner .nav-primary ul li{float:left;width:50%;margin:0;border-right:1px solid #ccc;border-bottom:1px solid #ccc;background:transparent}@media only screen and (min-width: 620px){.banner .nav-primary ul li{width:auto;border-right:0;border-bottom:0;border-left:1px solid #e3e3e3}.banner .nav-primary ul li:last-child{border-right:1px solid #e3e3e3}.banner .nav-primary ul li:last-child a{border-right:0}}@media only screen and (max-width: 620px){.banner .nav-primary ul li:nth-child(2n+2){border-right:0}}.banner .nav-primary ul li a,.banner .nav-primary ul li a:link,.banner .nav-primary ul li a:visited{font-size:1em;font-weight:300;position:relative;display:block;margin-bottom:0;padding:8px 10px;text-align:left;text-decoration:none;background-color:#ebebeb;font-smoothing:subpixel-antialiased;color:#333}@media only screen and (min-width: 620px){.banner .nav-primary ul li a,.banner .nav-primary ul li a:link,.banner .nav-primary ul li a:visited{font-size:.875em;padding:14px 14px 13px;text-align:center;border-left:1px solid #fff;background-color:transparent;color:#fff}}.banner .nav-primary ul li a:hover,.banner .nav-primary ul li a:link:hover,.banner .nav-primary ul li a:visited:hover{background:#f2f2f2}@media only screen and (min-width: 620px){.banner .nav-primary ul li a:hover,.banner .nav-primary ul li a:link:hover,.banner .nav-primary ul li a:visited:hover{background-color:#fff}}.banner .nav-primary ul li a.active,.banner .nav-primary ul li a:link.active,.banner .nav-primary ul li a:visited.active{background-color:#ddd}@media only screen and (min-width: 620px){.banner .nav-primary ul li a.active,.banner .nav-primary ul li a:link.active,.banner .nav-primary ul li a:visited.active{background-color:#e3e3e3;border-left:0}}.banner .nav-toggle{position:absolute;top:0;right:0;display:block;width:48px;height:48px;cursor:pointer;text-indent:-99999px;background-image:url("https://assets.ubuntu.com/v1/12387180-navigation-menu-plain.svg");background-repeat:no-repeat;background-position:center center;background-size:25px auto}@media only screen and (min-width: 620px){.banner .nav-toggle{display:none}}.banner .nav-toggle .nav-toggle__link{width:48px;height:48px}.banner .nav-toggle .nav-toggle__link.open{display:block}.banner .nav-toggle .nav-toggle__link.close{display:none}@media only screen and (max-width: 620px){.banner #nav:hover ul{display:block}.banner #nav:hover ul ul{display:none}.banner #nav:hover .nav-toggle .nav-toggle__link.open{display:none}.banner #nav:hover .nav-toggle .nav-toggle__link.close{display:block}}ol,ul{margin-left:20px;margin-bottom:20px}ol ol,ul ul,ol ul,ul ol{margin-bottom:0}nav ul,nav ol{list-style:none;list-style-image:none}li{font-size:1em;line-height:1.5;margin:0 0 .4em;padding:0}.list-ticks li{background-image:url("data:image/svg+xml;utf8, <svg xmlns='http://www.w3.org/2000/svg' width='14' height='14' viewBox='0 0 14 14'><circle fill='%23c4c4c4' cx='7' cy='7' r='7'/><path fill='%23fff' d='M6.1 10.813L2.41 8.105l1.184-1.613L5.9 8.187l4.393-4.394 1.414 1.414z' /></svg>")}.no-bullets{list-style:none;margin-left:0}.combined-list ul,.combined-list div{margin-bottom:0}.combined-list .last-item{border-bottom:1px dotted #888;padding-bottom:10px}.combined-list .last-col{margin-bottom:20px}.combined-list .last-col .last-item{border-bottom:0;padding-bottom:0}@media only screen and (min-width: 768px){.combined-list ul,.combined-list div{margin-bottom:20px}.combined-list .last-item{border-bottom:0;padding-bottom:0}}.inline-list{margin-left:0}.inline-list li{display:inline;list-style:none;margin-right:20px}.inline-list .last-item{margin-right:0}.list-step{list-style:none;margin-left:60px}@media only screen and (max-width: 1440px){.list-step__item:first-child{margin-top:10px}}@media only screen and (min-width: 1440px){.list-step__title{margin-bottom:0}}.list-step__bullet{box-shadow:0 1px 3px 1px rgba(51,51,51,0.2);background:#fff;border-radius:50%;padding:.3em .68em;display:inline-block;color:#f7f7f7;margin-right:.34375em;margin-bottom:.625em;margin-left:-60px}@media only screen and (max-width: 1440px){.list-step__bullet{position:absolute;top:-5px}}.row{border-bottom:1px dotted #888;clear:both;padding:20px 10px 0;position:relative}.row br{display:none}.row.no-padding-bottom{padding-bottom:0}.row::after{clear:both;content:'.';display:block;height:0;visibility:hidden}.row--light{background:#fff}.row--dark{background:#333;color:#fff}.row.row-grey,.row.row--grey{background:#f7f7f7;border:0;margin-top:-1px}.no-border{border:0}.row-hero{margin-top:20px;padding-top:0}.strip{width:100%;display:block}.strip-inner-wrapper{max-width:1440px;margin:auto}@media only screen and (min-width: 768px){.row-hero{margin-top:40px}.row{border-radius:0;margin:0;padding:40px 20px 20px}.row-grey{margin-top:-1px}}@media only screen and (min-width: 769px){.row br{display:block}}@media only screen and (min-width: 1440px){.row br{display:block}.row{padding:60px 20px 40px}.no-border{border:0}}.header-search [type="search"],.header-search [type="text"],.search-form [type="search"],.search-form [type="text"]{-webkit-appearance:none;background-color:#dedede;box-shadow:none;color:#333;display:block;float:left;font-size:1em;margin-bottom:0;padding:12px 10px;transition:all .5s ease-out;width:100%}.header-search .svg-search-handle,.search-form .svg-search-handle{fill:#333}.header-search .svg-search-frame,.search-form .svg-search-frame{stroke:#333}.header-search placeholder,.search-form placeholder{color:#333;opacity:1}.header-search input:placeholder,.search-form input:placeholder{color:#333;opacity:1}.header-search ::placeholder,.search-form ::placeholder{color:#333;opacity:1}.header-search [type="search"]:focus,.search-form [type="search"]:focus{background-color:#d3d3d3;border-color:#b3b3b3}.header-search [type=submit],.search-form [type=submit]{background:none;display:block;float:left;line-height:0;margin-left:-40px;overflow:visible;padding:3px 2px;width:auto}.header-search [type=submit]:hover,.search-form [type=submit]:hover{background:none}.header-search [type=submit] img,.search-form [type=submit] img{height:28px;width:28px;margin-top:2px}.banner .search-toggle{background-image:url("data:image/svg+xml;utf8, <svg xmlns='http://www.w3.org/2000/svg' width='28' height='28' viewBox='0 0 90 90'><g color='%23333'><path fill='none' stroke-width='4' overflow='visible' enable-background='accumulate' d='M0 0h90v90H0z'/><path d='M69 36.5a33 33.5 0 1 1-66 0 33 33.5 0 1 1 66 0z' transform='matrix(.636 0 0 .627 16.114 16.12)' fill='none' stroke='%23333' stroke-width='9.5' overflow='visible' enable-background='accumulate'/><path d='M55.77 52.92L52.94 55.75l14 14 2.83-2.83-14-14z' font-size='xx-small' fill='%23333' stroke-width='6' overflow='visible' enable-background='accumulate' font-family='Sans' class='s0'/><path d='M60.97 57.03c-1.55-3.04 1.02-3.63 2.46-0.58 1.44-0.21 3.21 4.29l9.19 9.2c2.68 2.85 3.26 2.46 5.64 2.38-2.38 2.77-2.79-0.08-5.65l-9.19-9.2c-0.73-0.75-1.78-1.19-2.83-1.19z' font-size='xx-small' fill='%23333' stroke-width='11.8' overflow='visible' enable-background='accumulate' font-family='Sans' class='s0'/></g></svg>");background-position:center center;background-repeat:no-repeat;background-size:20px 20px;display:block;height:48px;outline:none;overflow:hidden;position:absolute;right:0;text-indent:-999em;top:0;width:24px}.banner .search-toggle .search-toggle__link{width:48px;height:48px}.banner .search-toggle .search-toggle__link.open{display:block}.banner .search-toggle .search-toggle__link.close{display:none}#site-search:hover form{display:block}#site-search:hover .search-toggle .search-toggle__link.open{display:none}#site-search:hover .search-toggle .search-toggle__link.close{display:block}.header-search,.search-form{background:#f7f7f7;border:0;display:none;float:left;position:relative;margin:0;width:100%;z-index:3}.search-form.active,.header-search.active,.header-search.open{display:block}.search-form div,.header-search div{box-shadow:inset 0 -4px 4px -4px rgba(0,0,0,0.3),inset 0 5px 5px -5px rgba(0,0,0,0.3);background:#fff;margin:10px;position:relative;z-index:1}.search-form form [type="search"],.header-search form [type="search"]{background:#fff;border:0;box-shadow:0 2px 2px rgba(0,0,0,0.3) inset,0 -1px 3px rgba(0,0,0,0.2) inset,0 2px 0 rgba(255,255,255,0.4);color:#333;display:block;float:left;font-size:1em;height:auto;margin:0;padding:8px 10px;width:100%}.yes-js .header-inner .search-form,.yes-js .header-inner .header-search{display:none}.yes-js .header-inner .search-form form,.yes-js .header-inner .header-search form{margin-left:0;margin-right:0;overflow:hidden;padding:10px;position:relative;top:0;width:100%;z-index:999}@media only screen and (max-width: 620px){.banner .search-toggle{right:0}.no-svg .search-toggle,.opera-mini .search-toggle{background-image:url("https://assets.ubuntu.com/v1/75d8151d-search-white.png")}}@media only screen and (min-width: 620px){.banner .search-toggle{display:none}}@media only screen and (min-width: 620px){.search-form,.header-search{background:none;border-right:0 none;float:right;margin-bottom:0;max-width:220px;overflow:hidden;padding:7px 0 5px 14px}.search-form form [type="text"],.search-form form [type="search"],.header-search form [type="text"],.header-search form [type="search"]{box-shadow:0 2px 4px rgba(0,0,0,0.4) inset;box-sizing:content-box;background-color:#dedede;border:0 solid #d8d8d8;border-width:0 0 1px;color:#333;font-size:.813em;height:24px;margin-bottom:0;padding:.5em 2.5em .5em .5em;transition:all .5s ease 0s;width:86px}.search-form form [type="text"]:focus,.search-form form [type="search"]:focus,.header-search form [type="text"]:focus,.header-search form [type="search"]:focus{background-color:#d3d3d3;border-color:#c4c4c4}.search-form .svg-search-handle,.header-search .svg-search-handle{fill:#333}.search-form .svg-search-frame,.header-search .svg-search-frame{stroke:#333}.search-form placeholder,.header-search placeholder{color:#333}.search-form input:placeholder,.header-search input:placeholder{color:#333}.search-form ::placeholder,.header-search ::placeholder{color:#333}}@media only screen and (min-width: 620px){.header-search .svg-search-handle,.search-form .svg-search-handle{fill:#333}.header-search .svg-search-frame,.search-form .svg-search-frame{stroke:#333}.header-search placeholder,.search-form placeholder{color:#333}.header-search input:placeholder,.search-form input:placeholder{color:#333}.header-search ::placeholder,.search-form ::placeholder{color:#333}}@media only screen and (max-width: 620px){.banner .nav-primary .header-search{position:relative;top:0;width:100%}.banner .nav-primary .header-search [type="search"]{background-color:#ebebeb;color:#333}.banner .nav-primary .header-search [type="submit"]{background-color:transparent;height:38px;margin-top:0;width:32px}.banner .nav-primary .header-search [type="submit"] img,.banner .nav-primary .header-search [type="submit"] svg{max-width:none}.banner .nav-primary .header-search.open{display:block}.banner .search-toggle{background-image:url("data:image/svg+xml;utf8, <svg xmlns='http://www.w3.org/2000/svg' width='28' height='28' viewBox='0 0 90 90'><g color='%23333'><path fill='none' stroke-width='4' overflow='visible' enable-background='accumulate' d='M0 0h90v90H0z'/><path d='M69 36.5a33 33.5 0 1 1-66 0 33 33.5 0 1 1 66 0z' transform='matrix(.636 0 0 .627 16.114 16.12)' fill='none' stroke='%23333' stroke-width='9.5' overflow='visible' enable-background='accumulate'/><path d='M55.77 52.92L52.94 55.75l14 14 2.83-2.83-14-14z' font-size='xx-small' fill='%23333' stroke-width='6' overflow='visible' enable-background='accumulate' font-family='Sans' class='s0'/><path d='M60.97 57.03c-1.55-3.04 1.02-3.63 2.46-0.58 1.44-0.21 3.21 4.29l9.19 9.2c2.68 2.85 3.26 2.46 5.64 2.38-2.38 2.77-2.79-0.08-5.65l-9.19-9.2c-0.73-0.75-1.78-1.19-2.83-1.19z' font-size='xx-small' fill='%23333' stroke-width='11.8' overflow='visible' enable-background='accumulate' font-family='Sans' class='s0'/></g></svg>");background-position:center center;background-repeat:no-repeat;background-size:25px auto;cursor:pointer;display:block;height:48px;position:absolute;right:0;text-indent:-99999px;width:48px}.no-svg .banner .search-toggle,.opera-mini .banner .search-toggle{background-image:url("https://assets.ubuntu.com/v1/2196e362-search-white.svg")}.opera-mini x:-o-prefocus,.opera-mini .banner .search-toggle{background-size:25px auto}}@media only screen and (min-width: 620px){.search-form,.header-search{display:block}.search-form .svg-search-handle,.header-search .svg-search-handle{fill:#333}.search-form .svg-search-frame,.header-search .svg-search-frame{stroke:#333}.search-form form [type="text"]:focus,.header-search form [type="text"]:focus{width:160px}.search-form [type="search"],.search-form [type="text"],.header-search [type="search"],.header-search [type="text"]{padding:8px 10px}}@media only screen and (max-width: 1440px){.search-form,.header-search{margin-right:10px}}@media only screen and (max-width: 620px){.banner .search-toggle{right:48px}}.ubuntu-search .nav-secondary,.search-results .nav-secondary,.search-no-results .nav-secondary{display:none}.ubuntu-search section>h1,.ubuntu-search section article h1,.search-results section>h1,.search-results section article h1,.search-no-results section>h1,.search-no-results section article h1{padding-bottom:10px;font-size:1.438em;margin-bottom:0}.ubuntu-search section>h1,.search-results section>h1,.search-no-results section>h1{border-bottom:1px dotted #cdcdcd}.ubuntu-search .main-search,.search-results .main-search,.search-no-results .main-search{background-color:transparent;margin:0 0 20px;padding:20px 0}.ubuntu-search .main-search [type="search"],.search-results .main-search [type="search"],.search-no-results .main-search [type="search"]{border:1px solid #888;float:left;font-size:2em;padding:.2em 65px .2em .2em;width:100%}.ubuntu-search .main-search [type=submit],.search-results .main-search [type=submit],.search-no-results .main-search [type=submit]{width:32px;height:38px;background-repeat:no-repeat;background-position:center 8px;background-color:transparent;background-size:28px 28px;display:block;float:left;line-height:0;margin-left:-53px;margin-top:-4px;overflow:visible;padding:4px}.ubuntu-search .main-search [type=submit] img,.search-results .main-search [type=submit] img,.search-no-results .main-search [type=submit] img{height:45px;width:45px}.ubuntu-search .main-search [type=submit]:hover,.search-results .main-search [type=submit]:hover,.search-no-results .main-search [type=submit]:hover{background:none}.ubuntu-search .search-result h1 .title-main,.search-results .search-result h1 .title-main,.search-no-results .search-result h1 .title-main{margin-right:20px}.ubuntu-search .search-result h1 .result-url,.search-results .search-result h1 .result-url,.search-no-results .search-result h1 .result-url{color:#888;display:block;overflow:hidden;padding-bottom:2px;text-overflow:ellipsis;vertical-align:bottom}.ubuntu-search .search-result h1 .result-url a,.search-results .search-result h1 .result-url a,.search-no-results .search-result h1 .result-url a{color:#888}.ubuntu-search .search-result p,.search-results .search-result p,.search-no-results .search-result p{margin-bottom:0}.ubuntu-search .num-results,.search-results .num-results,.search-no-results .num-results{display:inline-block;margin-left:20px}.ubuntu-search .bottom-results-total,.search-results .bottom-results-total,.search-no-results .bottom-results-total{margin:0;overflow:visible;padding-top:20px;text-align:center;width:100%}.ubuntu-search .bottom-nav,.search-results .bottom-nav,.search-no-results .bottom-nav{margin-top:-26px;overflow:hidden}.ubuntu-search .bottom-nav ul,.search-results .bottom-nav ul,.search-no-results .bottom-nav ul{margin-bottom:0;margin-left:0;overflow:hidden;padding:0}.ubuntu-search .bottom-nav li,.search-results .bottom-nav li,.search-no-results .bottom-nav li{float:left;margin-left:15px}.ubuntu-search .bottom-nav li:first-child,.search-results .bottom-nav li:first-child,.search-no-results .bottom-nav li:first-child{margin-left:0}.ubuntu-search .nav-back,.search-results .nav-back,.search-no-results .nav-back{float:left}.ubuntu-search .nav-back li:before,.search-results .nav-back li:before,.search-no-results .nav-back li:before{color:#f7f7f7;content:'\2039';margin-right:5px}.ubuntu-search .nav-back .item-extreme:before,.search-results .nav-back .item-extreme:before,.search-no-results .nav-back .item-extreme:before{content:'\2039\2039'}.ubuntu-search .nav-forward,.search-results .nav-forward,.search-no-results .nav-forward{float:right}.ubuntu-search .nav-forward li:after,.search-results .nav-forward li:after,.search-no-results .nav-forward li:after{color:#f7f7f7;content:'\203A';margin-left:5px}.ubuntu-search .nav-forward .item-extreme:after,.search-results .nav-forward .item-extreme:after,.search-no-results .nav-forward .item-extreme:after{content:'\203A\203A'}.ubuntu-search .error-notification,.search-results .error-notification,.search-no-results .error-notification{background-color:#fff;color:#333;display:block;margin-top:20px;padding:20px;width:100%}.ubuntu-search .result-line,.search-results .result-line,.search-no-results .result-line{color:#888}.ubuntu-search .results-top,.search-results .results-top,.search-no-results .results-top{border-bottom:1px dotted #cdcdcd;padding-bottom:.5em}.ubuntu-search .search-container,.search-results .search-container,.search-no-results .search-container{padding-bottom:0}@media only screen and (min-width: 620px){.ubuntu-search .main-search [type=submit]{margin-left:-60px;margin-top:0}}@font-face{font-family:'Ubuntu';font-style:normal;font-weight:300;src:url("../assets/fonts/ubuntu-l-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntu-l-webfont.woff") format("woff")}@font-face{font-family:'Ubuntu';font-style:normal;font-weight:400;src:url("../assets/fonts/ubuntu-r-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntu-r-webfont.woff") format("woff")}@font-face{font-family:'Ubuntu';font-style:normal;font-weight:700;src:url("../assets/fonts/ubuntu-b-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntu-b-webfont.woff") format("woff")}@font-face{font-family:'Ubuntu';font-style:italic;font-weight:300;src:url("../assets/fonts/ubuntu-li-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntu-li-webfont.woff") format("woff")}@font-face{font-family:'Ubuntu';font-style:italic;font-weight:400;src:url("../assets/fonts/ubuntu-ri-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntu-ri-webfont.woff") format("woff")}@font-face{font-family:'Ubuntu';font-style:italic;font-weight:700;src:url("../assets/fonts/ubuntu-bi-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntu-bi-webfont.woff") format("woff")}@font-face{font-family:'Ubuntu Mono';font-style:normal;font-weight:400;src:url("../assets/fonts/ubuntumono-r-webfont.woff2") format("woff2"),url("../assets/fonts/ubuntumono-r-webfont.woff") format("woff")}body{color:#333;font-family:'Ubuntu', Arial, 'libra sans', sans-serif;font-size:16px;font-weight:300}a:focus{outline:thin dotted}a:hover,a:active{outline:0}a{color:#f7f7f7;text-decoration:none}a:hover,a:active,a:focus{text-decoration:underline}strong{font-weight:400}.caps-centered,.muted-heading{font-size:.875em;margin-bottom:20px;text-align:center;text-transform:uppercase}small,.smaller{font-size:13px}sub,sup{font-size:75%;line-height:0;position:relative;vertical-align:baseline}sup{vertical-align:text-top}sub{vertical-align:text-bottom}pre{border-radius:4px;background-color:#fff;padding:.6em 1em;white-space:pre-wrap;word-wrap:break-word}p+h2,ul+h2,ol+h2,pre+h2{margin-top:0.5625em}header nav a:link{font-weight:normal}p+h3,ul+h3,ol+h3,pre+h3{margin-top:.783em}p+h4,ul+h4,ol+h4,pre+h4{margin-top:1.21875}ol+h2,p+h2,pre+h2,ul+h2{margin-top:.563em}ol+h3,p+h3,pre+h3,ul+h3{margin-top:.783em}ol+h4,p+h4,pre+h4,ul+h4{margin-top:1.219em}.intro{font-size:1em;line-height:1.4}.row div p:last-child,.row ul p:last-child{margin-bottom:0}.four-col p:last-child{margin-bottom:0}.note{color:#888;font-size:.813em}h1,h2,h3,h4,h5,h6{font-family:Ubuntu, Arial, 'libra sans', sans-serif;font-weight:300;line-height:1.3}h1{font-size:2.8125em;margin-bottom:.5em}h2{font-size:2em;margin-bottom:.5em}h3{font-size:1.4375em;margin-bottom:.522em}h4{font-size:1.25em;font-weight:400;margin-bottom:.615em}h5{font-size:1em;font-weight:700;margin-bottom:1em}h6{font-size:.8125em;font-weight:400;margin-bottom:1em;letter-spacing:.1em;text-transform:uppercase}p,li{font-size:1em;line-height:1.5;margin:0;margin-bottom:.75em;padding:0}button,input,select,textarea{font-family:Ubuntu,Arial,'libra sans',sans-serif}@media only screen and (min-width: 768px) and (max-width: 1440px){h1{font-size:1.5234375em;margin-bottom:.5em}h2{font-size:1.348125em;margin-bottom:.5em}h3{font-size:1.1428125em;margin-bottom:.522em}h4{font-size:1.171875em;font-weight:400;margin-bottom:.615em}h5{font-size:.9375em;font-weight:700;margin-bottom:1em}h6{font-size:.6778125em;font-weight:400;margin-bottom:1em;letter-spacing:.1em;text-transform:uppercase}.intro{font-size:1.13333em}}@media only screen and (max-width: 768px){h1{font-size:1.421875em;margin-bottom:.4375em}h2{font-size:1.25825em;margin-bottom:.4375em}h3{font-size:1.066625em;margin-bottom:.45675em}h4{font-size:1.09375em;font-weight:400;margin-bottom:.538125em}h5{font-size:.875em;font-weight:700;margin-bottom:.875em}h6{font-size:.632625em;font-weight:400;margin-bottom:.875em;letter-spacing:.1em;text-transform:uppercase}p,li{font-size:.875em}}@media only screen and (min-width: 1440px){p,li,code,pre{font-size:16px;line-height:1.5;margin-bottom:.75em}.intro{font-size:1.25em}}dfn{font-style:italic}code,pre{font-family:'Ubuntu Mono', 'Consolas', 'Monaco', 'Lucida Console', 'Courier New', Courier, monospace}table{border-collapse:collapse;border-spacing:0;overflow-x:scroll;margin-bottom:20px;margin:0 0 2.5em;width:100%}table th,table td{background:#f7f7f7;border:1px dotted #888;padding:15px 10px}table td{text-align:center;vertical-align:middle}table thead th{border-collapse:separate;border-spacing:0 10px;background:#fee3d2;color:#333;font-weight:normal}table tbody th{font-weight:300;text-align:left}table th[scope='col']{text-align:center}table thead th:first-of-type{text-align:left}@media only screen and (max-width: 768px){table{display:block}}body footer.global #nav-global li:first-of-type a{margin-left:0}footer.global{background:none;border-top:0;box-shadow:inset 0 2px 2px -1px #cdcdcd;clear:both;display:block;padding:30px 10px 20px;position:relative;width:100%}footer.global .legal{background-image:none;clear:both;margin:0 auto;min-height:40px;position:relative;width:100%}footer.global .legal p,footer.global .legal ul{padding-left:0}footer.global h2{font-size:.75em;line-height:1.4;margin-bottom:0;padding-bottom:.5em}footer.global h2,footer.global h2 a:link,footer.global h2 a:visited{color:#333;font-weight:normal}footer.global ul{margin:0}footer.global li a:hover{color:#f7f7f7}footer.global li ul li a:link,footer.global li ul li a:visited{color:#333;font-size:.75em}footer.global h2 a:hover,footer.global h2 a:active{color:#f7f7f7}footer.global p{color:#333;font-size:12px;margin-bottom:0}@media only screen and (max-width: 768px){footer.no-global .legal{box-shadow:0 2px 2px -1px #cdcdcd inset;box-sizing:content-box;margin-left:-10px;padding-left:10px;padding-right:10px;padding-top:10px}}@media only screen and (min-width: 1440px){footer.global .legal{width:1440px}footer.global{padding:30px 0 20px}}svg:not(:root){overflow:hidden}figure{width:100%}figure caption{display:block;width:100%;text-align:center}object,iframe,embed,canvas,video,audio{display:block;max-width:100%;margin:0 auto 20px}audio:not([controls]){display:none;height:0}[hidden]{display:none}.video-container{position:relative;padding-bottom:56.25%;padding-top:30px;height:0;overflow:hidden}.video-container iframe{position:absolute;top:0;left:0;width:100%;height:100%}.video-container h3,.video-container .video-title{margin-top:20px}.inline-logos{float:left;margin-left:0;padding:0;text-align:center;width:100%}.inline-logos .inline-logos__item{display:inline-block;margin-right:20px;padding:0 0 20px;width:36%}@media only screen and (max-width: 300px){.inline-logos .inline-logos__item{margin:0 0 20px;width:100%}}.inline-logos .inline-logos__item.clear-row{clear:left}.inline-logos .inline-logos__item.last-item{border:0}.inline-logos .inline-logos__image{max-height:32px;max-width:115px;transition:all .3s ease-out;vertical-align:middle}@media only screen and (min-width: 768px){.inline-logos{padding-top:20px}.inline-logos .inline-logos__item{display:inline-block;height:56px;line-height:60px;margin:0 30px 30px;width:150px}.inline-logos .inline-logos__image{float:none;height:auto;max-height:56px;max-width:150px;vertical-align:middle}}@media only screen and (min-width: 1440px){.inline-logos__item{margin-bottom:40px}}@media only screen and (min-width: 768px){.equal-height__align-vertically{align-items:center;justify-content:center}}@media only screen and (min-width: 768px){.equal-height--vertical-divider{position:relative}.equal-height--vertical-divider__item{padding-left:10px;padding-right:10px}.equal-height--vertical-divider__item:before{content:'';position:absolute;right:-10px;top:0;height:100%;width:1px;border-right:1px dotted #888}.equal-height--vertical-divider .last-col,.equal-height--vertical-divider__item:last-of-type{padding-right:0}.equal-height--vertical-divider .last-col:before,.equal-height--vertical-divider__item:last-of-type:before{border-right:0}.equal-height--vertical-divider__item:first-of-type{padding-left:0}}@media only screen and (max-width: 768px){.equal-height--vertical-divider .equal-height--vertical-divider__item{border-bottom:1px dotted #888;padding-bottom:20px}.equal-height--vertical-divider .equal-height--vertical-divider__item:last-of-type{border-bottom:0;padding-bottom:inherit}}body{background:url("../assets/img/backgrounds/image-background-paper.png") repeat-y center top #f7f7f7}.button--neutral,.button--inline-neutral{color:#333;background-color:transparent;border:1px solid #cdcdcd}.button--neutral:focus,.button--neutral:active,.button--neutral:hover,.button--inline-neutral:focus,.button--inline-neutral:active,.button--inline-neutral:hover{background-color:rgba(0,0,0,0.1)}.button--neutral:disabled:focus,.button--neutral:disabled:active,.button--neutral:disabled:hover,.button--neutral.button--disabled:focus,.button--neutral.button--disabled:active,.button--neutral.button--disabled:hover,.button--inline-neutral:disabled:focus,.button--inline-neutral:disabled:active,.button--inline-neutral:disabled:hover,.button--inline-neutral.button--disabled:focus,.button--inline-neutral.button--disabled:active,.button--inline-neutral.button--disabled:hover{background-color:transparent}.button--inline-neutral{display:inline-block;width:auto}.button--positive,.button--inline-positive{color:#fff;background-color:#38b44a}.button--positive:focus,.button--positive:active,.button--positive:hover,.button--inline-positive:focus,.button--inline-positive:active,.button--inline-positive:hover{background-color:#2c8d3a}.button--positive:disabled:focus,.button--positive:disabled:active,.button--positive:disabled:hover,.button--positive.button--disabled:focus,.button--positive.button--disabled:active,.button--positive.button--disabled:hover,.button--inline-positive:disabled:focus,.button--inline-positive:disabled:active,.button--inline-positive:disabled:hover,.button--inline-positive.button--disabled:focus,.button--inline-positive.button--disabled:active,.button--inline-positive.button--disabled:hover{background-color:#38b44a}.button--inline-positive{display:inline-block;width:auto}.button--destructive,.button--inline-destructive{color:#fff;background-color:#df382c}.button--destructive:focus,.button--destructive:active,.button--destructive:hover,.button--inline-destructive:focus,.button--inline-destructive:active,.button--inline-destructive:hover{background-color:#bc271c}.button--destructive:disabled:focus,.button--destructive:disabled:active,.button--destructive:disabled:hover,.button--destructive.button--disabled:focus,.button--destructive.button--disabled:active,.button--destructive.button--disabled:hover,.button--inline-destructive:disabled:focus,.button--inline-destructive:disabled:active,.button--inline-destructive:disabled:hover,.button--inline-destructive.button--disabled:focus,.button--inline-destructive.button--disabled:active,.button--inline-destructive.button--disabled:hover{background-color:#df382c}.button--inline-destructive{display:inline-block;width:auto}.button--base,.button--inline-base{color:#333;background-color:transparent}.button--base:focus,.button--base:active,.button--base:hover,.button--inline-base:focus,.button--inline-base:active,.button--inline-base:hover{background-color:rgba(0,0,0,0.1)}.button--base:disabled:focus,.button--base:disabled:active,.button--base:disabled:hover,.button--base.button--disabled:focus,.button--base.button--disabled:active,.button--base.button--disabled:hover,.button--inline-base:disabled:focus,.button--inline-base:disabled:active,.button--inline-base:disabled:hover,.button--inline-base.button--disabled:focus,.button--inline-base.button--disabled:active,.button--inline-base.button--disabled:hover{background-color:transparent}.button--inline-base{display:inline-block;width:auto}.cookie-policy{box-shadow:0 -1px 2px rgba(0,0,0,0.2);background-color:#fae4dc;position:fixed;bottom:0;width:100%;z-index:100}.cookie-policy p{font-size:.8125em;margin-bottom:0;margin-left:0;padding:8px 0;width:100%}.cookie-policy .link-cta{background-image:url("https://assets.ubuntu.com/v1/3f057022-close-orange.svg");background-repeat:no-repeat;color:#fff;float:right;font-size:1em;height:15px;margin:12px 0;margin-top:12px;padding:0;text-decoration:none;text-indent:-9999px;width:16px}.no-svg .cookie-policy .link-cta,.opera-mini .cookie-policy .link-cta{background-image:url("https://assets.ubuntu.com/v1/898777ac-close-orange.png")}.deep-link{text-decoration:none}.deep-link:after{content:'';display:inline-block;margin-left:3px;opacity:0;position:relative;top:1px;width:1em;height:1em;background-image:url(https://assets.ubuntu.com/v1/128877a5-anchor_16.svg);background-position:0 80%;background-repeat:no-repeat;background-size:16px;transition:opacity .1s}.deep-link:hover{text-decoration:none}.deep-link:hover:after{opacity:1}.banner{background:#000;box-shadow:none}@media only screen and (min-width: 620px){.banner .nav-primary{width:auto}.banner ul{border-top:0;border-right:0;box-shadow:none}.banner li{width:auto;border-right:0;border-bottom:0}.banner li:last-child a{border-right:0}.banner a,.banner a:link,.banner a:visited{border-left:0}}@media only screen and (max-width: 768px){footer.global{padding-top:0}}.instruction{padding:0;position:relative;width:100%;border-bottom:1px dotted #ddd}.instruction .instruction__bullet,.instruction .instruction__details{padding:60px 40px;margin-bottom:0}.instruction .instruction__bullet{margin-right:0}.instruction .instruction__step{display:inline-block;border-radius:50%;width:50px;height:50px;line-height:50px;color:#fff;text-align:center;vertical-align:top;font-size:2em;font-weight:400}.instruction .instruction__title{display:inline;position:relative;top:6px;line-height:1.6}.instruction .command-line{position:relative;border-radius:4px;background-color:#2c001e;border:1px solid #2c001e;overflow:hidden;transition:all .2s}.instruction .command-line .command-line__input{border:0;background:transparent;font-size:1em;font-family:Ubuntu Mono;font-weight:300;padding:.7em 1em;color:#fff;width:100%}.instruction .command-line .command-line__copy-button{transition:background-color .2s;position:absolute;right:0;top:0;width:50px;border:0;height:100%;display:block;text-indent:-9999px;background-image:url("https://assets.ubuntu.com/v1/994e60f9-get-link-url_16.svg");background-repeat:no-repeat;background-position:center;background-color:#fff}.row{border-bottom:0;background-color:rgba(255,255,255,0.6)}@media only screen and (max-width: 768px){.row{padding-bottom:20px}}@media only screen and (max-width: 768px){.row-hero{padding-top:20px;margin-top:0}}.strip-white{background:#fff}.strip-dark{background:#333;color:#fff}.strip-dark a{color:#fff}.strip-trans{background:transparent}.social-list{list-style:none;padding-left:0;margin-left:0}.social-list__item--twitter,.social-list__item--google-plus,.social-list__item--facebook{margin-right:20px;float:left;text-indent:-99999px;background-image:url("https://assets.ubuntu.com/v1/ed986fc0-icon-social.svg");background-repeat:no-repeat;height:44px;width:44px;overflow:hidden;display:inline-block}.social-list__item--twitter:hover,.social-list__item--google-plus:hover,.social-list__item--facebook:hover{background-position-y:-45px}.social-list__item--twitter{background-position:0 0}.social-list__item--google-plus{background-position:-45px 0}.social-list__item--facebook{background-position:-90px 0}@media only screen and (min-width: 768px){.tooltip{position:relative;display:inline-block}.tooltip__content{position:absolute;z-index:98;left:-1000px;right:-1000px;top:-30px;font-weight:300;margin:auto;display:block;text-align:center;white-space:nowrap}.tooltip:hover .tooltip__content:after{display:table;z-index:98;margin:auto;color:#fff;border-radius:3px;background:#000;box-shadow:none;font-size:12px;content:attr(data-tooltip);padding:4px 6px;white-space:nowrap;text-align:center}.tooltip:hover .tooltip__content:before{position:absolute;top:100%;left:50%;margin-left:-5px;content:'';border:solid transparent;border-width:5px;border-top-color:#000}}a,.link{cursor:pointer;color:#f7f7f7;text-decoration:underline}pre{padding:10px 15px;background-color:#333;border:0;border-radius:2px;color:#cdcdcd}html{overflow-y:scroll;overflow-x:hidden;min-height:100%;color:#333;font:1em / 1.5 "'Ubuntu', Arial, 'libra sans', sans-serif"}a{color:#333;text-decoration:none;border-bottom:1px solid #c2c2c2}a:hover{text-decoration:none;color:#dd4814}p,li,code,pre{font-size:16px;font-size:1rem;line-height:1.5;margin-bottom:.75em}pre,code{text-align:left;white-space:pre-line;word-spacing:normal;word-wrap:break-word;tab-size:4;hyphens:none;direction:ltr}hr{margin-bottom:20px}h1,h2,h3,h4,h5{margin:10px 0;font-weight:300;line-height:1.5;color:#333}.gigantic{font-size:45px;font-size:2.8125rem}.mega{font-size:40px;font-size:2.5rem}input[type='text'],input[type='number'],input[type='search'],input[type='password'],input[type='email'],input[type='url'],input[type='tel']{padding:7px 10px;box-shadow:none;width:100%}input[type='text']::placeholder,input[type='number']::placeholder,input[type='search']::placeholder,input[type='password']::placeholder,input[type='email']::placeholder,input[type='url']::placeholder,input[type='tel']::placeholder{color:#888}input[type='text'][disabled],input[type='number'][disabled],input[type='search'][disabled],input[type='password'][disabled],input[type='email'][disabled],input[type='url'][disabled],input[type='tel'][disabled]{color:#888;background-color:transparent}input[type='number']{padding-right:15px}input[type='search']{appearance:textfield}input[type='search']::-webkit-search-decoration,input[type='search']::-webkit-search-cancel-button{appearance:none}input[type='checkbox']{margin:4px 0 0;margin-top:1px	9;line-height:normal}input[type='radio'],input[type='image']{display:inline-block;margin-right:10px}textarea{height:auto;min-height:175px;padding:7px 10px !important;box-shadow:none;width:100%;line-height:20px}textarea::placeholder{color:#888}textarea[disabled]{color:#888;background-color:transparent}select{display:block;clear:both;cursor:pointer;margin:0;background-image:url("../assets/images/forms/chevron-down.svg");background-repeat:no-repeat;background-position:top 14px right 10px;background-color:#fff;padding:7px 30px 7px 10px !important;box-shadow:none;width:100%;-moz-appearance:none;appearance:none;text-indent:.01px;text-overflow:''}select[multiple],select[size]{height:auto;background-image:none;padding-top:10px}select[disabled]{color:#888;background-image:none;background-color:transparent}select:-moz-focusring{color:transparent;text-shadow:0 0 0 #000}select::-ms-expand{display:none}fieldset{background:none;margin-left:0;padding:0}label{display:inline-block;max-width:100%;margin-bottom:5px}@media only screen and (max-width: 768px){label{width:100%;color:#888}}[type="checkbox"]:not(:checked),[type="checkbox"]:checked{position:absolute;left:-9999px}[type="checkbox"]+label{min-height:34px;position:relative;padding-left:25px;cursor:pointer;display:inline-block !important}[type="checkbox"]+label:before{content:'';position:absolute;left:0;top:11px;width:11px;height:11px;border:1px solid #cdcdcd;background:#fff;border-radius:2px}[type="checkbox"]:checked+label:before{background-color:#dd4814;border-color:#dd4814}[type="checkbox"]:checked+label:after{content:'✔';position:absolute;line-height:13px;top:0px;left:3px;font-size:10px;color:#fff;transition:all .2s}@-moz-document url-prefix(){[type="checkbox"]:checked+label:after{font-size:13px;left:1px}}[type="checkbox"][disabled]{cursor:not-allowed}[type="checkbox"][disabled]+label{cursor:not-allowed}[type="checkbox"][disabled]+label:before{opacity:.5}[type="radio"]:not(:checked),[type="radio"]:checked{position:absolute;left:-9999px}[type="radio"]+label{position:relative;padding-left:25px;cursor:pointer;display:inline-block !important}[type="radio"]+label:before{content:'';position:absolute;left:0;top:1px;width:13px;height:13px;border:1px solid #cdcdcd;background:#fff;border-radius:50%}[type="radio"]:checked+label:after{content:'';position:absolute;left:3px;top:3px;width:9px;height:9px;background:#dd4814;border-radius:50%}.tags{width:100%}.tags .tag-list{width:100%;margin:0}.tags .tag-list .tag-item{display:inline-block;float:left;line-height:36px;margin-bottom:0;margin-right:10px;word-wrap:break-word}.tags .tag-list .tag-item .remove-button{border-bottom:0}.tags .input{width:100% !important}.onoffswitch{position:relative;display:inline-block;vertical-align:middle;width:38px;margin:0 0 4px;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none}.onoffswitch[disabled] .onoffswitch-checkbox,.onoffswitch[disabled] .onoffswitch-label{opacity:.5;pointer-events:none}.onoffswitch-external-label{display:inline-block;margin-left:5px}.onoffswitch-checkbox{display:none !important}.onoffswitch-label{display:block !important;overflow:hidden;cursor:pointer;border-radius:2px;padding-left:0 !important;margin:0 !important;vertical-align:top;width:100% !important;min-height:16px !important;max-height:16px;line-height:16px !important;vertical-align:top !important}.onoffswitch-label:before,.onoffswitch-label:after{display:none}.onoffswitch-inner{display:block;width:200%;margin-left:-100%;transition:margin .3s ease-in 0s}.onoffswitch-inner:before,.onoffswitch-inner:after{display:block;float:left;width:50%;height:18px;padding:0;line-height:18px;font-size:14px;color:#fff;font-family:Trebuchet, Arial, sans-serif;font-weight:bold;box-sizing:border-box}.onoffswitch-inner:before{content:'';padding-left:10px;background-color:#38b44a;color:#fff}.onoffswitch-inner:after{content:'';padding-right:10px;background-color:#eee;color:#888;text-align:right}.onoffswitch-switch{display:block;width:19px;height:16px;margin:0;background:#fff;position:absolute;top:0;bottom:0;right:19px;border:1px solid #d2d2d2;border-radius:2px;transition:all .3s ease-in 0s;box-sizing:border-box}.onoffswitch-checkbox:checked+.onoffswitch-label .onoffswitch-inner{margin-left:0}.onoffswitch-checkbox:checked+.onoffswitch-label .onoffswitch-switch{right:0}dl dt{clear:left;color:#333}@media only screen and (max-width: 768px){dl dt{color:#888}}dl dd{color:#333;margin-left:0}@media only screen and (max-width: 768px){dl dd{text-align:right}}dl dt,dl dd{display:inline-block;float:left;line-height:36px;margin-bottom:10px !important;word-wrap:break-word}@media only screen and (max-width: 768px){dl dt,dl dd{width:50% !important}}dl dt:last-child,dl dd:last-child{margin-bottom:0}.wrapper{min-height:100%;height:auto !important;height:100%;margin:0 auto;position:relative;background:rgba(255,255,255,0.4)}.wrapper:after{content:'';position:absolute;display:block;top:0;right:0;bottom:0;left:0;background:url("../assets/images/backgrounds/background-paper.png");height:100%;width:100%;z-index:-1}.wrapper--inner{max-width:1440px;width:100%;margin:0 auto;clear:both;display:block}@media only screen and (max-width: 768px){.wrapper--inner{float:left;width:100%}}.row{background-color:transparent;border-top:1px dotted #cdcdcd}.row:first-of-type{border-top:0}.button--secondary{line-height:18px}.button--primary:hover,.button--destructive:hover,.button--positive:hover{color:#fff}.button--base:hover,.button--neutral:hover{color:#333}.button--large{height:auto;padding-top:15px;padding-bottom:15px}@media only screen and (max-width: 768px){[class^="button--"]{margin-top:0px}[class^="button--"]+[class^="button--"]{margin-top:10px}}.button-group{position:relative;width:100%;vertical-align:middle;clear:both;min-width:160px}@media only screen and (max-width: 768px){.button-group{width:100%}}.button-group.button-group--inline{display:inline-block;width:auto}@media only screen and (max-width: 768px){.button-group.button-group--inline{width:100%}}.button-group .button-group__link{display:inline-block;float:left;text-align:left;padding-right:35px}.button-group .button-group__link::after{content:'';display:inline-block;position:absolute;top:15px;right:15px;width:4px;height:4px;vertical-align:middle;border-right:1px solid;border-bottom:1px solid;transform:rotate(45deg)}.button-group .button-group__dropdown{border-radius:3px;left:0;margin:0;list-style:none;background:#fff;box-shadow:0 1px 1px rgba(0,0,0,0.1);z-index:20;max-height:1000px;transition:max-height .3s ease-in;position:absolute;top:36px;clear:both;min-width:160px}@media only screen and (max-width: 768px){.button-group .button-group__dropdown{width:100%}}.button-group .button-group__item{float:left;clear:both;padding:5px 10px;margin:0;width:100%}.button-group .button-group__item .button-group__item-link{color:#333;cursor:pointer;width:100%;float:left;margin:0;border:0;text-transform:lowercase;white-space:nowrap}.button-group .button-group__item .button-group__item-link:first-letter{text-transform:uppercase}.button-group .button-group__item .button-group__item-link:hover{color:#dd4814;text-decoration:none}.form .form__group{margin-bottom:10px}.form .form__group .form__group-input{position:relative}.form .form__group .form__group-input .form__group-remove{position:absolute;top:10px;right:10px;cursor:pointer;height:16px;width:16px;background:url("../assets/images/icons/remove.svg") no-repeat;background-size:75% 75%;background-position:center;display:inline-block}.form .form__group .form__group-input .icon,.form .form__group .form__group-input .icon--status-failed,.form .form__group .form__group-input .icon--status-in-progress,.form .form__group .form__group-input .icon--status-queued,.form .form__group .form__group-input .icon--status-succeeded,.form .form__group .form__group-input .icon--status-waiting{position:absolute;top:10px;right:10px;display:inline-block}.form .form__group.form__group--subtle .form__group-label{color:#888}.form .form__group.form__group--subtle input,.form .form__group.form__group--subtle select,.form .form__group.form__group--subtle textarea{border-color:#e3e3e3;background-color:transparent}.form .form__group.form__group--subtle input:hover,.form .form__group.form__group--subtle select:hover,.form .form__group.form__group--subtle textarea:hover{border-color:#cdcdcd;background-color:#fff;outline:none}.form .form__group.form__group--subtle input:active,.form .form__group.form__group--subtle input:focus,.form .form__group.form__group--subtle select:active,.form .form__group.form__group--subtle select:focus,.form .form__group.form__group--subtle textarea:active,.form .form__group.form__group--subtle textarea:focus{border-color:#888;background-color:#fff;outline:none}.form .form__fieldset .form__group:last-of-type{margin-bottom:0}.form .form__siblings{float:left;width:100%;margin-bottom:10px}.form .form__siblings:hover .form__group--subtle input,.form .form__siblings:hover .form__group--subtle select,.form .form__siblings:hover .form__group--subtle textarea{border-color:#cdcdcd;background-color:#fff;outline:none}.form .form__siblings.is-active .form__group--subtle input,.form .form__siblings.is-active .form__group--subtle select,.form .form__siblings.is-active .form__group--subtle textarea{border-color:#cdcdcd;background-color:#fff;outline:none}.form.form--stack .form__group{width:100%;float:left}.form.form--stack .form__group [class*='-col']{margin-bottom:0}.form.form--stack .form__group .form__group-label{margin-top:0;margin-bottom:0;line-height:34px;vertical-align:top}.form.form--stack .form__group .form__group-input{min-height:34px}.form.form--stack .form__group .form__group-input .onoffswitch{margin-top:9px}.form.form--inline .form__fieldset{display:inline-block;width:auto;margin:0;vertical-align:middle}.form.form--inline .form__group{display:inline-block;margin:0 20px 0 0;vertical-align:top}@media only screen and (max-width: 768px){.form.form--inline .form__group{display:block;width:100%}}.form.form--inline .form__group .form__group-label{margin-bottom:0;margin-right:10px;vertical-align:top;line-height:34px}@media only screen and (max-width: 768px){.form.form--inline .form__group .form__group-label{display:block;width:100%}}.form.form--inline .form__group .form__group-label .icon--left{position:absolute;left:0;top:12px}.form.form--inline .form__group input,.form.form--inline .form__group select,.form.form--inline .form__group .form__group-input{display:inline-block;width:auto;min-height:34px}@media only screen and (max-width: 768px){.form.form--inline .form__group input,.form.form--inline .form__group select,.form.form--inline .form__group .form__group-input{display:block;width:100%}}.form.form--inline .form__group [type='checkbox']+label,.form.form--inline .form__group [type='radio']+label{line-height:34px}.form.form--inline .form__group .onoffswitch{margin:9px 0 0}.form.form--inline .form__group .onoffswitch-external-label{margin:0;vertical-align:middle;line-height:34px}.form .form__help-text{font-size:14px;font-size:0.875rem;color:#888}.form__error{margin:0;color:#df382c}.form__error .form__error-item{margin:6px 0}.code-block{overflow:auto;margin:.5em 0;padding:1em;border:1px solid #cdcdcd;border-radius:.5em}.code-block .code-block__line{float:left;clear:both;margin:0}.code-block.code-block--terminal{color:#fff;border:0;background-color:#333}.code-block.code-block--numbering{counter-reset:line-numbering}.code-block.code-block--numbering .code-block__line::before{width:1.5em;padding-right:1em;content:counter(line-numbering);counter-increment:line-numbering;user-select:none;text-align:right;pointer-events:none;opacity:.5}.flash-messages{width:100%;float:left;margin:0}.flash-messages .flash-messages__item{box-sizing:border-box;border-radius:2px;list-style:none;padding:15px 20px;margin:0 0 20px;background:#fff;background-position:top 50% left 15px;background-repeat:no-repeat;box-shadow:0 1px 1px rgba(0,0,0,0.1)}.flash-messages .flash-messages__item:last-of-type{margin-bottom:0}.flash-messages .flash-messages__item.flash-messages__item--error{background:#fff url("../assets/images/icons/error.svg") no-repeat;background-size:16px 16px;background-position:15px center;padding-left:45px}.flash-messages .flash-messages__item.flash-messages__item--info{background:#fff url("../assets/images/icons/info.svg") no-repeat;background-size:16px 16px;background-position:15px center;padding-left:45px}.flash-messages .flash-messages__item.flash-messages__item--warning{background:#fff url("../assets/images/icons/warning.svg") no-repeat;background-size:16px 16px;background-position:15px center;padding-left:45px}.flash-messages .flash-messages__item.flash-messages__item--success{background:#fff url("../assets/images/icons/success.svg") no-repeat;background-size:16px 16px;background-position:15px center;padding-left:45px}.icon,.icon--status-failed,.icon--status-in-progress,.icon--status-queued,.icon--status-succeeded,.icon--status-waiting{width:16px;height:16px;padding:0;border-bottom:0 !important;display:inline-block;vertical-align:middle;text-indent:999em}.icon.icon--add,.icon--add.icon--status-failed,.icon--add.icon--status-in-progress,.icon--add.icon--status-queued,.icon--add.icon--status-succeeded,.icon--add.icon--status-waiting{background:url("../assets/images/icons/add.svg") no-repeat;background-size:100% 100%}.icon.icon--account,.icon--account.icon--status-failed,.icon--account.icon--status-in-progress,.icon--account.icon--status-queued,.icon--account.icon--status-succeeded,.icon--account.icon--status-waiting{background:url("../assets/images/icons/account.svg") no-repeat;background-size:100% 100%}.icon.icon--cross,.icon--cross.icon--status-failed,.icon--cross.icon--status-in-progress,.icon--cross.icon--status-queued,.icon--cross.icon--status-succeeded,.icon--cross.icon--status-waiting{background:url("../assets/images/icons/cross.svg") no-repeat;background-size:100% 100%}.icon.icon--cross-orange,.icon--cross-orange.icon--status-failed,.icon--cross-orange.icon--status-in-progress,.icon--cross-orange.icon--status-queued,.icon--cross-orange.icon--status-succeeded,.icon--cross-orange.icon--status-waiting{background:url("../assets/images/icons/cross-orange.svg") no-repeat;background-size:100% 100%}.icon.icon--debug,.icon--debug.icon--status-failed,.icon--debug.icon--status-in-progress,.icon--debug.icon--status-queued,.icon--debug.icon--status-succeeded,.icon--debug.icon--status-waiting{background:url("../assets/images/icons/debug.svg") no-repeat;background-size:100% 100%}.icon.icon--delete,.icon--delete.icon--status-failed,.icon--delete.icon--status-in-progress,.icon--delete.icon--status-queued,.icon--delete.icon--status-succeeded,.icon--delete.icon--status-waiting{background:url("../assets/images/icons/delete.svg") no-repeat;background-size:100% 100%}.icon.icon--edit,.icon--edit.icon--status-failed,.icon--edit.icon--status-in-progress,.icon--edit.icon--status-queued,.icon--edit.icon--status-succeeded,.icon--edit.icon--status-waiting{background:url("../assets/images/icons/edit.svg") no-repeat;background-size:100% 100%}.icon.icon--error,.icon--error.icon--status-failed,.icon--error.icon--status-in-progress,.icon--error.icon--status-queued,.icon--error.icon--status-succeeded,.icon--error.icon--status-waiting{background:url("../assets/images/icons/error.svg") no-repeat;background-size:100% 100%}.icon.icon--help,.icon--help.icon--status-failed,.icon--help.icon--status-in-progress,.icon--help.icon--status-queued,.icon--help.icon--status-succeeded,.icon--help.icon--status-waiting{background:url("../assets/images/icons/help.svg") no-repeat;background-size:100% 100%}.icon.icon--info,.icon--info.icon--status-failed,.icon--info.icon--status-in-progress,.icon--info.icon--status-queued,.icon--info.icon--status-succeeded,.icon--info.icon--status-waiting{background:url("../assets/images/icons/info.svg") no-repeat;background-size:100% 100%}.icon.icon--loading,.icon--loading.icon--status-failed,.icon--loading.icon--status-in-progress,.icon--loading.icon--status-queued,.icon--loading.icon--status-succeeded,.icon--loading.icon--status-waiting{background:url("../assets/images/icons/loading.png") no-repeat;background-size:100% 100%}.icon.icon--mount,.icon--mount.icon--status-failed,.icon--mount.icon--status-in-progress,.icon--mount.icon--status-queued,.icon--mount.icon--status-succeeded,.icon--mount.icon--status-waiting{background:url("../assets/images/icons/mount.svg") no-repeat;background-size:100% 100%}.icon.icon--unmount,.icon--unmount.icon--status-failed,.icon--unmount.icon--status-in-progress,.icon--unmount.icon--status-queued,.icon--unmount.icon--status-succeeded,.icon--unmount.icon--status-waiting{background:url("../assets/images/icons/unmount.svg") no-repeat;background-size:100% 100%}.icon.icon--partition,.icon--partition.icon--status-failed,.icon--partition.icon--status-in-progress,.icon--partition.icon--status-queued,.icon--partition.icon--status-succeeded,.icon--partition.icon--status-waiting{background:url("../assets/images/icons/partition.svg") no-repeat;background-size:100% 100%}.icon.icon--power-error,.icon--power-error.icon--status-failed,.icon--power-error.icon--status-in-progress,.icon--power-error.icon--status-queued,.icon--power-error.icon--status-succeeded,.icon--power-error.icon--status-waiting{background:url("../assets/images/icons/power-error.svg") no-repeat;background-size:100% 100%}.icon.icon--power-off,.icon--power-off.icon--status-failed,.icon--power-off.icon--status-in-progress,.icon--power-off.icon--status-queued,.icon--power-off.icon--status-succeeded,.icon--power-off.icon--status-waiting{background:url("../assets/images/icons/power-off.svg") no-repeat;background-size:100% 100%}.icon.icon--power-on,.icon--power-on.icon--status-failed,.icon--power-on.icon--status-in-progress,.icon--power-on.icon--status-queued,.icon--power-on.icon--status-succeeded,.icon--power-on.icon--status-waiting{background:url("../assets/images/icons/power-on.svg") no-repeat;background-size:100% 100%}.icon.icon--remove,.icon--remove.icon--status-failed,.icon--remove.icon--status-in-progress,.icon--remove.icon--status-queued,.icon--remove.icon--status-succeeded,.icon--remove.icon--status-waiting{background:url("../assets/images/icons/remove.svg") no-repeat;background-size:100% 100%}.icon.icon--success,.icon--success.icon--status-failed,.icon--success.icon--status-in-progress,.icon--success.icon--status-queued,.icon--success.icon--status-succeeded,.icon--success.icon--status-waiting{background:url("../assets/images/icons/success.svg") no-repeat;background-size:100% 100%}.icon.icon--success-mono,.icon--success-mono.icon--status-failed,.icon--success-mono.icon--status-in-progress,.icon--success-mono.icon--status-queued,.icon--success-mono.icon--status-succeeded,.icon--success-mono.icon--status-waiting{background:url("../assets/images/icons/success-mono.svg") no-repeat;background-size:100% 100%}.icon.icon--success-grey,.icon--success-grey.icon--status-failed,.icon--success-grey.icon--status-in-progress,.icon--success-grey.icon--status-queued,.icon--success-grey.icon--status-succeeded,.icon--success-grey.icon--status-waiting{background:url('data:image/svg+xml;utf8, <svg width="16px" height="16px" viewBox="0 0 16 16" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g id="success"><rect id="rect6708" x="0" y="9.99999989e-06" width="16" height="16"></rect><circle id="circle6710" stroke="#888" stroke-width="1.5" fill="#888" cx="8" cy="8.00001" r="7.2500086"></circle><polygon id="path6712" fill="#fff" points="11.80029 4.92345 11.73439 4.98115 6.99964 9.12837 4.22407 6.74743 3.38501 7.69631 7.00033 11.50002 12.5 5.71278 11.80029 4.92349"></polygon></g></g></svg>');background-size:100% 100%}.icon.icon--settings,.icon--settings.icon--status-failed,.icon--settings.icon--status-in-progress,.icon--settings.icon--status-queued,.icon--settings.icon--status-succeeded,.icon--settings.icon--status-waiting{background:url("../assets/images/icons/settings.svg") no-repeat;background-size:100% 100%}.icon.icon--sync,.icon--sync.icon--status-failed,.icon--sync.icon--status-in-progress,.icon--sync.icon--status-queued,.icon--sync.icon--status-succeeded,.icon--sync.icon--status-waiting{background:url("../assets/images/icons/sync.svg") no-repeat;background-size:100% 100%}.icon.icon--search,.icon--search.icon--status-failed,.icon--search.icon--status-in-progress,.icon--search.icon--status-queued,.icon--search.icon--status-succeeded,.icon--search.icon--status-waiting{background:url("../assets/images/icons/magnifying_glass.svg") no-repeat;background-size:100% 100%}.icon.icon--system-shutdown,.icon--system-shutdown.icon--status-failed,.icon--system-shutdown.icon--status-in-progress,.icon--system-shutdown.icon--status-queued,.icon--system-shutdown.icon--status-succeeded,.icon--system-shutdown.icon--status-waiting{background:url("../assets/images/icons/system-shutdown.svg") no-repeat;background-size:100% 100%}.icon.icon--tooltip,.icon--tooltip.icon--status-failed,.icon--tooltip.icon--status-in-progress,.icon--tooltip.icon--status-queued,.icon--tooltip.icon--status-succeeded,.icon--tooltip.icon--status-waiting{background:url("../assets/images/icons/tooltip.svg") no-repeat;background-size:100% 100%}.icon.icon--tags,.icon--tags.icon--status-failed,.icon--tags.icon--status-in-progress,.icon--tags.icon--status-queued,.icon--tags.icon--status-succeeded,.icon--tags.icon--status-waiting{background:url("../assets/images/icons/tags.svg") no-repeat;background-size:100% 100%}.icon.icon--tick,.icon--tick.icon--status-failed,.icon--tick.icon--status-in-progress,.icon--tick.icon--status-queued,.icon--tick.icon--status-succeeded,.icon--tick.icon--status-waiting{background:url("../assets/images/icons/tick.svg") no-repeat;background-size:100% 100%}.icon.icon--logical-volume,.icon--logical-volume.icon--status-failed,.icon--logical-volume.icon--status-in-progress,.icon--logical-volume.icon--status-queued,.icon--logical-volume.icon--status-succeeded,.icon--logical-volume.icon--status-waiting{background:url("../assets/images/icons/logical-volume.svg") no-repeat;background-size:100% 100%}.icon.icon--warning,.icon--warning.icon--status-failed,.icon--warning.icon--status-in-progress,.icon--warning.icon--status-queued,.icon--warning.icon--status-succeeded,.icon--warning.icon--status-waiting{background:url("../assets/images/icons/warning.svg") no-repeat;background-size:100% 100%}.icon.icon--warning-mono,.icon--warning-mono.icon--status-failed,.icon--warning-mono.icon--status-in-progress,.icon--warning-mono.icon--status-queued,.icon--warning-mono.icon--status-succeeded,.icon--warning-mono.icon--status-waiting{background:url("../assets/images/icons/warning-mono.svg") no-repeat}.icon.icon--open,.icon--open.icon--status-failed,.icon--open.icon--status-in-progress,.icon--open.icon--status-queued,.icon--open.icon--status-succeeded,.icon--open.icon--status-waiting{background:url("../assets/images/forms/chevron-down.svg") no-repeat;background-size:10px 4px;background-position:center 4px}.icon.icon--close,.icon--close.icon--status-failed,.icon--close.icon--status-in-progress,.icon--close.icon--status-queued,.icon--close.icon--status-succeeded,.icon--close.icon--status-waiting{background:url("../assets/images/forms/chevron-up.svg") no-repeat;background-size:10px 4px;background-position:center 4px}.icon--status-failed{content:url('data:image/svg+xml;utf8, <svg width="16px" height="16px" viewBox="0 0 14 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g id="status-queued" transform="translate(1, 1)"><g id="Page-1"><g id="status-queued"><g id="Group"><circle id="Oval" stroke="#ed3146" stroke-width="2" cx="6" cy="6" r="6"></circle><circle id="Oval-Copy" class="status-circle" fill="#ed3146" cx="6" cy="6" r="4"></circle></g></g></g></g></g></svg>')}.icon--status-in-progress{content:url('data:image/svg+xml;utf8, <svg width="16px" height="16px" viewBox="0 0 14 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g id="status-queued" transform="translate(1, 1)"><g id="Page-1"><g id="status-queued"><g id="Group"><circle id="Oval" stroke="#19b6ee" stroke-width="2" cx="6" cy="6" r="6"></circle><circle id="Oval-Copy" class="status-circle" fill="#19b6ee" cx="6" cy="6" r="4"></circle></g></g></g></g></g></svg>')}.icon--status-queued{content:url('data:image/svg+xml;utf8, <svg width="16px" height="16px" viewBox="0 0 14 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g id="status-queued" transform="translate(1, 1)"><g id="Page-1"><g id="status-queued"><g id="Group"><circle id="Oval" stroke="#888" stroke-width="2" cx="6" cy="6" r="6"></circle><circle id="Oval-Copy" class="status-circle" fill="#888" cx="6" cy="6" r="4"></circle></g></g></g></g></g></svg>')}.icon--status-succeeded{content:url('data:image/svg+xml;utf8, <svg width="16px" height="16px" viewBox="0 0 14 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g id="status-queued" transform="translate(1, 1)"><g id="Page-1"><g id="status-queued"><g id="Group"><circle id="Oval" stroke="#3eb34f" stroke-width="2" cx="6" cy="6" r="6"></circle><circle id="Oval-Copy" class="status-circle" fill="#3eb34f" cx="6" cy="6" r="4"></circle></g></g></g></g></g></svg>')}.icon--status-waiting{content:url('data:image/svg+xml;utf8, <svg width="16px" height="16px" viewBox="0 0 14 14" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"><g id="Page-1" stroke="none" stroke-width="1" fill="none" fill-rule="evenodd"><g id="status-queued" transform="translate(1, 1)"><g id="Page-1"><g id="status-queued"><g id="Group"><circle id="Oval" stroke="#f99b0f" stroke-width="2" cx="6" cy="6" r="6"></circle><circle id="Oval-Copy" class="status-circle" fill="#f99b0f" cx="6" cy="6" r="4"></circle></g></g></g></g></g></svg>')}.icon--small{width:14px;height:14px}.icon--large{width:20px;height:20px}.icon--mega{width:23px;height:23px}.icon--gigantic{width:32px;height:32px}h1 .icon,h1 .icon--status-failed,h1 .icon--status-in-progress,h1 .icon--status-queued,h1 .icon--status-succeeded,h1 .icon--status-waiting{margin-top:-6px}h2 .icon,h2 .icon--status-failed,h2 .icon--status-in-progress,h2 .icon--status-queued,h2 .icon--status-succeeded,h2 .icon--status-waiting,h3 .icon,h3 .icon--status-failed,h3 .icon--status-in-progress,h3 .icon--status-queued,h3 .icon--status-succeeded,h3 .icon--status-waiting{margin-top:-5px}h4 .icon,h4 .icon--status-failed,h4 .icon--status-in-progress,h4 .icon--status-queued,h4 .icon--status-succeeded,h4 .icon--status-waiting,h5 .icon,h5 .icon--status-failed,h5 .icon--status-in-progress,h5 .icon--status-queued,h5 .icon--status-succeeded,h5 .icon--status-waiting,p .icon,p .icon--status-failed,p .icon--status-in-progress,p .icon--status-queued,p .icon--status-succeeded,p .icon--status-waiting,label .icon,label .icon--status-failed,label .icon--status-in-progress,label .icon--status-queued,label .icon--status-succeeded,label .icon--status-waiting,button .icon,button .icon--status-failed,button .icon--status-in-progress,button .icon--status-queued,button .icon--status-succeeded,button .icon--status-waiting,a .icon,a .icon--status-failed,a .icon--status-in-progress,a .icon--status-queued,a .icon--status-succeeded,a .icon--status-waiting,th .icon,th .icon--status-failed,th .icon--status-in-progress,th .icon--status-queued,th .icon--status-succeeded,th .icon--status-waiting,td .icon,td .icon--status-failed,td .icon--status-in-progress,td .icon--status-queued,td .icon--status-succeeded,td .icon--status-waiting,.table__header .icon,.table__header .icon--status-failed,.table__header .icon--status-in-progress,.table__header .icon--status-queued,.table__header .icon--status-succeeded,.table__header .icon--status-waiting,.table__data .icon,.table__data .icon--status-failed,.table__data .icon--status-in-progress,.table__data .icon--status-queued,.table__data .icon--status-succeeded,.table__data .icon--status-waiting{margin-top:-3px}.list__tree{list-style:none;border-left:1px solid #d2d2d2;position:relative}.list__tree.list__tree--sub-level{margin-top:10px;margin-left:20px;clear:both}.list__tree.list__tree--sub-level .list__item .list__item-feedback{left:180px}.list__tree .list__item{list-style:none}.list__tree .list__item:before{content:'';width:12px;height:1px;background:#d2d2d2;display:inline-block;position:relative;top:-4px;margin-right:5px}.list__tree .list__item:last-child::after{content:'';width:4px;height:1em;position:absolute;display:block;left:-2px;bottom:-6px;background:#fafafa}.list__tree .list__item .list__item-feedback{position:relative;left:200px;margin-top:-24px}table,.table{border-color:#d2d2d2;border-spacing:0;overflow-x:scroll;margin:0 0 20px;width:100%;text-align:left;border-collapse:separate}table tr,.table table tr,table .table tr,.table .table__row{width:100%;border-color:#b2b2b2;border-bottom-style:dotted;border-bottom-width:1px}@media only screen and (max-width: 768px){table tr,.table table tr,table .table tr,.table .table__row{background:#fff;border:0;border-radius:2px;box-sizing:border-box;box-shadow:0 1px 1px rgba(0,0,0,0.1);display:block;margin-bottom:20px;width:100%;padding:0 10px;float:left}}table th,.table table th,table .table th,table td,.table table td,table .table td,.table .table__header,.table .table__data{font-size:16px;font-size:1rem;padding:10px;box-sizing:border-box;min-height:21px;background:none;border:0;text-align:left;border-collapse:separate;vertical-align:top;backface-visibility:hidden;position:relative}@media only screen and (max-width: 768px){table th,.table table th,table .table th,table td,.table table td,table .table td,.table .table__header,.table .table__data{box-sizing:border-box;display:block;margin:0;min-height:45px;padding-left:50%;width:100% !important;text-align:right;position:relative;border-bottom:0 !important;border-top:0}table th label,.table table th label,table .table th label,table td label,.table table td label,table .table td label,.table .table__header label,.table .table__data label,table th dl,.table table th dl,table .table th dl,table td dl,.table table td dl,table .table td dl,.table .table__header dl,.table .table__data dl,table th dt,.table table th dt,table .table th dt,table td dt,.table table td dt,table .table td dt,.table .table__header dt,.table .table__data dt,table th p,.table table th p,table .table th p,table td p,.table table td p,table .table td p,.table .table__header p,.table .table__data p{text-align:left}table th:first-child,table td:first-child,.table .table__header:first-child,.table .table__data:first-child{border-top:0}table th::before,.table table th::before,table .table th::before,table td::before,.table table td::before,table .table td::before,.table .table__header::before,.table .table__data::before{content:attr(aria-label);display:block;width:50%;height:100%;font-size:16px;color:#888;position:absolute;top:10px;left:10px;text-align:left !important}table th.table--mobile-controls,table td.table--mobile-controls,.table .table--mobile-controls.table__header,.table .table--mobile-controls.table__data{padding-left:10px;width:100% !important;text-align:center !important}}@media only screen and (max-width: 768px) and (max-width: 768px){table th.table--mobile-controls::before,table td.table--mobile-controls::before,.table .table--mobile-controls.table__header::before,.table .table--mobile-controls.table__data::before{display:none}}table thead tr,.table table thead tr,table thead .table tr,.table .table__head .table__row{color:#888;border-bottom:1px solid}table thead tr:hover,.table .table__head .table__row:hover{background-color:transparent}@media only screen and (max-width: 768px){table thead tr,.table table thead tr,table thead .table tr,.table .table__head .table__row{width:100%;border:0;display:block}table thead tr:hover,.table .table__head .table__row:hover{background-color:#fff}}table thead th,.table table thead th,table thead .table th,.table .table__head .table__header{font-size:13px;font-size:0.8125rem;background:none;color:#888;font-size:13px;display:table-cell}@media only screen and (max-width: 768px){table thead th,.table table thead th,table thead .table th,.table .table__head .table__header{display:none;width:100%}table thead th::before,.table table thead th::before,table thead .table th::before,.table .table__head .table__header::before{content:attr(aria-label);display:block;width:50%;height:100%;font-size:16px;color:#333;position:absolute;top:10px;left:10px;text-align:left !important;font-weight:300}table thead th.u-display--medium,.table .table__head .u-display--medium.table__header{display:block;border-bottom:0}}table thead th input[type="radio"]+label,.table .table__head .table__header input[type="radio"]+label,table thead th input[type="checkbox"]+label,.table .table__head .table__header input[type="checkbox"]+label{margin:0;top:0px;min-height:16px}table thead th input[type="radio"]+label:before,.table .table__head .table__header input[type="radio"]+label:before,table thead th input[type="checkbox"]+label:before,.table .table__head .table__header input[type="checkbox"]+label:before{top:0}@media only screen and (max-width: 768px){table thead th input[type="radio"]+label:before,.table .table__head .table__header input[type="radio"]+label:before,table thead th input[type="checkbox"]+label:before,.table .table__head .table__header input[type="checkbox"]+label:before{top:2px}}table thead th a:link,table thead th a:visited,.table .table__head .table__header .table__header-link{color:#888;border-bottom:0}table thead th a:hover:link,table thead th a:hover:visited,.table .table__head .table__header .table__header-link:hover{color:#333;text-decoration:none}table thead th a.is-active:link,table thead th a.is-active:visited,.table .table__head .table__header .is-active.table__header-link{color:#333;text-decoration:none}table thead th a.is-sorted:link,table thead th a.is-sorted:visited,.table .table__head .table__header .is-sorted.table__header-link{border-bottom:1px solid #333}table thead .divide,.table .table__head .table__header-divide,.table .table__head .divide{width:1px;display:inline-block;background:#888;height:10px;margin:0 5px}table .numerical,.table .numerical{text-align:right}table input[type="radio"]+label,.table input[type="radio"]+label,table input[type="checkbox"]+label,.table input[type="checkbox"]+label{margin:0;top:0px;min-height:16px}table input[type="radio"]+label:before,.table input[type="radio"]+label:before,table input[type="checkbox"]+label:before,.table input[type="checkbox"]+label:before{top:0}table th{color:#888;border-bottom:1px solid}table td{border-color:#b2b2b2;border-bottom-style:dotted;border-bottom-width:1px}table td a:link,table td a:visited{color:#333;border-bottom:1px solid #c2c2c2}table td a:link:hover,table td a:visited:hover{text-decoration:none;color:#dd4814}table thead{width:100%;display:table-header-group}@media only screen and (max-width: 768px){table thead{display:block;width:100%}}table thead.table__head--sticky{background:#fff;padding:0;width:100vw !important;left:0 !important;right:0 !important;height:auto}table thead.table__head--sticky tr{max-width:1440px;width:100%;margin:0 auto;display:block;float:none;border-bottom:0;overflow:hidden}table thead.table__head--sticky tr th{border-bottom:0;float:left}@media only screen and (max-width: 768px){table tbody{width:100%;display:block}}.table{display:table}@media only screen and (max-width: 768px){.table{display:block}}.table .table__row{float:left;display:table-row}.table .table__row:hover{background-color:#fff}.table .table__row:hover .table__controls{z-index:1;opacity:1}.table .table__row:hover .table__controls--secondary{z-index:1;opacity:1}.table .table__row.is-active{background-color:#fff}.table .table__row.is-active .table__controls{z-index:1;opacity:1}.table .table__row.is-active .table__controls--secondary{z-index:1;opacity:1}.table .table__row.is-active .table__dropdown .table__row{display:none}.table .table__row.is-active .table__dropdown .table__row.is-active{display:block}.table .table__header,.table .table__data{display:table-cell;float:left}@media only screen and (max-width: 768px){.table .table__data{border-color:#b2b2b2;border-bottom-style:dotted;border-bottom-width:1px;padding-left:50%}}.table .table__data a:link,.table .table__data a:visited{color:#333;border-bottom:1px solid #c2c2c2}.table .table__data a:link:hover,.table .table__data a:visited:hover{text-decoration:none;color:#dd4814}.table .table__head{display:table-head;width:100%;box-sizing:border-box;width:auto !important}.table .table__head.table__head--sticky{background:#fff;padding:0;width:100vw !important;left:0 !important;right:0 !important;border-top:1px dotted #d2d2d2;box-shadow:0 1px 1px rgba(0,0,0,0.1);height:auto !important}.table .table__head.table__head--sticky .table__row{max-width:1440px;width:100%;margin:0 auto;display:block;float:none;border-bottom:0;overflow:hidden}.table .table__body{display:table-row-group;float:left;width:100%}.table .table__footer{display:table-footer-group}.table .table__label{clear:both;display:block;margin-top:5px;line-height:16px;color:#bcbcbc}.table .table__label a{color:#bcbcbc}.table .table__label a:hover{color:#f7f7f7}.table .table__label.is-active a{color:#f7f7f7}.table .table__controls{width:100%;text-align:right;opacity:0;z-index:-1000}@media only screen and (max-width: 768px){.table .table__controls{opacity:1;z-index:1;text-align:center}.table .table__controls .icon,.table .table__controls .icon--status-failed,.table .table__controls .icon--status-in-progress,.table .table__controls .icon--status-queued,.table .table__controls .icon--status-succeeded,.table .table__controls .icon--status-waiting{transform:scale(1.5, 1.5);margin:20px !important}}.table .table__controls--secondary{opacity:0;z-index:-1000;width:auto;text-align:left}@media only screen and (max-width: 768px){.table .table__controls--secondary{opacity:1;z-index:1;text-align:center}}.table .table__controls a,.table .table__controls a:link,.table .table__controls a:visited{color:#333;border-bottom:1px solid #c2c2c2}.table .table__controls a:hover,.table .table__controls a:link:hover,.table .table__controls a:visited:hover{text-decoration:none;color:#dd4814}.table .table__input{display:inline-block;background-color:#fff;border-color:#d2d2d2;background-position:right 10px top 16px;margin:-7px 0}.table .table__input[disabled]{background-color:transparent;border-color:transparent;pointer-events:none;background-position:-9999px -9999px;color:#333}.table .table__dropdown{width:100%}@media only screen and (max-width: 768px){.table .table__dropdown{background:#fff;border:0;box-sizing:border-box;top:0;right:0;bottom:0;left:0;position:fixed;z-index:1000;overflow-y:scroll;width:100% !important;padding-top:74px;padding-bottom:10px}}.table .table__dropdown .table__dropdown-title{display:none}@media only screen and (max-width: 768px){.table .table__dropdown .table__dropdown-title{width:100%;float:left;display:block;box-shadow:0 1px 4px rgba(0,0,0,0.1);padding:15px 10px;margin:0;background:#fff;position:fixed;z-index:10;top:0}.table .table__dropdown .table__dropdown-title>h2{margin:0}}.table .table__dropdown .table__row{border-bottom:0;display:none;position:relative}@media only screen and (max-width: 768px){.table .table__dropdown .table__row{box-shadow:none}}.table .table__dropdown .table__row--indent{padding-left:41px;float:left;width:100%;box-sizing:border-box}@media only screen and (max-width: 768px){.table .table__dropdown .table__row--indent{padding-left:0}}.table .table__dropdown .table__row:before{display:block;margin:0 auto;width:calc(100% - 20px);border-top:1px dotted #d2d2d2;position:absolute;height:1px;content:'';top:0;left:10px}.table .table__dropdown .table__row.table__row--head{border-bottom:0}.table .table__dropdown .table__row.table__row--head .table__header{color:#bcbcbc;font-size:13px}.table .table__dropdown .table__row.u-border--none:before{display:none}.table .table__dropdown .table__row.u-border:before{display:block;margin:0 auto;width:calc(100% - 20px);border-top:1px dotted #d2d2d2;position:absolute;height:1px;content:'';top:0;left:10px}@media only screen and (max-width: 768px){.table .table__dropdown .table__row.is-active{padding:0 !important;display:block}}@media only screen and (max-width: 768px){.table .table__dropdown .table__row.is-active .table__data{padding:10px 10px 0;border-top:0}.table .table__dropdown .table__row.is-active .table__data:before{display:none}.table .table__dropdown .table__row.is-active .table__data .six-col{margin-bottom:10px}.table .table__dropdown .table__row.is-active .table__data .six-col:last-child{margin-bottom:0}}@media only screen and (max-width: 768px){.table .table__dropdown .table__row.is-active .table__data--children{padding:10px 10px 10px 50% !important;border-top:1px dotted #d2d2d2}.table .table__dropdown .table__row.is-active .table__data--children:before{display:block}}.table .table__dropdown .table__row.is-active .table__input{background-color:#fff;border-color:#d2d2d2;background-position:right 10px top 16px;pointer-events:all}.table .table__dropdown .table__row.is-active .table__input[disabled]{cursor:pointer}@media only screen and (max-width: 768px){.table .table__dropdown .table__row.is-active{margin-bottom:0}.table .table__dropdown .table__row.is-active::before{display:none}}@media only screen and (max-width: 768px) and (max-width: 768px){.table .table__dropdown .table__row.is-active.table__dropdown-controls{top:inherit !important;margin:0;z-index:2000;padding-bottom:0;height:auto !important;bottom:0 !important}.table .table__dropdown .table__row.is-active.table__dropdown-controls [class^="button--"]{width:48%;float:left;margin:0 1%;height:auto;overflow:hidden;white-space:normal;line-height:1.5}.table .table__dropdown .table__row.is-active.table__dropdown-controls [class^="button--"]+[class^="button--"]{width:48%;float:left;margin:0 1%}.table .table__dropdown .table__row.is-active.table__dropdown-controls.u-position--top{top:20px}.table .table__dropdown .table__row.is-active.table__dropdown-controls.u-remove--shadow{box-shadow:none !important}.table .table__dropdown .table__row.is-active.table__dropdown-controls .table__data{min-height:auto}.table .table__dropdown .table__row.is-active.table__dropdown-controls .table__data::before{display:none}}.table .table__dropdown--info .table__row{border-bottom:0}.table .table__dropdown--info .table__data{color:#bcbcbc}.form .form__group input,.form .form__group select{margin:0}.table--error{border-color:#df382c}.table--error .table__header,.table--error .table__data,.table--error th,.table--error td{border-color:#df382c;background-color:#fadfdd}.table--warning{border-color:#eca918}.table--warning .table__header,.table--warning .table__data,.table--warning th,.table--warning td{border-color:#eca918;background-color:#fcefd4}.table--success{border-color:#38b44a}.table--success .table__header,.table--success .table__data,.table--success th,.table--success td{border-color:#38b44a;background-color:#caeecf}.table--information{border-color:#19b6ee}.table--information .table__header,.table--information .table__data,.table--information th,.table--information td{border-color:#19b6ee;background-color:#d7f2fc}.table-col--1{width:1%}.table-col--2{width:2%}.table-col--3{width:3%}.table-col--4{width:4%}.table-col--5{width:5%}.table-col--6{width:6%}.table-col--7{width:7%}.table-col--8{width:8%}.table-col--9{width:9%}.table-col--10{width:10%}.table-col--11{width:11%}.table-col--12{width:12%}.table-col--13{width:13%}.table-col--14{width:14%}.table-col--15{width:15%}.table-col--16{width:16%}.table-col--17{width:17%}.table-col--18{width:18%}.table-col--19{width:19%}.table-col--20{width:20%}.table-col--21{width:21%}.table-col--22{width:22%}.table-col--23{width:23%}.table-col--24{width:24%}.table-col--25{width:25%}.table-col--26{width:26%}.table-col--27{width:27%}.table-col--28{width:28%}.table-col--29{width:29%}.table-col--30{width:30%}.table-col--31{width:31%}.table-col--32{width:32%}.table-col--33{width:33%}.table-col--34{width:34%}.table-col--35{width:35%}.table-col--36{width:36%}.table-col--37{width:37%}.table-col--38{width:38%}.table-col--39{width:39%}.table-col--40{width:40%}.table-col--41{width:41%}.table-col--42{width:42%}.table-col--43{width:43%}.table-col--44{width:44%}.table-col--45{width:45%}.table-col--46{width:46%}.table-col--47{width:47%}.table-col--48{width:48%}.table-col--49{width:49%}.table-col--50{width:50%}.table-col--51{width:51%}.table-col--52{width:52%}.table-col--53{width:53%}.table-col--54{width:54%}.table-col--55{width:55%}.table-col--56{width:56%}.table-col--57{width:57%}.table-col--58{width:58%}.table-col--59{width:59%}.table-col--60{width:60%}.table-col--61{width:61%}.table-col--62{width:62%}.table-col--63{width:63%}.table-col--64{width:64%}.table-col--65{width:65%}.table-col--66{width:66%}.table-col--67{width:67%}.table-col--68{width:68%}.table-col--69{width:69%}.table-col--70{width:70%}.table-col--71{width:71%}.table-col--72{width:72%}.table-col--73{width:73%}.table-col--74{width:74%}.table-col--75{width:75%}.table-col--76{width:76%}.table-col--77{width:77%}.table-col--78{width:78%}.table-col--79{width:79%}.table-col--80{width:80%}.table-col--81{width:81%}.table-col--82{width:82%}.table-col--83{width:83%}.table-col--84{width:84%}.table-col--85{width:85%}.table-col--86{width:86%}.table-col--87{width:87%}.table-col--88{width:88%}.table-col--89{width:89%}.table-col--90{width:90%}.table-col--91{width:91%}.table-col--92{width:92%}.table-col--93{width:93%}.table-col--94{width:94%}.table-col--95{width:95%}.table-col--96{width:96%}.table-col--97{width:97%}.table-col--98{width:98%}.table-col--99{width:99%}.table-col--100{width:100%}.tabs{display:inline-block}@media only screen and (max-width: 768px){.tabs{margin:0 0 20px;width:100%}}.tabs .tabs__tab{font-size:18px;font-size:1.125rem;display:inline-block;list-style-type:none;margin:0}.tabs .tabs__tab .tabs__tab-link{color:#888;text-decoration:none;border-bottom:0}.tabs .tabs__tab:hover .tabs__tab-link{border-bottom:3px solid #888}.tabs .tabs__tab:after{content:'';width:1px;display:inline-block;background:#d2d2d2;height:11px;padding:0;margin:0 5px}.tabs .tabs__tab:last-child::after{display:none}.tabs .is-active .tabs__tab-link,.tabs .is-active:hover .tabs__tab-link{border-bottom:3px solid #dd4814}.tooltip{position:relative}.tooltip::before{position:absolute;left:50%;transform:translateX(-50%) translateY(-14px);content:'';width:0;height:0;border-left:5px solid transparent;border-right:5px solid transparent;z-index:-1;border-top:5px solid #333;opacity:0;visibility:hidden;bottom:100%;margin-bottom:-11px}.tooltip::after{content:attr(aria-label);font-size:13px;font-weight:400;line-height:16px;position:absolute;z-index:-1;left:50%;bottom:100%;transform:translateX(-50%) translateY(-8px);background:#333;color:#fff;padding:10px;height:auto;text-indent:0;opacity:0;visibility:hidden;border-radius:5px;box-shadow:0 1px 3px 0 rgba(51,51,51,0.2);white-space:pre;box-sizing:border-box;font-style:normal}.tooltip:hover::after,.tooltip:hover::before{opacity:1 !important;z-index:1000;visibility:visible}@media only screen and (max-width: 768px){.tooltip:hover::after,.tooltip:hover::before{display:none}}.tooltip.tooltip--right::before{border-top:5px solid transparent;border-right:5px solid #333;border-bottom:5px solid transparent;left:100%;bottom:inherit;top:50%;transform:translateX(4px) translateY(-50%)}.tooltip.tooltip--right::after{left:100%;bottom:inherit;top:50%;margin-bottom:-14px;transform:translateX(14px) translateY(-50%)}.tooltip.tooltip--bottom::before{border-top:0;border-right:5px solid transparent;border-bottom:5px solid #333;border-left:5px solid transparent;left:50%;bottom:inherit;top:100%;transform:translateX(-50%) translateY(8px)}.tooltip.tooltip--bottom::after{left:50%;bottom:inherit;top:100%;margin-bottom:0;transform:translateX(-50%) translateY(13px)}.tooltip.tooltip--left::before{border-top:5px solid transparent;border-left:5px solid #333;border-bottom:5px solid transparent;left:-4px;bottom:inherit;top:50%;transform:translateX(-100%) translateY(-50%)}.tooltip.tooltip--left::after{left:-14px;bottom:inherit;top:50%;margin-bottom:-14px;transform:translateX(-100%) translateY(-50%)}.p-mobile-nav{display:none}@media only screen and (max-width: 768px){.p-mobile-nav{display:block;width:100%}.p-mobile-nav__link{width:100%;float:left;display:block;padding:15px 10px;margin:0;background:#fff;z-index:10;top:0}}.accordion{box-sizing:border-box;border-radius:2px;list-style:none;background:#fff;box-shadow:0 1px 1px rgba(0,0,0,0.1);margin-bottom:40px}@media only screen and (max-width: 768px){.accordion{margin-bottom:0}}.accordion.is-disabled{opacity:.5;pointer-events:none}.accordion .accordion__title{font-size:20px;font-size:1.25rem;border-bottom:1px dotted #b2b2b2;padding:8px 20px 9px;margin:0}@media only screen and (max-width: 768px){.accordion .accordion__title{border-bottom:0;background:transparent url("../assets/images/forms/chevron_down.svg") top 20px right 20px no-repeat}.accordion .accordion__title.is-active{border-bottom:1px dotted #b2b2b2;background-image:url("../assets/images/icons/cross.svg")}.accordion .accordion__title.is-active+.accordion__content{display:block}}@media only screen and (max-width: 768px){.accordion .accordion__content{display:none}}.accordion .accordion__tab{border-bottom:1px dotted #b2b2b2}.accordion .accordion__tab:last-of-type{border:0}.accordion .accordion__tab .accordion__tab-title.is-active{background-image:url("../assets/images/forms/chevron_up.svg")}.accordion .accordion__tab .accordion__tab-title.is-active+.accordion__tab-content{max-height:400px;transition:max-height .5s ease-in}@media only screen and (max-width: 768px){.accordion .accordion__tab .accordion__tab-title.is-active+.accordion__tab-content{max-height:150px}}.accordion .accordion__tab .accordion__tab-content{max-height:0;transition:max-height .5s ease-out;overflow:hidden;overflow:auto}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list{list-style-type:none;padding:0 20px 14px;margin:0}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item{margin-bottom:.15em}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item .accordion__tab-link{box-sizing:border-box;color:#333;width:100%;display:inline-block;padding-right:20px;border-bottom:0;outline:0}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item .accordion__tab-link:hover{color:#dd4814;text-decoration:none}.disabled .accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item .accordion__tab-link{color:#333}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item.is-active{margin-bottom:.15em;font-weight:400}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item.is-active .accordion__tab-link{background:transparent url("../assets/images/icons/cross.svg") top 7px right 0 no-repeat;text-decoration:none}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item.is-active:hover{color:#f7f7f7}.accordion .accordion__tab .accordion__tab-content .accordion__tab-list .accordion__tab-item.is-active:hover .accordion__tab-link{color:#dd4814;background-image:url("../assets/images/icons/cross_orange.svg")}.accordion .accordion__tab .accordion__tab-title,.accordion .accordion__tab .accordion__tab-title.is-active{padding:12px 20px;margin:0;color:#888;cursor:pointer;background:transparent url("../assets/images/forms/chevron_down.svg") top 20px right 20px no-repeat}.action-card{background:#fff;box-shadow:0 1px 1px rgba(0,0,0,0.1);margin-bottom:40px;padding:20px 20px 10px;box-sizing:border-box;width:100%;display:inline-block;clear:both}.action-card .action-card__title{font-size:23px;margin-bottom:20px}.action-card .action-card__controls{border-top:1px dotted #888;width:100%;display:inline-block;margin:10px 0 0;padding:10px 0 0;box-sizing:border-box}.banner{background-color:#000}.banner .nav-primary .logo a{border-bottom:0}.banner .nav-primary .logo a svg,.banner .nav-primary .logo a img{vertical-align:middle}.banner .nav-toggle a{border-bottom:0}@media only screen and (min-width: 620px){.banner{box-shadow:none}.banner .nav-primary{max-width:none}.banner .nav-primary .logo{height:auto;margin-top:12px}.banner .nav-primary .logo a{border-bottom:0}.banner .nav-primary ul li{border-left:1px solid #333}.banner .nav-primary ul li:last-child{border-right:1px solid #333;border-left:1px solid #333}.banner .nav-primary ul li a,.banner .nav-primary ul li a:link,.banner .nav-primary ul li a:visited{box-sizing:border-box;border-bottom:0;border-left:0}.banner .nav-primary ul li a:hover,.banner .nav-primary ul li a:link:hover,.banner .nav-primary ul li a:visited:hover{background-color:rgba(51,51,51,0.9)}.banner .nav-primary ul li a.active,.banner .nav-primary ul li a:link.active,.banner .nav-primary ul li a:visited.active{background-color:transparent;box-shadow:inset 0px -3px #dd4814}.banner .nav-primary ul li a.active:hover,.banner .nav-primary ul li a:link.active:hover,.banner .nav-primary ul li a:visited.active:hover{background-color:rgba(51,51,51,0.9)}}.page-header{box-sizing:border-box;background:#fff;box-shadow:0 1px 1px rgba(0,0,0,0.1);padding:0 20px;height:auto !important;width:100%;float:left;margin-bottom:20px;overflow:visible}@media only screen and (max-width: 768px){.page-header{padding:0 10px;width:100%}}.page-header .page-header__title{font-size:32px;font-size:2rem;margin:0;padding:34px 0;display:inline-block}@media only screen and (max-width: 768px){.page-header .page-header__title{font-size:1.5rem;padding:20px 0;width:100%}}.page-header .page-header__title [contenteditable="true"]{font-size:32px;font-size:2rem;padding:8px 10px;width:auto;box-sizing:border-box;border:1px solid transparent;margin:-20px 0 -20px -10px;border-radius:2px;color:#333;cursor:default;display:inline-block}@media only screen and (max-width: 768px){.page-header .page-header__title [contenteditable="true"]{border:1px solid #D2D2D2;margin:0;width:100%;font-size:1.5rem}}.page-header .page-header__title [contenteditable="true"].editmode,.page-header .page-header__title [contenteditable="true"].editable:hover{border:1px solid #D2D2D2;cursor:text}.page-header .page-header__title [contenteditable="true"].editmode:hover,.page-header .page-header__title [contenteditable="true"]:active,.page-header .page-header__title [contenteditable="true"]:focus{outline:none;background-color:#fff;border:1px solid #B2B2B2}.page-header .page-header__title [contenteditable="true"].invalid,.page-header .page-header__title [contenteditable="true"].invalid:hover,.page-header .page-header__title [contenteditable="true"].invalid:active,.page-header .page-header__title [contenteditable="true"].invalid:focus{border-color:#df382c}.page-header .page-header__title .page-header__title-dot{display:inline-block;width:auto;padding:0}@media only screen and (max-width: 768px){.page-header .page-header__title .page-header__title-dot{display:none}}.page-header .page-header__title .page-header__title-domain{font-size:32px;font-size:2rem;display:inline-block;width:auto;min-height:66px;max-height:66px;line-height:25px;margin:-20px 0;background-position:top 32px right 10px;padding:13px 10px !important}@media only screen and (max-width: 768px){.page-header .page-header__title .page-header__title-domain{width:100%;margin:20px 0 0;font-size:1.5rem}}.page-header .page-header__status{font-size:19px;font-size:1.1875rem;width:auto;display:inline-block;position:relative;margin-left:20px;color:#888}@media only screen and (max-width: 768px){.page-header .page-header__status{margin:0 0 20px;width:100%}}@media only screen and (max-width: 768px){.page-header .page-header__status .page-header__status-text{margin:0 !important;width:50%;display:inline-block}}.page-header .page-header__status .page-header__status-check{font-size:16px;font-size:1rem;margin-left:10px;color:inherit;border:0}.page-header .page-header__status .page-header__status-check:hover{text-decoration:underline}@media only screen and (max-width: 768px){.page-header .page-header__status .page-header__status-check{margin:0;width:50%;display:inline-block}}.page-header .page-header__controls{padding:40px 0;display:inline-block;float:right}@media only screen and (max-width: 768px){.page-header .page-header__controls{padding:0 0 20px;width:100%}.page-header .page-header__controls [class^="button--"]{margin-top:10px}.page-header .page-header__controls [class^="button--"]:first-child{margin-top:0}}.page-header .page-header__controls .page-header__controls-feedback{color:#333;text-decoration:none;border-bottom:1px solid #c2c2c2}.page-header .page-header__controls .page-header__controls-feedback:hover{text-decoration:none;color:#dd4814;cursor:pointer}@media only screen and (max-width: 768px){.page-header .page-header__controls .page-header__controls-feedback{margin-bottom:20px;display:inline-block;clear:both}}.page-header .page-header__dropdown{max-height:0;overflow:hidden;-webkit-transition:max-height 0.3s ease;-moz-transition:max-height 0.3s ease;transition:max-height 0.3s ease;width:100%}.page-header .page-header__dropdown.is-open{max-height:1000px}.page-header .page-header__dropdown .page-header__section{border-top:1px dotted #888;padding-top:20px;padding-bottom:20px}.page-header .page-header__dropdown .page-header__message{width:auto;display:inline-block;float:none;margin:0 20px 0 0;line-height:36px}.page-header .page-header__dropdown .page-header__message--error{background:url("../assets/images/icons/error.svg") no-repeat;background-size:16px 16px;background-position:0 10px;padding-left:25px}.page-header .page-header__dropdown .page-header__message--info{background:url("../assets/images/icons/info.svg") no-repeat;background-size:16px 16px;background-position:0 10px;padding-left:25px}.page-header .page-header__dropdown .page-header__message--warning{background:url("../assets/images/icons/warning.svg") no-repeat;background-size:16px 16px;background-position:0 10px;padding-left:25px}.page-header .page-header__dropdown .page-header__message--success{background:url("../assets/images/icons/success.svg") no-repeat;background-size:16px 16px;background-position:0 10px;padding-left:25px}.page-header .page-header__dropdown .page-header__controls{padding:0;float:right}.search{position:relative}.search .search__input{font-size:16px;font-size:1rem;box-sizing:border-box;border-radius:4px;list-style:none;background:#fff;box-shadow:0 1px 1px rgba(0,0,0,0.1);width:100%;border:0;padding:15px;appearance:textfield}.search .search__input::placeholder{color:#888}.search .search__input::-ms-clear{display:none}.search .search__input::-webkit-search-decoration,.search .search__input::-webkit-search-cancel-button,.search .search__input::-webkit-search-results-button,.search .search__input::-webkit-search-results-decoration{display:none}.search .search__input:disabled,.search .search__input.search--disabled{background-color:#fff;opacity:.5;pointer-events:none}.search .search__input:disabled+.search__submit,.search .search__input.search--disabled+.search__submit{pointer-events:none;opacity:.5}.search .search__submit{position:absolute;top:15px;right:15px;background-color:transparent;background-image:url("../assets/images/icons/magnifying_glass.svg");background-position:center;background-repeat:no-repeat;background-size:20px;text-indent:-999em;display:block;width:20px;height:20px;overflow:hidden;outline:none;padding:0;border:0}.search .search__submit:hover{background-color:transparent;background-image:url("../assets/images/icons/magnifying_glass.svg")}.search .search__submit.search__submit--close{background-image:url("../assets/images/icons/remove.svg");background-size:16px}.u-animation--spin{animation:spin 1s infinite linear !important}.u-animation--pulse{animation:pulse 1s ease-out !important;animation-iteration-count:infinite !important;opacity:0}@keyframes spin{0%{transform:rotate(0deg)}100%{transform:rotate(360deg)}}@keyframes pulse{0%{transform:scale(0.1, 0.1);opacity:0}50%{opacity:1}100%{transform:scale(1.2, 1.2);opacity:0}}.u-border{border:1px dotted #cdcdcd !important}.u-border--top{border-top:1px dotted #cdcdcd !important}.u-border--right{border-right:1px dotted #cdcdcd !important}.u-border--bottom{border-bottom:1px dotted #cdcdcd !important}.u-border--left{border-left:1px dotted #cdcdcd !important}.u-border--dotted{border-style:dotted !important}.u-border--solid{border-style:solid !important}.u-border--dashed{border-style:dashed !important}.u-border--none{border:0 !important}@media only screen and (min-width: 768px){.u-align--right{text-align:right !important}.u-align--left{text-align:left !important}.u-align--center{text-align:center !important}}.u-position--relative{position:relative !important}.u-position--fixed{position:fixed !important}.u-position--absolute{position:absolute !important}.u-float--right{float:right !important}.u-float--left{float:left !important}.u-float--none{float:none !important}.u-muted{opacity:.5 !important;filter:alpha(opacity=50) !important}.u-clear{clear:both !important}.u-display--block{display:block !important}@media only screen and (max-width: 768px){.u-display--block-mobile{display:block !important}}.u-display--inline{display:inline !important}.u-display--inline-block{display:inline-block !important}.u-display--hidden{display:none !important}@media only screen and (max-width: 768px){.u-display--desktop{display:none !important;position:absolute !important;visibility:hidden !important}}.u-display--mobile{display:none !important}@media only screen and (max-width: 768px){.u-display--mobile{display:block !important}}.u-width--full{width:100% !important}.u-width--half{width:50% !important}@media only screen and (max-width: 768px){.u-width--half{width:100% !important}}.u-width--third{width:calc(100%/3) !important}@media only screen and (max-width: 768px){.u-width--third{width:100% !important}}.u-width--quater{width:25% !important}@media only screen and (max-width: 768px){.u-width--quater{width:100% !important}}.u-scroll--none{overflow:hidden}.u-mobile--offscreen{left:-100vw;display:none !important}.u-margin{margin:20px}.u-margin--none{margin:0 !important}.u-margin--tiny{margin:5px !important}.u-margin--small{margin:10px !important}.u-margin--medium{margin:30px !important}.u-margin--large{margin:40px !important}.u-margin--huge{margin:80px !important}.u-margin--top{margin-top:20px !important}.u-margin--top-none{margin-top:0 !important}.u-margin--top-tiny{margin-top:5px !important}.u-margin--top-small{margin-top:10px !important}.u-margin--top-medium{margin-top:30px !important}.u-margin--top-large{margin-top:40px !important}.u-margin--top-huge{margin-top:80px !important}.u-margin--right{margin-right:20px !important}.u-margin--right-none{margin-right:0 !important}.u-margin--right-tiny{margin-right:5px !important}.u-margin--right-small{margin-right:10px !important}.u-margin--right-medium{margin-right:30px !important}.u-margin--right-large{margin-right:40px !important}.u-margin--right-huge{margin-right:80px !important}.u-margin--bottom{margin-bottom:20px !important}.u-margin--bottom-none{margin-bottom:0 !important}.u-margin--bottom-tiny{margin-bottom:5px !important}.u-margin--bottom-small{margin-bottom:10px !important}.u-margin--bottom-medium{margin-bottom:30px !important}.u-margin--bottom-large{margin-bottom:40px !important}.u-margin--bottom-huge{margin-bottom:80px !important}.u-margin--left{margin-left:20px !important}.u-margin--left-none{margin-left:0 !important}.u-margin--left-tiny{margin-left:5px !important}.u-margin--left-small{margin-left:10px !important}.u-margin--left-medium{margin-left:30px !important}.u-margin--left-large{margin-left:40px !important}.u-margin--left-huge{margin-left:80px !important}@media only screen and (max-width: 768px){.u-mobile-margin--bottom{margin-bottom:20px !important}}.u-padding{padding:20px}.u-padding--none{padding:0 !important}.u-padding--tiny{padding:5px !important}.u-padding--small{padding:10px !important}.u-padding--medium{padding:30px !important}.u-padding--large{padding:40px !important}.u-padding--huge{padding:80px !important}.u-padding--top{padding-top:20px !important}.u-padding--top-none{padding-top:0 !important}.u-padding--top-tiny{padding-top:5px !important}.u-padding--top-small{padding-top:10px !important}.u-padding--top-medium{padding-top:30px !important}.u-padding--top-large{padding-top:40px !important}.u-padding--top-huge{padding-top:80px !important}.u-padding--right{padding-right:20px !important}.u-padding--right-none{padding-right:0 !important}.u-padding--right-tiny{padding-right:5px !important}.u-padding--right-small{padding-right:10px !important}.u-padding--right-medium{padding-right:30px !important}.u-padding--right-large{padding-right:40px !important}.u-padding--right-huge{padding-right:80px !important}.u-padding--bottom{padding-bottom:20px !important}.u-padding--bottom-none{padding-bottom:0 !important}.u-padding--bottom-tiny{padding-bottom:5px !important}.u-padding--bottom-small{padding-bottom:10px !important}.u-padding--bottom-medium{padding-bottom:30px !important}.u-padding--bottom-large{padding-bottom:40px !important}.u-padding--bottom-huge{padding-bottom:80px !important}.u-padding--left{padding-left:20px !important}.u-padding--left-none{padding-left:0 !important}.u-padding--left-tiny{padding-left:5px !important}.u-padding--left-small{padding-left:10px !important}.u-padding--left-medium{padding-left:30px !important}.u-padding--left-large{padding-left:40px !important}.u-padding--left-huge{padding-left:80px !important}@media only screen and (max-width: 768px){.u-mobile-margin--bottom{margin-bottom:20px !important}}.u-text--subtle{color:#888;font-size:.9375rem}.u-text--error{color:#df382c !important}.u-border--error{border-color:#df382c !important}.u-background--error{background-color:#df382c !important}.has-error{border:1px solid #df382c !important}.has-error:focus{border:1px solid #df382c !important}.u-text--off{color:#d2d2d2 !important}.u-border--off{border-color:#d2d2d2 !important}.u-background--off{background-color:#d2d2d2 !important}.has-off{border:1px solid #d2d2d2 !important}.has-off:focus{border:1px solid #d2d2d2 !important}.u-text--warning{color:#eca918 !important}.u-border--warning{border-color:#eca918 !important}.u-background--warning{background-color:#eca918 !important}.has-warning{border:1px solid #eca918 !important}.has-warning:focus{border:1px solid #eca918 !important}.u-text--success{color:#38b44a !important}.u-border--success{border-color:#38b44a !important}.u-background--success{background-color:#38b44a !important}.has-success{border:1px solid #38b44a !important}.has-success:focus{border:1px solid #38b44a !important}.u-text--on{color:#38b44a !important}.u-border--on{border-color:#38b44a !important}.u-background--on{background-color:#38b44a !important}.has-on{border:1px solid #38b44a !important}.has-on:focus{border:1px solid #38b44a !important}.u-text--information{color:#19b6ee !important}.u-border--information{border-color:#19b6ee !important}.u-background--information{background-color:#19b6ee !important}.has-information{border:1px solid #19b6ee !important}.has-information:focus{border:1px solid #19b6ee !important}.u-text--loading{color:#2ab7ec !important}.u-border--loading{border-color:#2ab7ec !important}.u-background--loading{background-color:#2ab7ec !important}.has-loading{border:1px solid #2ab7ec !important}.has-loading:focus{border:1px solid #2ab7ec !important}.u-vertical-align{position:relative;top:50%;transform:translateY(-50%)}.u-text--truncate{white-space:nowrap;overflow:hidden;text-overflow:ellipsis}.hidden{display:none}.small-icon{width:12px}.images-info{text-align:center;padding:10px}.images-warning{box-sizing:border-box;border-radius:2px;list-style:none;padding:15px 20px 15px 45px;background-repeat:no-repeat;box-shadow:0 1px 1px rgba(0,0,0,0.1);background:#fff url("../assets/images/icons/error.svg") no-repeat;background-size:16px 16px;background-position:15px center;margin-bottom:20px}#loader{width:10px;margin:16px auto 0 auto}#importing{box-sizing:border-box;border-radius:2px;list-style:none;padding:15px;font-size:16px;margin:0 0 20px;background:#fff;background-position:top 50% left 15px;background-repeat:no-repeat;box-shadow:0 1px 1px rgba(0,0,0,0.1)}#importing .spinner{margin-top:-3px}.spinner{width:16px;height:16px;padding:0;border-bottom:0 !important;display:inline-block;vertical-align:middle;text-indent:999em;background:url("../assets/images/icons/loading.png") no-repeat;background-size:100% 100%}.spin{animation:spin 1s infinite linear !important}.importing-dot{opacity:0;-webkit-animation:dot 1.3s infinite;animation:dot 1.3s infinite}.accounts .logout .divide{padding:0 20px 0 30px;display:inline-block}.accounts .api li{position:relative}.accounts .api li input[type='text']{line-height:30px;padding-right:30px;width:100%}.accounts .api li input[type='text']::-webkit-input-placeholder{color:#333}.accounts .api li input[type='text']:-moz-placeholder{color:#333}.accounts .api li input[type='text']::-moz-placeholder{color:#333}.accounts .api li input[type='text']:-ms-input-placeholder{color:#333}.accounts .api li .delete-link{position:absolute;top:13px;right:15px}.script-content{width:100%;overflow:hidden}.script-content .content{margin:20px 0 0}

=== modified file 'src/maasserver/static/js/angular/directives/power_parameters.js'
--- src/maasserver/static/js/angular/directives/power_parameters.js	2016-09-06 14:04:02 +0000
+++ src/maasserver/static/js/angular/directives/power_parameters.js	2017-02-01 16:08:23 +0000
@@ -10,7 +10,7 @@
         '<div class="form__group">',
             '<label for="power-type" ',
                     'class="form__group-label two-col">Power type</label>',
-            '<div class="form__group-inpput three-col">',
+            '<div class="form__group-input three-col">',
                 '<select name="power-type" id="power-type" ',
                     'placeholder="Choose a power type" ',
                     'data-ng-disabled="ngDisabled" ',
@@ -18,7 +18,7 @@
                     'data-ng-model="ngModel.type" ',
                     'data-ng-options="',
                     'type as type.description ',
-                    'for type in maasPowerParameters">',
+                    'for type in maasPowerParameters track by type.name">',
                     '<option value="" disabled selected>',
                         'Select your power type',
                     '</option>',

=== modified file 'src/maasserver/static/js/angular/directives/tests/test_power_parameters.js'
--- src/maasserver/static/js/angular/directives/tests/test_power_parameters.js	2015-05-28 14:45:30 +0000
+++ src/maasserver/static/js/angular/directives/tests/test_power_parameters.js	2017-02-01 16:08:08 +0000
@@ -197,7 +197,7 @@
                 expect(select.attr("data-ng-model")).toBe("ngModel.type");
                 expect(select.attr("data-ng-options")).toBe(
                     "type as type.description for type " +
-                    "in maasPowerParameters");
+                    "in maasPowerParameters track by type.name");
                 expect(select.attr("data-ng-disabled")).toBe("ngDisabled");
             });
 
@@ -210,7 +210,7 @@
             $scope.powerTypes = [powerType];
             var directive = compileDirective("powerTypes", "value");
             var select = directive.find("select");
-            var option = select.find('option[value="0"]');
+            var option = select.find('option[value="test"]');
             expect(option.text()).toBe("Test Title");
         });
 

=== modified file 'src/maasserver/static/js/angular/directives/version_reloader.js'
--- src/maasserver/static/js/angular/directives/version_reloader.js	2016-09-21 14:49:23 +0000
+++ src/maasserver/static/js/angular/directives/version_reloader.js	2017-02-01 22:46:58 +0000
@@ -9,8 +9,8 @@
 
 
 angular.module('MAAS').directive('maasVersionReloader', [
-    '$window', 'GeneralManager', 'ManagerHelperService',
-    function($window, GeneralManager, ManagerHelperService) {
+    '$window', 'GeneralManager', 'ManagerHelperService', 'LogService',
+    function($window, GeneralManager, ManagerHelperService, LogService) {
         return {
             restrict: "A",
             controller: function($scope) {
@@ -25,12 +25,17 @@
                 ManagerHelperService.loadManager($scope, GeneralManager).then(
                     function() {
                         GeneralManager.enableAutoReload(true);
+                        LogService.info(
+                            'Version reloader: Monitoring MAAS "' +
+                            $scope.site + '"; version', $scope.version.text,
+                            "via", $window.location.href);
                         $scope.$watch("version.text",
                             function(newValue, oldValue) {
-                                console.log(
-                                    "Detected new MAAS version; " +
-                                    "forcing reload.");
                                 if(newValue !== oldValue) {
+                                    LogService.info(
+                                        "MAAS version changed from '" +
+                                        oldValue + "' to '" + newValue +
+                                        "'; forcing reload.");
                                     $scope.reloadPage();
                                 }
                             });

=== modified file 'src/maasserver/static/js/angular/factories/tests/test_region.js'
--- src/maasserver/static/js/angular/factories/tests/test_region.js	2016-09-20 20:25:54 +0000
+++ src/maasserver/static/js/angular/factories/tests/test_region.js	2017-01-30 13:05:29 +0000
@@ -394,10 +394,14 @@
         });
 
         it("includes csrftoken if cookie defined", function() {
-            csrftoken = makeName('csrftoken');
+            var csrftoken = makeName('csrftoken');
             // No need to organize a cleanup: cookies are reset before each
             // test.
-            $cookies.csrftoken = csrftoken;
+            if(angular.isFunction($cookies.put)) {
+                $cookies.put('csrftoken', csrftoken);
+            } else {
+                $cookies.csrftoken = csrftoken;
+            }
             expect(RegionConnection._buildUrl()).toBe(
                 "ws://" + $window.location.hostname + ":" +
                 $window.location.port + $window.location.pathname + "/ws" +

=== added file 'src/maasserver/static/js/angular/services/log.js'
--- src/maasserver/static/js/angular/services/log.js	1970-01-01 00:00:00 +0000
+++ src/maasserver/static/js/angular/services/log.js	2017-02-03 17:41:49 +0000
@@ -0,0 +1,93 @@
+/* Copyright 2017 Canonical Ltd.  This software is licensed under the
+ * GNU Affero General Public License version 3 (see the file LICENSE).
+ *
+ * Log Service
+ *
+ * Allows logging to be enabled or disabled, and allows a particular log level
+ * to be set, which will allow logs of a specified importance (or more) to be
+ * seen.
+ *
+ * Also appends a time index (in seconds, accurate to milliseconds) to the
+ * beginning of the log string.
+ */
+
+angular.module('MAAS').service('LogService', [
+    '$window', function($window) {
+        var self = this;
+
+        // Global enable/disable for MAAS logging. If set to `false`, this
+        // value takes precedence over the logLevel.
+        self.logging = true;
+
+        // Specifies the log level.
+        // Level 1: error() logging
+        // Level 2: error() and warn() logging
+        // Level 3: all of the above, plus info()
+        // Level 4: all of the above, plus log()
+        // Level 5: all of the above, plus debug()
+        self.logLevel = 5;
+
+        // Returns a monotonic time (in milliseconds) since page load.
+        self.now = function() {
+            return $window.performance ? $window.performance.now() : 0;
+        };
+
+        // Standard logging functions.
+        self._debug = console.debug;
+        self._log = console.log;
+        self._info = console.info;
+        self._warn = console.warn;
+        self._error = console.error;
+
+        // Formats the specified time (in milliseconds) in seconds.
+        this.formatMilliseconds = function(milliseconds) {
+            return parseFloat(milliseconds / 1000.0).toFixed(3);
+        };
+
+        // Internal function to log using the specified destination, with the
+        // specified list of arguments.
+        this.__log = function(destination, args) {
+            if(self.logging === true) {
+                // Add time index to the beginning of the log.
+                Array.prototype.unshift.call(args,
+                    "[" + self.formatMilliseconds(self.now()) + "]");
+                destination.apply(self, args);
+            }
+        };
+
+        // Wrapper to check the log level and then call self._debug().
+        this.debug = function() {
+            if(self.logLevel >= 5) {
+                self.__log(self._debug, arguments);
+            }
+        };
+
+        // Wrapper to check the log level and then call self._log().
+        this.log = function() {
+            if(self.logLevel >= 4) {
+                self.__log(self._log, arguments);
+            }
+        };
+
+        // Wrapper to check the log level and then call self._info().
+        this.info = function() {
+            if(self.logLevel >= 3) {
+                self.__log(self._info, arguments);
+            }
+        };
+
+        // Wrapper to check the log level and then call self._warn().
+        this.warn = function() {
+            if(self.logLevel >= 2) {
+                self.__log(self._warn, arguments);
+            }
+        };
+
+        // Wrapper to check the log level and then call self._err().
+        this.error = function() {
+            if(self.logLevel >= 1) {
+                self.__log(self._error, arguments);
+            }
+        };
+    }
+]);

=== added file 'src/maasserver/static/js/angular/services/tests/test_log.js'
--- src/maasserver/static/js/angular/services/tests/test_log.js	1970-01-01 00:00:00 +0000
+++ src/maasserver/static/js/angular/services/tests/test_log.js	2017-02-03 17:41:49 +0000
@@ -0,0 +1,112 @@
+/* Copyright 2017 Canonical Ltd.  This software is licensed under the
+ * GNU Affero General Public License version 3 (see the file LICENSE).
+ *
+ * Unit tests for LogService.
+ */
+
+describe("LogService", function() {
+
+    beforeEach(function() {
+        spyOn(console, 'debug');
+        spyOn(console, 'log');
+        spyOn(console, 'info');
+        spyOn(console, 'warn');
+        spyOn(console, 'error');
+    });
+
+    // Load the MAAS module.
+    beforeEach(module("MAAS"));
+
+    // Get the $injector so the test can grab the LogService.
+    var $injector;
+    var LogService;
+    beforeEach(inject(function(_$injector_) {
+        $injector = _$injector_;
+        LogService = $injector.get("LogService");
+    }));
+
+    var scenarios = [
+        {
+            'it': "console.error",
+            'logLevel': 1,
+            'func': 'error'
+        },
+        {
+            'it': "console.warn",
+            'logLevel': 2,
+            'func': 'warn'
+        },
+        {
+            'it': "console.info",
+            'logLevel': 3,
+            'func': 'info'
+        },
+        {
+            'it': "console.log",
+            'logLevel': 4,
+            'func': 'log'
+        },
+        {
+            'it': "console.debug",
+            'logLevel': 5,
+            'func': 'debug'
+        }
+    ];
+
+    // Do positive tests for appropriate log levels.
+    angular.forEach(scenarios, function(scenario) {
+        it("calls " + scenario.it + " for logLevel=" + scenario.logLevel,
+            function() {
+            LogService.logging = true;
+            LogService.logLevel = scenario.logLevel;
+            var logFunction = LogService[scenario.func];
+            var message = makeName();
+            logFunction(message);
+            expect(console[scenario.func]).toHaveBeenCalled();
+        });
+    });
+
+    // Do negative tests for log levels that do not include the level.
+    angular.forEach(scenarios, function(scenario) {
+        it("doesn't call " + scenario.it + " for logLevel=" +
+            (scenario.logLevel - 1), function() {
+            LogService.logging = true;
+            LogService.logLevel = scenario.logLevel - 1;
+            var logFunction = LogService[scenario.func];
+            var message = makeName();
+            logFunction(message);
+            expect(console[scenario.func]).not.toHaveBeenCalled();
+        });
+    });
+
+    describe("formatMilliseconds", function() {
+
+        it("formats milliseconds into equivalent decimal seconds", function() {
+            var result = LogService.formatMilliseconds(1234);
+            expect(result).toEqual("1.234");
+        });
+
+        it("zero-pads values", function() {
+            var result = LogService.formatMilliseconds(1000);
+            expect(result).toEqual("1.000");
+        });
+    });
+
+    describe("__log", function() {
+
+        it("appends timestamp to the beginning of the log", function() {
+            LogService.logging = true;
+            var message = makeName();
+            var outMessage = null;
+            LogService.now = function() {
+                return 0;
+            };
+            LogService.__log(function() {
+                // __log() will call the destination log function after
+                // appending the formatted time to the beginning.
+                outMessage = [arguments[0], arguments[1]];
+            }, [message]);
+            expect(outMessage).toEqual(["[0.000]", message]);
+        });
+    });
+});

=== added file 'src/maasserver/static/js/angular/testing/setup.js'
--- src/maasserver/static/js/angular/testing/setup.js	1970-01-01 00:00:00 +0000
+++ src/maasserver/static/js/angular/testing/setup.js	2017-02-01 22:46:58 +0000
@@ -0,0 +1,19 @@
+/* Copyright 2017 Canonical Ltd.  This software is licensed under the
+ * GNU Affero General Public License version 3 (see the file LICENSE).
+ *
+ * Test Setup
+ *
+ * Special setup that occurs only for unit testing.
+ */
+
+angular.module('MAAS').run(['LogService', function (LogService) {
+    // Silence logging by default in the tests.
+    LogService.logging = false;
+
+    // Make our own monotonic clock for testing, since the unit test suite
+    // won't have $window.performance.
+    var time = 0;
+    LogService.now = function() {
+        return time++;
+    };
+}]);

=== modified file 'src/maasserver/static/partials/dashboard.html'
--- src/maasserver/static/partials/dashboard.html	2017-02-01 18:13:33 +0000
+++ src/maasserver/static/partials/dashboard.html	2017-02-02 20:15:50 +0000
@@ -1,224 +1,188 @@
-<div data-ng-hide="loaded">
-    <header class="page-header" sticky media-query="min-width: 769px">
-        <div class="wrapper--inner">
-            <h1 class="page-header__title">Device discovery</h1>
-            <p class="page-header__status">
-                <span class="u-text--loading"><i class="icon u-margin--right-tiny icon--loading u-animation--spin"></i>Searching network for devices&hellip;</span>
-            </p>
+<header class="page-header" sticky media-query="min-width: 769px">
+    <div class="wrapper--inner">
+        <h1 class="page-header__title">Device discovery</h1>
+        <p class="page-header__status">{$ discoveredDevices.length $} items discovered <span class="u-text--loading" data-ng-if="!loaded"><i class="icon icon--loading u-animation--spin"></i> Loading...</span></p>
+        <div class="page-header__controls" data-ng-if="networkDiscovery">
+            <maas-obj-form obj="networkDiscovery" manager="configManager" inline="true">
+                <maas-obj-field type="onoffswitch" key="value" label="Discovery enabled"
+  on-value="enabled" off-value="disabled" subtle="true" label-left="true"
+  label-info="When enabled, MAAS will use passive techniques, such as listening to&#xa; ARP requests and DNS advertisements, to observe networks attached&#xa; to rack controllers. Active subnet mapping can also be enabled in the&#xa; Device Discovery section of the Settings page."></maas-obj-field>
+            </maas-obj-form>
         </div>
-    </header>
-    <section class="row">
-      <div class="wrapper--inner">
-          <div class="twelve-col">
-              <table>
-                  <thead>
-                      <tr>
-                          <th class="table-col--37">Name</th>
-                          <th class="table-col--15">Mac address</th>
-                          <th class="table-col--13">IP</th>
-                          <th class="table-col--15">Rack</th>
-                          <th class="table-col--20">Last seen</th>
-                      </tr>
-                  </thead>
-                  <tbody>
-                      <tr>
-                          <td colspan="5">
-                              No new discoveries
-                          </td>
-                      </tr>
-                  </tbody>
-              </table>
-          </div>
       </div>
-    </section>
+</header>
+<div class="wrapper--inner">
+    <maas-notifications></maas-notifications>
 </div>
-
-<maas-notifications />
-
-<div class="ng-hide" data-ng-show="loaded">
-    <header class="page-header" sticky media-query="min-width: 769px">
-        <div class="wrapper--inner">
-            <h1 class="page-header__title">Device discovery</h1>
-            <p class="page-header__status" >{$ discoveredDevices.length $} items discovered</p>
-            <div class="page-header__controls" data-ng-if="networkDiscovery">
-                <maas-obj-form obj="networkDiscovery" manager="configManager" inline="true">
-                    <maas-obj-field type="onoffswitch" key="value" label="Discovery enabled"
-      on-value="enabled" off-value="disabled" subtle="true" label-left="true"
-      label-info="When enabled, MAAS will use passive techniques, such as listening to&#xa; ARP requests and DNS advertisements, to observe networks attached&#xa; to rack controllers. Active subnet mapping can also be enabled in the&#xa; Device Discovery section of the Settings page."></maas-obj-field>
-                </maas-obj-form>
-            </div>
-          </div>
-    </header>
-    <section class="row">
-        <div class="wrapper--inner">
-            <div class="p-notification--warning" data-ng-if="networkDiscovery.value === 'disabled'">
-                <p class="p-notification__response">
-                  <span class="p-notification__status">Warning:</span>List of devices will not update as discovery is turned off</p>
-            </div>
-            <div class="twelve-col">
-                <div class="table" >
-                    <header class="table__head u-display--desktop" sticky sticky-class="table__head--sticky" offset="116" media-query="min-width: 768px">
-                        <div class="table__row">
-                            <div class="table__header table-col--37">Name</div>
-                            <div class="table__header table-col--15">Mac address</div>
-                            <div class="table__header table-col--13">IP</div>
-                            <div class="table__header table-col--15">Rack</div>
-                            <div class="table__header table-col--20">Last seen</div>
-                        </div>
-                    </header>
-                    <div class="table__body">
-                        <div class="table__row table--success" data-ng-if="convertTo.saved">
-                            <div class="table__data table-col--97">
-                                <i class="icon icon--success"></i> {$ convertTo.hostname $} has been add to {$ site $}.
-                                <a href="#/node/{$ convertTo.parent $}" data-ng-if="convertTo.parent">Go to the machine devices page.</a>
-                                <a href="#/nodes/?tab=devices" data-ng-if="!convertTo.parent">Go to the device listing.</a>
-                            </div>
+<section class="row">
+    <div class="wrapper--inner">
+        <div class="p-notification--warning" data-ng-if="networkDiscovery.value === 'disabled'">
+            <p class="p-notification__response">
+              <span class="p-notification__status">Warning:</span>List of devices will not update as discovery is turned off</p>
+        </div>
+        <div class="twelve-col">
+            <div class="table" >
+                <header class="table__head u-display--desktop" sticky sticky-class="table__head--sticky" offset="116" media-query="min-width: 768px">
+                    <div class="table__row">
+                        <div class="table__header table-col--37">Name</div>
+                        <div class="table__header table-col--15">Mac address</div>
+                        <div class="table__header table-col--13">IP</div>
+                        <div class="table__header table-col--15">Rack</div>
+                        <div class="table__header table-col--20">Last seen</div>
+                    </div>
+                </header>
+                <div class="table__body">
+                    <div class="table__row table--success" data-ng-if="convertTo.saved">
+                        <div class="table__data table-col--97">
+                            <i class="icon icon--success"></i> {$ convertTo.hostname $} has been add to {$ site $}.
+                            <a href="#/node/{$ convertTo.parent $}" data-ng-if="convertTo.parent">Go to the machine devices page.</a>
+                            <a href="#/nodes/?tab=devices" data-ng-if="!convertTo.parent">Go to the device listing.</a>
+                        </div>
+                        <div class="table__data table-col--3">
+                            <i class="icon icon--cross tooltip" aria-label="Clear"
+                                data-ng-click="convertTo.saved = false">Clear</i>
+                        </div>
+                    </div>
+                    <div class="table__row" data-ng-if="!discoveredDevices.length">
+                        <div class="table__data table-col--100">
+                            No new discoveries
+                        </div>
+                    </div>
+                    <div class="table__row"
+                        data-ng-repeat="discovery in discoveredDevices | orderBy:'-last_seen' track by discovery.first_seen"
+                        data-ng-class="{'is-active' : discovery.first_seen === selectedDevice}">
+                        <div data-ng-if="discovery.first_seen !== selectedDevice"
+                            data-ng-dblclick="toggleSelected(discovery.first_seen)">
+                            <div class="table__data table-col--37" aria-label="Name">
+                                {$ getDiscoveryName(discovery) $}
+                                <i data-ng-show="discovery.is_external_dhcp === true" class="icon icon--info tooltip u-margin--left-tiny ng-hide" aria-label="This device is providing DHCP."></i>
+                            </div>
+                            <div class="table__data table-col--15" aria-label="MAC">
+                                {$ discovery.mac_address $}<br/>
+                                <small class="u-text--subtle u-display--desktop">{$ discovery.mac_organization || 'Unknown' $}</small>
+                            </div>
+                            <div class="table__data table-col--13 u-text--truncate" aria-label="IP Address">
+                                {$ discovery.ip $}
+                            </div>
+                            <div class="table__data table-col--15" aria-label="Rack">
+                                {$ discovery.observer_hostname $}
+                            </div>
+                            <div class="table__data table-col--17" aria-label="Last seen">
+                                <time>{$ discovery.last_seen $}</time>
+                            </div>
+                            <div class="table__data table--mobile-controls  table-col--3">
+                                <div class="table__controls u-align--right">
+                                    <i class="icon icon--open tooltip u-display--desktop" aria-label="Open"
+                                        data-ng-click="toggleSelected(discovery.first_seen)">Open</i>
+                                    <a class="button--secondary u-display--mobile"
+                                        data-ng-click="toggleSelected(discovery.first_seen)">Add device</a>
+                                </div>
+                            </div>
+                        </div>
+                        <maas-obj-form data-ng-if="discovery.first_seen === selectedDevice"
+                            obj="convertTo" manager="proxyManager" pre-process="preProcess"
+                            after-save="afterSave" table-form="true" save-on-blur="false">
+                            <div class="table__data table-col--20">
+                                <maas-obj-field class="u-margin--bottom-none" type="text" key="hostname"
+                                    disable-label="true" placeholder="Hostname (optional)"></maas-obj-field>
+                            </div>
+                            <div class="table__data table-col--77"></div>
                             <div class="table__data table-col--3">
-                                <i class="icon icon--cross tooltip" aria-label="Clear"
-                                    data-ng-click="convertTo.saved = false">Clear</i>
-                            </div>
-                        </div>
-                        <div class="table__row" data-ng-if="!discoveredDevices.length">
-                            <div class="table__data table-col--100">
-                                No new discoveries
-                            </div>
-                        </div>
-                        <div class="table__row"
-                            data-ng-repeat="discovery in discoveredDevices | orderBy:'-last_seen' track by discovery.first_seen"
-                            data-ng-class="{'is-active' : discovery.first_seen === selectedDevice}">
-                            <div data-ng-if="discovery.first_seen !== selectedDevice"
-                                data-ng-dblclick="toggleSelected(discovery.first_seen)">
-                                <div class="table__data table-col--37" aria-label="Name">
-                                    {$ getDiscoveryName(discovery) $}
-                                    <i data-ng-show="discovery.is_external_dhcp === true" class="icon icon--info tooltip u-margin--left-tiny ng-hide" aria-label="This device is providing DHCP."></i>
-                                </div>
-                                <div class="table__data table-col--15" aria-label="MAC">
-                                    {$ discovery.mac_address $}<br/>
-                                    <small class="u-text--subtle u-display--desktop">{$ discovery.mac_organization || 'Unknown' $}</small>
-                                </div>
-                                <div class="table__data table-col--13 u-text--truncate" aria-label="IP Address">
-                                    {$ discovery.ip $}
-                                </div>
-                                <div class="table__data table-col--15" aria-label="Rack">
-                                    {$ discovery.observer_hostname $}
-                                </div>
-                                <div class="table__data table-col--17" aria-label="Last seen">
-                                    <time>{$ discovery.last_seen $}</time>
-                                </div>
-                                <div class="table__data table--mobile-controls  table-col--3">
-                                    <div class="table__controls u-align--right">
-                                        <i class="icon icon--open tooltip u-display--desktop" aria-label="Open"
-                                            data-ng-click="toggleSelected(discovery.first_seen)">Open</i>
-                                        <a class="button--secondary u-display--mobile"
-                                            data-ng-click="toggleSelected(discovery.first_seen)">Add device</a>
-                                    </div>
-                                </div>
-                            </div>
-                            <maas-obj-form data-ng-if="discovery.first_seen === selectedDevice"
-                                obj="convertTo" manager="proxyManager" pre-process="preProcess"
-                                after-save="afterSave" table-form="true" save-on-blur="false">
-                                <div class="table__data table-col--20">
-                                    <maas-obj-field class="u-margin--bottom-none" type="text" key="hostname"
-                                        disable-label="true" placeholder="Hostname (optional)"></maas-obj-field>
-                                </div>
-                                <div class="table__data table-col--77"></div>
-                                <div class="table__data table-col--3">
-                                    <i class="icon icon--close tooltip u-display--desktop" aria-label="Close"
-                                        data-ng-click="toggleSelected(discovery.first_seen)">Close</i>
-                                </div>
-                                <div class="table__dropdown">
-                                    <div class="table__dropdown-title">
-                                        <h2 data-ng-click="toggleSelected(selectedDevice)" class="u-float--left">Adding {$ getDiscoveryName(discovery) $}</h2>
-                                        <i data-ng-click="toggleSelected(selectedDevice)" class="icon icon--remove u-float--right u-margin--top-small u-margin--right-small"></i>
-                                    </div>
-                                    <div class="table__row is-active">
-                                        <div class="table__data table-col--100">
-                                            <div class="six-col">
-                                                <dl>
-                                                    <dt class="two-col">Mac</dt>
-                                                        <dd class="four-col last-col">{$ discovery.mac_address $}</dd>
-                                                    <dt class="two-col">IP</dt>
-                                                        <dd class="four-col last-col">{$ discovery.ip $}</dd>
-                                                    <dt class="two-col">Rack</dt>
-                                                        <dd class="four-col last-col">{$ discovery.observer_hostname $}</dd>
-                                                    <dt class="two-col">Last seen</dt>
-                                                        <dd class="four-col last-col"><time>{$ discovery.last_seen $}</time></dd>
-                                                </dl>
-                                            </div>
-                                            <div class="six-col last-col">
-                                                <dl>
-                                                    <dt class="two-col">Fabric</dt>
-                                                        <dd class="four-col last-col">{$ discovery.fabric_name $}</dd>
-                                                    <dt class="two-col">VLAN</dt>
-                                                        <dd class="four-col last-col">{$ getVLANName(discovery.vlan) $}</dd>
-                                                    <dt class="two-col">Subnet</dt>
-                                                        <dd class="four-col last-col">{$ getSubnetName(discovery.subnet) $}</dd>
-                                                </dl>
-                                            </div>
-                                        </div>
-                                    </div>
-                                    <div class="table__row is-active u-padding--top-small">
-                                        <div class="table__data table-col--100">
-                                            <fieldset class="form__fieldset six-col">
-                                                <div class="form__group u-display--mobile">
-                                                  <label for="Name" class="form__group-label two-col">Name</label>
-                                                  <div class="form__group-input three-col">
-                                                    <maas-obj-field type="text" key="hostname"
-                                                        disable-label="true" placeholder="Hostname (optional)"></maas-obj-field>
-                                                  </div>
-                                                </div>
-                                                <div class="form__group">
-                                                    <label class="form__group-label two-col u-margin--bottom-small">Type</label>
-                                                    <div class="form__group-input three-col">
-                                                        <select name="type" ng-model="convertTo.type">
-                                                            <option value="" disabled>Choose type</option>
-                                                            <option value="device">Device</option>
-                                                            <option value="interface">Interface</option>
-                                                        </select>
-                                                    </div>
-                                                </div>
-                                                <maas-obj-field type="options" key="domain" label="Domain" placeholder="Choose domain"
-                                                    label-width="two" input-width="three" subtle="false"
-                                                    options="domain as domain.name for domain in domains"
-                                                    data-ng-if="convertTo.type === 'device'"></maas-obj-field>
-                                                <maas-obj-field type="options" subtle="false" key="system_id" label="Device name" placeholder="Select device name"
-                                                    options="device.system_id as device.fqdn for device in devices | orderBy:'fqdn'"
-                                                    label-width="two" label-info="Create as an interface on the selected device." input-width="three"
-                                                    data-ng-if="convertTo.type === 'interface'"></maas-obj-field>
-                                            </fieldset>
-                                            <fieldset class="form__fieldset six-col last-col">
-                                                <maas-obj-field type="options" subtle="false" key="parent" label="Parent" placeholder="Select parent (optional)" placeholder-enabled="true"
-                                                    options="machine.system_id as machine.fqdn for machine in machines | filter:{status_code: 6} | orderBy:'fqdn'"
-                                                    label-width="two" label-info="Assign this device as a child of the parent machine." input-width="three"
-                                                    data-ng-if="convertTo.type === 'device'"></maas-obj-field>
-                                                <maas-obj-field type="options" subtle="false" key="ip_assignment" label="IP assignment" placeholder="Select IP assignment"
-                                                    options="option[0] as option[1] for option in convertTo.deviceIPOptions" label-width="two" input-width="three"></maas-obj-field>
-                                            </fieldset>
-                                        </div>
-                                    </div>
-                                    <div class="table__row is-active">
-                                        <div class="table__data u-float--left">
-                                            <maas-obj-errors></maas-obj-errors>
-                                        </div>
-                                        <div class="table__data u-float--right">
-                                            <a class="button--base button--inline u-display--desktop" data-ng-click="toggleSelected(selectedDevice)">Cancel</a>
-                                            <button class="button--secondary button--inline"
-                                                data-ng-click="convertTo.goTo = true"
-                                                data-ng-if="!convertTo.$maasForm.getValue('parent')"
-                                                maas-obj-save>Save and go to device listing</button>
-                                            <button class="button--secondary button--inline"
-                                                data-ng-click="convertTo.goTo = true"
-                                                data-ng-if="convertTo.$maasForm.getValue('parent')"
-                                                maas-obj-save>Save and go to machine details</button>
-                                            <button class="button--positive button--inline" maas-obj-save>Save</button>
-                                        </div>
-                                    </div>
-                                </div>
-                            </maas-obj-form>
-                        </div>
+                                <i class="icon icon--close tooltip u-display--desktop" aria-label="Close"
+                                    data-ng-click="toggleSelected(discovery.first_seen)">Close</i>
+                            </div>
+                            <div class="table__dropdown">
+                                <div class="table__dropdown-title">
+                                    <h2 data-ng-click="toggleSelected(selectedDevice)" class="u-float--left">Adding {$ getDiscoveryName(discovery) $}</h2>
+                                    <i data-ng-click="toggleSelected(selectedDevice)" class="icon icon--remove u-float--right u-margin--top-small u-margin--right-small"></i>
+                                </div>
+                                <div class="table__row is-active">
+                                    <div class="table__data table-col--100">
+                                        <div class="six-col">
+                                            <dl>
+                                                <dt class="two-col">Mac</dt>
+                                                    <dd class="four-col last-col">{$ discovery.mac_address $}</dd>
+                                                <dt class="two-col">IP</dt>
+                                                    <dd class="four-col last-col">{$ discovery.ip $}</dd>
+                                                <dt class="two-col">Rack</dt>
+                                                    <dd class="four-col last-col">{$ discovery.observer_hostname $}</dd>
+                                                <dt class="two-col">Last seen</dt>
+                                                    <dd class="four-col last-col"><time>{$ discovery.last_seen $}</time></dd>
+                                            </dl>
+                                        </div>
+                                        <div class="six-col last-col">
+                                            <dl>
+                                                <dt class="two-col">Fabric</dt>
+                                                    <dd class="four-col last-col">{$ discovery.fabric_name $}</dd>
+                                                <dt class="two-col">VLAN</dt>
+                                                    <dd class="four-col last-col">{$ getVLANName(discovery.vlan) $}</dd>
+                                                <dt class="two-col">Subnet</dt>
+                                                    <dd class="four-col last-col">{$ getSubnetName(discovery.subnet) $}</dd>
+                                            </dl>
+                                        </div>
+                                    </div>
+                                </div>
+                                <div class="table__row is-active u-padding--top-small">
+                                    <div class="table__data table-col--100">
+                                        <fieldset class="form__fieldset six-col">
+                                            <div class="form__group u-display--mobile">
+                                              <label for="Name" class="form__group-label two-col">Name</label>
+                                              <div class="form__group-input three-col">
+                                                <maas-obj-field type="text" key="hostname"
+                                                    disable-label="true" placeholder="Hostname (optional)"></maas-obj-field>
+                                              </div>
+                                            </div>
+                                            <div class="form__group">
+                                                <label class="form__group-label two-col u-margin--bottom-small">Type</label>
+                                                <div class="form__group-input three-col">
+                                                    <select name="type" ng-model="convertTo.type">
+                                                        <option value="" disabled>Choose type</option>
+                                                        <option value="device">Device</option>
+                                                        <option value="interface">Interface</option>
+                                                    </select>
+                                                </div>
+                                            </div>
+                                            <maas-obj-field type="options" key="domain" label="Domain" placeholder="Choose domain"
+                                                label-width="two" input-width="three" subtle="false"
+                                                options="domain as domain.name for domain in domains"
+                                                data-ng-if="convertTo.type === 'device'"></maas-obj-field>
+                                            <maas-obj-field type="options" subtle="false" key="system_id" label="Device name" placeholder="Select device name"
+                                                options="device.system_id as device.fqdn for device in devices | orderBy:'fqdn'"
+                                                label-width="two" label-info="Create as an interface on the selected device." input-width="three"
+                                                data-ng-if="convertTo.type === 'interface'"></maas-obj-field>
+                                        </fieldset>
+                                        <fieldset class="form__fieldset six-col last-col">
+                                            <maas-obj-field type="options" subtle="false" key="parent" label="Parent" placeholder="Select parent (optional)" placeholder-enabled="true"
+                                                options="machine.system_id as machine.fqdn for machine in machines | filter:{status_code: 6} | orderBy:'fqdn'"
+                                                label-width="two" label-info="Assign this device as a child of the parent machine." input-width="three"
+                                                data-ng-if="convertTo.type === 'device'"></maas-obj-field>
+                                            <maas-obj-field type="options" subtle="false" key="ip_assignment" label="IP assignment" placeholder="Select IP assignment"
+                                                options="option[0] as option[1] for option in convertTo.deviceIPOptions" label-width="two" input-width="three"></maas-obj-field>
+                                        </fieldset>
+                                    </div>
+                                </div>
+                                <div class="table__row is-active">
+                                    <div class="table__data u-float--left">
+                                        <maas-obj-errors></maas-obj-errors>
+                                    </div>
+                                    <div class="table__data u-float--right">
+                                        <a class="button--base button--inline u-display--desktop" data-ng-click="toggleSelected(selectedDevice)">Cancel</a>
+                                        <button class="button--secondary button--inline"
+                                            data-ng-click="convertTo.goTo = true"
+                                            data-ng-if="!convertTo.$maasForm.getValue('parent')"
+                                            maas-obj-save>Save and go to device listing</button>
+                                        <button class="button--secondary button--inline"
+                                            data-ng-click="convertTo.goTo = true"
+                                            data-ng-if="convertTo.$maasForm.getValue('parent')"
+                                            maas-obj-save>Save and go to machine details</button>
+                                        <button class="button--positive button--inline" maas-obj-save>Save</button>
+                                    </div>
+                                </div>
+                            </div>
+                        </maas-obj-form>
                     </div>
                 </div>
             </div>
         </div>
-    </section>
-</div>
+    </div>
+</section>

=== modified file 'src/maasserver/static/partials/nodes-list.html'
--- src/maasserver/static/partials/nodes-list.html	2017-02-01 18:13:33 +0000
+++ src/maasserver/static/partials/nodes-list.html	2017-02-02 20:15:50 +0000
@@ -1,5 +1,3 @@
-<maas-notifications />
-
 <header class="page-header" sticky media-query="min-width: 769px">
     <div class="wrapper--inner">
         <!-- XXX ricgard 2016-06-16 - Need to add e2e test. -->
@@ -610,7 +608,10 @@
       </div>
     </div>
 </header>
-<div class="row u-padding--top-none">
+<div class="wrapper--inner">
+    <maas-notifications></maas-notifications>
+</div>
+<div class="u-padding--top-none">
     <div class="wrapper--inner">
     <aside class="three-col">
         <div class="accordion maas-accordion ng-hide" data-ng-show="currentpage === 'nodes'"

=== modified file 'src/maasserver/views/combo.py'
--- src/maasserver/views/combo.py	2017-01-27 16:29:35 +0000
+++ src/maasserver/views/combo.py	2017-02-01 22:18:19 +0000
@@ -91,6 +91,7 @@
             "js/angular/services/browser.js",
             "js/angular/services/converter.js",
             "js/angular/services/json.js",
+            "js/angular/services/log.js",
             "js/angular/directives/accordion.js",
             "js/angular/directives/boot_images.js",
             "js/angular/directives/call_to_action.js",

=== modified file 'src/maasserver/websockets/handlers/tests/test_notification.py'
--- src/maasserver/websockets/handlers/tests/test_notification.py	2017-01-24 07:38:06 +0000
+++ src/maasserver/websockets/handlers/tests/test_notification.py	2017-02-02 17:12:11 +0000
@@ -47,6 +47,7 @@
         "updated": Equals(dehydrate_datetime(ntfn.updated)),
         "created": Equals(dehydrate_datetime(ntfn.created)),
         "message": Equals(ntfn.render()),
+        "category": Equals(ntfn.category),
     })
 
 

=== modified file 'src/maastesting/fixtures.py'
--- src/maastesting/fixtures.py	2017-01-26 15:57:00 +0000
+++ src/maastesting/fixtures.py	2017-02-02 14:13:44 +0000
@@ -32,6 +32,7 @@
 
 import fixtures
 from fixtures import EnvironmentVariable
+from maastesting import root
 from testtools.monkey import MonkeyPatcher
 from twisted.python.reflect import namedObject
 
@@ -409,15 +410,11 @@
     """
 
     def _setUp(self):
-        super(MAASRootFixture, self)._setUp()
-        maasroot = os.environ.get("MAAS_ROOT")
-        if maasroot is None:
-            raise NotADirectoryError("MAAS_ROOT is not defined.")
-        elif os.path.isdir(maasroot):
+        skel = Path(root).joinpath("run-skel")
+        if skel.is_dir():
             self.path = self.useFixture(TempDirectory()).join("run")
             # Work only in `run`; reference the old $MAAS_ROOT.
             etc = Path(self.path).joinpath("etc")
-            src = Path(maasroot)
             # Create and populate $MAAS_ROOT/run/etc/{ntp,ntp.conf}. The
             # `.keep` file is not strictly necessary, but it's created for
             # consistency with the source tree's `run` directory.
@@ -425,16 +422,16 @@
             ntp.mkdir(parents=True)
             ntp.joinpath(".keep").touch()
             ntp_conf = etc.joinpath("ntp.conf")
-            ntp_conf.write_bytes(src.joinpath("etc", "ntp.conf").read_bytes())
+            ntp_conf.write_bytes(skel.joinpath("etc", "ntp.conf").read_bytes())
             # Create and populate $MAAS_ROOT/run/etc/maas.
             maas = etc.joinpath("maas")
             maas.mkdir(parents=True)
             maas.joinpath("drivers.yaml").symlink_to(
-                src.joinpath("etc", "maas", "drivers.yaml").resolve())
+                skel.joinpath("etc", "maas", "drivers.yaml").resolve())
             maas.joinpath("templates").symlink_to(
-                src.joinpath("etc", "maas", "templates").resolve())
+                skel.joinpath("etc", "maas", "templates").resolve())
             # Update the environment.
             self.useFixture(EnvironmentVariable("MAAS_ROOT", self.path))
         else:
             raise NotADirectoryError(
-                "MAAS_ROOT (%r) is not a directory." % maasroot)
+                "Skeleton MAAS_ROOT (%s) is not a directory." % skel)

=== modified file 'src/maastesting/tests/test_fixtures.py'
--- src/maastesting/tests/test_fixtures.py	2017-01-26 15:57:00 +0000
+++ src/maastesting/tests/test_fixtures.py	2017-02-02 14:13:44 +0000
@@ -12,7 +12,10 @@
 from unittest.mock import call
 
 from fixtures import EnvironmentVariable
-from maastesting import root
+from maastesting import (
+    fixtures as fixtures_module,
+    root,
+)
 from maastesting.factory import factory
 from maastesting.fixtures import (
     CaptureStandardIO,
@@ -256,34 +259,28 @@
 
     def setUp(self):
         super(TestMAASRootFixture, self).setUp()
-        self.maasroot = os.path.join(root, "run-skel")
-        self.useFixture(EnvironmentVariable("MAAS_ROOT", self.maasroot))
+        self.skel = os.path.join(root, "run-skel")
+        self.useFixture(EnvironmentVariable("MAAS_ROOT", "/"))
 
     def test_creates_populates_and_removes_new_directory(self):
         fixture = MAASRootFixture()
         with fixture:
             self.assertThat(fixture.path, PathExists())
-            self.assertThat(fixture.path, Not(SamePath(self.maasroot)))
-            files_expected = set(listdirs(self.maasroot))
+            self.assertThat(fixture.path, Not(SamePath(self.skel)))
+            files_expected = set(listdirs(self.skel))
             files_observed = set(listdirs(fixture.path))
             self.assertThat(files_observed, Equals(files_expected))
         self.assertThat(fixture.path, Not(PathExists()))
 
     def test_updates_MAAS_ROOT_in_the_environment(self):
-        self.assertThat(os.environ["MAAS_ROOT"], SamePath(self.maasroot))
+        self.assertThat(os.environ["MAAS_ROOT"], Not(SamePath(self.skel)))
         with MAASRootFixture() as fixture:
             self.assertThat(os.environ["MAAS_ROOT"], SamePath(fixture.path))
-        self.assertThat(os.environ["MAAS_ROOT"], SamePath(self.maasroot))
-
-    def test_breaks_when_MAAS_ROOT_is_not_defined(self):
-        fixture = MAASRootFixture()
-        del os.environ["MAAS_ROOT"]
-        error = self.assertRaises(NotADirectoryError, fixture._setUp)
-        self.assertThat(str(error), Equals("MAAS_ROOT is not defined."))
+        self.assertThat(os.environ["MAAS_ROOT"], Not(SamePath(self.skel)))
 
     def test_breaks_when_MAAS_ROOT_is_not_a_directory(self):
+        self.patch(fixtures_module, "root", self.make_file())
         fixture = MAASRootFixture()
-        os.environ["MAAS_ROOT"] = self.make_file()
         error = self.assertRaises(NotADirectoryError, fixture._setUp)
         self.assertThat(str(error), DocTestMatches(
-            "MAAS_ROOT (...) is not a directory."))
+            "Skeleton MAAS_ROOT (...) is not a directory."))

=== modified file 'src/metadataserver/models/scriptset.py'
--- src/metadataserver/models/scriptset.py	2017-02-02 00:36:16 +0000
+++ src/metadataserver/models/scriptset.py	2017-02-03 23:47:03 +0000
@@ -32,15 +32,24 @@
 
 class ScriptSetManager(Manager):
 
-    def _clean_old(self, node, result_type):
-        script_sets = self.filter(
-            node=node, result_type=result_type).order_by('last_ping').reverse()
+    def _clean_old(self, node, result_type, new_script_set):
+        # Gather the list of existing script results of the given type for this
+        # node.
+        script_sets = self.filter(node=node, result_type=result_type)
+        # Exclude the newly created script_set so we don't try to remove it.
+        # This can happen when multiple script_sets have last_ping = None.
+        script_sets = script_sets.exclude(id=new_script_set.id)
+        # Sort by last_ping in reverse order so we only remove older entrees.
+        script_sets = script_sets.order_by('last_ping').reverse()
         config_var = {
             RESULT_TYPE.COMMISSIONING: 'max_node_commissioning_results',
             RESULT_TYPE.TESTING: 'max_node_testing_results',
             RESULT_TYPE.INSTALLATION: 'max_node_installation_results',
         }
         script_set_limit = Config.objects.get_config(config_var[result_type])
+        # Remove one from the script_set_limit to account for the newly created
+        # script_set.
+        script_set_limit -= 1
         if script_sets.count() > script_set_limit:
             for script_set in script_sets[script_set_limit:]:
                 script_set.delete()
@@ -57,7 +66,7 @@
 
         script_set = self.create(
             node=node, result_type=RESULT_TYPE.COMMISSIONING)
-        self._clean_old(node, RESULT_TYPE.COMMISSIONING)
+        self._clean_old(node, RESULT_TYPE.COMMISSIONING, script_set)
 
         for script_name, data in NODE_INFO_SCRIPTS.items():
             if node.is_controller and not data['run_on_controller']:
@@ -111,7 +120,7 @@
                 script_set=script_set, status=SCRIPT_STATUS.PENDING,
                 script=script)
 
-        self._clean_old(node, RESULT_TYPE.TESTING)
+        self._clean_old(node, RESULT_TYPE.TESTING, script_set)
         return script_set
 
     def create_installation_script_set(self, node):
@@ -127,7 +136,7 @@
             script_set=script_set, status=SCRIPT_STATUS.PENDING,
             script_name=CURTIN_INSTALL_LOG)
 
-        self._clean_old(node, RESULT_TYPE.INSTALLATION)
+        self._clean_old(node, RESULT_TYPE.INSTALLATION, script_set)
         return script_set
 
 

=== modified file 'src/metadataserver/models/tests/test_scriptset.py'
--- src/metadataserver/models/tests/test_scriptset.py	2017-02-02 00:36:16 +0000
+++ src/metadataserver/models/tests/test_scriptset.py	2017-02-03 23:47:03 +0000
@@ -22,6 +22,28 @@
 class TestScriptSetManager(MAASServerTestCase):
     """Test the ScriptSet manager."""
 
+    def test_clean_old_ignores_new_script_set(self):
+        # Make sure the created script_set isn't cleaned up. This can happen
+        # when multiple script_sets last_ping are set to None.
+        script_set_limit = Config.objects.get_config(
+            'max_node_installation_results')
+        node = factory.make_Node()
+        for _ in range(script_set_limit * 2):
+            ScriptSet.objects.create(
+                node=node, result_type=RESULT_TYPE.INSTALLATION,
+                last_ping=None)
+
+        script_set = ScriptSet.objects.create_installation_script_set(node)
+        # If the new script_set was cleaned up this will fail.
+        node.current_installation_script_set = script_set
+        node.save()
+
+        self.assertEquals(
+            script_set_limit,
+            ScriptSet.objects.filter(
+                node=node,
+                result_type=RESULT_TYPE.INSTALLATION).count())
+
     def test_create_commissioning_script_set(self):
         custom_scripts = [
             factory.make_Script(script_type=SCRIPT_TYPE.COMMISSIONING)

=== modified file 'src/provisioningserver/drivers/pod/__init__.py'
--- src/provisioningserver/drivers/pod/__init__.py	2017-01-26 23:43:32 +0000
+++ src/provisioningserver/drivers/pod/__init__.py	2017-02-02 20:33:52 +0000
@@ -158,8 +158,21 @@
     OVER_COMMIT = 'over_commit'
 
 
+class AttrHelperMixin:
+    """Mixin to add the `fromdict` and `asdict` to the classes."""
+
+    @classmethod
+    def fromdict(cls, data):
+        """Convert from a dictionary."""
+        return cls(**data)
+
+    def asdict(self):
+        """Convert to a dictionary."""
+        return attr.asdict(self)
+
+
 @attr.s
-class DiscoveredMachineInterface:
+class DiscoveredMachineInterface(AttrHelperMixin):
     """Discovered machine interface."""
     mac_address = attr.ib(convert=str)
     vid = attr.ib(convert=int, default=-1)
@@ -167,7 +180,7 @@
 
 
 @attr.s
-class DiscoveredMachineBlockDevice:
+class DiscoveredMachineBlockDevice(AttrHelperMixin):
     """Discovered machine block device."""
     model = attr.ib(convert=convert_obj(str, optional=True))
     serial = attr.ib(convert=convert_obj(str, optional=True))
@@ -178,7 +191,7 @@
 
 
 @attr.s
-class DiscoveredMachine:
+class DiscoveredMachine(AttrHelperMixin):
     """Discovered machine."""
     architecture = attr.ib(convert=str)
     cores = attr.ib(convert=int)
@@ -193,7 +206,7 @@
 
 
 @attr.s
-class DiscoveredPodHints:
+class DiscoveredPodHints(AttrHelperMixin):
     """Discovered pod hints.
 
     Hints provide helpful information to a user trying to compose a machine.
@@ -207,7 +220,7 @@
 
 
 @attr.s
-class DiscoveredPod:
+class DiscoveredPod(AttrHelperMixin):
     """Discovered pod information."""
     architectures = attr.ib(convert=convert_list(str))
     cores = attr.ib(convert=int)
@@ -221,6 +234,32 @@
     machines = attr.ib(
         convert=convert_list(DiscoveredMachine), default=[])
 
+
+@attr.s
+class RequestedMachineBlockDevice(AttrHelperMixin):
+    """Requested machine block device information."""
+    size = attr.ib(convert=int)
+
+
+@attr.s
+class RequestedMachineInterface(AttrHelperMixin):
+    """Requested machine interface information."""
+    # Currently has no parameters.
+
+
+@attr.s
+class RequestedMachine(AttrHelperMixin):
+    """Requested machine information."""
+    architecture = attr.ib(convert=str)
+    cores = attr.ib(convert=int)
+    memory = attr.ib(convert=int)
+    block_devices = attr.ib(convert=convert_list(RequestedMachineBlockDevice))
+    interfaces = attr.ib(convert=convert_list(RequestedMachineInterface))
+
+    # Optional fields.
+    cpu_speed = attr.ib(
+        convert=convert_obj(int, optional=True), default=None)
+
     @classmethod
     def fromdict(cls, data):
         """Convert from a dictionary."""
@@ -245,11 +284,14 @@
         """
 
     @abstractmethod
-    def compose(self, system_id, context):
+    def compose(self, system_id, context, request):
         """Compose a node from parameters in context.
 
         :param system_id: Pod system_id.
         :param context: Pod settings.
+        :param request: Requested machine.
+        :type request: `RequestedMachine`.
+        :returns: Tuple with (`DiscoveredMachine`, `DiscoveredPodHints`).
         """
 
     @abstractmethod

=== modified file 'src/provisioningserver/drivers/pod/tests/test_base.py'
--- src/provisioningserver/drivers/pod/tests/test_base.py	2017-01-26 23:43:32 +0000
+++ src/provisioningserver/drivers/pod/tests/test_base.py	2017-02-02 20:33:52 +0000
@@ -25,6 +25,9 @@
     PodConnError,
     PodDriverBase,
     PodError,
+    RequestedMachine,
+    RequestedMachineBlockDevice,
+    RequestedMachineInterface,
 )
 from testtools.matchers import (
     Equals,
@@ -403,6 +406,132 @@
         ))
 
 
+class TestRequestClasses(MAASTestCase):
+
+    def test_block_device_size(self):
+        size = random.randint(512, 512 * 1024)
+        device = RequestedMachineBlockDevice(size=size)
+        self.assertEquals(size, device.size)
+
+    def test_machine(self):
+        cores = random.randint(1, 8)
+        cpu_speed = random.randint(1000, 2000)
+        memory = random.randint(4096, 8192)
+        interfaces = [
+            RequestedMachineInterface()
+            for _ in range(3)
+        ]
+        block_devices = [
+            RequestedMachineBlockDevice(
+                size=random.randint(512, 1024))
+            for _ in range(3)
+        ]
+        machine = RequestedMachine(
+            architecture='amd64/generic',
+            cores=cores, cpu_speed=cpu_speed, memory=memory,
+            interfaces=interfaces,
+            block_devices=block_devices)
+        self.assertEquals(cores, machine.cores)
+        self.assertEquals(cpu_speed, machine.cpu_speed)
+        self.assertEquals(memory, machine.memory)
+        self.assertEquals(interfaces, machine.interfaces)
+        self.assertEquals(block_devices, machine.block_devices)
+
+    def test_machine_without_cpu_speed(self):
+        cores = random.randint(1, 8)
+        memory = random.randint(4096, 8192)
+        interfaces = [
+            RequestedMachineInterface()
+            for _ in range(3)
+        ]
+        block_devices = [
+            RequestedMachineBlockDevice(
+                size=random.randint(512, 1024))
+            for _ in range(3)
+        ]
+        machine = RequestedMachine(
+            architecture='amd64/generic',
+            cores=cores, cpu_speed=None, memory=memory,
+            interfaces=interfaces,
+            block_devices=block_devices)
+        self.assertEquals(cores, machine.cores)
+        self.assertIsNone(machine.cpu_speed)
+        self.assertEquals(memory, machine.memory)
+        self.assertEquals(interfaces, machine.interfaces)
+        self.assertEquals(block_devices, machine.block_devices)
+
+    def test_machine_asdict(self):
+        cores = random.randint(1, 8)
+        cpu_speed = random.randint(1000, 2000)
+        memory = random.randint(4096, 8192)
+        interfaces = [
+            RequestedMachineInterface()
+            for _ in range(3)
+        ]
+        block_devices = [
+            RequestedMachineBlockDevice(size=random.randint(512, 1024))
+            for _ in range(3)
+        ]
+        machine = RequestedMachine(
+            architecture='amd64/generic',
+            cores=cores, cpu_speed=cpu_speed, memory=memory,
+            interfaces=interfaces, block_devices=block_devices)
+        self.assertThat(machine.asdict(), MatchesDict({
+            "architecture": Equals("amd64/generic"),
+            "cores": Equals(cores),
+            "cpu_speed": Equals(cpu_speed),
+            "memory": Equals(memory),
+            "interfaces": MatchesListwise([
+                MatchesDict({})
+                for interface in interfaces
+            ]),
+            "block_devices": MatchesListwise([
+                MatchesDict({
+                    "size": Equals(block_device.size),
+                })
+                for block_device in block_devices
+            ]),
+        }))
+
+    def test_machine_fromdict(self):
+        cores = random.randint(1, 8)
+        cpu_speed = random.randint(1000, 2000)
+        memory = random.randint(4096, 8192)
+        interfaces = [
+            dict()
+            for _ in range(3)
+        ]
+        block_devices = [
+            dict(size=random.randint(512, 1024))
+            for _ in range(3)
+        ]
+        machine_data = dict(
+            architecture='amd64/generic',
+            cores=cores, cpu_speed=cpu_speed, memory=memory,
+            interfaces=interfaces, block_devices=block_devices)
+        machine = RequestedMachine.fromdict(machine_data)
+        self.assertThat(machine, IsInstance(RequestedMachine))
+        self.assertThat(machine, MatchesStructure(
+            architecture=Equals('amd64/generic'),
+            cores=Equals(cores),
+            cpu_speed=Equals(cpu_speed),
+            memory=Equals(memory),
+            interfaces=MatchesListwise([
+                IsInstance(RequestedMachineInterface)
+                for interface in interfaces
+            ]),
+            block_devices=MatchesListwise([
+                MatchesAll(
+                    IsInstance(RequestedMachineBlockDevice),
+                    MatchesStructure(
+                        size=Equals(block_device['size']),
+                    ),
+                )
+                for block_device in block_devices
+            ]),
+        ))
+
+
 class FakePodDriverBase(PodDriverBase):
 
     name = ""
@@ -420,7 +549,7 @@
     def discover(self, system_id, context):
         raise NotImplementedError
 
-    def compose(self, system_id, context):
+    def compose(self, system_id, context, request):
         raise NotImplementedError
 
     def decompose(self, system_id, context):
@@ -504,7 +633,8 @@
         fake_driver = make_pod_driver_base()
         self.assertRaises(
             NotImplementedError,
-            fake_driver.compose, sentinel.system_id, sentinel.context)
+            fake_driver.compose,
+            sentinel.system_id, sentinel.context, sentinel.request)
 
     def test_decompose_raises_not_implemented(self):
         fake_driver = make_pod_driver_base()

=== modified file 'src/provisioningserver/drivers/pod/tests/test_virsh.py'
--- src/provisioningserver/drivers/pod/tests/test_virsh.py	2017-01-26 00:07:40 +0000
+++ src/provisioningserver/drivers/pod/tests/test_virsh.py	2017-01-30 19:50:11 +0000
@@ -945,7 +945,8 @@
         driver = VirshPodDriver()
         self.assertRaises(
             NotImplementedError,
-            driver.compose, sentinel.system_id, sentinel.context)
+            driver.compose,
+            sentinel.system_id, sentinel.context, sentinel.request)
 
     def test_decompose_raises_not_implemented(self):
         driver = VirshPodDriver()

=== modified file 'src/provisioningserver/drivers/pod/virsh.py'
--- src/provisioningserver/drivers/pod/virsh.py	2017-01-26 23:43:32 +0000
+++ src/provisioningserver/drivers/pod/virsh.py	2017-01-30 19:50:11 +0000
@@ -628,13 +628,13 @@
         return discovered_pod
 
     @inlineCallbacks
-    def compose(self, system_id, context):
-        """Compose machine(s)."""
+    def compose(self, system_id, context, request):
+        """Compose machine."""
         raise NotImplementedError
 
     @inlineCallbacks
     def decompose(self, system_id, context):
-        """Decompose machine(s)."""
+        """Decompose machine."""
         raise NotImplementedError
 
 

=== modified file 'src/provisioningserver/rpc/arguments.py'
--- src/provisioningserver/rpc/arguments.py	2017-01-23 19:55:12 +0000
+++ src/provisioningserver/rpc/arguments.py	2017-01-31 16:10:58 +0000
@@ -222,3 +222,54 @@
         from provisioningserver.drivers.pod import DiscoveredPod
         data = super(AmpDiscoveredPod, self).fromString(inString)
         return DiscoveredPod.fromdict(data)
+
+
+class AmpDiscoveredPodHints(StructureAsJSON):
+    """Encode and decode `DiscoveredPodHints` over the wire."""
+
+    def toString(self, inObject):
+        # Circular imports.
+        from provisioningserver.drivers.pod import DiscoveredPodHints
+        if not isinstance(inObject, DiscoveredPodHints):
+            raise TypeError("%r is not of type DiscoveredPodHints." % inObject)
+        return super(AmpDiscoveredPodHints, self).toString(inObject.asdict())
+
+    def fromString(self, inString):
+        # Circular imports.
+        from provisioningserver.drivers.pod import DiscoveredPodHints
+        data = super(AmpDiscoveredPodHints, self).fromString(inString)
+        return DiscoveredPodHints.fromdict(data)
+
+
+class AmpDiscoveredMachine(StructureAsJSON):
+    """Encode and decode `DiscoveredMachine` over the wire."""
+
+    def toString(self, inObject):
+        # Circular imports.
+        from provisioningserver.drivers.pod import DiscoveredMachine
+        if not isinstance(inObject, DiscoveredMachine):
+            raise TypeError("%r is not of type DiscoveredMachine." % inObject)
+        return super(AmpDiscoveredMachine, self).toString(inObject.asdict())
+
+    def fromString(self, inString):
+        # Circular imports.
+        from provisioningserver.drivers.pod import DiscoveredMachine
+        data = super(AmpDiscoveredMachine, self).fromString(inString)
+        return DiscoveredMachine.fromdict(data)
+
+
+class AmpRequestedMachine(StructureAsJSON):
+    """Encode and decode `RequestedMachine` over the wire."""
+
+    def toString(self, inObject):
+        # Circular imports.
+        from provisioningserver.drivers.pod import RequestedMachine
+        if not isinstance(inObject, RequestedMachine):
+            raise TypeError("%r is not of type RequestedMachine." % inObject)
+        return super(AmpRequestedMachine, self).toString(inObject.asdict())
+
+    def fromString(self, inString):
+        # Circular imports.
+        from provisioningserver.drivers.pod import RequestedMachine
+        data = super(AmpRequestedMachine, self).fromString(inString)
+        return RequestedMachine.fromdict(data)

=== modified file 'src/provisioningserver/rpc/cluster.py'
--- src/provisioningserver/rpc/cluster.py	2017-01-26 22:02:18 +0000
+++ src/provisioningserver/rpc/cluster.py	2017-01-31 16:10:58 +0000
@@ -35,8 +35,11 @@
 
 from provisioningserver.rpc import exceptions
 from provisioningserver.rpc.arguments import (
+    AmpDiscoveredMachine,
     AmpDiscoveredPod,
+    AmpDiscoveredPodHints,
     AmpList,
+    AmpRequestedMachine,
     Bytes,
     CompressedAmpList,
     IPAddress,
@@ -638,6 +641,36 @@
     }
 
 
+class ComposeMachine(amp.Command):
+    """Compose a machine in a pod.
+
+    :since: 2.2
+    """
+    arguments = [
+        (b"pod_id", amp.Integer()),
+        (b"name", amp.Unicode()),
+        (b"type", amp.Unicode()),
+        # We can't define a tighter schema here because this is a highly
+        # variable bag of arguments from a variety of sources.
+        (b"context", StructureAsJSON()),
+        (b"request", AmpRequestedMachine()),
+    ]
+    response = [
+        (b"machine", AmpDiscoveredMachine()),
+        (b"hints", AmpDiscoveredPodHints()),
+    ]
+    errors = {
+        exceptions.UnknownPodType: (
+            b"UnknownPodType"),
+        NotImplementedError: (
+            b"NotImplementedError"),
+        exceptions.PodActionFail: (
+            b"PodActionFail"),
+        exceptions.PodInvalidResources: (
+            b"PodInvalidResources"),
+    }
+
+
 class ScanNetworks(amp.Command):
     """Requests an immediate scan of attached networks.
 

=== modified file 'src/provisioningserver/rpc/clusterservice.py'
--- src/provisioningserver/rpc/clusterservice.py	2017-01-26 22:02:18 +0000
+++ src/provisioningserver/rpc/clusterservice.py	2017-02-03 19:24:55 +0000
@@ -600,7 +600,18 @@
         :py:class:`~provisioningserver.rpc.cluster.DiscoverPod`.
         """
         return pods.discover_pod(
-            type, context, pod_id=id, name=name)
+            type, context, pod_id=pod_id, name=name)
+
+    @cluster.ComposeMachine.responder
+    def compose_machine(
+            self, type, context, request, pod_id, name):
+        """ComposeMachine()
+
+        Implementation of
+        :py:class:`~provisioningserver.rpc.cluster.ComposeMachine`.
+        """
+        return pods.compose_machine(
+            type, context, request, pod_id=pod_id, name=name)
 
     @cluster.ScanNetworks.responder
     def scan_all_networks(

=== modified file 'src/provisioningserver/rpc/exceptions.py'
--- src/provisioningserver/rpc/exceptions.py	2017-01-23 19:55:12 +0000
+++ src/provisioningserver/rpc/exceptions.py	2017-01-31 15:34:04 +0000
@@ -131,6 +131,11 @@
     """Raised when performing a pod action."""
 
 
+class PodInvalidResources(Exception):
+    """Raised when a pod cannot compose a machine because of
+    invalid resources."""
+
+
 class CannotRegisterCluster(Exception):
     """The cluster could not be registered."""
 

=== modified file 'src/provisioningserver/rpc/pods.py'
--- src/provisioningserver/rpc/pods.py	2017-01-23 19:55:12 +0000
+++ src/provisioningserver/rpc/pods.py	2017-02-03 17:53:48 +0000
@@ -8,7 +8,9 @@
 ]
 
 from provisioningserver.drivers.pod import (
+    DiscoveredMachine,
     DiscoveredPod,
+    DiscoveredPodHints,
     get_error_message,
 )
 from provisioningserver.drivers.pod.registry import PodDriverRegistry
@@ -18,6 +20,7 @@
 )
 from provisioningserver.rpc.exceptions import (
     PodActionFail,
+    PodInvalidResources,
     UnknownPodType,
 )
 from provisioningserver.utils.twisted import asynchronous
@@ -41,14 +44,18 @@
         raise UnknownPodType(pod_type)
     d = pod_driver.discover(pod_id, context)
     if not isinstance(d, Deferred):
-        raise PodActionFail("bad pod driver; did not return Deferred.")
+        raise PodActionFail(
+            "bad pod driver '%s'; 'discover' did not return Deferred." % (
+                pod_type))
 
     def convert(result):
         """Convert the result to send over RPC."""
         if result is None:
             raise PodActionFail("unable to discover pod information.")
         elif not isinstance(result, DiscoveredPod):
-            raise PodActionFail("bad pod driver; invalid result.")
+            raise PodActionFail(
+                "bad pod driver '%s'; 'discover' returned invalid result." % (
+                    pod_type))
         else:
             return {
                 "pod": result
@@ -67,3 +74,62 @@
     d.addCallback(convert)
     d.addErrback(catch_all)
     return d
+
+
+@asynchronous
+def compose_machine(pod_type, context, request, pod_id, name):
+    """Compose a machine that at least matches equal to or greater than
+    `request`.
+
+    The region controller handles parsing the outputed `DiscoveredMachine` and
+    updating the database as required.
+    """
+    pod_driver = PodDriverRegistry.get_item(pod_type)
+    if pod_driver is None:
+        raise UnknownPodType(pod_type)
+    d = pod_driver.compose(pod_id, context, request)
+    if not isinstance(d, Deferred):
+        raise PodActionFail(
+            "bad pod driver '%s'; 'compose' did not return Deferred." % (
+                pod_type))
+
+    def convert(result):
+        """Convert the result to send over RPC."""
+        if result is None:
+            # None is allowed when a machine could not be composed with the
+            # driver. This means it could not match the request. Returning None
+            # allows the region to try another pod if available to compose
+            # that machine.
+            raise PodInvalidResources()
+        else:
+            if (isinstance(result, tuple) and
+                    len(result) == 2 and
+                    isinstance(result[0], DiscoveredMachine) and
+                    isinstance(result[1], DiscoveredPodHints)):
+                return {
+                    "machine": result[0],
+                    "hints": result[1],
+                }
+            else:
+                raise PodActionFail(
+                    "bad pod driver '%s'; 'compose' returned "
+                    "invalid result." % pod_type)
+
+    def catch_all(failure):
+        """Convert all failures into `PodActionFail` unless already a
+        `PodActionFail`, `PodInvalidResources` or `NotImplementedError`."""
+        if failure.check(PodInvalidResources):
+            # Driver returned its own invalid resource exception instead of
+            # None. Just pass this onto the region.
+            return failure
+
+        # Log locally to help debugging.
+        log.err(failure, "%s: Failed to compose machine: %s" % (name, request))
+        if failure.check(NotImplementedError, PodActionFail):
+            return failure
+        else:
+            raise PodActionFail(get_error_message(failure.value))
+
+    d.addCallback(convert)
+    d.addErrback(catch_all)
+    return d

=== modified file 'src/provisioningserver/rpc/tests/test_arguments.py'
--- src/provisioningserver/rpc/tests/test_arguments.py	2017-01-23 19:55:12 +0000
+++ src/provisioningserver/rpc/tests/test_arguments.py	2017-02-02 20:33:52 +0000
@@ -12,8 +12,14 @@
 from maastesting.testcase import MAASTestCase
 import netaddr
 from provisioningserver.drivers.pod import (
+    DiscoveredMachine,
+    DiscoveredMachineBlockDevice,
+    DiscoveredMachineInterface,
     DiscoveredPod,
     DiscoveredPodHints,
+    RequestedMachine,
+    RequestedMachineBlockDevice,
+    RequestedMachineInterface,
 )
 from provisioningserver.rpc import arguments
 from testtools import ExpectedException
@@ -222,3 +228,80 @@
         self.assertThat(encoded, IsInstance(bytes))
         decoded = argument.fromString(encoded)
         self.assertThat(decoded, Equals(self.example))
+
+
+class TestDiscoveredPodHints(MAASTestCase):
+
+    example = DiscoveredPodHints(
+        cores=random.randint(1, 8),
+        cpu_speed=random.randint(1000, 2000),
+        memory=random.randint(1024, 8192), local_storage=0)
+
+    def test_round_trip(self):
+        argument = arguments.AmpDiscoveredPodHints()
+        encoded = argument.toString(self.example)
+        self.assertThat(encoded, IsInstance(bytes))
+        decoded = argument.fromString(encoded)
+        self.assertThat(decoded, Equals(self.example))
+
+
+class TestDiscoveredMachine(MAASTestCase):
+
+    example = DiscoveredMachine(
+        architecture='amd64/generic',
+        cores=random.randint(1, 8),
+        cpu_speed=random.randint(1000, 2000),
+        memory=random.randint(1024, 8192),
+        power_state=factory.make_name('unknown'),
+        power_parameters={
+            'power_id': factory.make_name('power_id'),
+        },
+        interfaces=[
+            DiscoveredMachineInterface(
+                mac_address=factory.make_mac_address())
+            for _ in range(3)
+        ],
+        block_devices=[
+            DiscoveredMachineBlockDevice(
+                model=factory.make_name("model"),
+                serial=factory.make_name("serial"),
+                size=random.randint(512, 1024),
+                id_path=factory.make_name("/dev/vda"))
+            for _ in range(3)
+        ],
+        tags=[
+            factory.make_name("tag")
+            for _ in range(3)
+        ])
+
+    def test_round_trip(self):
+        argument = arguments.AmpDiscoveredMachine()
+        encoded = argument.toString(self.example)
+        self.assertThat(encoded, IsInstance(bytes))
+        decoded = argument.fromString(encoded)
+        self.assertThat(decoded, Equals(self.example))
+
+
+class TestRequestedMachine(MAASTestCase):
+
+    example = RequestedMachine(
+        architecture='amd64/generic',
+        cores=random.randint(1, 8),
+        cpu_speed=random.randint(1000, 2000),
+        memory=random.randint(1024, 8192),
+        interfaces=[
+            RequestedMachineInterface()
+            for _ in range(3)
+        ],
+        block_devices=[
+            RequestedMachineBlockDevice(
+                size=random.randint(512, 1024))
+            for _ in range(3)
+        ])
+
+    def test_round_trip(self):
+        argument = arguments.AmpRequestedMachine()
+        encoded = argument.toString(self.example)
+        self.assertThat(encoded, IsInstance(bytes))
+        decoded = argument.fromString(encoded)
+        self.assertThat(decoded, Equals(self.example))

=== modified file 'src/provisioningserver/rpc/tests/test_clusterservice.py'
--- src/provisioningserver/rpc/tests/test_clusterservice.py	2017-01-26 22:02:18 +0000
+++ src/provisioningserver/rpc/tests/test_clusterservice.py	2017-02-03 17:53:48 +0000
@@ -61,8 +61,12 @@
     OperatingSystemRegistry,
 )
 from provisioningserver.drivers.pod import (
+    DiscoveredMachine,
     DiscoveredPod,
     DiscoveredPodHints,
+    RequestedMachine,
+    RequestedMachineBlockDevice,
+    RequestedMachineInterface,
 )
 from provisioningserver.drivers.power import PowerError
 from provisioningserver.drivers.power.registry import PowerDriverRegistry
@@ -2792,7 +2796,57 @@
             })
         self.assertThat(
             mock_discover_pod, MockCalledOnceWith(
-                pod_type, context, pod_id=id, name=name))
+                pod_type, context, pod_id=pod_id, name=name))
+
+
+class TestClusterProtocol_ComposeMachine(MAASTestCase):
+
+    def test__is_registered(self):
+        protocol = Cluster()
+        responder = protocol.locateResponder(
+            cluster.ComposeMachine.commandName)
+        self.assertIsNotNone(responder)
+
+    def test_calls_discover_pod(self):
+        mock_compose_machine = self.patch_autospec(
+            pods, 'compose_machine')
+        mock_compose_machine.return_value = succeed({
+            "machine": DiscoveredMachine(
+                architecture='amd64/generic',
+                cores=random.randint(1, 8),
+                cpu_speed=random.randint(1000, 3000),
+                memory=random.randint(1024, 8192),
+                block_devices=[], interfaces=[]),
+            "hints": DiscoveredPodHints(
+                cores=random.randint(1, 8),
+                cpu_speed=random.randint(1000, 2000),
+                memory=random.randint(1024, 8192), local_storage=0),
+            })
+        pod_type = factory.make_name('pod_type')
+        context = {
+            "data": factory.make_name("data"),
+        }
+        request = RequestedMachine(
+            architecture='amd64/generic',
+            cores=random.randint(1, 8),
+            cpu_speed=random.randint(1000, 3000),
+            memory=random.randint(1024, 8192),
+            block_devices=[
+                RequestedMachineBlockDevice(size=random.randint(8, 16))],
+            interfaces=[
+                RequestedMachineInterface()])
+        pod_id = random.randint(1, 100)
+        name = factory.make_name('pod')
+        call_responder(Cluster(), cluster.ComposeMachine, {
+            'type': pod_type,
+            'context': context,
+            'request': request,
+            'pod_id': pod_id,
+            'name': name,
+            })
+        self.assertThat(
+            mock_compose_machine, MockCalledOnceWith(
+                pod_type, context, request, pod_id=pod_id, name=name))
 
 
 class TestClusterProtocol_DisableAndShutoffRackd(MAASTestCase):

=== modified file 'src/provisioningserver/rpc/tests/test_pods.py'
--- src/provisioningserver/rpc/tests/test_pods.py	2017-01-23 19:55:12 +0000
+++ src/provisioningserver/rpc/tests/test_pods.py	2017-02-03 17:53:48 +0000
@@ -15,8 +15,12 @@
     MAASTwistedRunTest,
 )
 from provisioningserver.drivers.pod import (
+    DiscoveredMachine,
     DiscoveredPod,
     DiscoveredPodHints,
+    RequestedMachine,
+    RequestedMachineBlockDevice,
+    RequestedMachineInterface,
 )
 from provisioningserver.drivers.pod.registry import PodDriverRegistry
 from provisioningserver.rpc import (
@@ -50,7 +54,9 @@
             PodDriverRegistry, "get_item").return_value = fake_driver
         with ExpectedException(
                 exceptions.PodActionFail,
-                re.escape("bad pod driver; did not return Deferred.")):
+                re.escape(
+                    "bad pod driver '%s'; 'discover' did not "
+                    "return Deferred." % fake_driver.name)):
             yield pods.discover_pod(fake_driver.name, {})
 
     @inlineCallbacks
@@ -74,7 +80,9 @@
             PodDriverRegistry, "get_item").return_value = fake_driver
         with ExpectedException(
                 exceptions.PodActionFail,
-                re.escape("bad pod driver; invalid result.")):
+                re.escape(
+                    "bad pod driver '%s'; 'discover' returned "
+                    "invalid result." % fake_driver.name)):
             yield pods.discover_pod(fake_driver.name, {})
 
     @inlineCallbacks
@@ -124,3 +132,163 @@
                 exceptions.PodActionFail,
                 re.escape("Failed talking to pod: " + fake_exception_msg)):
             yield pods.discover_pod(fake_driver.name, {})
+
+
+class TestComposeMachine(MAASTestCase):
+
+    run_tests_with = MAASTwistedRunTest.make_factory(timeout=5)
+
+    def make_requested_machine(self):
+        return RequestedMachine(
+            architecture='amd64/generic',
+            cores=random.randint(1, 8),
+            cpu_speed=random.randint(1000, 3000),
+            memory=random.randint(1024, 8192),
+            block_devices=[
+                RequestedMachineBlockDevice(size=random.randint(8, 16))],
+            interfaces=[
+                RequestedMachineInterface()])
+
+    @inlineCallbacks
+    def test_unknown_pod_raises_UnknownPodType(self):
+        unknown_type = factory.make_name("unknown")
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        with ExpectedException(exceptions.UnknownPodType):
+            yield pods.compose_machine(
+                unknown_type, {}, fake_request, pod_id=pod_id, name=pod_name)
+
+    @inlineCallbacks
+    def test_handles_driver_not_returning_Deferred(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_driver.compose.return_value = None
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        with ExpectedException(
+                exceptions.PodActionFail,
+                re.escape(
+                    "bad pod driver '%s'; 'compose' did not "
+                    "return Deferred." % fake_driver.name)):
+            yield pods.compose_machine(
+                fake_driver.name, {}, fake_request,
+                pod_id=pod_id, name=pod_name)
+
+    @inlineCallbacks
+    def test_handles_driver_resolving_to_None(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_driver.compose.return_value = succeed(None)
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        with ExpectedException(exceptions.PodInvalidResources):
+            yield pods.compose_machine(
+                fake_driver.name, {}, fake_request,
+                pod_id=pod_id, name=pod_name)
+
+    @inlineCallbacks
+    def test_handles_driver_not_resolving_to_tuple(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_driver.compose.return_value = succeed({})
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        with ExpectedException(
+                exceptions.PodActionFail,
+                re.escape(
+                    "bad pod driver '%s'; 'compose' returned "
+                    "invalid result." % fake_driver.name)):
+            yield pods.compose_machine(
+                fake_driver.name, {}, fake_request,
+                pod_id=pod_id, name=pod_name)
+
+    @inlineCallbacks
+    def test_handles_driver_not_resolving_to_tuple_of_discovered(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_driver.compose.return_value = succeed((object(), object()))
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        with ExpectedException(
+                exceptions.PodActionFail,
+                re.escape(
+                    "bad pod driver '%s'; 'compose' returned "
+                    "invalid result." % fake_driver.name)):
+            yield pods.compose_machine(
+                fake_driver.name, {}, fake_request,
+                pod_id=pod_id, name=pod_name)
+
+    @inlineCallbacks
+    def test_handles_driver_resolving_to_tuple_of_discovered(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        machine = DiscoveredMachine(
+            architecture='amd64/generic',
+            cores=random.randint(1, 8),
+            cpu_speed=random.randint(1000, 3000),
+            memory=random.randint(1024, 8192),
+            block_devices=[], interfaces=[])
+        hints = DiscoveredPodHints(
+            cores=random.randint(1, 8),
+            cpu_speed=random.randint(1000, 2000),
+            memory=random.randint(1024, 8192), local_storage=0)
+        fake_driver.compose.return_value = succeed((machine, hints))
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        result = yield pods.compose_machine(
+            fake_driver.name, {}, fake_request, pod_id=pod_id, name=pod_name)
+        self.assertEquals({
+            "machine": machine,
+            "hints": hints,
+        }, result)
+
+    @inlineCallbacks
+    def test_handles_driver_raising_NotImplementedError(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_driver.compose.return_value = fail(NotImplementedError())
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        with ExpectedException(NotImplementedError):
+            yield pods.compose_machine(
+                fake_driver.name, {}, fake_request,
+                pod_id=pod_id, name=pod_name)
+
+    @inlineCallbacks
+    def test_handles_driver_raising_any_Exception(self):
+        fake_driver = MagicMock()
+        fake_driver.name = factory.make_name("pod")
+        fake_exception_type = factory.make_exception_type()
+        fake_exception_msg = factory.make_name("error")
+        fake_exception = fake_exception_type(fake_exception_msg)
+        fake_driver.compose.return_value = fail(fake_exception)
+        fake_request = self.make_requested_machine()
+        pod_id = random.randint(1, 10)
+        pod_name = factory.make_name("pod")
+        self.patch(
+            PodDriverRegistry, "get_item").return_value = fake_driver
+        with ExpectedException(
+                exceptions.PodActionFail,
+                re.escape("Failed talking to pod: " + fake_exception_msg)):
+            yield pods.compose_machine(
+                fake_driver.name, {}, fake_request,
+                pod_id=pod_id, name=pod_name)

=== modified file 'src/provisioningserver/templates/commissioning-user-data/snippets/maas_ipmi_autodetect.py'
--- src/provisioningserver/templates/commissioning-user-data/snippets/maas_ipmi_autodetect.py	2016-09-23 16:21:57 +0000
+++ src/provisioningserver/templates/commissioning-user-data/snippets/maas_ipmi_autodetect.py	2017-02-03 14:51:31 +0000
@@ -142,14 +142,34 @@
     bmc_set('Lan_Conf:IP_Address_Source', source)
 
 
+def _bmc_get_ipmi_addresses(address_type):
+    try:
+        return bmc_get(address_type)
+    except subprocess.CalledProcessError:
+        return ""
+
+
 def get_ipmi_ip_address():
-    output = bmc_get('Lan_Conf:IP_Address')
     show_re = re.compile(
         '((?:[0-9]{1,3}\.){3}[0-9]{1,3}|[0-9a-fA-F]*:[0-9a-fA-F:.]+)')
-    res = show_re.search(output)
-    if res is None:
-        return None
-    return res.group()
+    for address_type in [
+            'Lan_Conf:IP_Address',
+            'Lan6_Conf:IPv6_Static_Addresses',
+            'Lan6_Conf:IPv6_Dynamic_Addresses']:
+        output = _bmc_get_ipmi_addresses(address_type)
+        # Loop through the addreses by preference: IPv4, static IPv6, dynamic
+        # IPv6.  Return the first valid, non-link-local address we find.
+        # While we could conceivably allow link-local addresses, we would need
+        # to devine which of our interfaces is the correct link, and then we
+        # would need support for link-local addresses in freeipmi-tools.
+        res = show_re.findall(output)
+        for ip in res:
+            if ip.lower().startswith('fe80::') or ip == '0.0.0.0':
+                time.sleep(2)
+                continue
+            return ip
+    # No valid IP address was found.
+    return None
 
 
 def verify_ipmi_user_settings(user_number, user_settings):
@@ -281,15 +301,15 @@
 
     # get the IP address
     IPMI_IP_ADDRESS = get_ipmi_ip_address()
-    if IPMI_IP_ADDRESS == "0.0.0.0":
-        # if IPMI_IP_ADDRESS is 0.0.0.0, wait 60 seconds and retry.
+    if IPMI_IP_ADDRESS is None:
+        # if IPMI_IP_ADDRESS not set (or reserved), wait 60 seconds and retry.
         set_ipmi_network_source("Static")
         time.sleep(2)
         set_ipmi_network_source("Use_DHCP")
         time.sleep(60)
         IPMI_IP_ADDRESS = get_ipmi_ip_address()
 
-    if IPMI_IP_ADDRESS is None or IPMI_IP_ADDRESS == "0.0.0.0":
+    if IPMI_IP_ADDRESS is None:
         # Exit (to not set power params in MAAS) if no IPMI_IP_ADDRESS
         # has been detected
         exit(1)

=== modified file 'src/provisioningserver/templates/commissioning-user-data/snippets/tests/test_maas_ipmi_autodetect.py'
--- src/provisioningserver/templates/commissioning-user-data/snippets/tests/test_maas_ipmi_autodetect.py	2016-09-23 19:44:23 +0000
+++ src/provisioningserver/templates/commissioning-user-data/snippets/tests/test_maas_ipmi_autodetect.py	2017-02-01 22:53:37 +0000
@@ -497,19 +497,74 @@
 
     scenarios = [
         ('none', dict(
-            output='  IP_Address    \n\n', expected=None)),
+            output4='  IP_Address    \n\n',
+            output_st='', output_dy='', expected=None)),
         ('bogus', dict(
-            output='  IP_Address    bogus\n\n', expected=None)),
+            output4='  IP_Address    bogus\n\n',
+            output_st='', output_dy='', expected=None)),
         ('ipv4', dict(
-            output='  IP_Address    192.168.1.1\n\n', expected='192.168.1.1')),
-        ('ipv6', dict(
-            output='  IP_Address    2001:db8::3\n\n', expected='2001:db8::3')),
+            output4='  IP_Address    192.168.1.1\n\n',
+            output_st='', output_dy='', expected='192.168.1.1')),
+        ('ipv4-ipv6', dict(
+            output4='  IP_Address    192.168.1.1\n\n',
+            output_st='  IPv6_Static_Addresses 2001:db8::3\n\n',
+            output_dy='', expected='192.168.1.1')),
+        ('static6', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='  IPv6_Static_Addresses 2001:db8::3\n\n',
+            output_dy='',
+            expected='2001:db8::3')),
+        ('static6 multiple', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='  IPv6_Static_Addresses fe80::3:7 2001:db8::3\n\n',
+            output_dy='',
+            expected='2001:db8::3')),
+        ('mixed6', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='  IPv6_Static_Addresses 2001:db8::9:5\n\n',
+            output_dy='  ## IPv6_Dynamic_Addresses 2001:db8::3\n\n',
+            expected='2001:db8::9:5')),
+        ('dynamic6', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='',
+            output_dy='  ## IPv6_Dynamic_Addresses 2001:db8::3\n\n',
+            expected='2001:db8::3')),
+        ('dynamic6 with link-local', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='',
+            output_dy='  ## IPv6_Dynamic_Addresses fe80::3:7 2001:db8::3\n\n',
+            expected='2001:db8::3')),
+        ('dynamic6 multiple', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='',
+            output_dy=(
+                '  ## IPv6_Dynamic_Addresses fe80::7 2001:db8::3 2001::5\n\n'
+            ),
+            expected='2001:db8::3')),
         ('link-local', dict(
-            output='  IP_Address    fe80::3:7\n\n', expected='fe80::3:7')),
+            output4='',
+            output_st='',
+            output_dy='  ## IPv6_Dynamic_IP_Addresses  fe80::7 fe80::3:9\n\n',
+            expected=None)),
+        ('0.0.0.0', dict(
+            output4='  IP_Address    0.0.0.0\n\n',
+            output_st='',
+            output_dy='',
+            expected=None)),
     ]
 
     def test_get_ipmi_ip_address(self):
-        run_command = self.patch(maas_ipmi_autodetect, 'run_command')
-        run_command.return_value = self.output
+        ret_values = {
+            'Lan_Conf:IP_Address': self.output4,
+            'Lan6_Conf:IPv6_Static_Addresses': self.output_st,
+            'Lan6_Conf:IPv6_Dynamic_Addresses': self.output_dy
+        }
+
+        def ret_val(arg):
+            return ret_values[arg]
+
+        self.patch(
+            maas_ipmi_autodetect,
+            '_bmc_get_ipmi_addresses').side_effect = ret_val
         actual = get_ipmi_ip_address()
         self.assertEqual(self.expected, actual)

=== modified file 'versions.cfg'
--- versions.cfg	2017-01-23 09:19:14 +0000
+++ versions.cfg	2017-02-02 20:19:50 +0000
@@ -4,7 +4,7 @@
 collective.recipe.sphinxbuilder = 1.0
 Pygments = 2.1.3
 snowballstemmer = 1.2.1
-Sphinx = 1.5.1
+Sphinx = 1.3.6
 sphinx-rtd-theme = 0.1.9
 
 # Development and testing.

